/* -------------------------------------------------------
 *
 * (C)Copyright 2001,2006
 * International Business Machines Corporation,
 *
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of IBM Corporation nor the names of its
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Filename : ed_compare.c
 *
 * Purpose  : Emitter Data Routines
 *
 * ------------------------------------------------------- */

#include "common/mambo.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emitter/ed.h"

int ed_compare (EMIT_DATA *edata1, EMIT_DATA *edata2)
{
    switch (edata1->header.head.tag)
        {
        case TAG_HEADER:
        case TAG_FOOTER:
            return(memcmp(edata1, edata2, sizeof(HEADER)));
            break;
        case TAG_MEMDFAULT:
        case TAG_MEMWRITE:
        case TAG_MEMREAD:
            return(memcmp(edata1, edata2, sizeof(IO_INFO)));
            break;
        case TAG_INST:
            return(memcmp(edata1, edata2, sizeof(INST_INFO)));
            break;
        case TAG_L1I_CACHE_MISS:
        case TAG_L1D_CACHE_MISS:
        case TAG_L2_CACHE_MISS:
            return(memcmp(edata1, edata2, sizeof(CACHE_INFO)));
            break;
        case TAG_DISK_REQ:
            return(memcmp(edata1, edata2, sizeof(DISK_INFO)));
            break;
        case TAG_MSR:
            return(memcmp(edata1, edata2, sizeof(MSR_INFO)));
            break;
        case TAG_IDLE:
            return(memcmp(edata1, edata2, sizeof(IDLE_INFO)));
            break;
        case TAG_EXEC_LOAD:
        case TAG_EXEC_UNLOAD:
            return(memcmp(edata1, edata2, sizeof(LOAD_INFO)));
            break;
        case TAG_PID_KILL:
        case TAG_PID_RESUME:
        case TAG_PID_CREATE:
        case TAG_PID_EXECEXIT:
        case TAG_PID_EXEC:
            return(memcmp(edata1, edata2, sizeof(PID_INFO)));
            break;
        case TAG_CONFIG:
            return(memcmp(edata1, edata2, sizeof(CONFIG_INFO)));
            break;
        case TAG_CACHE_ADDR:
            return(memcmp(edata1, edata2, sizeof(CACHE_OP_INFO)));
            break;
        case TAG_SLB_MISS_INST:
        case TAG_SLB_MISS_DATA:
            return(memcmp(edata1, edata2, sizeof(SLB_INFO)));
            break;
        case TAG_ITLB_MISS:
        case TAG_DTLB_MISS:
        case TAG_TLB_MISS_INST:
        case TAG_TLB_MISS_DATA:
            return(memcmp(edata1, edata2, sizeof(TLB_INFO)));
            break;
        default:
            fprintf(stderr, "WARNING: ed_compare: Unknown tag of %d (0x%08X) encountered\n",
                    edata1->header.head.tag,
                    edata1->header.head.tag);
            return(edata1->header.head.tag != edata2->header.head.tag);
            break;
        }
    fprintf(stderr, "ERROR: ed_compare: shouldn't reach here\n");
    exit(-1);
}
