/* -------------------------------------------------------
 *
 * (C)Copyright 2001,2006
 * International Business Machines Corporation,
 *
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of IBM Corporation nor the names of its
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Filename : ed_print.c
 *
 * Purpose  : Emitter Data Print Routines
 *
 * ------------------------------------------------------- */

#include "common/mambo.h"
#include <stdio.h>                                /* FILE struct, fprintf() */
#include "emitter/ed.h"

int ed_print_good (FILE *out, EMIT_DATA *edata)
{
    switch (edata->header.head.tag)
        {
        case TAG_HEADER:
            fprintf(out,"%d: #"Format64Dec" HEADER\n", edata->io.head.cpu, edata->io.head.seqid);
            break;
        case TAG_FOOTER:
            fprintf(out,"%d: #"Format64Dec" FOOTER\n", edata->io.head.cpu, edata->io.head.seqid);
            break;
        case TAG_MEMDFAULT:
            fprintf(out,"%d: #"Format64Dec" MemDataFault: EA:"Format64" LENGTH:%d\n",
                    edata->io.head.cpu,
                    edata->io.head.seqid,
                    edata->io.eaddr,
                    edata->io.length);
            break;
        case TAG_MEMWRITE:
            fprintf(out,"%d: #"Format64Dec" MemWrite: EA:" Format64 " VSID:"Format64" PA:"Format64" LENGTH:%d VALUE="Format64"\n",
                    edata->io.head.cpu,
                    edata->io.head.seqid,
                    edata->io.eaddr,
                    edata->io.vsid_unshifted,
                    edata->io.paddr,
                    edata->io.length,
                    edata->io.value);

            break;
        case TAG_MEMREAD:
            fprintf(out,"%d: #"Format64Dec" MemRead: EA:"Format64" PA:"Format64" LENGTH:%d VALUE="Format64"\n",
                    edata->io.head.cpu,
                    edata->io.head.seqid,
                    edata->io.eaddr,
                    edata->io.paddr,
                    edata->io.length,
                    edata->io.value);
            break;
        case TAG_INST:
            fprintf(out,"%d: #"Format64Dec" CYCLE:"Format64Dec" INST: EA:"Format64" VSID:"Format64" PA:"Format64" INST="Format32"\n",
                    edata->inst.head.cpu,
                    edata->inst.head.seqid,
                    edata->inst.head.timestamp,
                    edata->inst.eaddr,
                    edata->inst.vsid_unshifted,
                    edata->inst.paddr,
                    edata->inst.inst);

            break;
        case TAG_L1I_CACHE_MISS:
            fprintf(out,"%d: #"Format64Dec" L1I CACHE MISS: EA:"Format64" PA:"Format64" IDX:%d CC=%d\n",
                    edata->cache.head.cpu,
                    edata->cache.head.seqid,
                    edata->cache.eaddr,
                    edata->cache.paddr,
                    edata->cache.index,
                    edata->cache.cc);
            break;
        case TAG_L1D_CACHE_MISS:
            fprintf(out,"%d: #"Format64Dec" L1D CACHE MISS: EA:"Format64" PA:"Format64" IDX:%d CC=%d\n",
                    edata->cache.head.cpu,
                    edata->cache.head.seqid,
                    edata->cache.eaddr,
                    edata->cache.paddr,
                    edata->cache.index,
                    edata->cache.cc);
            break;
        case TAG_L2_CACHE_MISS:
            fprintf(out,"%d: #"Format64Dec" L2 CACHE MISS: EA:"Format64" PA:"Format64" IDX:%d CC=%d\n",
                    edata->cache.head.cpu,
                    edata->cache.head.seqid,
                    edata->cache.eaddr,
                    edata->cache.paddr,
                    edata->cache.index,
                    edata->cache.cc);
            break;
        case TAG_DISK_REQ:
            fprintf(out,"%d: #"Format64Dec" DISK-REQ: OP:%s SECTORS:%d SECTOR:%d OFFSET:%d\n",
                    edata->disk.head.cpu,
                    edata->disk.head.seqid,
                    (edata->disk.operation == E_DISK_WRITE ? "WRITE" :
                     (edata->disk.operation == E_DISK_READ ? "READ" : "SYSTEM")),
                    edata->disk.sectors,
                    edata->disk.sector,
                    edata->disk.offset);

            break;
        case TAG_MSR:
            fprintf(out,"%d: #"Format64Dec" MSR: "Format64Dec"\n",
                    edata->msr.head.cpu,
                    edata->msr.head.seqid,
                    edata->msr.value);

            break;
        case TAG_IDLE:
            fprintf(out,"%d: #"Format64Dec" IDLE_RANGE: "Format64" <-> "Format64"\n",
                    edata->idle.head.cpu,
                    edata->idle.head.seqid,
                    edata->idle.start,
                    edata->idle.end);
            break;
        case TAG_EXEC_LOAD:
            fprintf(out,"%d: #"Format64Dec" EXEC_LOAD: PID:"FormatPid" PATHNAME:'%s' TXT_EA:"Format64" TXT_VSID:"Format64" TXT_LEN:%d DATA_EA:"Format64" DATA_VSID:"Format64" DATA_LEN:%d\n",
                    edata->load.head.cpu,
                    edata->load.head.seqid,
                    edata->load.pid,
                    edata->load.pathname,
                    edata->load.text_ea,
                    edata->load.text_vsid,
                    edata->load.text_len,
                    edata->load.data_ea,
                    edata->load.data_vsid,
                    edata->load.data_len);
            break;
        case TAG_EXEC_UNLOAD:
            fprintf(out,"%d: #"Format64Dec" EXEC_UNLOAD: PID:"FormatPid" PATHNAME:'%s' TXT_EA:"Format64" TXT_VSID:"Format64" TXT_LEN:%d DATA_EA:"Format64" DATA_VSID:"Format64" DATA_LEN:%d\n",
                    edata->load.head.cpu,
                    edata->load.head.seqid,
                    edata->load.pid,
                    edata->load.pathname,
                    edata->load.text_ea,
                    edata->load.text_vsid,
                    edata->load.text_len,
                    edata->load.data_ea,
                    edata->load.data_vsid,
                    edata->load.data_len);
            break;
        case TAG_PID_KILL:
            fprintf(out,"%d: #"Format64Dec" PID_KILL: PID:"FormatPid"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid);
            break;
        case TAG_PID_CREATE:
            fprintf(out,"%d: #"Format64Dec" PID_CREATE: PID:"FormatPid" FLAG:%d TIMESTAMP: "Format64Dec"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.flag,
                    edata->pid.head.timestamp);
            break;
        case TAG_KERNEL_THREAD_CREATE:
            fprintf(out,"%d: #"Format64Dec" KERNEL_THREAD_CREATE: PID:"FormatPid" FLAG:%d TIMESTAMP: "Format64Dec"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.flag,
                    edata->pid.head.timestamp);
            break;
        case TAG_PID_EXECEXIT:
            fprintf(out,"%d: #"Format64Dec" PID_EXECEXIT: PID:"FormatPid" timestamp: "Format64Dec"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.head.timestamp);
            break;
        case TAG_PID_EXEC:
            fprintf(out,"%d: #"Format64Dec" PID_EXEC: PID:"FormatPid" PATHNAME:%s TIMESTAMP: "Format64Dec"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.pathname,
                    edata->pid.head.timestamp);
            break;
        case TAG_PID_RESUME:
            fprintf(out,"%d: #"Format64Dec" PID_RESUME: PID:"FormatPid" TID:%d TIMESTAMP: "Format64Dec"\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.tid,
                    edata->pid.head.timestamp);
            break;
        case TAG_FIT_INTERRUPT:
            fprintf(out,"%d: #"Format64Dec" FIT_INTERRUPT: eaddr: " Format64 " inst_form: %d ins:" Format32 " length:%d\n",
                    edata->interrupt.head.cpu,
                    edata->interrupt.head.seqid,
                    edata->interrupt.eaddr,
                    edata->interrupt.inst_form,
                    edata->interrupt.ins,
                    edata->interrupt.length);
            break;
        case TAG_PIT_INTERRUPT:
            fprintf(out,"%d: #"Format64Dec" PIT_INTERRUPT: EA:"Format64"  inst_form: %d ins:" Format32 " length:%d\n",
                    edata->interrupt.head.cpu,
                    edata->interrupt.head.seqid,
                    edata->interrupt.eaddr,
                    edata->interrupt.inst_form,
                    edata->interrupt.ins,
                    edata->interrupt.length);
            break;
        case TAG_EXTERNAL_INTERRUPT:
            fprintf(out,"%d: #"Format64Dec" EXTERNAL_INTERRUPT: EA:"Format64"  inst_form: %d ins:" Format32 " length:%d\n",
                    edata->interrupt.head.cpu,
                    edata->interrupt.head.seqid,
                    edata->interrupt.eaddr,
                    edata->interrupt.inst_form,
                    edata->interrupt.ins,
                    edata->interrupt.length);
            break;
        case TAG_ALIGN_EXCEPTION:
            fprintf(out,"%d: #"Format64Dec" ALIGN_EXCEPT: EA:"Format64"  inst_form: %d ins:" Format32 " length:%d\n",
                    edata->interrupt.head.cpu,
                    edata->interrupt.head.seqid,
                    edata->interrupt.eaddr,
                    edata->interrupt.inst_form,
                    edata->interrupt.ins,
                    edata->interrupt.length);
            break;
        case TAG_IPAGE_MISS:
            fprintf(out,"%d: #"Format64Dec" IPAGE_MISS: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->page.eaddr);
            break;
        case TAG_ITLB_HIT:
            fprintf(out,"%d: #"Format64Dec" ITLB HIT : EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_ITLB_MISS:
            fprintf(out,"%d: #"Format64Dec" ITLB MISS : EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_DTLB_HIT:
            fprintf(out,"%d: #"Format64Dec" DTLB HIT : EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_DTLB_MISS:
            fprintf(out,"%d: #"Format64Dec" DTLB MISS : EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_TLB_HIT_INST:
            fprintf(out,"%d: #"Format64Dec" TLB HIT INST: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_TLB_MISS_INST:
            fprintf(out,"%d: #"Format64Dec" TLB MISS INST: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_TLB_HIT_DATA:
            fprintf(out,"%d: #"Format64Dec" TLB HIT DATA: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_TLB_MISS_DATA:
            fprintf(out,"%d: #"Format64Dec" TLB MISS DATA: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->tlb.ea);
            break;
        case TAG_SLB_MISS_INST:
            fprintf(out,"%d: #"Format64Dec" SLB MISS INST: EADDR:"Format64" INDEX:"Format32"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->slb.ea,
                    edata->slb.slb_index);
            break;
        case TAG_SLB_MISS_DATA:
            fprintf(out,"%d: #"Format64Dec" SLB MISS DATA: EADDR:"Format64" INDEX:"Format32"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->slb.ea,
                    edata->slb.slb_index);
            break;
        case TAG_DPAGE_MISS:
            fprintf(out,"%d: #"Format64Dec" DPAGE_MISS: EADDR:"Format64"\n",
                    edata->page.head.cpu,
                    edata->page.head.seqid,
                    edata->page.eaddr);
            break;
        case TAG_CONFIG:
            fprintf(out,"%d: #"Format64Dec" CONFIG: CPUS:%d MEM_SIZE:"Format64Dec" CLOCK_FREQ:"Format64Dec" DISK_LAT:%d CPI:%f L1D_BLK_SIZE:%d L1I_BLK_SIZE:%d\n",
                    edata->config.head.cpu,
                    edata->config.head.seqid,
                    edata->config.cpus,
                    edata->config.mem_size,
                    edata->config.clock_frequency,
                    edata->config.disk_latency,
                    edata->config.cpi,
                    edata->config.l1d_block_size,
                    edata->config.l1i_block_size);
            break;
        case TAG_CACHE_ADDR:
            fprintf(out,"%d: #"Format64Dec" Cache Addr: EA:"Format64" VSID:"Format64" PA:"Format64"\n",
                    edata->cache_op.head.cpu,
                    edata->cache_op.head.seqid,
                    edata->cache_op.eaddr,
                    edata->cache_op.vsid_unshifted,
                    edata->cache_op.paddr);
            break;
        case TAG_CACHE_I_INVALIDATE:
            fprintf(out,"%d: #"Format64Dec" Cache I Invalidate: EA:"Format64" VSID:"Format64" PA:"Format64"\n",
                    edata->cache_op.head.cpu,
                    edata->cache_op.head.seqid,
                    edata->cache_op.eaddr,
                    edata->cache_op.vsid_unshifted,
                    edata->cache_op.paddr);
            break;
        case TAG_CACHE_D_FLUSH:
            fprintf(out,"%d: #"Format64Dec" Cache D Flush: EA:"Format64" VSID:"Format64" PA:"Format64"\n",
                    edata->cache_op.head.cpu,
                    edata->cache_op.head.seqid,
                    edata->cache_op.eaddr,
                    edata->cache_op.vsid_unshifted,
                    edata->cache_op.paddr);
            break;
        case TAG_TID_CREATE:
            fprintf(out,"%d: #"Format64Dec" TID_CREATE: PID:"FormatPid" TID:%d FLAG:%d\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.tid,
                    edata->pid.flag);
            break;
        case TAG_TID_KILL:
            fprintf(out,"%d: #"Format64Dec" TID_KILL: PID:"FormatPid" TID:%d\n",
                    edata->pid.head.cpu,
                    edata->pid.head.seqid,
                    edata->pid.pid,
                    edata->pid.tid);
            break;
            /* mco Idle enter/exit cases */
        case TAG_IDLE_ENTER:
            fprintf(out,"%d: #"Format64Dec" IDLE_ENTER: "Format64"\n",
                    edata->idle_enter.head.cpu,
                    edata->idle_enter.head.seqid,
                    edata->idle_enter.head.timestamp);
            break;
        case TAG_IDLE_EXIT:
            fprintf(out,"%d: #"Format64Dec" IDLE_EXIT: "Format64"\n",
                    edata->idle_exit.head.cpu,
                    edata->idle_exit.head.seqid,
                    edata->idle_exit.head.timestamp);
            break;
        case TAG_SUPERVISOR_MODE:
            fprintf(out,"%d: #"Format64Dec" ENTER_SUPERVISOR_MODE: "Format64"\n",
                    edata->enter_super_mode.head.cpu,
                    edata->enter_super_mode.head.seqid,
                    edata->enter_super_mode.head.timestamp);
            break;
        case TAG_PROBLEM_STATE_MODE:
            fprintf(out,"%d: #"Format64Dec" EXIT_SUPERVISOR_MODE: "Format64"\n",
                    edata->exit_super_mode.head.cpu,
                    edata->exit_super_mode.head.seqid,
                    edata->exit_super_mode.head.timestamp);
            break;
        case TAG_STATS_FILENAME:
            fprintf(out,"%d: stats filename = %s\n",edata->path.head.cpu,edata->path.pathname);
            break;
        case TAG_START_STATS:
            fprintf(out,"%d: start stats\n",edata->exit_super_mode.head.cpu);
            break;
        case TAG_STOP_STATS:
            fprintf(out,"%d: stop stats\n",edata->header.head.cpu);
            break;
        case TAG_DUMP_STATS:
            fprintf(out,"%d: dump stats\n",edata->header.head.cpu);
            break;
        default:
            fprintf(out,"Unknown tag of %d (0x%08X) encountered\n",
                    edata->header.head.tag,
                    edata->header.head.tag);
            return(0);
            break;
        }
    return(1);
}
