/* -------------------------------------------------------
 *
 * (C)Copyright 2001,2007
 * International Business Machines Corporation,
 *
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of IBM Corporation nor the names of its
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Filename : reader.c
 *
 * Purpose  : Dummy emitter reader
 *
 * ------------------------------------------------------- */

#include "common/mambo.h"
#include "emitter/emit_collector.h"



EMITTER_CONTROLS eControl;
int reader;

int main (int argc, char **argv)
{
    int end = 0;
    int pid = 0;
    EMIT_DATA *ePtr;


    printf("Reader Started\n");

    if (argc < 2)
        {
            printf("ERROR: Usage: %s simulator-process-id \n", argv[0]);
            exit(-1);
        }

    pid = atoi(argv[1]);

    /* Attach Reader to simulator */
    reader = EMIT_AttachReader(pid, &eControl, 0, TRUE);
    if (reader == -1)
        {
            fprintf(stderr, "ERROR: Failed to attach reader\n");
            return(0);
        }

    EMIT_ReadyWait(&eControl, reader, TRUE);

    while (!end)
        {
            EMIT_AdvanceCurrent(&eControl, reader);
            ePtr = EMIT_CurrentEntry(&eControl, reader);
            /* only print the test cases */
            switch (ePtr->header.head.tag)
                {
                case TAG_HEADER:
                case TAG_FOOTER:
                case TAG_MEMWRITE:
                case TAG_MEMREAD:
                case TAG_INST:
                case TAG_L1I_CACHE_MISS:
                case TAG_L1D_CACHE_MISS:
                case TAG_L2_CACHE_MISS:
                case TAG_DISK_REQ:
                case TAG_MSR:
                case TAG_IDLE:
                case TAG_EXEC_LOAD:
                case TAG_EXEC_UNLOAD:
                case TAG_PID_KILL:
                case TAG_PID_CREATE:
                case TAG_PID_EXECEXIT:
                case TAG_PID_EXEC:
                case TAG_PID_RESUME:
                case TAG_IPAGE_MISS:
                case TAG_DPAGE_MISS:
                case TAG_CONFIG:
                case TAG_MEMDFAULT:
                case TAG_CACHE_ADDR:
                case TAG_CACHE_D_FLUSH:
                case TAG_CACHE_I_INVALIDATE:
                case TAG_TID_CREATE:
                case TAG_TID_KILL:
                case TAG_JIT_ADD:
                case TAG_JIT_REMOVE:
                case TAG_ERAT_INIT:
                case TAG_ERAT_INVALIDATE:
                case TAG_ERAT_FIND_MAPPING:
                case TAG_ERAT_ADD_MAPPING:
                case TAG_ERAT_SEGTAB_LOOKUP:
                case TAG_ERAT_PAGETAB_LOOKUP:
                case TAG_ITLB_MISS:
                case TAG_DTLB_MISS:
                case TAG_TLB_MISS_INST:
                case TAG_TLB_MISS_DATA:
                case TAG_SLB_MISS_INST:
                case TAG_SLB_MISS_DATA:
                case TAG_IDLE_ENTER:
                case TAG_IDLE_EXIT:
                case TAG_SUPERVISOR_MODE:
                case TAG_PROBLEM_STATE_MODE:
                case TAG_KERNEL_THREAD_CREATE:
                case TAG_FIT_INTERRUPT:
                case TAG_PIT_INTERRUPT:
                case TAG_ALIGN_EXCEPTION:
                case TAG_EXTERNAL_INTERRUPT:
                case TAG_L1D_CACHE_HIT:
                case TAG_L1I_CACHE_HIT:
                case TAG_L2_CACHE_HIT:
                case TAG_DTLB_HIT:
                case TAG_ITLB_HIT:
                case TAG_TLB_HIT_INST:
                case TAG_TLB_HIT_DATA:
                case TAG_SLB_HIT:
                case TAG_KERNEL_FINISHED_LOADING:
                case TAG_START_STATS:
                case TAG_STOP_STATS:
                case TAG_DUMP_STATS:
                case TAG_STATS_FILENAME:
                    ed_print_good(stdout,ePtr);
                default:
                    /* nada */ ;
                }
            if (ePtr->header.head.tag == TAG_FOOTER)
                end = 1;
        }

    EMIT_Quit(&eControl, reader, TRUE);

    printf("Reader Terminating\n");

    return(1);
}
