/* -------------------------------------------------------
 *
 * (C)Copyright 2001,2007
 * International Business Machines Corporation,
 *
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of IBM Corporation nor the names of its
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Filename : spu-ttrace.c
 *
 * Purpose  : Simple reader to collect transaction traces
 *
 * ------------------------------------------------------- */

#define MCONFIG_SPU                            1
#define MCONFIG_STI_EMITTER                    1

#include "common/mambo.h"
#include "emitter/emit_collector.h"
#include "trans_id.h"
#include <memory.h>

EMITTER_CONTROLS eControl;
int reader;

int main (int argc, char **argv)
{
    int         end = 0;
    int         pid = 0;
    EMIT_DATA   *ePtr = NULL;
    FILE        *fp = NULL;

    printf("Reader Started\n");

    if (argc < 3)
        {
            printf("ERROR: Usage: %s simulator-process-id csv-dump-file\n", argv[0]);
            exit(-1);
        }

    pid = atoi(argv[1]);

    /* Open the output file */
    if ((fp = fopen(argv[2], "w")) == NULL)
        {
            fprintf(stderr, "Cannot open file '%s'\n", argv[2]);
            exit(-1);
        }

    /* Attach Reader to simulator */
    reader = EMIT_AttachReader(pid, &eControl, 0, TRUE);
    if (reader == -1)
        {
            fprintf(stderr, "ERROR: Failed to attach reader\n");
            return(0);
        }

    EMIT_ReadyWait(&eControl, reader, TRUE);

    do
        {
            EMIT_AdvanceCurrent(&eControl, reader);
            ePtr = EMIT_CurrentEntry(&eControl, reader);
            /* only print the test cases */
            switch (ePtr->header.head.tag)
                {
                case TAG_TRANSACTION:
                    {
                        char id[32];
                        TRID_ToString(&(ePtr->trans.id), id, sizeof(id));
                        fprintf(fp, "[%lld]", ePtr->trans.head.timestamp);
                        fprintf(fp, "%s> '%s' at '%s' (sid=%d, addr=%llx, len=%d) '%s'\n",
                                id,
                                ePtr->trans.desc,
                                ePtr->trans.where,
                                ePtr->trans.src_id,
                                ePtr->trans.addr,
                                ePtr->trans.length,
                                ePtr->trans.text);
                    }
                    break;

                case TAG_APU_OP:
                    {
                        fprintf(fp, "[%lld]SPU(%d)::%s> '%s'\n",
                                ePtr->apu_op.cycle, ePtr->apu_op.apu,
                                ePtr->apu_op.unit, ePtr->apu_op.text);
                    }
                    break;

                case TAG_APU_PIPE_TRANS:
                    {
                        char id[32];
                        TRID_ToString(&(ePtr->pipe_trans.id.trans), id, sizeof(id));
                        fprintf(fp, "[%lld] ", ePtr->pipe_trans.head.timestamp);
                        fprintf(fp, "(trid:%s,be:%d,spu:%d) a=%llx p=%s s=%s '%s'\n",
                                id,
                                ePtr->pipe_trans.id.be,
                                ePtr->pipe_trans.id.spu,
                                ePtr->pipe_trans.addr,
                                ePtr->pipe_trans.pipe,
                                ePtr->pipe_trans.stage,
                                ePtr->pipe_trans.text);
                    }
                    break;

                case TAG_APU_PERF_STATS:
                    {
                        fprintf(fp, "[%lld]SPU(%d)--\n",
                                ePtr->perf_stats.head.timestamp,
                                ePtr->perf_stats.apu);
                        fprintf(fp, "\tinst_count            :%lld\n",
                                ePtr->perf_stats.inst_count);
                        fprintf(fp, "\tcyc_count             :%lld\n",
                                ePtr->perf_stats.cyc_count);
                        fprintf(fp, "\tnop_count             :%lld\n",
                                ePtr->perf_stats.nop_count);

                        fprintf(fp, "\tbranch.taken          :%lld\n",
                                ePtr->perf_stats.branch.taken);
                        fprintf(fp, "\tbranch.not_taken      :%lld\n",
                                ePtr->perf_stats.branch.not_taken);
                        fprintf(fp, "\tbranch.mispred_stall  :%lld\n",
                                ePtr->perf_stats.branch.mispred_stall);

                        fprintf(fp, "\thint.count            :%lld\n",
                                ePtr->perf_stats.hint.count);
                        fprintf(fp, "\thint.hit              :%lld\n",
                                ePtr->perf_stats.hint.hit);
                        fprintf(fp, "\thint.stall            :%lld\n",
                                ePtr->perf_stats.hint.stall);

                        fprintf(fp, "\tprefetch.ls_contention:%lld\n",
                                ePtr->perf_stats.prefetch.ls_contention);
                        fprintf(fp, "\tprefetch.miss_stall   :%lld\n",
                                ePtr->perf_stats.prefetch.miss_stall);

                        fprintf(fp, "\tchannel.stall         :%lld\n",
                                ePtr->perf_stats.channel.stall);

                        fprintf(fp, "\tissue.sp_block        :%lld\n",
                                ePtr->perf_stats.issue.sp_block);
                        fprintf(fp, "\tissue.dp_block        :%lld\n",
                                ePtr->perf_stats.issue.dp_block);

                        fprintf(fp, "\texec.single           :%lld\n",
                                ePtr->perf_stats.exec.single);
                        fprintf(fp, "\texec.dual             :%lld\n",
                                ePtr->perf_stats.exec.dual);
                        fprintf(fp, "\texec.nop              :%lld\n",
                                ePtr->perf_stats.exec.nop);
                        fprintf(fp, "\texec.dep_stall        :%lld\n",
                                ePtr->perf_stats.exec.dep_stall);
                        fprintf(fp, "\texec.siss_stall       :%lld\n",
                                ePtr->perf_stats.exec.siss_stall);

                        fprintf(fp, "\tsbi.contention        :%lld\n",
                                ePtr->perf_stats.sbi.contention);
                    }
                    break;

                case TAG_APU_PERF_PIPE_STATS:
                    {
                        fprintf(fp, "[%lld]SPU(%d)--",
                                ePtr->pipe_stats.head.timestamp,
                                ePtr->pipe_stats.apu);
                        fprintf(fp, "'%s': inst=%lld, dep_stall=%lld, siss_stall=%lld\n",
                                ePtr->pipe_stats.pipe_name,
                                ePtr->pipe_stats.inst_count,
                                ePtr->pipe_stats.dep_stall,
                                ePtr->pipe_stats.siss_stall);
                    }
                    break;

                default:
                    /* Ignore */
                    break;
                }

            if (ePtr->header.head.tag == TAG_FOOTER)
                end = 1;
        }
    while (!end);

    EMIT_Quit(&eControl, reader, TRUE);

    printf("Reader Terminating\n");
    fclose(fp);

    return(1);
}
