/* -------------------------------------------------------
 *
 * (C)Copyright 2001,2007
 * International Business Machines Corporation,
 *
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of IBM Corporation nor the names of its
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Filename : sti-annoperf.cpp
 *
 * Purpose  : Simple reader to show performance stats according to annotations
 *
 * ------------------------------------------------------- */

#define MCONFIG_SPU                            1
#define MCONFIG_STI_EMITTER                    1

#include "environment.h"

#ifndef MAMBO_CPP
extern "C" {
#endif /* #ifndef MAMBO_CPP */
#include "common/mambo.h"
#include "emitter/emit_collector.h"
#include <memory.h>
#ifndef MAMBO_CPP
}
#endif /* #ifndef MAMBO_CPP */

#include <iostream>
#include <string>
#include <map>

using namespace std;

typedef map<string, unsigned long long>  RegionMap;
typedef RegionMap::iterator                    RegionMapIter;
typedef RegionMap::const_iterator              RegionMapConstIter;

static RegionMap                               region_map;

EMITTER_CONTROLS eControl;
int reader;

int main (int argc, char **argv)
{
    int         end = 0;
    int         pid = 0;
    EMIT_DATA   *ePtr = NULL;

    printf("Reader Started\n");

    if (argc < 2)
        {
            printf("ERROR: Usage: %s simulator-process-id\n", argv[0]);
            exit(-1);
        }

    pid = atoi(argv[1]);

    /* Attach Reader to simulator */
    reader = EMIT_AttachReader(pid, &eControl, 0, TRUE);
    if (reader == -1)
        {
            fprintf(stderr, "ERROR: Failed to attach reader\n");
            return(0);
        }

    EMIT_ReadyWait(&eControl, reader, TRUE);

    string curr_region("Unspecified");

    do
        {
            EMIT_AdvanceCurrent(&eControl, reader);
            ePtr = EMIT_CurrentEntry(&eControl, reader);
            /* only print the test cases */
            switch (ePtr->header.head.tag)
                {
                case TAG_APU_ANNOTATE_STATS:
                    curr_region = ePtr->apu_annotation.text;
                    break;

                case TAG_APU_PERF_STATS:
                    region_map[curr_region] += ePtr->perf_stats.cyc_count;
                    break;

                default:
                    /* Ignore */
                    break;
                }

            if (ePtr->header.head.tag == TAG_FOOTER)
                end = 1;
        }
    while (!end);

    EMIT_Quit(&eControl, reader, TRUE);

    // Dump the cycle counts per region
    for (RegionMapIter i = region_map.begin(); i != region_map.end(); ++i)
        {
            cout << "Region: '" << i->first << "', Cycles: " << i->second << endl;
        }

    printf("Reader Terminating\n");

    return(1);
}
