/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2007
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : external.h
 *
 * Purpose  : Definition of all external procedures
 *
 *
 * ------------------------------------------------------- */

/* we depend upon "mambo.h" being the first file included in every *.c file;
 * check that it has been included */
#if !defined(_MAMBO_H_) && defined(MAMBO)
#error "common/mambo.h" not included first
#endif /* #if !defined(_MAMBO_H_) && defined(MAMBO) */



#ifndef _EXTERNAL_H_
#define _EXTERNAL_H_


/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

/* To determine what Operating System we are running:
 *
 *   __APPLE__ for Macintosh Darwin, Tiger, Panther, ...
 *   _AIX      for IBM's AIX
 *   __linux__ for Linux
 *
 */

/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

/* Determine which compiler we have. */
/* We have two compilers in mind:  gcc and xlc */
/* Define additional ones if need be. */
#undef HAVE_GCC
#undef HAVE_XLC

#ifdef __GNUC__
#define HAVE_GCC
#endif /* #ifdef __GNUC__ */
#ifdef GCC
#define HAVE_GCC
#endif /* #ifdef GCC */


#ifdef __IBMC__
#define HAVE_XLC
#endif /* #ifdef __IBMC__ */
#if defined(__xlC__)
#define HAVE_XLC
#endif /* #if defined(__xlC__) */
#ifdef __xlc__
#define HAVE_XLC
#endif /* #ifdef __xlc__ */

/* Separately, you can check cplusplus for C++ or g++ or xlC */

/* We want to use INT32_MIN, INT32_MAX, and INT_LEAST32_MAX.
 * These are defined in /usr/include/stdint.h, but, some times,
 * they are skipped over unless __STDC_LIMIT_MACROS is defined.
 * So we define it.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS   1
#endif /* #ifndef __STDC_LIMIT_MACROS */


/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

#ifdef MAMBO
/* Configure should have produced a config.h file that defines
   the properties of the system we are running on.  Include
   that file, so we can use that throughout Mambo. */
#include "../config/config.h"

#ifndef HAVE_UINT
typedef unsigned int uint;
#define HAVE_UINT 1
#endif /* #ifndef HAVE_UINT */

#ifdef HAVE_OPENPTY
#ifdef __APPLE__
#include <util.h>
#else /* #ifdef __APPLE__ */
#include <pty.h>
#endif /* #ifdef __APPLE__ */
#endif /* #ifdef HAVE_OPENPTY */

#endif /* #ifdef MAMBO */


/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

/* The _LONGLONG is supposed to signal that we have the 64-bit
 *  "long long" type for gcc */
#ifndef _LONGLONG
#define _LONGLONG 1
#endif /* #ifndef _LONGLONG */


/* The _LONG_LONG is supposed to signal that we have the 64-bit
 * "long long" type for xlc */
#ifndef _LONG_LONG
#define _LONG_LONG 1
#endif /* #ifndef _LONG_LONG */

/* _LARGE_FILES ...redefines all of the relevant data types,
   structures, and subroutine names to their large-file enabled
   counterparts, so we can access at least full 4G files. */

/* g++ on AIX doesn't work with _LARGE_FILES */
#if defined(_AIX) && defined(HAVE_GCC)
#ifndef __cplusplus
/* Don't set _LARGE_FILES if _LARGE_FILE_API is defined -- this will
   cause compile errors on AIX */
#ifndef _LARGE_FILE_API
#define _LARGE_FILES 1
#endif /* #ifndef _LARGE_FILE_API */
#else /* #ifndef __cplusplus */
#undef _LARGE_FILES
#endif /* #ifndef __cplusplus */
#endif /* #if defined(_AIX) && defined(HAVE_GCC) */

/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

/* Some systems (like Linux) provide 64-bit interfaces to
   files -- this particularly affects stat and mmap.  Other
   systems (like Apple's OS/X) don't have separate 64-bit
   functions.  */

#if defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64)
#define API64
#endif /* #if defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64) */

#if defined(API64) && !defined(_LARGEFILE64_SOURCE)
#define _LARGEFILE64_SOURCE   /* to get struct stat64 */
#endif /* #if defined(API64) && !defined(_LARGEFILE64_SOURCE) */


#include <errno.h>
#include <stddef.h>
#include <stdlib.h>

#ifdef _AIX
#include <unistd.h>
#else /* #ifdef _AIX */
#include <sys/unistd.h>
#endif /* #ifdef _AIX */

#include <sys/types.h>

#include <grp.h>
#include <limits.h>
#include <stdarg.h>

#include <signal.h>

#include <fcntl.h>
#include <termios.h>

#define __USE_ISOC99 1        /* define vfscanf and vsscanf in stdio.h */
#include <stdio.h>

#include <sys/stat.h>

#ifdef _AIX
#include <strings.h>                              /* bzero, bcopy, ... */
#else /* #ifdef _AIX */
#include <string.h>                               /* bzero, bcopy, ... */
#endif /* #ifdef _AIX */
#include <ctype.h>

#include <sys/mman.h>

/* Some platforms (e.g. Darwin) define MAP_ANON instead of MAP_ANONYMOUS */
#ifndef MAP_ANONYMOUS
#define MAP_ANONYMOUS MAP_ANON
#endif /* #ifndef MAP_ANONYMOUS */



#include <sys/shm.h>

#include <sys/time.h>
#include <time.h>

#include <sys/ipc.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>                            /* inet_addr */

#ifdef __cplusplus
extern "C" {
#endif /* #ifdef __cplusplus */


#ifdef __APPLE__
#include <net/ethernet.h>                         /* ether_ntoa */
    MEXTERN struct ether_addr *ether_aton (const char *);
#elif defined(_AIX)
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>                            /* ether_aton, ether_ntoa */
#include <netinet/if_ether.h>
#include <netinet/if_ether6.h>                    /* ETHER_ADDR_LEN */
#elif defined(__CYGWIN__)
#include <netinet/if_ether.h>
    extern struct ether_addr *ether_aton (char *);
#else /* #ifdef __APPLE__ */
#include <netinet/ether.h>
#endif /* #ifdef __APPLE__ */

#ifdef __cplusplus
}
#endif /* #ifdef __cplusplus */


#ifdef __APPLE__
/* on darwin (probably BSD) socklen_t is not defined -- use int */
typedef int SOCKLEN_t;
#else /* #ifdef __APPLE__ */
/* AIX and Linux have socklen_t defined */
typedef socklen_t SOCKLEN_t;
#endif /* #ifdef __APPLE__ */

#include <math.h>



#ifndef O_LARGEFILE
/* Some OSes are always large file */
#define O_LARGEFILE 0
#endif /* #ifndef O_LARGEFILE */


/* ******************************************************** */
/* some code does not include mambo.h.  For them we need to
   repeat how to declare an inline function. So this is the same
   as in mambo.h.  We do it before including tcl.h, because tcl.h
   will also define INLINE, if it is not already defined.
*/

#ifndef INLINE
/* The AIX xlc compiler does not seem to like the gcc
   "static inline" declaration. */
#if defined(HAVE_GCC) && !__STRICT_ANSI__
#define INLINE static inline
#else /* #if defined(HAVE_GCC) && !__STRICT_ANSI__ */
#define INLINE static
#endif /* #if defined(HAVE_GCC) && !__STRICT_ANSI__ */
#endif /* #ifndef INLINE */



#endif /* #ifndef _EXTERNAL_H_ */
