/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2006
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : mambo.h
 *
 * Purpose  : All mambo common macro definitions
 *
 * ------------------------------------------------------- */

#ifndef _MAMBO_H_
#define _MAMBO_H_


/* ******************************************************** */
/* Figure out any properties of the compilation that we want to
 * be generally available -- 32/64-bit and C/C++
 */
#include "definecpp.h"
#include "define64bit.h"


/* ******************************************************** */
/* all the external function definitions */
#include "common/external.h"

#ifdef MAMBO

#ifndef BUILD_ONCE
/* Every mambo source file needs to include the configuration settings.
 * These configuration #defines may affect just about anything else */
#include "common/config_before.h"
#include "common/m_config.h"
#include "common/m_define.h"
#include "common/p_config.h"
#include "common/p_define.h"
#include "common/config_after.h"
#endif /* #ifndef BUILD_ONCE */

#endif /* #ifdef MAMBO */

#ifdef EMITTER
#define MCONFIG_TCL 1
#endif /* #ifdef EMITTER */

/* if we are using the TCL system, include the external header
   files to define TCL and TK functions. */
#ifdef MCONFIG_TCL
#include "common/tcltk_external.h"
#endif /* #ifdef MCONFIG_TCL */

/*
 * tcl 8.4.x changes the prototype for Tcl_CmdProc by making argv[] a
 * 'const char *'.  One day they will realize that they want it to be
 * 'char *const []' (and possibly retain the leading const as well) so
 * hopefully this logic will make that transition easier.
 * NOTE: we use the same macro that TCL uses to perform the const!
 */
#ifndef CONST84
#define CONST84
#endif /* #ifndef CONST84 */

#define TCL_CONST CONST84


/* ******************************************************** */
/* The AIX xlc compiler does not seem to like the gcc
   "static inline" declaration. */
#if defined(HAVE_GCC) && !__STRICT_ANSI__
#define INLINE static inline
#else /* #if defined(HAVE_GCC) && !__STRICT_ANSI__ */
#define INLINE static
#endif /* #if defined(HAVE_GCC) && !__STRICT_ANSI__ */

#ifndef HAVE_GCC
#undef HOSTPPC             /* HOSTPPC means use gcc asm */
#endif /* #ifndef HAVE_GCC */


/* ******************************************************** */
/* All (VOID *) pointers should be declared (VOID *).  This
 * allows them to be changed to (char *) or (VOID *) easily.
 * The problem is that for (VOID *), the compiler will not
 * complain if a parameter that should be a (VOID *) is actually
 * a (VOID **), but it will for a (char *) and a (char **). */
#define VOID void

/* ******************************************************** */
/* We would like to be able to easily find all variables with a
 * global scope.  This tag can be either empty or "static".
 */
#define GLOBAL


/* ******************************************************** */
/* SWAP_BYTES takes a 16-bit value and swaps the
 * lower two bytes */
#define SWAP_BYTES(val) (((val & 0xFF) << 8) | ((val >> 8) & 0xFF))


/* ******************************************************** */
/* To make it easy to find the type casts in this language,
 * define a macro to take a type and an expression, and do the
 * cast. */
#define CAST(t,e) ((t)(e))
#define CAST2(t1,t2,e) CAST(t1,CAST(t2,e))


/* ******************************************************** */
/* ALIGN_DOWN(a,s) will align the address, a, to the size s */
/* !! size s MUST be a power of two !! */
/* Note: we need size to be a signed number, so that the
   ~(size-1) expression will do sign extension for larger sizes */
#define ALIGN_DOWN(addr,size) ((addr) & ~(size-1))
#define ALIGN_UP(addr,size)   ALIGN_DOWN((addr)+(size)-1, size)


#define NUMBER_IN_ARRAY(a)  (CAST(int,sizeof(a)/sizeof(a[0])))

/* Does a reference (from addr to addr+size-1) cross a boundary? */
#define CROSSES_BOUNDARY(addr,size,boundary) (ALIGN_DOWN(addr,boundary) != ALIGN_DOWN(addr+size-1,boundary))

/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

/* basic types for all code */
#include "basictypes_t.h"


/* ******************************************************** */

#ifdef MAMBO    /* Should only apply when building Mambo executable */

/* tsim support */
//#include "tsim/api.h"


/* mambo simulation types for all code */
#include "common/simtypes.h"
#include "processor/simtime.h"
#include "processor/domains.h"


/* include the info about how to print */
#include "utils/sim_error.h"
#include "common/debug_commands.h"

/* include the info about stop_simulation */
#include "utils/sim_stop.h"

/* statistics types for all code */
#include "common/mambo_stats.h"

/* information common to all mambo code */
#include "common/mambo_common_t.h"                /* CommonMamboState */
extern CommonMamboState mambo_common;

/* include the info about  tracing */
#include "mtrace_t.h"

/* abstract malloc interface */
#include "utils/freespace.h"

/* support for setting device pathnames */
#include "common/device_path.h"

#endif /* #ifdef MAMBO */

#endif /* #ifndef _MAMBO_H_ */
