/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2006
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : definecpp.h
 *
 * Purpose  : Figure out if the host is 32-bit or 64-bit
 *
 * ------------------------------------------------------- */

#ifndef _DEFINECPP_H_
#define _DEFINECPP_H_

/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */

#ifdef __cplusplus
#define MEXTERN extern "C"
#else /* #ifdef __cplusplus */
#define MEXTERN extern
#endif /* #ifdef __cplusplus */


/* Mambo is written in two pieces: a C piece and a C++ piece.  In
   addition we refer to standard C library functions.

   We need to compile the C++ pieces with g++.  We normally
   compile the C pieces with gcc.  This means we need to declare,
   to the C++ pieces, that the functions that the C++ piece calls
   that are in the C part are C functions.  For this we wrapper
   the C header functions with <extern "C">.

   But since C is a subset of C++, we should also be able to
   compile the C pieces with C++.  In this case, we don't need to
   tell the C++ pieces that the C pieces are <extern "C"> since
   they will be compiled with g++, and everything is the same
   calling convention. However, we would need, in this case to
   declare all the standard C libraries as <extern "C">.

   We can tell if we are compiling with g++, because MAMBO_CPP
   will be true. (__cplusplus will also be true but that will
   sometimes be true if we are compiling a C++ piece even though
   most of mambo is being compiled with gcc.)

   So we have two interfaces:

   (1) The internal one between the C++ and C parts of Mambo.
   (2) The external one between the C parts of Mambo and the
       standard C libraries.

   We define two macros to facilitate this:

   BEGIN_MAMBO_C/END_MAMBO_C -- is used to surround code that
   is the C part of Mambo. (for the prototypes).

   BEGIN_MAMBO_CPP/END_MAMBO_CPP -- is used to surround code
   that is the C++ part of Mambo.
*/

#ifdef MAMBO_CPP

/* Case 1: compile everything with g++ */
#define BEGIN_MAMBO_CPP
#define END_MAMBO_CPP
#ifdef __cplusplus
#define BEGIN_MAMBO_C    extern "C" {
#define END_MAMBO_C      }
#else /* #ifdef __cplusplus */
#define BEGIN_MAMBO_C
#define END_MAMBO_C

#endif /* #ifdef __cplusplus */

#else /* #ifdef MAMBO_CPP */

/* Case 2: compile some with g++, most with gcc */
#ifdef __cplusplus
#define BEGIN_MAMBO_CPP  extern "C" {
#define END_MAMBO_CPP    }
#else /* #ifdef __cplusplus */
#define BEGIN_MAMBO_CPP
#define END_MAMBO_CPP
#endif /* #ifdef __cplusplus */
#define BEGIN_MAMBO_C
#define END_MAMBO_C

#endif /* #ifdef MAMBO_CPP */



#endif /* #ifndef _DEFINECPP_H_ */
