/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2006
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : emitter_data.h
 *
 * Purpose  :
 *
 *  This defines the data structures contained in the
 *  emitter stream.   The emitter stream is a circular
 *  buffer of these entries.  Each entry is actually a
 *  union of the various structures listed here.
 *  Each structure contains a common header.  The common
 *  header contains a tag field which determines which
 *  structure you should use to look at that entry.
 *
 * ------------------------------------------------------- */
/* we depend upon "mambo.h" being the first file included in every *.c file;
 * check that it has been included */
#if !defined(_MAMBO_H_) && defined(MAMBO)
#error "common/mambo.h" not included first
#endif /* #if !defined(_MAMBO_H_) && defined(MAMBO) */

#ifndef _EMITTER_DATA_H_
#define _EMITTER_DATA_H_

#include "basictypes_t.h"
#include "emitter/atomic.h"
#include "emitter/emitter_data_t.h"


#define EMIT_FirstIndex(eControl, rdr)  ((eControl)->rdata->reader[(rdr)].first)
#define EMIT_CurrentIndex(eControl, rdr)  ((eControl)->rdata->reader[(rdr)].current)
#define EMIT_NextWriteIndex(eControl)  ((eControl)->rdata->next_write)
#define EMIT_GetEntry(eControl, rdr, index)  (&((eControl)->edata[EINDEX(index)]))
#define EMIT_FirstEntry(eControl, rdr)  (&((eControl)->edata[EINDEX(EMIT_FirstIndex(eControl, rdr))]))
#define EMIT_CurrentEntry(eControl, rdr)  (&((eControl)->edata[EINDEX(EMIT_CurrentIndex(eControl, rdr))]))


/* EMIT_AttachReader
 *  Called by reader to attach to the simulator shared memory region
 *  specified by the simulator process ID of pid
 */
extern int EMIT_AttachReader (int pid, EMITTER_CONTROLS *eControl, int history, boolean verbose);


/* EMIT_AdvanceCurrent
 *  Called by reader to advance the active reader pointer by one.
 */
extern void EMIT_AdvanceCurrent (EMITTER_CONTROLS *eControl, int rdr);


/* EMIT_ReadyWait
 *  Called by reader to wait for the header to be written out
 *  This indicates the start of the emitter stream
 */
extern void EMIT_ReadyWait (EMITTER_CONTROLS *eControl, int rdr, boolean verbose);


/* EMIT_Quit
 *  Called by reader to detach and clean up the shared memory
 *  buffers
 */
extern void EMIT_Quit (EMITTER_CONTROLS *eControl, int rdr, boolean verbose);


/* EMIT_SetReadersExpected
 *  Called by reader to indicate how many readers simulator should
 *  wait to attach before it starts to emit records
 */
extern int EMIT_SetReadersExpected (int pid, int readers, boolean verbose);

#endif /* #ifndef _EMITTER_DATA_H_ */
