/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2007
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : emitter_data_t.h
 *
 * Purpose  :
 *
 *  This defines the data structures contained in the
 *  emitter stream.   The emitter stream is a circular
 *  buffer of these entries.  Each entry is actually a
 *  union of the various structures listed here.
 *  Each structure contains a common header.  The common
 *  header contains a tag field which determines which
 *  structure you should use to look at that entry.
 *
 * ------------------------------------------------------- */
/* we depend upon "mambo.h" being the first file included in every *.c file;
 * check that it has been included */
#if !defined(_MAMBO_H_) && defined(MAMBO)
#error "common/mambo.h" not included first
#endif /* #if !defined(_MAMBO_H_) && defined(MAMBO) */

#ifndef _EMITTER_DATA_T_H_
#define _EMITTER_DATA_T_H_

#include "trans_id_t.h"
#include "mpid_t.h"

#ifdef CONFIG_PARALLEL
#include <pthread.h>
#endif /* #ifdef CONFIG_PARALLEL */

#define TAG_HEADER                  0x01
#define TAG_FOOTER                  0x02
#define TAG_MEMWRITE                0x03
#define TAG_MEMREAD                 0x04
#define TAG_INST                    0x05
#define TAG_DISK_REQ                0x09
#define TAG_MSR                     0x0A
#define TAG_IDLE                    0x0B
#define TAG_EXEC_LOAD               0x0C
#define TAG_EXEC_UNLOAD             0x0D
#define TAG_PID_KILL                0x0E
#define TAG_PID_CREATE              0x0F
#define TAG_PID_EXECEXIT            0x10
#define TAG_PID_EXEC                0x11
#define TAG_PID_RESUME              0x12
#define TAG_IPAGE_MISS              0x13
#define TAG_DPAGE_MISS              0x14
#define TAG_CONFIG                  0x15
#define TAG_MEMDFAULT               0x16
#define TAG_NMP_L3PREFETCH          0x1B /* Added by mew 09022004 */
#define TAG_MUREAD                  0x1C /* Added by mew 09232004 */
#define TAG_MUWRITE                 0x1D /* Added by mew 09232004 */
#define TAG_DRAM_WRITE              0x1E
#define TAG_DRAM_READ               0x1F

/*JRUBIO:BEGIN*/
#define TAG_TID_CREATE              0x20
#define TAG_TID_KILL                0x21
#define TAG_JIT_ADD                 0x22
#define TAG_JIT_REMOVE              0x23
/*JRUBIO:END*/

#define TAG_ERAT_INIT               0x24
#define TAG_ERAT_INVALIDATE         0x25
#define TAG_ERAT_FIND_MAPPING       0x26
#define TAG_ERAT_ADD_MAPPING        0x27
#define TAG_ERAT_SEGTAB_LOOKUP      0x28
#define TAG_ERAT_PAGETAB_LOOKUP     0x29

#define TAG_ITLB_MISS               0x2A
#define TAG_DTLB_MISS               0x2B
#define TAG_TLB_MISS_INST           0x2C
#define TAG_TLB_MISS_DATA           0x2D

#define TAG_SLB_MISS_INST           0x2E
#define TAG_SLB_MISS_DATA           0x2F

/* mco linux idle enter and exit */
#define TAG_IDLE_ENTER              0x32
#define TAG_IDLE_EXIT               0x33
#define TAG_SUPERVISOR_MODE         0x34
#define TAG_PROBLEM_STATE_MODE      0x35
#define TAG_KERNEL_THREAD_CREATE    0x36
#define TAG_FIT_INTERRUPT           0x37
#define TAG_PIT_INTERRUPT           0x38
#define TAG_ALIGN_EXCEPTION         0x39
#define TAG_EXTERNAL_INTERRUPT      0x3A
#define TAG_DTLB_HIT                0x3E
#define TAG_ITLB_HIT                0x3F
#define TAG_TLB_HIT_INST            0x40
#define TAG_TLB_HIT_DATA            0x41
#define TAG_SLB_HIT                 0x42
/* mco: signal to the emitter reader that we can start collecting stats */
#define TAG_KERNEL_FINISHED_LOADING 0x43
#define TAG_KERNEL_THREAD_CALL      0x44
#define TAG_KERNEL_MMAP_CALL        0x45

#define TAG_PERIODIC_POWER          0x46
#define TAG_POWER_COMPOUND_EVENT    0x47
#define TAG_POWER_BASIC_EVENT       0x48

/*****************************/
/* TAGS 0x50 - 0x5F RESERVED */
/*****************************/

#define TAG_CHANGE_FLOW             0x60
#define TAG_DATA_PREFIX             0x61
#define TAG_INTERRUPT               0x62
#define TAG_STRING_BYTECOUNT        0x63

/* Cache events */
#define TAG_L1I_CACHE_HIT           0x70
#define TAG_L1I_CACHE_MISS          0x71
#define TAG_L1D_CACHE_HIT           0x72
#define TAG_L1D_CACHE_MISS          0x73
#define TAG_L2_CACHE_HIT            0x74
#define TAG_L2_CACHE_MISS           0x75
#define TAG_L3_CACHE_HIT            0x76
#define TAG_L3_CACHE_MISS           0x77
#define TAG_CACHE_D_FLUSH           0x78
#define TAG_CACHE_I_INVALIDATE      0x79
#define TAG_CACHE_ADDR              0x7A
#define TAG_LOCAL_CACHE_MISS        0x7B
#define TAG_PTE_ACCESS              0x7C
#define TAG_IS_CACHE_EVENT(e)       ((e) >= TAG_L1I_CACHE_HIT && (e) <= TAG_PTE_ACCESS)


#define TAG_BGQ_PE_HIT              0x7D
#define TAG_BGQ_PE_MISS             0x7E


/*****************************/
/* TAGS 0xA0 - 0xAF RESERVED */
/*****************************/

#define TAG_STATS_FILENAME          0xED
#define TAG_START_STATS             0xEE
#define TAG_STOP_STATS              0xEF
#define TAG_DUMP_STATS              0xF0
#define TAG_REAL_MEMWRITE           0xF1
#define TAG_REAL_MEMREAD            0xF2

#define TAG_BUS_WAIT                0xF3
#define TAG_LPIDR                   0xF4

#define TAG_ANNOTATE_STATS          0xF5
#define TAG_TRANSACTION             0xF6

#define TAG_TIME_PRINT              0xF7
#define TAG_MTRACE                  0xF8

/* Disk access types */
#define E_DISK_WRITE    0
#define E_DISK_READ     1
#define E_DISK_SYSTEM   2

#define INVALID_CPU    (CAST(uint8,0xFF))
#define INVALID_MCM    (CAST(uint8,0xFF))
#define INVALID_THREAD (CAST(uint8,0xFF))

/* Cache hit type */
#define EMIT_CACHE_HIT_L1       0
#define EMIT_CACHE_HIT_L1_M     1
#define EMIT_CACHE_HIT_L2_M     2
#define EMIT_CACHE_HIT_L2       3
#define EMIT_CACHE_HIT_L21      4
#define EMIT_CACHE_HIT_L25      5
#define EMIT_CACHE_HIT_L275     6
#define EMIT_CACHE_HIT_L3       7
#define EMIT_CACHE_HIT_L31      8
#define EMIT_CACHE_HIT_L35      9
#define EMIT_CACHE_HIT_L375     10
#define EMIT_CACHE_HIT_LMEM     11
#define EMIT_CACHE_HIT_RMEM     12
#define EMIT_CACHE_HIT_MIP      13

struct emit_head
{
    uint8 tag;
    uint8 cpu;    /* can be 0xFF */
    uint8 thread; /* can be 0xFF */
    uint8 mcm;    /* can be 0xFF */
    uint8 pad[4]; /* make sure seqid is 8-byte aligned */
    uint64 seqid;
    uint64 timestamp;
};
typedef struct emit_head EMIT_HEAD;


struct config_info
{
    EMIT_HEAD head;
    int cpus;
    uint64 mem_size;
    uint64 clock_frequency;
    int disk_latency;
    float cpi;
    int l1d_block_size;
    int l1i_block_size;
    int l2_block_size;
    int l1i_max_index;
    int l1d_max_index;
    int l2_max_index;
    int l1s_per_l2;
    int threads;
};
typedef struct config_info CONFIG_INFO;


struct inst_info
{
    EMIT_HEAD head;
    uint64 eaddr;
    uint64 vsid_unshifted;
    uint64 paddr;
    uint32 inst;
    uint32 inst_type;
};
typedef struct inst_info INST_INFO;


/* XXX */
/* The following structure is bad for the union due to
 * its size.  This should be looked at in the future */
/* NOTE: This must be greater than 20 for annotations to work */
#define MAX_EMITTER_PATHNAME    80

struct path_info
{
    EMIT_HEAD head;
    char pathname[MAX_EMITTER_PATHNAME];
};
typedef struct path_info PATH_INFO;

struct load_info
{
    EMIT_HEAD head;
    mpid_t pid;
    char pathname[MAX_EMITTER_PATHNAME];
    uint64 text_vsid;
    uint64 text_ea;
    int text_len;
    uint64 data_vsid;
    uint64 data_ea;
    int data_len;
};
typedef struct load_info LOAD_INFO;


struct mmap_info
{
    EMIT_HEAD head;
    char pathname[MAX_EMITTER_PATHNAME];
    uint64 eaddr;
    int len;
    int prot;
    int flags;
    int file_offset;
};
typedef struct mmap_info MMAP_INFO;


struct pid_info
{
    EMIT_HEAD head;
    char pathname[MAX_EMITTER_PATHNAME];
    mpid_t ppid;
    mpid_t pid;
    int tid;
    int flag;
};
typedef struct pid_info PID_INFO;


struct idle_info
{
    EMIT_HEAD head;
    uint64 start;
    uint64 end;
};
typedef struct idle_info IDLE_INFO;


/* mco IDLE and MODE handling */
struct idle_enter_info
{
    EMIT_HEAD head;
};
typedef struct idle_enter_info IDLE_ENTER_INFO;


struct idle_exit_info
{
    EMIT_HEAD head;
};
typedef struct idle_exit_info IDLE_EXIT_INFO;


struct supervisor_mode_enter_info
{
    EMIT_HEAD head;
    uint64    pc_addr;
};
typedef struct supervisor_mode_enter_info SUPERVISOR_MODE_ENTER_INFO;


struct supervisor_mode_exit_info
{
    EMIT_HEAD head;
    uint64    pc_addr;
};
typedef struct supervisor_mode_exit_info SUPERVISOR_MODE_EXIT_INFO;


struct interrupt_info
{
    EMIT_HEAD head;
    uint64 eaddr;
    uint32 inst_form;
    uint32 ins;
    int length;
};
typedef struct interrupt_info INTERRUPT_INFO;


struct kernel_started_info
{
    EMIT_HEAD head;
};
typedef struct kernel_started_info KERNEL_STARTED_INFO;

/* end mco */


struct msr_info
{
    EMIT_HEAD head;
    uint64 value;
};
typedef struct msr_info MSR_INFO;


struct lpidr_info
{
    EMIT_HEAD head;
    uint64 value;
};
typedef struct lpidr_info LPIDR_INFO;


struct disk_info
{
    EMIT_HEAD head;
    uint16 operation;
    uint16 sectors;
    uint16 sector;
    uint16 offset;
};
typedef struct disk_info DISK_INFO;


struct io_info
{
    EMIT_HEAD head;
    uint64 pc;
    uint64 vsid_unshifted;
    uint64 value;
    uint64 paddr;
    uint64 eaddr;
    char length;
    unsigned char color;
};
typedef struct io_info IO_INFO;


struct cache_info
{
    EMIT_HEAD head;
    uint64 eaddr;
    uint64 vsid_unshifted;
    uint64 paddr;
    int length;            /* number of bytes being accessed */
    int index;               /* line index */
    int cc;              /* way index */
    /* applies to l2 misses, = 1 if caused by ifetch */
    int ifetch;
    uint8 isload;  /* 1 = load, 0 otherwise.
                      may or may not be set by simulator, so be careful */
    uint8 issync; /* 1 = is sync (larx/stcx), 0 = regular */
    unsigned char color;
    uint8 hittype;
    uint64 pc;
};
typedef struct cache_info CACHE_INFO;


struct cache_op_info
{
    EMIT_HEAD head;
    uint64 eaddr;
    uint64 vsid_unshifted;
    uint64 paddr;
};
typedef struct cache_op_info CACHE_OP_INFO;


struct bus_wait_info
{
    EMIT_HEAD   head;
    uint64      cycles;
};
typedef struct bus_wait_info BUS_WAIT_INFO;


struct header
{
    EMIT_HEAD head;
};
typedef struct header HEADER;


struct page_info
{
    EMIT_HEAD head;
    uint64 eaddr;
    unsigned char color;
    uint64 pc;
};
typedef struct page_info PAGE_INFO;


struct erat_info
{
    EMIT_HEAD head;
    uint64 instructions;
    uint64 ea;
    uint32 row;
    uint16 col;
    uint8 flags;
    uint8 resv;
};
typedef struct erat_info ERAT_INFO;


struct slb_info
{
    EMIT_HEAD head;
    uint64 ea;
    uint64 vsid_unshifted;
    int32 slb_index;
    unsigned char color;
    uint64 pc;
};
typedef struct slb_info SLB_INFO;


struct tlb_info
{
    EMIT_HEAD head;
    uint64 ea;
    uint32 pid;  /* pid register for TLB */
    unsigned char color;
    uint64 pc;
};
typedef struct tlb_info TLB_INFO;


#ifdef CONFIG_ENERGY_SIMULATION
#include "processor/power_model_s.h"
struct periodic_power_info
{
    EMIT_HEAD head;
    double P_IFU_power;
    double P_IDU_power;
    double P_ISU_power;
    double P_FPU_power;
    double P_FXU_power;
    double P_LSU_power;
    double P_CHIPRAS_power;
    double P_CIU_power;
    double P_CORERAS_power;
    double P_GX_power;
    double P_L2_power;
    double P_L3_power;
    double P_MC_power;
    double P_NCU_power;
    double P_TOTAL_power;
};
typedef struct periodic_power_info PERIODIC_POWER_INFO;

struct power_compound_event_info
{
    EMIT_HEAD head;
    power_compound_event event;
};
typedef struct power_compound_event_info POWER_COMPOUND_EVENT_INFO;

struct power_basic_event_info
{
    EMIT_HEAD head;
    power_basic_event event;
};
typedef struct power_basic_event_info POWER_BASIC_EVENT_INFO;


#endif /* #ifdef CONFIG_ENERGY_SIMULATION */


struct power_info
{
    EMIT_HEAD head;
    uint64 pc_addr;
    Power_t IdlePower;
    Power_t DCachePower;
    Power_t ICachePower;
    Power_t BranchPower;
    Power_t InstrPower;
    Power_t TotalPower; /* total computed power */
};
typedef struct power_info POWER_INFO;


struct kernel_thread_info
{
    EMIT_HEAD head;
    mpid_t pid;
    char function_name[MAX_EMITTER_PATHNAME];
};
typedef struct kernel_thread_info KERNEL_THREAD_INFO;


#define MAX_EMITTER_ANNOTATION_STACK_NAME   20
#define MAX_EMITTER_ANNOTATION              (MAX_EMITTER_PATHNAME - MAX_EMITTER_ANNOTATION_STACK_NAME)
struct stats_annotation_info
{
    EMIT_HEAD head;
    unsigned int    stk_handle;
    unsigned int    stk_depth;
    char            stk_name[MAX_EMITTER_ANNOTATION_STACK_NAME];
    char            text[MAX_EMITTER_ANNOTATION];
};
typedef struct stats_annotation_info STATS_ANNOTATION_INFO;


#define MAX_EMITTER_TRANS_TEXT              64
#define MAX_EMITTER_TRANS_DESC_TXT          32
#define MAX_EMITTER_TRANS_LOC_TXT           32
struct trans_info
{
    EMIT_HEAD   head;
    TRID        id;                                 /* Transaction ID */
    uint16      src_id;                             /* Source ID */
    uint64      addr;                               /* Address (Physical or Effective) */
    uint32      length;                             /* Number of bytes requested */
    char        desc[MAX_EMITTER_TRANS_DESC_TXT];   /* Description of transaction purpose */
    char        where[MAX_EMITTER_TRANS_LOC_TXT];   /* Description of current location */
    char        text[MAX_EMITTER_TRANS_TEXT];       /* Note */
};
typedef struct trans_info TRANS_INFO;

#define MAX_EMITTER_MTRACE_TEXT    255
struct mtrace_info
{
    EMIT_HEAD head;
    uint64    id;                                  /* Transaction ID */
    char      where[MAX_EMITTER_MTRACE_TEXT + 1];  /* Description of current location */
};
typedef struct mtrace_info MTRACE_INFO;

#ifdef MCONFIG_SPU
#include "emitter/sti_emitter_data_t.h"
#endif /* #ifdef MCONFIG_SPU */

union emit_data
{
    PATH_INFO path;
    CONFIG_INFO config;
    LOAD_INFO load;
    MMAP_INFO kernmmap;
    IO_INFO io;
    DISK_INFO disk;
    INST_INFO inst;
    CACHE_INFO cache;
    CACHE_OP_INFO cache_op;
    HEADER header;
    MSR_INFO msr;
    LPIDR_INFO lpidr;
    IDLE_INFO idle;
    PID_INFO pid;
    PAGE_INFO page;
    ERAT_INFO erat;
    IDLE_ENTER_INFO idle_enter;
    IDLE_EXIT_INFO idle_exit;
    SUPERVISOR_MODE_ENTER_INFO enter_super_mode;
    SUPERVISOR_MODE_EXIT_INFO exit_super_mode;
    TLB_INFO tlb;
    SLB_INFO slb;
    INTERRUPT_INFO interrupt;
    KERNEL_STARTED_INFO kernel_status;
    KERNEL_THREAD_INFO kernel_thread;
    BUS_WAIT_INFO bus_wait;

#ifdef MCONFIG_SPU
#include "emitter/sti_emitter_union_t.h"
#endif /* #ifdef MCONFIG_SPU */


#ifdef CONFIG_ENERGY_SIMULATION
    PERIODIC_POWER_INFO periodic_power;
    POWER_COMPOUND_EVENT_INFO power_compound_event;
    POWER_BASIC_EVENT_INFO power_basic_event;
#endif /* #ifdef CONFIG_ENERGY_SIMULATION */

    STATS_ANNOTATION_INFO   annotation;
    TRANS_INFO              trans;
    MTRACE_INFO             mtrace;

};
typedef union emit_data EMIT_DATA;

#ifdef __APPLE__
#define EMIT_DATA_ENTRIES  0x001000     /* 4K */
#else /* #ifdef __APPLE__ */
#ifndef _AIX
#define EMIT_DATA_ENTRIES  0x010000     /* 64K * sizeof(EMIT_DATA), shared memory segment must be <= 32M (linux) */
#else /* #ifndef _AIX */
#define EMIT_DATA_ENTRIES  0x200000     /* 2M */
#endif /* #ifndef _AIX */
#endif /* #ifdef __APPLE__ */
#define EINDEX(a) ((a) & (EMIT_DATA_ENTRIES - 1))
/*
 * Emitter Reader Data Structure
 *  The following is read and written by the readers.
 */
#define MAX_READERS 10

struct per_reader
{
    volatile sint64 first;
    volatile sint64 current;
    volatile int history;
    volatile boolean ready;
};
typedef struct per_reader per_reader_s;


struct reader_data
{
    volatile boolean all_ready;   /* Flag for whether writer is ready to go */

    volatile int readers_attached;/* Number of readers attached */
    volatile int readers_expected;/* Number of readers expected */

    uint64 emitter_waits;
    volatile sint64 next_write;
    volatile sint64 min_first;   /* Minimum first entry */
    per_reader_s     reader[MAX_READERS];

#ifdef CONFIG_PARALLEL
    pthread_mutex_t emit_mutex;
#endif /* #ifdef CONFIG_PARALLEL */
};
typedef struct reader_data READER_DATA;


#define KEY_READER_DATA 1
#define KEY_WRITER_DATA 2

struct EmitterControls
{
    int pid;
    int key;
    int eShmid;
    int rShmid;
    READER_DATA * rdata;
    EMIT_DATA * edata;
};
typedef struct EmitterControls EMITTER_CONTROLS;



#endif /* #ifndef _EMITTER_DATA_T_H_ */
