/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2007
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : sti_emitter_data_t.h
 *
 * Purpose  :
 *
 * STI Specific data structures.  This file gets included
 * in the main emitter_data.h file
 *
 * ------------------------------------------------------- */
/* we depend upon "mambo.h" being the first file included in every *.c file;
 * check that it has been included */
#if !defined(_MAMBO_H_) && defined(MAMBO)
#error "common/mambo.h" not included first
#endif /* #if !defined(_MAMBO_H_) && defined(MAMBO) */

#ifndef _STI_EMITTER_DATA_T_H_
#define _STI_EMITTER_DATA_T_H_

#define TAG_APU_INST                    0x50     /* record is an APU_INST_INFO */

#define TAG_APU_MEMREAD                 0x51     /* record is an APU_MEM_INFO */
#define TAG_APU_MEMWRITE                0x52     /* record is an APU_MEM_INFO */

#define TAG_APU_DMA_ISSUE               0x53     /* record is an APU_DMA_INFO */
#define TAG_PPU_DMA_ISSUE               0x54     /* record is an APU_DMA_INFO */

#define TAG_APU_TASK_ASSIGN             0x55     /* record is an APU_TASK_INFO */
#define TAG_APU_TASK_START              0x56     /* record is an APU_TASK_INFO */
#define TAG_APU_TASK_DONE               0x57     /* record is an APU_TASK_INFO */

#define TAG_APU_WAIT                    0x58     /* record is an APU_WAIT_INFO */

#define TAG_APU_ANNOTATE_STATS          0x5A     /* record is an APU_ANNOTATION_INFO */
#define TAG_APU_PIPE_INST               0x5B     /* record is an APU_PIPE_INST_INFO */
#define TAG_APU_OP                      0x5C     /* record is an APU_OP_INFO */
#define TAG_APU_PIPE_TRANS              0x5D     /* record is an APU_PIPE_TRANS_INFO */
#define TAG_APU_PERF_STATS              0x5E     /* record is an APU_PERF_STATS_INFO */
#define TAG_APU_PERF_PIPE_STATS         0x5F     /* record is an APU_PIPE_PERF_STATS_INFO */

#define TAG_APU_DMA_TG_STATUS_RQ        0xA0     /* record is an APU_DMA_TG_STATUS_RQ_INFO */
#define TAG_APU_DMA_TG_COMPLETE         0xA1     /* record is an APU_DMA_TG_COMPLETE_INFO */
#define TAG_APU_DMA_TG_STATUS_QRY       0xA2     /* record is an APU_DMA_TG_STATUS_QRY_INFO */

#define TAG_PPU_DMA_TG_STATUS_RQ        0xA3     /* record is an PPU_DMA_TG_STATUS_RQ_INFO */
#define TAG_PPU_DMA_TG_COMPLETE         0xA4     /* record is an PPU_DMA_TG_COMPLETE_INFO */
#define TAG_PPU_DMA_TG_INTR             0xA5     /* record is an PPU_DMA_TG_INTR_INFO */
#define TAG_PPU_DMA_TG_STATUS_QRY       0xA6     /* record is an PPU_DMA_TG_STATUS_QRY_INFO */

#define TAG_SPU_LOAD                    0xA7     /* record is an SPU_LOAD_STORE_INFO */
#define TAG_SPU_STORE                   0xA8     /* record is an SPU_LOAD_STORE_INFO */

struct apu_inst_info
{
    EMIT_HEAD head;
    int apu;
    uint64 cycle;
    uint64 paddr;
    uint32 inst;
};
typedef struct apu_inst_info APU_INST_INFO;

struct apu_pipe_inst_info
{
    EMIT_HEAD head;
    int apu;
    uint64 cycle;
    uint64 paddr;
    uint32 inst;
    uint32 pipe;
};
typedef struct apu_pipe_inst_info APU_PIPE_INST_INFO;


struct apu_mem_info
{
    EMIT_HEAD head;
    int apu;
    uint64 paddr;
    int length;
    uint64 value;
};
typedef struct apu_mem_info APU_MEM_INFO;


struct mfc_dma_info
{
    EMIT_HEAD head;                   /* Header */
    int       apu;                    /* APU number */
    uint64    clock;
    uint64    addr;                   /* Effective Address of DMA command */
    int       length;                 /* Length of DMA command */
    uint32    ls_addr;                /* Local store address of DMA command */
    uint32    cmd;                    /* DMA command */
    uint32    gid;                    /* Tag Group ID of DMA command */
};
typedef struct mfc_dma_info MFC_DMA_INFO;


#define APU_TASK_MAX_STR 25
struct apu_task_info
{
    EMIT_HEAD head;
    int apu;
    uint64 clock;
    char name[APU_TASK_MAX_STR];
};
typedef struct apu_task_info APU_TASK_INFO;


struct apu_wait_info
{
    EMIT_HEAD head;
    int apu;
    uint64 clock;
    int channel_num;
    int channel_stall;
};
typedef struct apu_wait_info APU_WAIT_INFO;


struct apu_stats_annotation_info
{
    EMIT_HEAD head;
    int apu;
    unsigned int    stk_handle;
    unsigned int    stk_depth;
    char            stk_name[MAX_EMITTER_ANNOTATION_STACK_NAME];
    char            text[MAX_EMITTER_ANNOTATION];
};
typedef struct apu_stats_annotation_info APU_STATS_ANNOTATION_INFO;

#define MAX_EMITTER_APU_OP_UNIT_TEXT    32
#define MAX_EMITTER_APU_OP_TEXT         64
struct apu_op_info
{
    EMIT_HEAD   head;
    int     apu;
    uint64  cycle;
    char    unit[MAX_EMITTER_APU_OP_UNIT_TEXT];   /* Description of current unit */
    char    text[MAX_EMITTER_APU_OP_TEXT];        /* Note */
};
typedef struct apu_op_info APU_OP_INFO;

#define MAX_EMITTER_APU_PIPE_TRANS_TEXT     64
#define MAX_EMITTER_APU_PIPE_DESC_TXT       16
#define MAX_EMITTER_APU_PIPE_STAGE_TXT      16

struct trans_id
{
    TRID        trans;                          /* Transaction ID */
    uint8       spu;                            /* SPU ID */
    uint8       be;                             /* BE ID */
};
typedef struct trans_id trans_id;


struct apu_pipe_trans_info
{
    EMIT_HEAD   head;
    trans_id    id;
    uint64      addr;                                   /* Address (Physical or Effective) */
    char        pipe[MAX_EMITTER_APU_PIPE_DESC_TXT];    /* Description of pipe */
    char        stage[MAX_EMITTER_APU_PIPE_STAGE_TXT];  /* Description of pipe stage */
    char        text[MAX_EMITTER_APU_PIPE_TRANS_TEXT];  /* Note */
};
typedef struct apu_pipe_trans_info APU_PIPE_TRANS_INFO;


struct apu_perf_stats_info
{
    EMIT_HEAD   head;                   /* Header */
    int         apu;                    /* APU number */
    uint64      inst_count;             /* Instruction Count */
    uint64      cyc_count;              /* Cycle Count */
    uint64      nop_count;              /* NOP Instruction Count */
    struct {
        uint64  taken;                  /* Count of branches taken */
        uint64  not_taken;              /* Count of branches not taken */
        uint64  mispred_stall;          /* Count of branch mispredict stall cycles */
    } branch;
    struct {
        uint64  count;                  /* Count of HBR hint instructions */
        uint64  hit;                    /* Count of HBR hint hits */
        uint64  stall;                  /* Count of hint stall cycles */
    } hint;
    struct {
        uint64  ls_contention;          /* Count of cycles LS instruction issue contended with prefetch for LS pipe */
        uint64  miss_stall;             /* Count of prefetch miss stall cycles */
    } prefetch;
    struct {
        uint64  stall;                  /* Count of channel stall cycles */
    } channel;
    struct {
        uint64  sp_block;               /* Total SP dual-issue block cycles */
        uint64  dp_block;               /* Total DP dual-issue block cycles */
    } issue;
    struct {
        uint64  single;                 /* Count of single execution cycles */
        uint64  dual;                   /* Count of dual execution cycles */
        uint64  nop;                    /* Count of NOP execution cycles */
        uint64  dep_stall;              /* Total operand dependency stall cycles (all execution pipelines) */
        uint64  siss_stall;             /* Total structural issue stall cycles (all execution pipelines) */
    } exec;
    struct {
        uint64  contention;             /* Count of cycles LS interactions contended with SBI */
    } sbi;
};
typedef struct apu_perf_stats_info APU_PERF_STATS_INFO;


#define MAX_EMITTER_APU_PIPE_NAME_TXT  16
struct apu_pipe_perf_stats_info
{
    EMIT_HEAD   head;                   /* Header */
    int         apu;                    /* APU number */
    char        pipe_name[MAX_EMITTER_APU_PIPE_NAME_TXT];
    uint64      inst_count;             /* Per-pipeline executed instruction count */
    uint64      dep_stall;              /* Per-pipeline operand dependency stall cycles */
    uint64      siss_stall;             /* Per-pipeline structural issue stall cycles */
};
typedef struct apu_pipe_perf_stats_info APU_PIPE_PERF_STATS_INFO;

/*
 * The APU_DMA_TG_STATUS_RQ_INFO record is produced as soon as a tag group status request
 * is made by the SPU (writing to channel 23, 0x17)
 */
struct apu_dma_tg_status_rq_info
{
    EMIT_HEAD   head;                   /* Header */
    int         apu;                    /* APU number */
    uint32      type;                   /* Type of request: 0=immediate, 1=any, 2=all */
    uint32      mask;                   /* Query mask */
};
typedef struct apu_dma_tg_status_rq_info    APU_DMA_TG_STATUS_RQ_INFO;

/*
 * The APU_DMA_TG_COMPLETE_INFO record is produced as soon as all pending commands for a given
 * unmasked SPU tag group are completed at the DMA controller
 */
struct apu_dma_tg_complete_info
{
    EMIT_HEAD   head;                   /* Header */
    int         apu;                    /* APU number */
    uint32      gid;                    /* Group ID (unmasked groups reported only) */
};
typedef struct apu_dma_tg_complete_info    APU_DMA_TG_COMPLETE_INFO;

/*
 * The APU_DMA_TG_STATUS_QRY_INFO record is produced under two circumstances:
 *
 * 1. Whenever an SPU-initiated tag status query is explicitly
 *    performed (via read of the tag status channel, 24 (0x18)). In
 *    this case, the "pushed" flag will be 0
 *
 * 2. Whenever the SPU tag status is written to the status channel (24,
 *    0x18) by the DMA controller. Status results will be written
 *    to this channel to free a potentially blocked read request
 *    which was issued after requesting a conditional update. In
 *    this case the "pushed" flag will be 1. Circumstances triggering
 *    this condition are:
 *
 *    A. When the request type made to channel 23 (0x17) is IMMEDIATE
 *    B. When the request is conditional, but satisfied immediately
 *       at the time the write was made to the request channel (23, 0x17)
 *    C. When the request is conditional, and satisfied during DMA
 *       command completion as SPU DMAQ entries are retired
 */
struct apu_dma_tg_status_qry_info
{
    EMIT_HEAD   head;                   /* Header */
    int         apu;                    /* APU number */
    uint32      tags;                   /* Completed tags mask */
    uint32      mask;                   /* Query mask */
    uint32      pushed;                 /* 1 if pushed by DMA to SPU channel, 0 if obtained by SPU read channel */
};
typedef struct apu_dma_tg_status_qry_info    APU_DMA_TG_STATUS_QRY_INFO;

/*
 * The PPU_DMA_TG_STATUS_RQ_INFO record is produced as soon as the PU writes a request
 * to the query type status register mapped to the SPU problem
 * memory space
 */
struct ppu_dma_tg_status_rq_info
{
    EMIT_HEAD   head;                   /* Header */
    uint32      type;                   /* Type of request: 0=none/cancel, 1=any, 2=all */
    uint32      mask;                   /* Query mask */
};
typedef struct ppu_dma_tg_status_rq_info    PPU_DMA_TG_STATUS_RQ_INFO;

/*
 * The PPU_DMA_TG_COMPLETE_INFO record is produced as soon as all pending commands for a given
 * unmasked PU tag group are completed at the DMA controller
 */
struct ppu_dma_tg_complete_info
{
    EMIT_HEAD   head;                   /* Header */
    uint32      gid;                    /* Group ID (unmasked groups reported only) */
};
typedef struct ppu_dma_tg_complete_info    PPU_DMA_TG_COMPLETE_INFO;

/*
 * The PPU_DMA_TG_INTR_INFO record is produced as soon as an unmasked DMA tag group completion
 * interrupt is raised for the PU
 */
struct ppu_dma_tg_intr_info
{
    EMIT_HEAD   head;                   /* Header */
    uint32      type;                   /* Type of request: 0=none/cancel, 1=any, 2=all */
    uint32      tags;                   /* Completed tag groups mask */
    uint32      mask;                   /* Query mask */
};
typedef struct ppu_dma_tg_intr_info    PPU_DMA_TG_INTR_INFO;

/*
 * The PPU_DMA_TG_STATUS_QRY_INFO record is produced as soon as the PU reads from
 * the DMA tag status register in the SPU problem state memory map area
 */
struct ppu_dma_tg_status_qry_info
{
    EMIT_HEAD   head;                   /* Header */
    uint32      tags;                   /* Completed tags mask */
    uint32      mask;                   /* Query mask */
};
typedef struct ppu_dma_tg_status_qry_info    PPU_DMA_TG_STATUS_QRY_INFO;

/*
 * The SPU_LOAD_STORE_INFO record is produced whenever an SPU performs a load or store
 * operation to the SPU's local store.
 */
struct spu_load_store_info
{
    EMIT_HEAD   head;                   /* Header */
    uint32      ls_addr;                /* Local store address of load or store */
    uint32      size;                   /* Size of load or store */
};
typedef struct spu_load_store_info    SPU_LOAD_STORE_INFO;

#endif /* #ifndef _STI_EMITTER_DATA_T_H_ */
