/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2006
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : endianfix.h
 *
 * Purpose  : Fix Cross Endian Problems
 *
 * ------------------------------------------------------- */

#ifndef _ENDIANFIX_H_
#define _ENDIANFIX_H_


/* Let the machine include files define the BYTE_ORDER */
#ifdef linux
#include <endian.h>
#include <asm/types.h>
#elif defined(_AIX)
#include <sys/machine.h>
#elif defined(__APPLE__)
#include <machine/endian.h>
#elif defined(__CYGWIN__)
#include <sys/param.h>
#include <asm/types.h>
#endif /* #ifdef linux */

#ifndef BYTE_ORDER
#error "BYTE_ORDER MUST BE DEFINED"
#endif /* #ifndef BYTE_ORDER */


#if !__STRICT_ANSI__
/* Define host-specific swab functions */
#if defined(i386) || defined(__i386) || defined(__i386__)
#define swab32(x) swab32_asm(x)
INLINE uint32 swab32_asm(uint32 value)
{
    asm("bswap %0" : "=r" (value) : "0" (value));
    return(value);
}
#define swab64(x) swab64_asm(x)

struct st1_s
{
    uint32 hi;
    uint32 lo;
};
typedef struct st1_s st1_s;

union ut2_u
{
    st1_s  bytes4;
    uint64 bytes8;
};
typedef union ut2_u convert64;

INLINE uint64 swab64_asm(uint64 value)
{
    convert64 x;
    convert64 r;
    uint32 t;
    x.bytes8 = value;
    t = swab32(x.bytes4.lo);
    r.bytes4.hi = t;
    t = swab32(x.bytes4.hi);
    r.bytes4.lo = t;
    return(r.bytes8);
}
#endif /* #if defined(i386) || defined(__i386) || defined(__i386__) */
#endif /* #if !__STRICT_ANSI__ */

/* Define default swab functions */

#ifndef swab16
#define swab16(x)                                                       \
    (CAST(uint16,                                                       \
          ((CAST(uint16,(x)) & CAST(uint16,0x00FFU)) << 8) |            \
          ((CAST(uint16,(x)) & CAST(uint16,0xFF00U)) >> 8)))
#endif /* #ifndef swab16 */

#ifndef swab32
#define swab32(x)                                                       \
    (CAST(uint32,                                                       \
          ((CAST(uint32,(x)) & CAST(uint32,0x000000FFUL)) << 24) |      \
          ((CAST(uint32,(x)) & CAST(uint32,0x0000FF00UL)) <<  8) |      \
          ((CAST(uint32,(x)) & CAST(uint32,0x00FF0000UL)) >>  8) |      \
          ((CAST(uint32,(x)) & CAST(uint32,0xFF000000UL)) >> 24)))
#endif /* #ifndef swab32 */

#ifndef swab64
#define swab64(x)                                                       \
    (CAST(uint64,                                                       \
          ((CAST(uint64,(x)) & CAST(uint64,0x00000000000000FFULL)) << 56) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x000000000000FF00ULL)) << 40) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x0000000000FF0000ULL)) << 24) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x00000000FF000000ULL)) <<  8) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x000000FF00000000ULL)) >>  8) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x0000FF0000000000ULL)) >> 24) | \
          ((CAST(uint64,(x)) & CAST(uint64,0x00FF000000000000ULL)) >> 40) | \
          ((CAST(uint64,(x)) & CAST(uint64,0xFF00000000000000ULL)) >> 56)))
#endif /* #ifndef swab64 */

#if BYTE_ORDER == LITTLE_ENDIAN

#define BE2HO_1(_x) CAST(uint8,_x)
#define BE2HO_2(_x) swab16(_x)
#define BE2HO_4(_x) swab32(_x)
#define BE2HO_8(_x) swab64(_x)

#define HO2BE_1(_x) (_x)
#define HO2BE_2(_x) swab16(_x)
#define HO2BE_4(_x) swab32(_x)
#define HO2BE_8(_x) swab64(_x)

#define LE2HO_1(_x) (_x)
#define LE2HO_2(_x) (_x)
#define LE2HO_4(_x) (_x)
#define LE2HO_8(_x) (_x)

#define HO2LE_1(_x) (_x)
#define HO2LE_2(_x) (_x)
#define HO2LE_4(_x) (_x)
#define HO2LE_8(_x) (_x)

#else /* #if BYTE_ORDER == LITTLE_ENDIAN */

#define BE2HO_1(_x) (_x)
#define BE2HO_2(_x) (_x)
#define BE2HO_4(_x) (_x)
#define BE2HO_8(_x) (_x)

#define HO2BE_1(_x) (_x)
#define HO2BE_2(_x) (_x)
#define HO2BE_4(_x) (_x)
#define HO2BE_8(_x) (_x)

#define LE2HO_1(_x) CAST(uint8,_x)
#define LE2HO_2(_x) swab16(_x)
#define LE2HO_4(_x) swab32(_x)
#define LE2HO_8(_x) swab64(_x)

#define HO2LE_1(_x) (_x)
#define HO2LE_2(_x) swab16(_x)
#define HO2LE_4(_x) swab32(_x)
#define HO2LE_8(_x) swab64(_x)
#endif /* #if BYTE_ORDER == LITTLE_ENDIAN */



/* Map types to sizes */
#define BE2HO_HALF      BE2HO_2
#define BE2HO_WORD      BE2HO_4
#define BE2HO_OFF32     BE2HO_4
#define BE2HO_OFF64     BE2HO_8
#define BE2HO_ADDR32    BE2HO_4
#define BE2HO_ADDR64    BE2HO_8
#define BE2HO_XWORD32   BE2HO_4
#define BE2HO_XWORD64   BE2HO_8




/* ************************************************************ */
/*                                                              */
/*                                                              */
/* ************************************************************ */
/*
 * Take a pointer to a big endian value in "maddr" and
 * covert it to a host value in "result".
 */
/*  *maddr -> result */
/* void BE_HO_Swizzle(void *maddr, int size, int result) */
#define BE_HO_Swizzle(maddr, size, result)                              \
    {                                                                   \
        switch (size)                                                   \
            {                                                           \
            case 1:                                                     \
                result = *CAST(uint8 *, maddr);                         \
                break;                                                  \
            case 2:                                                     \
                result = CAST(uint16, BE2HO_2(*CAST(uint16 *, maddr))); \
                break;                                                  \
            case 4:                                                     \
                result = CAST(uint32, BE2HO_4(*CAST(uint32 *, maddr))); \
                break;                                                  \
            case 8:                                                     \
                result = CAST(uint64, BE2HO_8(*CAST(uint64 *, maddr))); \
                break;                                                  \
            default:                                                    \
                result = 0;                                             \
            }                                                           \
    }

#define BE_HO_Swizzle_ConvertSize(maddr, size, result)                              \
    {                                                                   \
        switch (size)                                                   \
            {                                                           \
            case 1:                                                     \
                {                                                       \
                    uint8   val8 = CAST(uint8, *maddr);                 \
                    result = val8;                                      \
                }                                                       \
                break;                                                  \
            case 2:                                                     \
                {                                                       \
                    uint16  val16 = CAST(uint16, *maddr);               \
                    result = CAST(uint16, swab16(val16));              \
                }                                                       \
                break;                                                  \
            case 4:                                                     \
                {                                                       \
                    uint32  val32 = CAST(uint32, *maddr);               \
                    result = CAST(uint32, swab32(val32)); \
                }                                                       \
                break;                                                  \
            case 8:                                                     \
                {                                                       \
                    uint64  val64 = CAST(uint64, *maddr);               \
                    result = CAST(uint64, swab64(val64));              \
                }                                                       \
                break;                                                  \
            default:                                                    \
                result = 0;                                             \
            }                                                           \
    }

INLINE void BE_HO_Swizzle_Inline(void *maddr, int size)
{
    switch (size)
        {
        case 2:
            *CAST(uint16 *, maddr) = BE2HO_2(*CAST(uint16 *, maddr));
            break;
        case 4:
            *CAST(uint32 *, maddr) = BE2HO_4(*CAST(uint32 *, maddr));
            break;
        case 8:
            *CAST(uint64 *, maddr) = BE2HO_8(*CAST(uint64 *, maddr));
            break;
        case 16:
            {
#if BYTE_ORDER == LITTLE_ENDIAN
                uint64  tmp = BE2HO_8(*CAST(uint64 *, maddr));
                *CAST(uint64 *, maddr) = BE2HO_8(*CAST(uint64 *, CAST(uint8*, maddr) + 8));
                *CAST(uint64 *, CAST(uint8*, maddr) + 8) = tmp;
#endif /* #if BYTE_ORDER == LITTLE_ENDIAN */
            }
            break;
        default:
            break;
        }
}

/*
 * Take host "value" of size and covert it to a big endian value
 * in "maddr".
 */
/* value -> *maddr */
/* void HO_BE_Swizzle(void *maddr, int size, int value) */
#define HO_BE_Swizzle(maddr, size, value)                               \
    {                                                                   \
        switch (size)                                                   \
            {                                                           \
            case 1:                                                     \
                *CAST(uint8 *, maddr) = value;                          \
                break;                                                  \
            case 2:                                                     \
                *CAST(uint16 *, maddr) = HO2BE_2(value);                \
                break;                                                  \
            case 4:                                                     \
                *CAST(uint32 *, maddr) = HO2BE_4(value);                \
                break;                                                  \
            case 8:                                                     \
                *CAST(uint64 *, maddr) = HO2BE_8(value);                \
                break;                                                  \
            default:                                                    \
                break;                                                  \
            }                                                           \
    }

/*
 * Take a pointer to a little endian value in "maddr" and
 * covert it to a host value in "result".
 */
/* *maddr -> result */
/* void LE_HO_Swizzle(void *maddr, int size, int result) */
#define LE_HO_Swizzle(maddr, size, result)                              \
    {                                                                   \
        switch (size)                                                   \
            {                                                           \
            case 1:                                                     \
                result = *CAST(uint8 *, maddr);                         \
                break;                                                  \
            case 2:                                                     \
                result = CAST(uint16, LE2HO_2(*CAST(uint16 *, maddr))); \
                break;                                                  \
            case 4:                                                     \
                result = CAST(uint32, LE2HO_4(*CAST(uint32 *, maddr))); \
                break;                                                  \
            case 8:                                                     \
                result = CAST(uint64, LE2HO_8(*CAST(uint64 *, maddr))); \
                break;                                                  \
            default:                                                    \
                result = 0;                                             \
            }                                                           \
    }

/*
 * Take a pointer to a host "value" of size and covert it
 * to a little endian value in "maddr".
 */
/* value -> *maddr */
/* void HO_LE_Swizzle(void *maddr, int size, int value) */
#define HO_LE_Swizzle(maddr, size, value)                               \
    {                                                                   \
        switch (size)                                                   \
            {                                                           \
            case 1:                                                     \
                *CAST(uint8 *, maddr) = value;                          \
                break;                                                  \
            case 2:                                                     \
                *CAST(uint16 *, maddr) = HO2LE_2(value);                \
                break;                                                  \
            case 4:                                                     \
                *CAST(uint32 *, maddr) = HO2LE_4(value);                \
                break;                                                  \
            case 8:                                                     \
                *CAST(uint64 *, maddr) = HO2LE_8(value);                \
                break;                                                  \
            default:                                                    \
                break;                                                  \
            }                                                           \
    }

#endif /* #ifndef _ENDIANFIX_H_ */
