/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2001, 2007
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : it_codes.h
 *
 * Purpose  : defining instruction type codes
 *
 * ------------------------------------------------------- */

#ifndef _IT_CODES_H_
#define _IT_CODES_H_

/* define a type for these codes; each code is a bit; up to 32 bits. */
typedef uint32 it_code_type;
typedef uint32 at_code_type;

/*
 * There are two flag fields per instruction:
 *
 * inst_type_flags and addl_type_flags.
 *
 * The first is for pure instruction classification.
 *
 * The second is for internal mambo computed properties,
 * used for cycle-accurate work.
 */

/* ************************************************************ */
/*                                                              */
/*  For the first one: inst_type_flags                          */
/*                                                              */
/* ************************************************************ */

/* Instruction Type */
/* arithmetic */
#define IT_ARITH                  CAST(it_code_type,0x00000001)
/* add,addc,adde,addi,addic,addic.,addis,addme,addze,andi.,andis.,
   divd,divdu,divw,divwu,isel,macchw,macchws,macchwsu,macchwu,
   machhw,machhws,machhwsu,machhwu,maclhw,maclhws,maclhwsu,maclhwu,
   mulchw,mulchwu,mulhd,mulhdu,mulhhw,mulhhwu,mulhw,mulhwu,mulld,
   mullhw,mullhwu,mulli,mullw,neg,nmacchw,nmacchws,nmachhw,
   nmachhws,nmaclhw,nmaclhws,subf,subfc,subfe,subfic,subfme,subfze */

/* branch */
#define IT_BRANCH                 CAST(it_code_type,0x00000002)
/* b,ba,bc,bc_uncond,bca,bcctr,bcctrl,bcl,bcl_uncond,bcla,bclr,bclrl,
   bdnz,bdz,bf,bl,bla,bt */

/* cache control: */
#define IT_CACHE                  CAST(it_code_type,0x00000004)
/* dcbf,dcbi,dcbst,dcbt,dcbtst,dcbz,dcbz_l,dccci,dcread,icbi,icbt,iccci */

/* compare instruction */
#define IT_COMPARE                CAST(it_code_type,0x00000008)
/* breakpt,cmp,cmpdi,cmpl,cmpldi,cmplwi,cmpwi,fcmpo,fcmpu,
   ps_cmpo0,ps_cmpo1,ps_cmpu0,ps_cmpu1,
   ps_merge00,ps_merge01,ps_merge10,ps_merge11 */

#define IT_CR                     CAST(it_code_type,0x00000010) /* operator for CR register */ \
                                  /* crand,crandc,creqv,crnand,crnor,cror,crorc,crxor,mcrf,mcrfs,mcrxr,mfcr,mtcrf */

/* eciwx and ecowx */
#define IT_EXT_CONTROL            CAST(it_code_type,0x00000020)

/* floating point */
#define IT_FLOAT                  CAST(it_code_type,0x00000040)

/* traps, system calls, rfi forms */
#define IT_INTERRUPT              CAST(it_code_type,0x00000080)
/* hrfid,rfci,rfi,rfid,sc,td,tdi,tw,twi */

/* loads from memory */
#define IT_LOAD                   CAST(it_code_type,0x00000100)
/* lbz,lbzu,lbzux,lbzx,ld,ldarx,ldbrx,ldu,ldux,ldx,lfd,lfdu,lfdux,
   lfdx,lfs,lfsu,lfsux,lfsx,lha,lhau,lhaux,lhax,lhbrx,lhz,lhzu,lhzux,
   lhzx,lmd,lmw,lswi,lswx,lvebx,lvehx,lvewx,lvewx128,lvlx,lvlx128,
   lvlxl,lvlxl128,lvrx,lvrx128,lvrxl,lvrxl128,lvsl,lvsr,lvx,lvx128,
   lvxl,lvxl128,lwa,lwarx,lwaux,lwax,lwbrx,lwz,lwzu,lwzux,lwzx,
   psq_l,psq_lu,psq_lux,psq_lx,reld,relfd,relw */

/* logical operations (like arithmetic) */
#define IT_LOGICAL                CAST(it_code_type,0x00000200)
/* and,andc,cntlzd,cntlzw,dlmzb,eqv,extsb,extsh,extsw,
   nand,nor,or,orc,ori,oris,rldcl,rldcr,rldic,rldicl,rldicr,
   rldimi,rlwimi,rlwinm,rlwnm,sld,slw,srad,sradi,
   sraw,srawi,srd,srw,xor,xori,xoris,cmpb */

/* Not a real instruction -- mambo extension */
#define IT_SPECIAL                CAST(it_code_type,0x00000400)
/* rtascall,support,support-old,support_halt */

/* special purpose registers */
#define IT_SPR                    CAST(it_code_type,0x00000800)
/* mfdcr,mfmsr,mfspr,mfsr,mfsrin,mftb,mtdcr,mtmsr,mtmsrd,
   mtspr,mtsr,mtsrd,mtsrdin,mtsrin,
   slbia,slbie,slbmfee,slbmfev,slbmte,wrtee,wrteei */

/* stores to memory */
#define IT_STORE                  CAST(it_code_type,0x00001000)
/* psq_st,psq_stu,psq_stux,psq_stx,spld,splfd,splw,stb,stbu,stbux,
   stbx,std,stdbrx,stdcx.,stdu,stdux,stdx,stfd,stfdu,stfdux,stfdx,
   stfiwx,stfs,stfsu,stfsux,stfsx,sth,sthbrx,sthu,sthux,sthx,stmd,
   stmw,stswi,stswx,stvebx,stvehx,stvewx,stvewx128,stvlx,stvlx128,
   stvlxl,stvlxl128,stvrx,stvrx128,stvrxl,stvrxl128,stvx,stvx128,
   stvxl,stvxl128,stw,stwbrx,stwcx.,stwu,stwux,stwx */

/* eieio, sync, isync */
#define IT_SYNC                   CAST(it_code_type,0x00002000)

/* address translation */
#define IT_TRANS                  CAST(it_code_type,0x00004000)
/* mfsr,mfsrin,mtsr,mtsrd,mtsrdin,mtsrin,
   slbia,slbie,slbmfee,slbmfev,slbmte,
   tlbia,tlbie,tlbiel,tlbre,tlbsx,tlbsync,tlbwe */

/* VMX32 -- base VMX */
#define IT_VMX32                  CAST(it_code_type,0x00008000)

/* Support instructions like attn */
#define IT_SUPPORT                CAST(it_code_type,0x00010000)

/* lwarx,ldarx,stwcx,stdcx (SRR0 bit during trace exceptions) */
#define IT_LARX_STCX              CAST(it_code_type,0x00020000)

/* VMX128 */
#define IT_VMX128                 CAST(it_code_type,0x00040000)

/* Algebraic form of loads */
#define IT_ALGEBRAIC              CAST(it_code_type,0x00080000)
/* lha,lhau,lhaux,lhax,lwa,lwaux,lwax */

/* load or store multiples */
#define IT_LOAD_STORE_MULTIPLE    CAST(it_code_type,0x00100000)
/* lmd,lmw,stmd,stmw */

/* load or store strings */
#define IT_LOAD_STORE_STRING      CAST(it_code_type,0x00200000)
/* lswi,lswx,stswi,stswx */

/* Instr is a floating point MADD/MSUB */
#define IT_FMAS                   CAST(it_code_type,0x00400000)

/* Instr not grouped with others */
#define IT_DCBZ                   CAST(it_code_type,0x00800000)

/* load or store multiples */
#define IT_LOAD_STORE_FLOAT_PAIR  CAST(it_code_type,0x01000000)
/* lfdp,lfpx,stfdp,stfdpx */

/* load or store q */
#define IT_LOAD_STORE_Q           CAST(it_code_type,0x02000000)
/* when loading and storing to/from cache inhibited
 * this instructions should fire data storage exception */

/* Cache inhibited load and store instructions */
#define IT_CACHE_INHIB_LOAD_STORE CAST(it_code_type,0x04000000)

/* VSX */
#define IT_VSX                    CAST(it_code_type,0x08000000)

/* floating point move */
#define IT_NO_FPSCR               CAST(it_code_type,0x10000000)


/* floating point move */
#define IT_QPX                    CAST(it_code_type,0x20000000)

/* Store data */
#define IT_ST_DATA                CAST(it_code_type,0x40000000)

/* Instr has prefix for large reg number(s) */
#define IT_PREFIXED               CAST(it_code_type,0x80000000)




/* ************************************************************ */
/*  Macros for the first one                                    */
/* ************************************************************ */

#define II_IS_BRANCH(ii)       (((ii)->inst_type_flags & IT_BRANCH) != 0)
#define II_IS_COMPARE(ii)      (((ii)->inst_type_flags & IT_COMPARE) != 0)
#define II_IS_LARX_STCX(ii)    (((ii)->inst_type_flags & IT_LARX_STCX) != 0)
#define II_IS_DATA_STORE(ii)   (((ii)->inst_type_flags & IT_STORE) != 0)
#define II_IS_DATA_LOAD(ii)    (((ii)->inst_type_flags & IT_LOAD) != 0)
#define II_IS_FLOAT(ii)        (((ii)->inst_type_flags & IT_FLOAT) != 0)
#define II_IS_FMADD(ii)        (((ii)->inst_type_flags & IT_FMAS) != 0)
#define II_IS_SPR(ii)          (((ii)->inst_type_flags & IT_SPR) != 0)
#define II_IS_SYNC(ii)         (((ii)->inst_type_flags & IT_SYNC) != 0)
#define II_IS_TRANS(ii)        (((ii)->inst_type_flags & IT_TRANS) != 0)
#define II_IS_VMX32(ii)        (((ii)->inst_type_flags & IT_VMX32) != 0)
#define II_IS_VMX128(ii)       (((ii)->inst_type_flags & IT_VMX128) != 0)
#define II_IS_VSX(ii)          (((ii)->inst_type_flags & IT_VSX) != 0)
#define II_IS_INTERRUPT(ii)    (((ii)->inst_type_flags & IT_INTERRUPT) != 0)

#define II_IS_STORE(ii)        (((ii)->inst_type_flags & (IT_STORE | IT_DCBZ)) != 0)
#define II_IS_LOAD(ii)         (((ii)->inst_type_flags & IT_LOAD) != 0)
#define II_IS_SYNC_TYPE(ii)    (((ii)->inst_type_flags & (IT_SYNC | IT_TRANS)) != 0)
#define II_IS_VMX(ii)          (((ii)->inst_type_flags & (IT_VMX32 | IT_VMX128)) != 0)

#define II_IS_QPX(ii)          (((ii)->inst_type_flags & IT_QPX) != 0)

#define II_IS_STCX(ii)                                                  \
        (((ii)->inst_type_flags & (IT_STORE | IT_LARX_STCX)) == (IT_STORE | IT_LARX_STCX))
#define II_IS_LARX(ii)                                                  \
        (((ii)->inst_type_flags & (IT_LOAD | IT_LARX_STCX)) == (IT_LOAD | IT_LARX_STCX))
#define II_IS_FPU_FLOAT(ii)                                             \
        (((ii)->inst_type_flags & (IT_FLOAT | IT_VSX | IT_VMX128 | IT_VMX32)) == IT_FLOAT)

#define II_IS_SPECIAL(ii)   (((ii)->inst_type_flags & IT_SPECIAL) != 0)
#define II_IS_ST_DATA(ii)   (((ii)->inst_type_flags & IT_ST_DATA) != 0)
#define II_NEEDS_FPSCR(ii)  (((ii)->inst_type_flags & (IT_FLOAT | IT_NO_FPSCR)) == IT_FLOAT)

/* ************************************************************ */
/*                                                              */
/*  For the second one: addl_type_flags                         */
/*                                                              */
/* ************************************************************ */

/* ************************************************************ */
/* This next four must be together, consecutive, and in fixed order */
#define AT_SERIALIZE_BITS_SHIFT   24
/* Instr forces fetch serialization */
#define AT_SERIALIZE_IFETCH       CAST(at_code_type, 0x01000000)

/* Instr forces dispatch serialization */
#define AT_SERIALIZE_DISPATCH     CAST(at_code_type, 0x02000000)

/* Instr forces execution serialization */
#define AT_SERIALIZE_EXEC         CAST(at_code_type, 0x04000000)

/* Instr not grouped with others */
#define AT_NO_GROUPING            CAST(at_code_type, 0x08000000)
/* ************************************************************ */

/* Must be the first in a group */
#define AT_FIRST_IN_GROUP         CAST(at_code_type, 0x00000001)

/* Must be the last in a group */
#define AT_LAST_IN_GROUP          CAST(at_code_type, 0x00000002)

/* Not pipelined */
#define AT_NONPIPELINED           CAST(at_code_type, 0x00000004)

/* Fusable with the proceeding bc+8 instruction */
#define AT_FUSABLE_W_BR           CAST(at_code_type, 0x00000008)

/* Special VSU instructions. */
#define AT_STALL_ISSUE            CAST(at_code_type, 0x00000010)



/* ************************************************************ */
/*  Macros for the second one                                   */
/* ************************************************************ */

#define II_SERIALIZES_IFETCH(ii)   (((ii)->addl_type_flags & AT_SERIALIZE_IFETCH) != 0)
#define II_SERIALIZES_DISPATCH(ii) (((ii)->addl_type_flags & AT_SERIALIZE_DISPATCH) != 0)
#define II_SERIALIZES_EXEC(ii)     (((ii)->addl_type_flags & AT_SERIALIZE_EXEC) != 0)

#define II_IS_NO_GROUPING(ii)      (((ii)->addl_type_flags & AT_NO_GROUPING) != 0)
#define II_IS_FIRST_IN_GROUP(ii)   (((ii)->addl_type_flags & AT_FIRST_IN_GROUP) != 0)
#define II_IS_LAST_IN_GROUP(ii)    (((ii)->addl_type_flags & AT_LAST_IN_GROUP) != 0)

#define II_IS_FUSABLE_W_BR(ii) (((ii)->addl_type_flags & AT_FUSABLE_W_BR) != 0)
#define II_IS_NONPIPELINED(ii) (((ii)->addl_type_flags & AT_NONPIPELINED) != 0)
#define II_STALLS_ISSUE(ii)    (((ii)->addl_type_flags & AT_STALL_ISSUE) != 0)


#endif /* #ifndef _IT_CODES_H_ */
