/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2004, 2005
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : mpid_t.h
 *
 * Purpose  : PID type definition
 *
 * ------------------------------------------------------- */
#ifndef _MPID_T_H_
#define _MPID_T_H_

/*
  There are 3 different uses of "pid" in the Mambo code:

  1. The PID register -- a part of the PowerPC thread state
  that is used for address translaiton, particularly in
  connection with the TLBs. A 32-bit unsigned int.

  2. A Linux level process ID that is used when we are
  trying to attach an emitter to Mambo.  This pid is
  taken from a command line argument or TCL command,
  and is passed to EMIT_AttachReader.  This is an int,
  or maybe it should be a pid_t.

  3. A process ID for the operating system that is
  being simulated by Mambo.  This comes to us
  thru a set of sim_support calls that identify
  when processes are created for the simulated
  system.  Mambo keeps track of these, in process.c
  and trackpid.c, and will emit them as part of
  the emitter data stream.  This is an mpid_t.

  The code assumes that mpid_t is something that
  we can index with -- a "small" integer.
*/


/* This is used in Linux */
#define MAX_PID        CAST(mpid_t,(32 * 1024))
#define INVALID_PID    CAST(mpid_t,-1)
#define INIT_PID       CAST(mpid_t,1)


typedef int mpid_t;

#define FormatPid  "%5d"


#endif /* #ifndef _MPID_T_H_ */
