/* -------------------------------------------------------
 *
 * Licensed Materials - Property of IBM.
 * (C) Copyright IBM Corporation 2004, 2006
 * All Rights Reserved.
 *
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corporation.
 *
 * Filename : trans_id_t.h
 *
 * Purpose    : Transaction ID's for hop tracing
 *
 * ------------------------------------------------------- */

/*
 * The transaction ID has two components:
 * - Major ID, which is uniquely determined at the time
 *   the transaction to be monitored is created
 *
 * - Fragment ID's, which are created when a transaction
 *   is fragmented into smaller parts (often necessary
 *   as a transaction migrates across smaller buses).
 *   A list of these is kept, since fragments may be
 *   further decomposed.
 */

#ifndef _TRANS_ID_T_H_
#define _TRANS_ID_T_H_


typedef uint64      TRID_Major;         /* Major transaction ID */
#define TRID_INVALID_MAJOR  ((TRID_Major)(-1))


typedef uint16      TRID_Frag;          /* Fragment transaction ID */
#define TRID_INVALID_FRAG   ((TRID_Frag)(-1))


/* Fragment Description */
#define TRID_FRAG_DESC_SIZE 16
struct TRID_FragDesc
{
    char  c[TRID_FRAG_DESC_SIZE];
};
typedef struct TRID_FragDesc TRID_FragDesc;


#define TRID_MAX_BRANCHES  2
struct _TRID_Frag_Node_
{
    TRID_Frag       fid;    /* Fragment ID */
    TRID_FragDesc   fdes;   /* Fragment Description */
};
typedef struct _TRID_Frag_Node_ TRID_Frag_Node;


#define TRID_DESC_SIZE 31

struct _TRID_
{
    TRID_Major      mid;                        /* Major ID */
    char            desc[TRID_DESC_SIZE];       /* Description */
    unsigned char   done;                       /* Non-zero if no more operations to follow on this TRID */
    uint32          level;                      /* Branch level of this ID */
    TRID_Frag_Node  frags[TRID_MAX_BRANCHES];   /* Fragment List */
};
typedef struct _TRID_ TRID;

#endif /* #ifndef _TRANS_ID_T_H_ */
