#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2005, 2006
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#   axon_dev_tree.tcl
#
#   TCL procs for creating firmware device tree nodes for Axon Southbridge.
#------------------------------------------------------------------------

namespace eval axon_dev_tree {
    variable version       "1.0"
}


proc axon_dev_tree::debug_puts { msg } {
    if {[simdebug status open_firmware] != 0} {
        puts $msg
    }
}

proc axon_dev_tree::build_axon_node { sim mcm parent be_num } {
    global errorInfo
    puts "building axon device tree node"
    if { [catch {

        #############################################################################
        # Node: axon
        #############################################################################
        set hex_addr [$sim config query axon/start]
        set node2 [$sim mcm $mcm of addchild $parent "axon" [format "%lx" $hex_addr]]
        $sim mcm $mcm of addprop  $node2 "string" "name" "axon"
        $sim mcm $mcm of addprop  $node2 "string" "device_type" "axon"
        $sim mcm $mcm of addprop  $node2 "string" "model" "Axon"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node2 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node2 "array" "#size-cells" tmp_array
        set tmp_array [list 0x100 0x0 0xe0 0x0]
        $sim mcm $mcm of addprop  $node2 "array" "reg" tmp_array

        set tmp_array [list 0x0        0x0 0x100 0x0 0x10 0x0 \
                            0x20000020 0x0 0x120 0x0 0x10 0x0 \
                            0x40000040 0x0 0x140 0x0 0x10 0x0 \
                            0x50000050 0x0 0x150 0x0 0x10 0x0 \
                            0x80000080 0x0 0x180 0x0 0x10 0x0 \
                            0xa00000a0 0x0 0x1a0 0x0 0x10 0x0 \
                            0xc00000c0 0x0 0x1c0 0x0 0x10 0x0 \
                            0xd00000d0 0x0 0x1d0 0x0 0x10 0x0]
        $sim mcm $mcm of addprop  $node2 "array" "ranges" tmp_array

        set tmp_array [list 0x48a]
        $sim mcm $mcm of addprop  $node2 "array" "ioid" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node2 "array" "ioif" tmp_array

        # dcr-parent property added below

        # interrupt-parent added below

        set tmp_array [list 0x0 0x0 0x0 0x0 0x80000000]
        $sim mcm $mcm of addprop  $node2 "array" "ibm,dma-window" tmp_array

        set tmp_array [list 0x4 0x0 0x0 $be_num $be_num]
        $sim mcm $mcm of addprop  $node2 "array" "ibm,associativity" tmp_array

        debug_puts "Created axon@[format "%lx" $hex_addr]"

        #############################################################################
        # Node: axon/interrupt-controller
        #############################################################################
        set node3 [$sim mcm $mcm of addchild $node2 "interrupt-controller" ""]
        $sim mcm $mcm of addprop  $node3 "string" "name" "interrupt-controller"
        $sim mcm $mcm of addprop  $node3 "string" "device_type" "platform-open-pic"
        $sim mcm $mcm of addprop  $node3 "string" "compatible" "CBEA,platform-open-pic"
        $sim mcm $mcm of addprop  $node3 "string" "interrupt-controller" ""
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xffc00000 0x30000]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x20b]
        $sim mcm $mcm of addprop  $node3 "array" "interrupts" tmp_array

        # Node: interrupt-controller (Property interrupt-parent)
        set bp_base    [$sim be $be_num display bp_base]
        set be_intc_node  [$sim mcm $mcm of find_device /be@[format "%lx" $bp_base]/interrupt-controller@508400]
        if { $be_intc_node == -1 } { error "Can't find intcontroller for be $be_num!" }
        $sim mcm $mcm of addprop  $node3 "int" "interrupt-parent" $be_intc_node

        debug_puts "Created axon/interrupt-controller"

        set axon_intc_node $node3

        # Node: axon (Property interrupt-parent)
        $sim mcm $mcm of addprop  $node2 "int" "interrupt-parent" $axon_intc_node

        #############################################################################
        # Node: axon/plb5
        #############################################################################
        set node3 [$sim mcm $mcm of addchild $node2 "plb5" ""]
        $sim mcm $mcm of addprop  $node3 "string" "name" "plb5"
        $sim mcm $mcm of addprop  $node3 "string" "device_type" "plb5"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#size-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x13d92d40]
        $sim mcm $mcm of addprop  $node3 "array" "clock-frequency" tmp_array
        set tmp_array [list 0x200 0x40]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x40 0x2 0x41 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5"

        #############################################################################
        # Node: axon/plb5/plb5to4
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "plb5to4" ""]
        $sim mcm $mcm of addprop  $node4 "string" "name" "plb5to4"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xc0 0x10]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x42 0x2 0x43 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/plb5to4"

        #############################################################################
        # Node: axon/plb5/mc
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "mc" "2000002000000000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "mc"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "dma-memory"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x20000020 0x0 0x0 0x0]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x14 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x26 0x2 0x27 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array
        set tmp_array [list 0x20]
        $sim mcm $mcm of addprop  $node4 "array" "threshold-ce" tmp_array
        set tmp_array [list 0x60]
        $sim mcm $mcm of addprop  $node4 "array" "threshold-period-hours" tmp_array

        set node4 [$sim mcm $mcm of addchild $node3 "mc" "0"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "mc"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "dma-memory"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x0 0x0 0x0 0x0]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x10 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x23 0x2 0x24 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array
        set tmp_array [list 0x20]
        $sim mcm $mcm of addprop  $node4 "array" "threshold-ce" tmp_array
        set tmp_array [list 0x60]
        $sim mcm $mcm of addprop  $node4 "array" "threshold-period-hours" tmp_array

        debug_puts "Created axon/plb5/mc nodes"

        #############################################################################
        # Node: axon/plb5/msi
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "msi" "4000004400003000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "msi"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "msi"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x40000044 0x3000 0x0 0x1000]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x6f 0x7]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x72 0x3 0x73 0x3]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/msi"

        #############################################################################
        # Node: axon/plb5/mailbox
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "mailbox" "4000004400002000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "mailbox"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "mailbox"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x40000044 0x2000 0x0 0x800 0x40000044 0x2800 0x0 0x800]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x68 0x7]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x74 0x3 0x75 0x3]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/mailbox"

        #############################################################################
        # Node: axon/plb5/dma-controller
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "dma-controller" "4000004400001000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "dma-controller"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "dma-controller"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x40000044 0x1000 0x0 0x1000]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x77 0x2 0x78 0x2 0x79 0x2 0x7a 0x2 0x7b 0x2 0x7c 0x2 0x7d 0x2 0x7e 0x2 0x7f 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/dma-controller"

        #############################################################################
        # Node: axon/plb5/pciex
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "pciex" "a00000a200000000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "pciex"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "pciex"
        $sim mcm $mcm of addprop  $node4 "string" "compatible" "pciex"
        $sim mcm $mcm of addprop  $node4 "string" "model" "Axon"
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node4 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#size-cells" tmp_array
        set tmp_array [list 0x280 0x40]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0xa00000a2 0x0 0x2 0x0]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x1000000 0x0 0x0 0xa00000a3 0x0 0x0 0x10000 0x2000000 0x0 0x80000000 0xd00000d0 0x80000000 0x0 0x40000000 0x42000000 0x0 0xc0000000 0xd00000d0 0xc0000000 0x0 0x40000000]
        $sim mcm $mcm of addprop  $node4 "array" "ranges" tmp_array
        set tmp_array [list 0x0 0xff]
        $sim mcm $mcm of addprop  $node4 "array" "bus-range" tmp_array
        set tmp_array [list 0x48a]
        $sim mcm $mcm of addprop  $node4 "array" "ioid" tmp_array
        set tmp_array [list 0x0 0x0 0x0 0x0 $axon_intc_node 0x6a 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupt-map" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xf800 0x0 0x0 0x7]
        $sim mcm $mcm of addprop  $node4 "array" "interrupt-map-mask" tmp_array

        debug_puts "Created axon/plb5/pciex"

        #############################################################################
        # Node: axon/plb5/pciex/pci
        #############################################################################
        set node5 [$sim mcm $mcm of addchild $node4 "pci" "0"]
        $sim mcm $mcm of addprop  $node5 "string" "name" "pci"
        set tmp_array [list 0x1014]
        $sim mcm $mcm of addprop  $node5 "array" "vendor-id" tmp_array
        set tmp_array [list 0x32c]
        $sim mcm $mcm of addprop  $node5 "array" "device-id" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "revision-id" tmp_array
        set tmp_array [list 0x60400]
        $sim mcm $mcm of addprop  $node5 "array" "class-code" tmp_array
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node5 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node5 "array" "#size-cells" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "devsel-speed" tmp_array
        $sim mcm $mcm of addprop  $node5 "string" "device_type" "pci"
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "primary_bus" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "secondary_bus" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "subordinate_bus" tmp_array
        set tmp_array [list 0x1 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "bus-range" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "slot-names" tmp_array
        set tmp_array [list 0xc3000010 0x1 0x0 0x0 0x80000000]
        $sim mcm $mcm of addprop  $node5 "array" "assigned-addresses" tmp_array
        $sim mcm $mcm of addprop  $node5 "string" "interrupt-map" ""
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xf800 0x0 0x0 0x7]
        $sim mcm $mcm of addprop  $node5 "array" "interrupt-map-mask" tmp_array
        set tmp_array [list 0x0 0x0 0x0 0x0 0x0 0x43000010 0x0 0x0 0x0 0x80000000]
        $sim mcm $mcm of addprop  $node5 "array" "reg" tmp_array

        debug_puts "Created axon/plb5/pciex/pci"

        #############################################################################
        # Node: axon/plb5/pciex
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "pciex" "a00000a000000000"]
        $sim mcm $mcm of addprop  $node4 "string" "name" "pcie"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "pci"
        $sim mcm $mcm of addprop  $node4 "string" "compatible" "pci"
        $sim mcm $mcm of addprop  $node4 "string" "model" "Axon"
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node4 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#size-cells" tmp_array
        set tmp_array [list 0x240 0x40]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0xa00000a0 0x0 0x2 0x0]
        $sim mcm $mcm of addprop  $node4 "array" "reg" tmp_array
        set tmp_array [list 0x1000000 0x0 0x0 0xa00000a1 0x0 0x0 0x10000 0x2000000 0x0 0x80000000 0xc00000c0 0x80000000 0x0 0x40000000 0x42000000 0x0 0xc0000000 0xc00000c0 0xc0000000 0x0 0x40000000]
        $sim mcm $mcm of addprop  $node4 "array" "ranges" tmp_array
        set tmp_array [list 0x0 0xff]
        $sim mcm $mcm of addprop  $node4 "array" "bus-range" tmp_array
        set tmp_array [list 0x48a]
        $sim mcm $mcm of addprop  $node4 "array" "ioid" tmp_array
        set tmp_array [list 0x0 0x0 0x0 0x0 $axon_intc_node 0x62 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupt-map" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xf800 0x0 0x0 0x7]
        $sim mcm $mcm of addprop  $node4 "array" "interrupt-map-mask" tmp_array

        debug_puts "Created axon/plb5/pciex"

        #############################################################################
        # Node: axon/plb5/pciex/pci
        #############################################################################
        set node5 [$sim mcm $mcm of addchild $node4 "pci" "0"]
        $sim mcm $mcm of addprop  $node5 "string" "name" "pci"
        set tmp_array [list 0x1014]
        $sim mcm $mcm of addprop  $node5 "array" "vendor-id" tmp_array
        set tmp_array [list 0x32c]
        $sim mcm $mcm of addprop  $node5 "array" "device-id" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "revision-id" tmp_array
        set tmp_array [list 0x60400]
        $sim mcm $mcm of addprop  $node5 "array" "class-code" tmp_array
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node5 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node5 "array" "#size-cells" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "devsel-speed" tmp_array
        $sim mcm $mcm of addprop  $node5 "string" "device_type" "pci"
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "primary_bus" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "secondary_bus" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "subordinate_bus" tmp_array
        set tmp_array [list 0x1 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "bus-range" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "slot-names" tmp_array
        set tmp_array [list 0xc3000010 0x1 0x0 0x0 0x80000000]
        $sim mcm $mcm of addprop  $node5 "array" "assigned-addresses" tmp_array
        $sim mcm $mcm of addprop  $node5 "string" "interrupt-map" ""
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xf800 0x0 0x0 0x7]
        $sim mcm $mcm of addprop  $node5 "array" "interrupt-map-mask" tmp_array
        set tmp_array [list 0x0 0x0 0x0 0x0 0x0 0x43000010 0x0 0x0 0x0 0x80000000]
        $sim mcm $mcm of addprop  $node5 "array" "reg" tmp_array

        debug_puts "Created axon/plb5/pcix/pci"

        #############################################################################
        # Node: axon/plb5/plb4
        #############################################################################
        set node4 [$sim mcm $mcm of addchild $node3 "plb4" ""]
        $sim mcm $mcm of addprop  $node4 "string" "name" "plb4"
        $sim mcm $mcm of addprop  $node4 "string" "device_type" "plb4"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "#size-cells" tmp_array
        set tmp_array [list 0x16 0x2 0x80 0x10 0x100 0x40]
        $sim mcm $mcm of addprop  $node4 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x31 0x2]
        $sim mcm $mcm of addprop  $node4 "array" "interrupts" tmp_array
        set tmp_array [list 0x7ed6b40]
        $sim mcm $mcm of addprop  $node4 "array" "clock-frequency" tmp_array

        debug_puts "Created axon/plb5/plb4"

        #############################################################################
        # Node: axon/plb5/plb4/opb
        #############################################################################
        set node5 [$sim mcm $mcm of addchild $node4 "opb" ""]
        $sim mcm $mcm of addprop  $node5 "string" "name" "opb"
        $sim mcm $mcm of addprop  $node5 "string" "device_type" "opb"
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "#address-cells" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "#size-cells" tmp_array
        set tmp_array [list 0x0 0x40000045 0x0 0x80000000 0x80000000 0x40000045 0x80000000 0x80000000]
        $sim mcm $mcm of addprop  $node5 "array" "ranges" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "plb4-priority" tmp_array
        set tmp_array [list 0x90 0x10]
        $sim mcm $mcm of addprop  $node5 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x7ed6b40]
        $sim mcm $mcm of addprop  $node5 "array" "clock-frequency" tmp_array

        debug_puts "Created axon/plb5/plb4/opb"

        #############################################################################
        # Node: axon/plb5/plb4/opb/opb-arbiter
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "opb-arbiter" "40000c00"]
        $sim mcm $mcm of addprop  $node6 "string" "name" "opb-arbiter"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "opb-arbiter"
        set tmp_array [list 0x40000c00 0x40]
        $sim mcm $mcm of addprop  $node6 "array" "reg" tmp_array

        debug_puts "Created opb-arbiter"

        #############################################################################
        # Node: axon/plb5/plb4/opb/gpio
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "gpio" "40000700"]
        $sim mcm $mcm of addprop  $node6 "string" "name" "gpio"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "gpio"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x40000700 0x80]
        $sim mcm $mcm of addprop  $node6 "array" "reg" tmp_array
        set tmp_array [list 0x17 0x1 0x18 0x1 0x19 0x1 0x1a 0x1 0x1b 0x1 0x1c 0x1 0x1d 0x1 0x32 0x1 0x33 0x1 0x34 0x1 0x35 0x1 0x36 0x1 0x37 0x1 0x45 0x1 0x46 0x1 0x47 0x1 0x48 0x1 0x49 0x1 0x4a 0x1 0x4b 0x1 0x4c 0x1 0x4d 0x1 0x4e 0x1 0x4f 0x1 0x50 0x1 0x51 0x1 0x52 0x1 0x53 0x1 0x54 0x1 0x55 0x1 0x56 0x1 0x57 0x1]
        $sim mcm $mcm of addprop  $node6 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/plb4/opb/gpio"

        #############################################################################
        # Node: axon/plb5/plb4/opb/spi
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "spi" "40000000"]
        $sim mcm $mcm of addprop  $node6 "string" "name" "spi"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "spi"
        set tmp_array [list 0x40000000 0x10]
        $sim mcm $mcm of addprop  $node6 "array" "reg" tmp_array

        debug_puts "Created axon/plb5/plb4/opb/spi"

        #############################################################################
        # Node: axon/plb5/plb4/opb/i2c
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "i2c" "40000400"]
        $sim mcm $mcm of addprop  $node6 "string" "name" "i2c"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "i2c"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x2 0x3]
        $sim mcm $mcm of addprop  $node6 "array" "interrupts" tmp_array
        set tmp_array [list 0x40000400 0x20]
        $sim mcm $mcm of addprop  $node6 "array" "reg" tmp_array

        debug_puts "Created axon/plb5/plb4/opb/i2c"

        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "ebc" ""]
        $sim mcm $mcm of addprop  $node6 "string" "name" "ebc"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "ebc"
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node6 "array" "#address-cells" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node6 "array" "#size-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "#interrupt-cells" tmp_array
        $sim mcm $mcm of addprop  $node6 "string" "ranges" ""
        set tmp_array [list 0x12 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x55 0x3]
        $sim mcm $mcm of addprop  $node6 "array" "interrupts" tmp_array

        debug_puts "Created axon/plb5/plb4/opb/ebc"

        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/ipmi
        #############################################################################
        set node7 [$sim mcm $mcm of addchild $node6 "ipmi" "2600008"]
        $sim mcm $mcm of addprop  $node7 "string" "name" "ipmi"
        $sim mcm $mcm of addprop  $node7 "string" "device_type" "ipmi"
        $sim mcm $mcm of addprop  $node7 "string" "compatible" "ipmi-kcs"
        set tmp_array [list 0x2600008 0x8]
        $sim mcm $mcm of addprop  $node7 "array" "reg" tmp_array
        set tmp_array [list 0x4]
        $sim mcm $mcm of addprop  $node7 "array" "reg-spacing" tmp_array
        $sim mcm $mcm of addprop  $node7 "string" "model" "IBM,BMC.BNBT04A"


        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/ibm,pmi
        #############################################################################
        set node7 [$sim mcm $mcm of addchild $node6 "ibm,pmi" "2600000"]
        $sim mcm $mcm of addprop  $node7 "string" "name" "ibm,pmi"
        $sim mcm $mcm of addprop  $node7 "string" "device_type" "ibm,pmi"
        set tmp_array [list 0x706d6900 0x706d692d 0x312e3000]
        $sim mcm $mcm of addprop  $node7 "array" "compatible" tmp_array
        set tmp_array [list 0x2600000 0x8]
        $sim mcm $mcm of addprop  $node7 "array" "reg" tmp_array
        set tmp_array [list 0x1a 0x0]
        $sim mcm $mcm of addprop  $node7 "array" "interrupts" tmp_array


        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/rtc
        #############################################################################
        set node7 [$sim mcm $mcm of addchild $node6 "rtc" "2200000"]
        $sim mcm $mcm of addprop  $node7 "string" "name" "rtc"
        $sim mcm $mcm of addprop  $node7 "string" "device_type" "rtc"
        set tmp_array [list 0x2200000 0x10]
        $sim mcm $mcm of addprop  $node7 "array" "reg" tmp_array


        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/nvram
        #############################################################################
        set node7 [$sim mcm $mcm of addchild $node6 "nvram" "2000000"]
        $sim mcm $mcm of addprop  $node7 "string" "name" "nvram"
        $sim mcm $mcm of addprop  $node7 "string" "device_type" "nvram"
        set tmp_array [list 0x2000000 0x100000]
        $sim mcm $mcm of addprop  $node7 "array" "reg" tmp_array
        set tmp_array [list 0x100000]
        $sim mcm $mcm of addprop  $node7 "array" "#bytes" tmp_array


        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/flash
        #############################################################################
        set node7 [$sim mcm $mcm of addchild $node6 "flash" "0"]
        $sim mcm $mcm of addprop  $node7 "string" "name" "flash"
        $sim mcm $mcm of addprop  $node7 "string" "device_type" "flash"
        set tmp_array [list 0x0 0x2000000]
        $sim mcm $mcm of addprop  $node7 "array" "reg" tmp_array
        set tmp_array [list 0x2000000]
        $sim mcm $mcm of addprop  $node7 "array" "#bytes" tmp_array


        #############################################################################
        # Node: axon/plb5/plb4/opb/ebc/flash/boot-rom
        #############################################################################
        set node8 [$sim mcm $mcm of addchild $node7 "boot-rom" ""]
        $sim mcm $mcm of addprop  $node8 "string" "name" "boot-rom"
        $sim mcm $mcm of addprop  $node8 "string" "device_type" "boot-rom"


        #############################################################################
        # Node: axon/plb5/plb4/plb4to5
        #############################################################################
        set node5 [$sim mcm $mcm of addchild $node4 "plb4to5" ""]
        $sim mcm $mcm of addprop  $node5 "string" "name" "plb4to5"
        $sim mcm $mcm of addprop  $node5 "string" "device_type" "plb4to5"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node5 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "plb4-priority" tmp_array
        set tmp_array [list 0xd0 0x10]
        $sim mcm $mcm of addprop  $node5 "array" "dcr-reg" tmp_array

        debug_puts "Created axon/plb5/plb4/plb4to5"

        #############################################################################
        # Node: axon/plb5/plb4/pcix
        #############################################################################
        set node5 [$sim mcm $mcm of addchild $node4 "pcix" "4000004600000000"]
        $sim mcm $mcm of addprop  $node5 "string" "name" "pcix"
        $sim mcm $mcm of addprop  $node5 "string" "device_type" "pci"
        $sim mcm $mcm of addprop  $node5 "string" "compatible" "pci"
        $sim mcm $mcm of addprop  $node5 "string" "model" "Axon"
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node5 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node5 "array" "#size-cells" tmp_array
        set tmp_array [list 0x40000046 0x0 0x1a 0x0]
        $sim mcm $mcm of addprop  $node5 "array" "reg" tmp_array
        set tmp_array [list 0x1000000 0x0 0x0 0x40000046 0x8000000 0x0 0x10000 0x2000000 0x0 0x80000000 0x40000047 0x80000000 0x0 0x40000000 0x42000000 0x0 0xc0000000 0x40000047 0xc0000000 0x0 0x40000000]
        $sim mcm $mcm of addprop  $node5 "array" "ranges" tmp_array
        set tmp_array [list 0x0 0xff]
        $sim mcm $mcm of addprop  $node5 "array" "bus-range" tmp_array
        set tmp_array [list 0x48a]
        $sim mcm $mcm of addprop  $node5 "array" "ioid" tmp_array
				
        set tmp_array [list 0x800 0x0 0x0 0x1 $axon_intc_node 0x0 0x2 0x900 0x0 0x0 0x2 $axon_intc_node 0x1 0x2 0x3800 0x0 0x0 0x1 $axon_intc_node 0x1D 0x2]
        $sim mcm $mcm of addprop  $node5 "array" "interrupt-map" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node5 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0xf800 0x0 0x0 0x7]
        $sim mcm $mcm of addprop  $node5 "array" "interrupt-map-mask" tmp_array

        debug_puts "Created axon/plb5/plb4/pcix"

        #############################################################################
        # Node: axon/plb5/plb4/pcix/ide
        #############################################################################
        set node6 [$sim mcm $mcm of addchild $node5 "ide" "7"]
        $sim mcm $mcm of addprop  $node6 "string" "name" "ide"
        $sim mcm $mcm of addprop  $node6 "string" "device_type" "ide"
        set tmp_array [list 0x1095]
        $sim mcm $mcm of addprop  $node6 "array" "vendor-id" tmp_array
        set tmp_array [list 0x680]
        $sim mcm $mcm of addprop  $node6 "array" "device-id" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "revision-id" tmp_array
        set tmp_array [list 0x10185]
        $sim mcm $mcm of addprop  $node6 "array" "class-code" tmp_array
        set tmp_array [list 0x3]
        $sim mcm $mcm of addprop  $node6 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node6 "array" "#size-cells" tmp_array
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node6 "array" "devsel-speed" tmp_array
        $sim mcm $mcm of addprop  $node6 "string" "fast-back-to-back" ""
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $node6 "array" "interrupts" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node6 "array" "min-grant" tmp_array
        set tmp_array [list 0x0]
        $sim mcm $mcm of addprop  $node6 "array" "max-latency" tmp_array
        set tmp_array [list 0x680]
        $sim mcm $mcm of addprop  $node6 "array" "subsystem-id" tmp_array
        set tmp_array [list 0x1095]
        $sim mcm $mcm of addprop  $node6 "array" "subsystem-vendor-id" tmp_array
        set tmp_array [list 0x82003830 0x0 0x80080000 0x0 0x80000 0x81003810 0x0 0x1000 0x0 0x8 0x81003814 0x0 0x1008 0x0 0x4 0x81003818 0x0 0x1010 0x0 0x8 0x8100381c 0x0 0x1018 0x0 0x4 0x81003820 0x0 0x1020 0x0 0x10 0x82003824 0x0 0x80040000 0x0 0x100]
        $sim mcm $mcm of addprop  $node6 "array" "assigned-addresses" tmp_array
        set tmp_array [list 0x3800 0x0 0x0 0x0 0x0 0x1003810 0x0 0x0 0x0 0x8 0x1003814 0x0 0x0 0x0 0x4 0x1003818 0x0 0x0 0x0 0x8 0x100381c 0x0 0x0 0x0 0x4 0x1003820 0x0 0x0 0x0 0x10 0x2003824 0x0 0x0 0x0 0x100 0x2003830 0x0 0x0 0x0 0x80000]
        $sim mcm $mcm of addprop  $node6 "array" "reg" tmp_array

        debug_puts "Created axon/plb5/plb4/pcix/ide"

        #############################################################################
        # Node: axon/plb5/plb4/pcix/ethernet -- removed
        #############################################################################

        #############################################################################
        # Node: axon/c3po
        #############################################################################
        set node3 [$sim mcm $mcm of addchild $node2 "c3po" "4000004400000000"]
        $sim mcm $mcm of addprop  $node3 "string" "name" "c3po"
        $sim mcm $mcm of addprop  $node3 "string" "device_type" "c3po"
        set tmp_array [list 0x40 0x40]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-reg" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x5d 0x2 0x5e 0x2 0x5f 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "interrupts" tmp_array
        set tmp_array [list 0x40000044 0x0 0x0 0x1000]
        $sim mcm $mcm of addprop  $node3 "array" "reg" tmp_array

        debug_puts "Created axon/c3po"

        #############################################################################
        # Node: axon/scratchpad
        #############################################################################
        set node3 [$sim mcm $mcm of addchild $node2 "scratchpad" "4000004400000c00"]
        $sim mcm $mcm of addprop  $node3 "string" "name" "scratchpad"
        $sim mcm $mcm of addprop  $node3 "string" "device_type" "scratchpad"
        set tmp_array [list 0x40000044 0xc00 0x0 0x100]
        $sim mcm $mcm of addprop  $node3 "array" "reg" tmp_array

        debug_puts "Created axon/scratchpad"

        #############################################################################
        # Node: axon/dcr
        #############################################################################
        set node3 [$sim mcm $mcm of addchild $node2 "dcr" "8000008000000000"]
        $sim mcm $mcm of addprop  $node3 "string" "name" "dcr"
        $sim mcm $mcm of addprop  $node3 "string" "device_type" "dcr"
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#interrupt-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#address-cells" tmp_array
        set tmp_array [list 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "#size-cells" tmp_array
        set tmp_array [list 0x80000080 0x0 0x1 0x0]
        $sim mcm $mcm of addprop  $node3 "array" "reg" tmp_array
        set tmp_array [list 0x18 0x8]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-reg" tmp_array
        $sim mcm $mcm of addprop  $node3 "string" "dcr-controller" ""
        set tmp_array [list 0x80000080 0x0 0x1 0x0]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-mmio-space" tmp_array
        set tmp_array [list 0x80000080 0x0 0x1 0x0]
        $sim mcm $mcm of addprop  $node3 "array" "dcr-mmio-base" tmp_array
        $sim mcm $mcm of addprop  $node3 "string" "dcr-access-method" "axon-mmio"
        set tmp_array [list 0x10]
        $sim mcm $mcm of addprop  $node3 "array" "axon-mmio-stride" tmp_array
        set tmp_array [list 0x1f 0x2]
        $sim mcm $mcm of addprop  $node3 "array" "interrupts" tmp_array

        debug_puts "Created axon/dcr"

        # axon dcr-parent property
        $sim mcm $mcm of addprop  $node2 "int" "dcr-parent" $node3

    } ] } then {
        puts $errorInfo
    }
}
