#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2005, 2006
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#   dev_tree.tcl
#
#   TCL procs for creating firmware device tree.
#------------------------------------------------------------------------


namespace eval dev_tree_cell {
    variable version       "1.0"
}

proc dev_tree_cell::add_spe_node { sim mcm parent spe_num spe_base spe_isrc } {

    set spe_node [ $sim mcm $mcm of addchild $parent "spe" [format "%x" $spe_base] ]
    # reg is the < offset, size > of the SPE LS, priv1, priv2 and ...
    set prob_offset [expr 0x400000 + ($spe_num*0x2000)]
    set reg_prop [list [expr $spe_base + 0]       0x40000 \
                       [expr $spe_base + 0x40000] 0x20000 \
                       [expr $spe_base + 0x60000] 0x20000 \
                       $prob_offset                0x2000]
    $sim mcm $mcm of addprop $spe_node "array"    "reg"                reg_prop
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    set interrupts_prop [list $spe_isrc [expr 0x100+$spe_isrc] [expr 0x200+$spe_isrc]]
    $sim mcm $mcm of addprop $spe_node "array"    "interrupts"         interrupts_prop

    return $spe_node
}

proc dev_tree_cell::add_be_node { sim mcm parent be_num } {

    set bp_base    [$sim be $be_num display bp_base]
    set bp_base_hi [rshift64 $bp_base 32]

    set be_node [ $sim mcm $mcm of addchild $parent "be" [format64 "%llx" $bp_base] ]
    $sim mcm $mcm of addprop $be_node "string"    "name"               "be"
    $sim mcm $mcm of addprop $be_node "string"    "device_type"        "be"
    $sim mcm $mcm of addprop $be_node "string"    "model"              "IBM,CBEA"
    $sim mcm $mcm of addprop $be_node "string"    "ibm,dt-version"     "1.0"
    $sim mcm $mcm of addprop $be_node "int"       "#address-cells"     1
    $sim mcm $mcm of addprop $be_node "int"       "#size-cells"        1
    # reg is the < address, size > of the CBEs MMIO registers
    set reg_prop [list $bp_base_hi 0x00000000 0x00000000 0x00800000]
    $sim mcm $mcm of addprop $be_node "array"     "reg"                reg_prop
    # ranges is a triple < child, parent, size >
    set ranges_prop [list 0x00000000 $bp_base_hi 0x00000000 0x00800000]
    $sim mcm $mcm of addprop $be_node "array"     "ranges"             ranges_prop
    set assoc_prop [list 4 0 0 $be_num $be_num]
    $sim mcm $mcm of addprop $be_node "array"     "ibm,associativity"  assoc_prop

    set cpus_prop [ $sim mcm $mcm of find_device /cpus/PowerPC,BE@$be_num ]
    $sim mcm $mcm of addprop $be_node int         "cpus"               $cpus_prop

    # ioc node
    set hex_addr 0x00510000 
    set ioc_node   [ $sim mcm $mcm of addchild $be_node "ioc" [format "%x" $hex_addr] ]
    # reg specifies the two MMIO ranges for the IOC 
    set reg_prop [list $hex_addr 0x00001000 [expr $hex_addr+0x1000] 0x00001000]
    $sim mcm $mcm of addprop $ioc_node "array"     "reg"                reg_prop
    $sim mcm $mcm of addprop $ioc_node "string"    "device_type"        "ioc"

    # bic0 node
    set hex_addr 0x00512000 
    set bic0_node   [ $sim mcm $mcm of addchild $be_node "bic0" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $bic0_node "string"    "name"               "bic0"
    # reg specifies the MMIO ranges for the BIC0
    set reg_prop [list $hex_addr 0x00001000]
    $sim mcm $mcm of addprop $bic0_node "array"     "reg"                reg_prop
    $sim mcm $mcm of addprop $bic0_node "string"    "device_type"        "bic0"

    # bic1 node
    set hex_addr 0x00513000 
    set bic1_node   [ $sim mcm $mcm of addchild $be_node "bic1" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $bic1_node "string"    "name"               "bic1"
    # reg specifies the MMIO ranges for the BIC1
    set reg_prop [list $hex_addr 0x00001000]
    $sim mcm $mcm of addprop $bic1_node "array"     "reg"                reg_prop
    $sim mcm $mcm of addprop $bic1_node "string"    "device_type"        "bic1"

    # mic-tm node
    set hex_addr 0x0050a000 
    set mic_node   [ $sim mcm $mcm of addchild $be_node "mic-tm" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $mic_node "string"    "name"               "mic-tm"
    # reg specifies the MMIO ranges for the MIC-TM
    set reg_prop [list $hex_addr 0x00001000]
    $sim mcm $mcm of addprop $mic_node "array"     "reg"                reg_prop
    $sim mcm $mcm of addprop $mic_node "string"    "device_type"        "mic-tm"

    # pervasive node
    set hex_addr 0x00509000 
    set pervasive_node   [ $sim mcm $mcm of addchild $be_node "pervasive" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $pervasive_node "string"    "name"         "pervasive"
    # reg specifies the MMIO ranges for the pervasive unit
    set reg_prop [list $hex_addr 0x00001000]
    $sim mcm $mcm of addprop $pervasive_node "array"     "reg"          reg_prop
    $sim mcm $mcm of addprop $pervasive_node "string"    "device_type"  "pervasive"

    # ppe-mmio node
    set hex_addr 0x00500000 
    set ppe_mmio_node   [ $sim mcm $mcm of addchild $be_node "ppe-mmio" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $ppe_mmio_node "string"    "name"          "ppe-mmio"
    # reg specifies the MMIO ranges for the PPE-MMIO
    set reg_prop [list $hex_addr 0x00001000]
    $sim mcm $mcm of addprop $ppe_mmio_node "array"     "reg"           reg_prop
    $sim mcm $mcm of addprop $ppe_mmio_node "string"    "device_type"   "ppe-mmio"

    # spe nodes
    array set spe_isrc {0 0x4 1 0x7 2 0x3 3 0x8 4 0x2 5 0x9 6 0x1 7 0xa}
    for { set spe_num 0 } { $spe_num < 8 } { incr spe_num } {
        set hex_addr [expr $spe_num*0x80000]
        set isrc [expr $spe_isrc($spe_num) + $be_num*0x1000000]
        set spe_node($spe_num) [add_spe_node $sim $mcm $be_node $spe_num $hex_addr $isrc]
    }

    # interrupt-controller node
    set hex_addr 0x00508400 
    set intc_node   [ $sim mcm $mcm of addchild $be_node "interrupt-controller" [format "%x" $hex_addr] ]
    $sim mcm $mcm of addprop $intc_node "string"        "name"          "interrupt-controller"
    $sim mcm $mcm of addprop $intc_node "string"        "interrupt-controller" ""
    # reg specifies the MMIO offset of the interrupt controller, one range for each
    # thread and one thread for the common MMIO
    set reg_prop [list $hex_addr                                        0x20 \
                       [format64 "0x%llX" [add64 $hex_addr 0x20]]       0x20 \
                       [format64 "0x%llX" [and64 $hex_addr 0xFFFFF000]] 0x1000 ]
    $sim mcm $mcm of addprop $intc_node "array"         "reg"           reg_prop
    $sim mcm $mcm of addprop $intc_node "string"        "device_type"   "CBEA-Internal-Interrupt-Controller"
    $sim mcm $mcm of addprop $intc_node "string"        "compatible"    "IBM,CBEA-Internal-Interrupt-Controller"
    $sim mcm $mcm of addprop $intc_node "int"           "#interrupt-cells"  0x1
    set cpu_num [expr $be_num*2]
    set isranges [list $cpu_num [expr $cpu_num+1]]
    $sim mcm $mcm of addprop $intc_node "array"   "ibm,interrupt-server-ranges"  isranges

    $sim mcm $mcm of addprop $be_node   "int"     "interrupt-parent"     $intc_node

    # Add spe vicinity properties
    set spe_node(-2) $mic_node
    set spe_node(-1) $mic_node
    set spe_node(8)  $bic0_node
    set spe_node(9)  $bic0_node
    for { set spe_num 0 } { $spe_num < 8 } { incr spe_num } {
        set vicinity_prop [list $spe_node([expr $spe_num-2]) $spe_node([expr $spe_num+2])]
        $sim mcm $mcm of addprop $spe_node($spe_num) "array"    "vicinity"    vicinity_prop
    }

}

proc dev_tree_cell::add_cpu { sim mcm parent be_num } {

    set cpu_num [expr $be_num*2]
    set cpu_node [ $sim mcm $mcm of addchild $parent "PowerPC,BE" $be_num ]
    $sim mcm $mcm of addprop  $cpu_node "string"    "name"               "PowerPC,BE"
    $sim mcm $mcm of addprop  $cpu_node "int"       "reg"                $cpu_num
    $sim mcm $mcm of addprop  $cpu_node "string"    "status"             "okay"
    $sim mcm $mcm of addprop  $cpu_node "string"    "device_type"        "cpu"
    $sim mcm $mcm of addprop  $cpu_node "string"    "ibm,bpa"            ""
    $sim mcm $mcm of addprop  $cpu_node "string"    "64-bit"             ""
    set associativity [list 4 0 0 $be_num $be_num]
    $sim mcm $mcm of addprop  $cpu_node "array" "ibm,associativity"       associativity
    set int_server [list $cpu_num [expr $cpu_num + 1]]
    $sim mcm $mcm of addprop  $cpu_node "array"     "ibm,ppc-interrupt-server#s" int_server
    $sim mcm $mcm of addprop  $cpu_node "int"       "ioid"               0x0000000e
    $sim mcm $mcm of addprop  $cpu_node "int"       "node-id"            $be_num

    set bp_base_hi [rshift64 [$sim be $be_num display bp_base] 32]
    set iic_prop [list $bp_base_hi 0x00508000 0x00001000]
    $sim mcm $mcm of addprop $cpu_node "array"      "iic"                iic_prop

    of::add_machine_info_props $cpu_node $sim $mcm

    return $cpu_node
}


proc dev_tree_cell::add_memory_nodes { sim mcm parent } {

    set retval         -1

    for {set region_num 0} {$region_num < [$sim config query memory_regions]} {incr region_num} {
        # Each memory region starts on a 4GB boundary
        set region_start [mul64 $region_num 0x100000000]

        # memory node for BE 0
        set memory_start [add64 $region_start [$sim config query memory_start]]
        set memory_size [memsize_toint [$sim config query memory_size]]
        set memory_node [$sim mcm $mcm of addchild $parent "memory" [format64 "%llx" $memory_start]]
        $sim mcm $mcm of addprop $memory_node "string" "name"        "memory"
        $sim mcm $mcm of addprop $memory_node "string" "device_type" "memory"
        set memory_reg [list [rshift64 $memory_start 32] [and64 $memory_start 0xffffffff] \
                             [rshift64 $memory_size  32] [and64 $memory_size  0xffffffff] ]
        $sim mcm $mcm of addprop $memory_node "array" "reg"           memory_reg
        set associativity [list 4 0 0 0 0]
        $sim mcm $mcm of addprop $memory_node "array" "ibm,associativity" associativity
        $sim mcm $mcm of addprop $memory_node "enum"    "available"   [$sim mcm $mcm of enuminfo MEMORY_MAP]

        if {$retval < 0} {set retval $memory_node}

        if { [$sim config query number_of_MCMs] > 1 } {
            # memory node for BE 1
            set memory_start [add64 $region_start [$sim config query mcm/1/memory_start]]
            set memory_size [memsize_toint [$sim config query mcm/1/memory_size]]
            set memory_node [$sim mcm $mcm of addchild $parent "memory" [format64 "%llx" $memory_start]]
            $sim mcm $mcm of addprop  $memory_node "string" "name"        "memory"
            $sim mcm $mcm of addprop  $memory_node "string" "device_type" "memory"
            set memory_reg [list [rshift64 $memory_start 32] [and64 $memory_start 0xffffffff] \
                                 [rshift64 $memory_size  32] [and64 $memory_size  0xffffffff] ]
            $sim mcm $mcm of addprop $memory_node "array"   "reg"         memory_reg
            set associativity [list 4 0 0 1 1]
            $sim mcm $mcm of addprop  $memory_node "array" "ibm,associativity" associativity
            $sim mcm $mcm of addprop $memory_node "enum"    "available"   [$sim mcm $mcm of enuminfo MEMORY_MAP]
        }        
    }

    return $retval
}

proc dev_tree_cell::build_firmware_tree { { sim mysim } { mcm 0 } } {

    global errorInfo
    puts "building tree...."
    if { [ catch { 
        
        puts "clearing existing Openfirmware tree"
        $sim mcm $mcm of clear
        set root_node [ $sim mcm $mcm of peer 0 ] 

        # Properties for root node
        $sim mcm $mcm of addprop  $root_node "string" "name"           "device-tree"
        $sim mcm $mcm of addprop  $root_node "string" "model"          "IBM,SystemSim-Cell"
        $sim mcm $mcm of addprop  $root_node "string" "device_type"    "chrp-cbea"
        $sim mcm $mcm of addprop  $root_node "string" "compatible"     "ibm,cbea"
        $sim mcm $mcm of addprop  $root_node "int"    "#address-cells" 2
        $sim mcm $mcm of addprop  $root_node "int"    "#size-cells"    2

        set tmp_array [list 0x180 0xfc000000 0x0 0x1000]
        $sim mcm $mcm of addprop $root_node "array" "platform-open-pic" tmp_array

        # Add Systemsim/Mambo node for bogus devices
        of::add_sim_node $sim $mcm
        of::add_mambo_node $sim $mcm

        # cpus node
        set cpus_node [ $sim mcm $mcm of addchild $root_node "cpus" "" ]
        $sim mcm $mcm of addprop  $cpus_node "string" "name"      "cpus"
        $sim mcm $mcm of addprop  $cpus_node "int"    "#address-cells" 1
        $sim mcm $mcm of addprop  $cpus_node "int"    "#size-cells"    0

        # Add cpu node for mcm 0
        set cpu_node [add_cpu $sim $mcm $cpus_node 0]

        # Add be node for mcm 0
        add_be_node $sim $mcm $root_node 0

        for { set i 1 } { $i < [$sim display number_of_MCMs] } { incr i } {

            # Add cpu node for mcm i
            add_cpu $sim $mcm $cpus_node $i

            # Add be node for mcm $i
            add_be_node $sim $mcm $root_node $i
        }

        # memory nodes
        set memory_node [ add_memory_nodes $sim $mcm $root_node ]

        # openprom node
        set prom_node [ $sim mcm $mcm of addchild $root_node "openprom" "" ]
        $sim mcm $mcm of addprop  $prom_node "string" "name"                 "openprom"
        $sim mcm $mcm of addprop  $prom_node "string" "device_type"          "BootROM"
        $sim mcm $mcm of addprop  $prom_node "string" "model"                "OpenFirmware Cell 1.0"
        $sim mcm $mcm of addprop  $prom_node "string" "relative-addressing" ""
        
        # options node
        set options_node [ $sim mcm $mcm of addchild $root_node "options" "" ]
        $sim mcm $mcm of addprop  $options_node "string" "name"                  "options"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-clientipaddr"   "127.0.0.2"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-gatewayipaddr"  "255.255.255.255"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-netmask"        "127.0.0.1"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,smt-enabled"       "dynamic"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,power-mgmt"        "on"
        $sim mcm $mcm of addprop  $options_node "string" "sms-bootmode?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-pxe_path" ""
        $sim mcm $mcm of addprop  $options_node "string" "sms-multiboot?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-protocol?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-spanning?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-duplex?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id3" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id2" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id1" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#spinup" "6 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#tftp_retries" "5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#tftp_blocksize" "200 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#bootp_retries" "5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot4" "8 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot3" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot2" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot1" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet4" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet3" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet2" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet1" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip4" "60 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip3" "78 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip2" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip1" "d5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip4" "6a "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip3" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip2" "c8 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip1" "65 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip4" "4 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip3" "e5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip2" "81 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip1" "9 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#language" "1 "
        $sim mcm $mcm of addprop  $options_node "string" "boot-list" "netboot"
        $sim mcm $mcm of addprop  $options_node "string" "real-mode?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "sms-boot?" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "direct-serial?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "use-nvramrc?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "selftest-#megs" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "security-password" ""
        $sim mcm $mcm of addprop  $options_node "string" "security-mode" ""
        $sim mcm $mcm of addprop  $options_node "string" "security-#badlogins" ""
        $sim mcm $mcm of addprop  $options_node "string" "screen-#rows" "18 "
        $sim mcm $mcm of addprop  $options_node "string" "screen-#columns" "50 "
        $sim mcm $mcm of addprop  $options_node "string" "output-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "oem-logo?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "oem-logo" "0 0"
        $sim mcm $mcm of addprop  $options_node "string" "oem-banner?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "oem-banner" ""
        $sim mcm $mcm of addprop  $options_node "string" "nvramrc" ""
        $sim mcm $mcm of addprop  $options_node "string" "input-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "fcode-debug?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "diag-switch?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "diag-file" ""
        $sim mcm $mcm of addprop  $options_node "string" "diag-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "boot-command" "boot"
        $sim mcm $mcm of addprop  $options_node "string" "boot-file" ""
        $sim mcm $mcm of addprop  $options_node "string" "auto-boot?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "boot-device" ""

        # options RTAS
        set rtas_node [ $sim mcm $mcm of addchild $root_node "rtas" "" ]
        $sim mcm $mcm of addprop  $rtas_node "string" "name"                    "rtas"
        $sim mcm $mcm of addprop  $rtas_node "int"    "rtas-version"            1
        $sim mcm $mcm of addprop  $rtas_node "int"    "rtas-size"               0xcc00
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $rtas_node "array"  "ibm,flash-block-version" tmp_array
        $sim mcm $mcm of addprop  $rtas_node "int"    "ibm,display-line-length" 80
        set assoc_ref_points [list 0x2 0x3]
        $sim mcm $mcm of addprop  $rtas_node "array" "ibm,associativity-reference-points" assoc_ref_points
        set max_assoc_domains [list 4 1 1 2 2]
        $sim mcm $mcm of addprop  $rtas_node "array" "ibm,max-associativity-domains" max_assoc_domains
        of::add_rtas_function $rtas_node "restart-rtas" $sim $mcm
        of::add_rtas_function $rtas_node "nvram-fetch" $sim $mcm
        of::add_rtas_function $rtas_node "nvram-store" $sim $mcm
        of::add_rtas_function $rtas_node "get-time-of-day" $sim $mcm
        of::add_rtas_function $rtas_node "set-time-of-day" $sim $mcm
        of::add_rtas_function $rtas_node "set-time-for-power-on" $sim $mcm
        of::add_rtas_function $rtas_node "freeze-time-base" $sim $mcm
        of::add_rtas_function $rtas_node "thaw-time-base" $sim $mcm
        of::add_rtas_function $rtas_node "event-scan" $sim $mcm
        of::add_rtas_function $rtas_node "check-exception" $sim $mcm
        of::add_rtas_function $rtas_node "read-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "write-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,read-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,write-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "set-indicator" $sim $mcm
        of::add_rtas_function $rtas_node "display-character" $sim $mcm
        of::add_rtas_function $rtas_node "system-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,nmi-register" $sim $mcm
        of::add_rtas_function $rtas_node "start-cpu" $sim $mcm
        of::add_rtas_function $rtas_node "get-term-char" $sim $mcm
        of::add_rtas_function $rtas_node "put-term-char" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,cbe-perftools" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,set-eeh-option" $sim $mcm
        of::add_rtas_function $rtas_node "get-sensor-state" $sim $mcm
        of::add_rtas_function $rtas_node "system-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,update-flash-64-and-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,update-flash-64" $sim $mcm
        of::add_rtas_function $rtas_node "stop-self" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,cbe-perftools" $sim $mcm
        
        # chosen node
        set chosen_node [ $sim mcm $mcm of addchild $root_node "chosen" "" ]
        $sim mcm $mcm of addprop  $chosen_node "string" "name"      "chosen"
        $sim mcm $mcm of addprop  $chosen_node "int"    "cpu"       $cpu_node
        $sim mcm $mcm of addprop  $chosen_node "int"    "memory"    $memory_node
        $sim mcm $mcm of addprop  $chosen_node "string" "bootargs"  "lpj=8000000 console=bogus0 root=/dev/mambobd0 rw"
        $sim mcm $mcm of addprop  $chosen_node "int"    "stdout"    0x10001
        $sim mcm $mcm of addprop  $chosen_node "int"    "stdin"     0x10000
        $sim mcm $mcm of addprop  $chosen_node "int"    "mmu"       0x10020

    } ] } then {
        puts $errorInfo
    }
    puts "done building tree."
}

# dev_tree_cpbw creates a device tree consistent with the initial CPBW/QS20 device tree.
# This is being preserved for the old Cell SDK tests which use kernels that expect
# a device tree with this format.
namespace eval dev_tree_cpbw {
    variable version       "1.0"
}

proc dev_tree_cpbw::add_cpu_0 { sim mcm cpus_node } {

    set be_num 0
    set cpu_num 0
    set cpu_node [ $sim mcm $mcm of addchild $cpus_node "PowerPC,BE" $be_num ]
    $sim mcm $mcm of addprop  $cpu_node "int"       "reg"                $cpu_num
    $sim mcm $mcm of addprop  $cpu_node "string"    "status"             "okay"
    $sim mcm $mcm of addprop  $cpu_node "string"    "name"               "PowerPC,BE"
    $sim mcm $mcm of addprop  $cpu_node "string"    "device_type"        "cpu"
    $sim mcm $mcm of addprop  $cpu_node "string"    "isolate-enabled"    "off"
    $sim mcm $mcm of addprop  $cpu_node "string"    "precise-enabled"    "off"
    set associativity [list 4 0 0 0 0]
    $sim mcm $mcm of addprop  $cpu_node "array" "ibm,associativity" associativity
    set int_server [format "%08d%08d" $cpu_num [expr $cpu_num + 1]]
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ibm,ppc-interrupt-server#s" $int_server
    $sim mcm $mcm of addprop  $cpu_node "int"       "ioid"               0x0000000e
    $sim mcm $mcm of addprop  $cpu_node "int"       "node-id"            0x00000000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bp-base"         000002000000000000800000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioif1"           000002400000000010000000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioif0"           000002800000000010000000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "iic"             000002000050800000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "pervasive"       000002000050900000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "mic-tm"          000002000050a00000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioc-translation" 000002000051000000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioc-cache"       000002000051100000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bif0"            000002000051200000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bif1"            000002000051300000001000
    of::add_machine_info_props $cpu_node $sim $mcm
    
    set spes_node [ $sim mcm $mcm of addchild $cpu_node "spes" "" ]
    $sim mcm $mcm of addprop  $spes_node "string"    "name"               "spes"

    set spe_num 0
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000400000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000400000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000000000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200000400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000002000006000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040000000002000

    set spe_num 1
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000700000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000700000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000008000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200000c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000200000e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040200000002000

    set spe_num 2
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000300000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000300000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000010000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200001400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000002000016000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040400000002000

    set spe_num 3
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000800000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000800000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000018000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200001c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000200001e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040600000002000

    set spe_num 4
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000200000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000200000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000020000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200002400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000002000026000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040800000002000

    set spe_num 5
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000900000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000900000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000028000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200002c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000200002e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040a00000002000

    set spe_num 6
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000100000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000100000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000030000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200003400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000002000036000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040c00000002000

    set spe_num 7
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000a00000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000a00000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000002000038000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000200003c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000200003e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000002000040e00000002000

    set pus_node [ $sim mcm $mcm of addchild $cpu_node "pus" "" ]
    $sim mcm $mcm of addprop  $pus_node "string"    "name"               "pus"

    set pu_num 0
    set pu_node [ $sim mcm $mcm of addchild $pus_node "pu" $pu_num ]
    $sim mcm $mcm of addprop $pu_node "string"   "device_type" "pu"
    $sim mcm $mcm of addprop $pu_node "int"      "reg"         $pu_num
    $sim mcm $mcm of addprop $pu_node byte_array "priv"        000002000050000000001000
    $sim mcm $mcm of addprop $pu_node "string"   "name"        "pu"

    return $cpu_node
}

proc dev_tree_cpbw::add_cpu_1 { sim mcm cpus_node } {

    set be_num 1
    set cpu_num 2
    set cpu_node [ $sim mcm $mcm of addchild $cpus_node "PowerPC,BE" $be_num ]
    $sim mcm $mcm of addprop  $cpu_node "int"       "reg"                $cpu_num
    $sim mcm $mcm of addprop  $cpu_node "string"    "status"             "okay"
    $sim mcm $mcm of addprop  $cpu_node "string"    "name"               "PowerPC,BE"
    $sim mcm $mcm of addprop  $cpu_node "string"    "device_type"        "cpu"
    $sim mcm $mcm of addprop  $cpu_node "string"    "isolate-enabled"    "off"
    $sim mcm $mcm of addprop  $cpu_node "string"    "precise-enabled"    "off"
    set associativity [list 4 0 0 1 1]
    $sim mcm $mcm of addprop  $cpu_node "array" "ibm,associativity" associativity
    set int_server [format "%08d%08d" $cpu_num [expr $cpu_num + 1]]
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ibm,ppc-interrupt-server#s" $int_server
    $sim mcm $mcm of addprop  $cpu_node "int"       "ioid"               0x0000000e
    $sim mcm $mcm of addprop  $cpu_node "int"       "node-id"            0x00000001
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bp-base"         000003000000000000800000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioif1"           000003400000000010000000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioif0"           000003800000000010000000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "iic"             000003000050800000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "pervasive"       000003000050900000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "mic-tm"          000003000050a00000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioc-translation" 000003000051000000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "ioc-cache"       000003000051100000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bif0"            000003000051200000001000
    $sim mcm $mcm of addprop  $cpu_node byte_array  "bif1"            000003000051300000001000
    of::add_machine_info_props $cpu_node $sim $mcm
    
    set spes_node [ $sim mcm $mcm of addchild $cpu_node "spes" "" ]
    $sim mcm $mcm of addprop  $spes_node "string"    "name"               "spes"

    set spe_num 0
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000400000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000400000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000000000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300000400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000003000006000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040000000002000

    set spe_num 1
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000700000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000700000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000008000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300000c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000300000e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040200000002000

    set spe_num 2
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000300000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000300000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000010000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300001400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000003000016000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040400000002000

    set spe_num 3
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000800000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000800000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000018000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300001c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000300001e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040600000002000

    set spe_num 4
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000200000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000200000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000020000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300002400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000003000026000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040800000002000

    set spe_num 5
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000900000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000900000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000028000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300002c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000300002e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040a00000002000

    set spe_num 6
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000100000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000100000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000030000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300003400000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       000003000036000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040c00000002000

    set spe_num 7
    set spe_node [ $sim mcm $mcm of addchild $spes_node "spe" $spe_num ]
    $sim mcm $mcm of addprop $spe_node "string"   "device_type" "spe"
    $sim mcm $mcm of addprop $spe_node "string"   "name"        "spe"
    $sim mcm $mcm of addprop $spe_node "int"      "reg"         $spe_num
    $sim mcm $mcm of addprop $spe_node byte_array "isrc"        0000000a00000000
    $sim mcm $mcm of addprop $spe_node byte_array "unit-id"     0000000a00000000
    $sim mcm $mcm of addprop $spe_node byte_array "local-store" 000003000038000000040000
    $sim mcm $mcm of addprop $spe_node byte_array "problem"     00000300003c00000001d000
    $sim mcm $mcm of addprop $spe_node byte_array "priv2"       00000300003e000000006000
    $sim mcm $mcm of addprop $spe_node byte_array "priv1"       000003000040e00000002000

    set pus_node [ $sim mcm $mcm of addchild $cpu_node "pus" "" ]
    $sim mcm $mcm of addprop  $pus_node "string"    "name"               "pus"

    set pu_num 0
    set pu_node [ $sim mcm $mcm of addchild $pus_node "pu" $pu_num ]
    $sim mcm $mcm of addprop $pu_node "string"   "device_type" "pu"
    $sim mcm $mcm of addprop $pu_node "int"      "reg"         $pu_num
    $sim mcm $mcm of addprop $pu_node byte_array "priv"        000003000050000000001000
    $sim mcm $mcm of addprop $pu_node "string"   "name"        "pu"

    return $cpu_node
}

proc dev_tree_cpbw::add_memory_nodes { sim mcm host_node } {

    set retval         -1

    for {set region_num 0} {$region_num < [$sim config query memory_regions]} {incr region_num} {
        # Each memory region starts on a 4GB boundary
        set region_start [mul64 $region_num 0x100000000]

        # memory node for BE 0
        set memory_start [add64 $region_start [$sim config query memory_start]]
        set memory_size [memsize_toint [$sim config query memory_size]]
        set memory_node [$sim mcm $mcm of addchild $host_node "memory" [format64 "%llx" $memory_start]]
        $sim mcm $mcm of addprop $memory_node "string" "name"        "memory"
        $sim mcm $mcm of addprop $memory_node "string" "device_type" "memory"
        set associativity [list 4 0 0 0 0]
        $sim mcm $mcm of addprop $memory_node "array" "ibm,associativity" associativity
        set memory_reg [list [format64 "0x%llx" $memory_start] $memory_size]
        $sim mcm $mcm of addprop $memory_node "array64" "reg"         memory_reg
        $sim mcm $mcm of addprop $memory_node "enum"    "available"   [$sim mcm $mcm of enuminfo MEMORY_MAP]

        if {$retval < 0} {set retval $memory_node}

        if { [$sim config query number_of_MCMs] > 1 } {
            # memory node for BE 1
            set memory_start [add64 $region_start [$sim config query mcm/1/memory_start]]
            set memory_size [memsize_toint [$sim config query mcm/1/memory_size]]
            set memory_node [$sim mcm $mcm of addchild $host_node "memory" [format64 "%llx" $memory_start]]
            $sim mcm $mcm of addprop  $memory_node "string" "name"        "memory"
            $sim mcm $mcm of addprop  $memory_node "string" "device_type" "memory"
            set associativity [list 4 0 0 1 1]
            $sim mcm $mcm of addprop  $memory_node "array" "ibm,associativity" associativity
            set memory_reg [list [format64 "0x%llx" $memory_start] $memory_size]
            $sim mcm $mcm of addprop $memory_node "array64" "reg"         memory_reg
            $sim mcm $mcm of addprop $memory_node "enum"    "available"   [$sim mcm $mcm of enuminfo MEMORY_MAP]
        }        
    }

    return $retval
}

proc dev_tree_cpbw::build_firmware_tree { { sim mysim } { mcm 0 } } {

    global errorInfo
    puts "building tree...."
    if { [ catch { 
        
        $sim mcm $mcm memory display 0 1

        puts "clearing existing Openfirmware tree"
        $sim mcm $mcm of clear
        set root_node [ $sim mcm $mcm of peer 0 ] 

        # Add Systemsim/Mambo node for bogus devices
        of::add_sim_node $sim $mcm
        of::add_mambo_node $sim $mcm

        # host bus properties
        #set host_node [ $sim mcm $mcm of addchild $root_node "tcl" "" ]
        set host_node $root_node
        $sim mcm $mcm of addprop  $host_node "string" "model"          "IBM,CPBW-Mambo, Simulated-System"
        $sim mcm $mcm of addprop  $host_node "string" "name"           "device-tree"
        $sim mcm $mcm of addprop  $host_node "string" "device_type"    "Simulated-Machine"
        $sim mcm $mcm of addprop  $host_node "string" "compatible"     "IBM,CPBW-SystemSim"
        $sim mcm $mcm of addprop  $host_node "int"    "#address-cells" 2
        $sim mcm $mcm of addprop  $host_node "int"    "#size-cells"    2
        $sim mcm $mcm of addprop  $host_node byte_array  "platform-spider-pic" 000002400000800000000800

        # cpu node
        set cpus_node [ $sim mcm $mcm of addchild $host_node "cpus" "" ]
        $sim mcm $mcm of addprop  $cpus_node "string" "name"      "cpus"
        
        set cpu_node [add_cpu_0 $sim $mcm $cpus_node]

        # interrupt controller
        set hex_addr 0x0000020000508400 
        set intc_node [ $sim mcm $mcm of addchild $host_node "interrupt-controller@$hex_addr" "" ]
        $sim mcm $mcm of addprop  $intc_node "string" "name"        "interrupt-controller"
        $sim mcm $mcm of addprop  $intc_node "string" "device_type"  "IBM,CBEA-Internal-Interrupt-Controller"
        $sim mcm $mcm of addprop  $intc_node "string" "compatible"   "IBM,CBEA-Internal-Interrupt-Controller"
        $sim mcm $mcm of addprop  $intc_node "int" "#interrupt-cells"  0x4
        set isranges [list 0x00000000 0x00000001]
        $sim mcm $mcm of addprop  $intc_node "array" "ibm,interrupt-server-ranges"  isranges
        set regval [list \
            $hex_addr 0x20 \
            [format64 "0x%llX" [add64 $hex_addr 0x20]] 0x20]
        $sim mcm $mcm of addprop  $intc_node "array64" "reg"  regval
        
        if { [$sim display number_of_MCMs] > 1 } {
            add_cpu_1 $sim $mcm $cpus_node

            # interrupt controller
            set hex_addr 0x0000030000508400 
            set intc_node [ $sim mcm $mcm of addchild $host_node "interrupt-controller@$hex_addr" "" ]
            $sim mcm $mcm of addprop  $intc_node "string" "name"        "interrupt-controller"
            $sim mcm $mcm of addprop  $intc_node "string" "device_type"  "IBM,CBEA-Internal-Interrupt-Controller"
            $sim mcm $mcm of addprop  $intc_node "string" "compatible"   "IBM,CBEA-Internal-Interrupt-Controller"
            $sim mcm $mcm of addprop  $intc_node "int" "#interrupt-cells"  0x4
            set isranges [list 0x00000002 0x00000003]
            $sim mcm $mcm of addprop  $intc_node "array" "ibm,interrupt-server-ranges"  isranges
            set regval [list \
                $hex_addr 0x20 \
                [format64 "0x%llX" [add64 $hex_addr 0x20]] 0x20]
            $sim mcm $mcm of addprop  $intc_node "array64" "reg"  regval

        }

        # memory nodes
        set memory_node [ add_memory_nodes $sim $mcm $host_node ]

        # openprom node
        set prom_node [ $sim mcm $mcm of addchild $host_node "openprom" "" ]
        $sim mcm $mcm of addprop  $prom_node "string" "name"                 "openprom"
        $sim mcm $mcm of addprop  $prom_node "string" "device_type"          "BootROM"
        $sim mcm $mcm of addprop  $prom_node "string" "model"                "OpenFirmware STI 1.0"
        $sim mcm $mcm of addprop  $prom_node "string" "relative-addressing" ""
        
        # options node
        set options_node [ $sim mcm $mcm of addchild $host_node "options" "" ]
        $sim mcm $mcm of addprop  $options_node "string" "name"                  "options"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-clientipaddr"   "127.0.0.2"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-gatewayipaddr"  "255.255.255.255"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,fw-netmask"        "127.0.0.1"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,smt-enabled"       "dynamic"
        $sim mcm $mcm of addprop  $options_node "string" "ibm,power-mgmt"        "on"
        $sim mcm $mcm of addprop  $options_node "string" "sms-bootmode?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-pxe_path" ""
        $sim mcm $mcm of addprop  $options_node "string" "sms-multiboot?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-protocol?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-spanning?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-duplex?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id3" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id2" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#scsi_id1" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#spinup" "6 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#tftp_retries" "5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#tftp_blocksize" "200 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#bootp_retries" "5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot4" "8 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot3" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot2" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#defaultboot1" "7 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet4" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet3" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet2" "ff "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#subnet1" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip4" "60 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip3" "78 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip2" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#gatewayip1" "d5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip4" "6a "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip3" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip2" "c8 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#serverip1" "65 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip4" "4 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip3" "e5 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip2" "81 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#clientip1" "9 "
        $sim mcm $mcm of addprop  $options_node "string" "sms-#language" "1 "
        $sim mcm $mcm of addprop  $options_node "string" "boot-list" "netboot"
        $sim mcm $mcm of addprop  $options_node "string" "real-mode?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "sms-boot?" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "direct-serial?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "use-nvramrc?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "selftest-#megs" "0 "
        $sim mcm $mcm of addprop  $options_node "string" "security-password" ""
        $sim mcm $mcm of addprop  $options_node "string" "security-mode" ""
        $sim mcm $mcm of addprop  $options_node "string" "security-#badlogins" ""
        $sim mcm $mcm of addprop  $options_node "string" "screen-#rows" "18 "
        $sim mcm $mcm of addprop  $options_node "string" "screen-#columns" "50 "
        $sim mcm $mcm of addprop  $options_node "string" "output-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "oem-logo?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "oem-logo" "0 0"
        $sim mcm $mcm of addprop  $options_node "string" "oem-banner?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "oem-banner" ""
        $sim mcm $mcm of addprop  $options_node "string" "nvramrc" ""
        $sim mcm $mcm of addprop  $options_node "string" "input-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "fcode-debug?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "diag-switch?" "false"
        $sim mcm $mcm of addprop  $options_node "string" "diag-file" ""
        $sim mcm $mcm of addprop  $options_node "string" "diag-device" ""
        $sim mcm $mcm of addprop  $options_node "string" "boot-command" "boot"
        $sim mcm $mcm of addprop  $options_node "string" "boot-file" ""
        $sim mcm $mcm of addprop  $options_node "string" "auto-boot?" "true"
        $sim mcm $mcm of addprop  $options_node "string" "boot-device" ""

        # options RTAS
        set rtas_node [ $sim mcm $mcm of addchild $host_node "rtas" "" ]
        $sim mcm $mcm of addprop  $rtas_node "string" "name"                    "rtas"
        $sim mcm $mcm of addprop  $rtas_node "int"    "rtas-version"            1
        $sim mcm $mcm of addprop  $rtas_node "int"    "rtas-size"               0xcc00
        set tmp_array [list 0x1]
        $sim mcm $mcm of addprop  $rtas_node "array"  "ibm,flash-block-version" tmp_array
        $sim mcm $mcm of addprop  $rtas_node "int"    "ibm,display-line-length" 80
        set assoc_ref_points [list 0x2 0x3]
        $sim mcm $mcm of addprop  $rtas_node "array" "ibm,associativity-reference-points" assoc_ref_points
        set max_assoc_domains [list 4 1 1 2 2]
        $sim mcm $mcm of addprop  $rtas_node "array" "ibm,max-associativity-domains" max_assoc_domains
        of::add_rtas_function $rtas_node "restart-rtas" $sim $mcm
        of::add_rtas_function $rtas_node "nvram-fetch" $sim $mcm
        of::add_rtas_function $rtas_node "nvram-store" $sim $mcm
        of::add_rtas_function $rtas_node "get-time-of-day" $sim $mcm
        of::add_rtas_function $rtas_node "set-time-of-day" $sim $mcm
        of::add_rtas_function $rtas_node "set-time-for-power-on" $sim $mcm
        of::add_rtas_function $rtas_node "freeze-time-base" $sim $mcm
        of::add_rtas_function $rtas_node "thaw-time-base" $sim $mcm
        of::add_rtas_function $rtas_node "event-scan" $sim $mcm
        of::add_rtas_function $rtas_node "check-exception" $sim $mcm
        of::add_rtas_function $rtas_node "read-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "write-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,read-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,write-pci-config" $sim $mcm
        of::add_rtas_function $rtas_node "set-indicator" $sim $mcm
        of::add_rtas_function $rtas_node "display-character" $sim $mcm
        of::add_rtas_function $rtas_node "system-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,nmi-register" $sim $mcm
        of::add_rtas_function $rtas_node "start-cpu" $sim $mcm
        of::add_rtas_function $rtas_node "get-term-char" $sim $mcm
        of::add_rtas_function $rtas_node "put-term-char" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,cbe-perftools" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,set-eeh-option" $sim $mcm
        of::add_rtas_function $rtas_node "get-sensor-state" $sim $mcm
        of::add_rtas_function $rtas_node "system-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,update-flash-64-and-reboot" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,update-flash-64" $sim $mcm
        of::add_rtas_function $rtas_node "stop-self" $sim $mcm
        of::add_rtas_function $rtas_node "ibm,cbe-perftools" $sim $mcm
        
        # chosen node
        set chosen_node [ $sim mcm $mcm of addchild $host_node "chosen" "" ]
        $sim mcm $mcm of addprop  $chosen_node "string" "name"      "chosen"
        $sim mcm $mcm of addprop  $chosen_node "int"    "cpu"       $cpu_node
        $sim mcm $mcm of addprop  $chosen_node "int"    "memory"    $memory_node
        #$sim mcm $mcm of addprop  $chosen_node "string" "bootpath"  "/pci@fff7f09000/ide@1/hd@1:1,booter"
#        $sim mcm $mcm of addprop  $chosen_node "string" "bootpath"  "/axon/plb5/plb4/pcix/ethernet@1"
        $sim mcm $mcm of addprop  $chosen_node "string" "bootargs"  "lpj=8000000 console=bogus0 root=/dev/mambobd0 rw"
#        $sim mcm $mcm of addprop  $chosen_node "string" "bootargs"  ""
        $sim mcm $mcm of addprop  $chosen_node "int"    "stdout"    0x10001
        $sim mcm $mcm of addprop  $chosen_node "int"    "stdin"     0x10000
        $sim mcm $mcm of addprop  $chosen_node "int"    "mmu"       0x10020

    } ] } then {
        puts $errorInfo
    }
    puts "done building tree."
}

proc build_firmware_tree { { version cell } { sim mysim } { mcm 0 } } {

    switch -- $version {
        cpbw { dev_tree_cpbw::build_firmware_tree $sim $mcm }
        cell { dev_tree_cell::build_firmware_tree $sim $mcm }
        default {
            error "Unknown device tree version requested"
        }
    }
}
