#------------------------------------------------------------------------
#
#    IBM Software
#    Licensed Material - Property of IBM
#    (C) Copyright International Business Machines Corp. 2004, 2007
#    All Rights Reserved.
#
#    U.S. Government Users Restricted Rights - Use, duplication or
#    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  BE_treesim.tcl
#------------------------------------------------------------------------

#------------------------------------------------------------------
# This is the BE version of the treesim_cmds_init command.  
# It is called by treesim::init to override the default tree commands.
#------------------------------------------------------------------
proc treesim_cmds_init {args} {
   set treesim::cmds(add_cpu_entries)  "BE_treesim::add_cpu_entries"
   set treesim::cmds(refresh_cpu)      "BE_treesim::refresh_cpu"
   set treesim::cmds(add_mcm_multiple) "BE_treesim::add_mcm_multiple"
}

#------------------------------------------------------------------
# BE tree
#------------------------------------------------------------------

namespace eval BE_treesim {
    namespace import  -force ::treesim::*
}

#------------------------------------------------------------------
# 
#------------------------------------------------------------------
proc BE_treesim::get_spu_entries { dataname } {

    upvar $dataname spudata
    array set spuinfo $spudata

    set spucmd $spuinfo(spucmd)
    
    set model [$spucmd display model]
#    set stack_checking "[SPU_StackCheck_state spuinfo]"

    # Experimental TVU support can be included in the list:
    #            "TVU"       managed "TVU_SPU::Init"
    return [list "SPUTrack" managed "SPUTrack_Init" \
                 "SPUCore"  managed "SPUCore_Init" \
                 "SPUChannel" managed "SPUChannel_Init" \
                 "SPUMemory" managed "SPUMemory_Init" \
                 "MFC"      managed "MFC_Init" \
                 "MFC_XLate" managed "SPU_xlate::Init" \
                 "SPUStats"  managed "SPUStats_Init" \
                 "LS_Stats" managed "SPULS_Init" \
                 "Model:$model" dialog  "BE_treesim::SPU_ChangeModel %W %#" \
                 "[SPU_StackCheck_label spuinfo]" dialog  "BE_treesim::SPU_StackCheck_toggle %W %#" \
                 "Load-Exec"  dialog  "SelectSPUElfFile" \
           ]
}

#------------------------------------------------------------------
#  Add more than 1 mcms to tree
#  use default function, but change names from MCM to BE
#------------------------------------------------------------------
proc BE_treesim::add_mcm_multiple { widget sim simidx num_mcms mcmname} {

    #puts "BE add_mcm_multiple"
    treesim::add_mcm_multiple $widget $sim $simidx $num_mcms "BE"
}

#------------------------------------------------------------------
# 
#------------------------------------------------------------------
proc BE_treesim::add_cpu_entries {widget cpuidx cpu} {

    #puts "add cpu children for cpu $cpu idx $cpuidx "

    # read current data to get mcmcmd 
    array set current [getdata $widget $cpuidx] 
    set parentcmd  $current(mcmcmd)

    set prefix [string range $cpu 0 2] 
    switch $prefix {
        "PPE" {
            add_ppc_entries $widget $cpuidx $cpu $parentcmd
        }
        "SPE" {
            add_spu_entries $widget $cpuidx $cpu $parentcmd
        }
        default {
            puts "unexpected cpu prefix : $prefix "
        }
    }
}

#------------------------------------------------------------------
# refresh the entries under a cpu node
#------------------------------------------------------------------
proc BE_treesim::refresh_cpu {widget cpuidx cpu} {
    array set current [getdata $widget $cpuidx] 

    set myTree [$widget cget -tree]

    set prefix [string range $cpu 0 2] 
    switch $prefix {
        "PPE" {
        }
        "SPE" {
            # Update label of model widget
            set model [$current(spucmd) display model]
            set idx [$myTree find $cpuidx -regexp Model]
            $widget entry configure $idx -label "Model:$model"

            # Update label of stackchecking widget
            set idx [$myTree find $cpuidx -regexp StackChecking]
            $widget entry configure $idx -label "[SPU_StackCheck_label current]"

        }
        default {
            puts "BE_treesim::refresh_cpu -- unexpected cpu prefix: $prefix"
        }
    }
}

#------------------------------------------------------------------
# Add entries under SPU node
#
# parentcmd is a tcl command that will take SPU as an argument
#                such as 'mysim' or an alias for 'mysim be 0'
#------------------------------------------------------------------
proc BE_treesim::add_spu_entries {widget spuidx spu parentcmd } {

    set dataname [treedataname $widget $spuidx]

    set spunum [string range $spu 3 end ]   ;# assumes something like SPU1
    set spucmd [alias create_unique $parentcmd spu $spunum]
    set mfccmd [alias create_unique $parentcmd mfc $spunum]

    lappend $dataname spunum $spunum spucmd $spucmd mfccmd $mfccmd

    set spu_entries [ get_spu_entries $dataname ]

    add_children $widget $spuidx $spu_entries

}

#------------------------------------------------------------------------
# Model functions
#------------------------------------------------------------------------

proc BE_treesim::SPU_ChangeModel { _upstate widget treeidx args } {
    upvar $_upstate  spuinfo

    set spucmd $spuinfo(spucmd)
    switch [$spucmd display model] {
        "pipeline"    { $spucmd set model instruction }
        "instruction" { $spucmd set model fast }
        "fast"        { $spucmd set model pipeline }
        default       { puts "Error: Unknown model" }
    }

    $widget entry configure $treeidx -label "Model:[$spucmd display model]"
}

#------------------------------------------------------------------------
# StackCheck functions
#------------------------------------------------------------------------

proc BE_treesim::SPU_StackCheck_label { _spuinfo } {
    upvar $_spuinfo spuinfo

    set spu $spuinfo(spunum)
    set be  $spuinfo(mcmnum)
    set sim $spuinfo(sim)

    if {[spu_stack_check::is_enabled $spu $be $sim]} {
        return "StackChecking:on"
    } else {
        return "StackChecking:off"
    }
}

proc BE_treesim::SPU_StackCheck_toggle { _upstate widget treeidx args } {
    upvar $_upstate  spuinfo

    set spu $spuinfo(spunum)
    set be  $spuinfo(mcmnum)
    set sim $spuinfo(sim)

    if {[spu_stack_check::is_enabled $spu $be $sim]} {
        spu_stack_check::disable $spu $be $sim
    } else {
        spu_stack_check::enable $spu $be $sim
    }

    $widget entry configure $treeidx -label "[SPU_StackCheck_label spuinfo]"
}
