#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  mfc.tcl
#
#------------------------------------------------------------------------

######################################################################
#
# MFC_Init -- Initalize the status window
#
######################################################################

proc MFC_Init {root _upstate args} {
    global MFCSim MFCSPU
    global SPUMFC

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate SPUMFC $root       ;# create SPUMFC($root,x)

    set sim $SPUMFC($root,sim)
    set spu $SPUMFC($root,spunum)
    set spucmd $SPUMFC($root,spucmd)
    set mfccmd $SPUMFC($root,mfccmd)

    # Tuck this data away so the Refresh can use it
    set MFCSim($root) $sim
    set MFCSPU($root) $spu

    ################### Queue Widgets ########################################
    label $root.dmaqLabel -text "DMA Queues" -foreground white -background black
    label $root.dmaqStatus -text "Status : [$mfccmd display status]"

    # SPU Queue
    frame ${root}.spuq -relief sunken -bd 2
    label ${root}.spuq.label -text "SPE DMA Queue"
    text ${root}.spuq.head -relief groove -font textbox_font \
         -height 2 -wrap none -width 70 \
         -xscrollcommand "${root}.spuq.xsb set"
	text ${root}.spuq.data -wrap none -width 70 -height 5 \
         -yscrollcommand "${root}.spuq.ysb set" -font textbox_font
    scrollbar ${root}.spuq.xsb -orient horizontal -command \
        [list MFC_ScrollTogether [list ${root}.spuq.head ${root}.spuq.data] xview]
    scrollbar ${root}.spuq.ysb -orient vertical -command "${root}.spuq.data yview"

    table ${root}.spuq \
        ${root}.spuq.label      0,0 -cspan 2 -fill x \
        ${root}.spuq.head       1,0 -cspan 1 -fill x -anchor w\
        ${root}.spuq.data       2,0 -cspan 1 -fill both \
        ${root}.spuq.ysb        2,1 -cspan 1 -fill y \
        ${root}.spuq.xsb        3,0 -cspan 1 -fill x

    table configure ${root}.spuq c* r* -resize none
    table configure ${root}.spuq r2 c0 -resize both

    # Proxy Queue
    frame ${root}.puq -relief sunken -bd 2
    label ${root}.puq.label -text "Proxy DMA Queue"
    text ${root}.puq.head -relief groove -font textbox_font \
         -height 2 -wrap none -width 70 \
         -xscrollcommand "${root}.puq.xsb set"
	text ${root}.puq.data -wrap none -width 70 -height 5 \
         -yscrollcommand "${root}.puq.ysb set" -font textbox_font
    scrollbar ${root}.puq.xsb -orient horizontal -command \
        [list MFC_ScrollTogether [list ${root}.puq.head ${root}.puq.data] xview]
    scrollbar ${root}.puq.ysb -orient vertical -command "${root}.puq.data yview"

    table ${root}.puq \
        ${root}.puq.label      0,0 -cspan 2 -fill x \
        ${root}.puq.head       1,0 -cspan 1 -fill x -anchor w\
        ${root}.puq.data       2,0 -cspan 1 -fill both \
        ${root}.puq.ysb        2,1 -cspan 1 -fill y \
        ${root}.puq.xsb        3,0 -cspan 1 -fill x

    table configure ${root}.puq c* r* -resize none
    table configure ${root}.puq r2 c0 -resize both

    ################### Interrupt Widgets ########################################
    frame $root.int_state

    label $root.int_state.intLabel -text "Interrupt State" -foreground white -background black
    label $root.int_state.iclass0 -text "ICLASS0 : mask=[$mfccmd display imask 0] status=[$mfccmd display istatus 0] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 0] [$mfccmd display istatus 0]]]"
    label $root.int_state.iclass1 -text "ICLASS1 : mask=[$mfccmd display imask 1] status=[$mfccmd display istatus 1] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 1] [$mfccmd display istatus 1]]]"
    label $root.int_state.iclass2 -text "ICLASS2 : mask=[$mfccmd display imask 2] status=[$mfccmd display istatus 2] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 2] [$mfccmd display istatus 2]]]"
    label $root.int_state.cer -text "CER : [$mfccmd display priv cer]"
    label $root.int_state.ctfr -text "CTFR : [$mfccmd display priv dsir]"

    table $root.int_state \
        $root.int_state.intLabel     0,0 -cspan 2 -fill x \
        $root.int_state.iclass0      1,0 -cspan 2 -fill x \
        $root.int_state.iclass1      2,0 -cspan 2 -fill x \
        $root.int_state.iclass2      3,0 -cspan 2 -fill x \
        $root.int_state.cer          4,0 -fill x \
        $root.int_state.ctfr         4,1 -fill x

    ################### Misc Widgets ########################################
    frame $root.misc
    label $root.misc.label -text "Misc." -foreground white -background black
    label $root.misc.mfc_control -text "DMA_Control : [$mfccmd display priv mfc_control]"
    label $root.misc.accr -text "ACCR : [$mfccmd display priv accr]"

    table $root.misc \
        $root.misc.label            0,0 -cspan 2 -fill x \
        $root.misc.mfc_control      1,0 -fill x \
        $root.misc.accr             1,1 -fill x

    ################### Main table ########################################
    table $root \
        $root.dmaqLabel     	0,0 -fill x \
        $root.dmaqStatus     	1,0 -fill y \
        ${root}.spuq           	2,0 -fill both \
        ${root}.puq            	3,0 -fill both \
        $root.int_state         4,0 -fill both \
        $root.misc          	5,0 -fill both

    table configure $root c* r* -resize none
    #table configure $root r1 r2 c1 -resize expand
    table configure $root c0 r2 r3 -resize expand

    set title [treesim::get_path_name $SPUMFC($root,widget) $SPUMFC($root,treeidx) 1 ] 
    wm title $root "$title: MFC"
    return "MFC_Refresh"
}

#
# UpdateLabel will color changed label values red, otherwise they are grey
#
proc UpdateLabel { thelabel newvalue } {
    if { [string compare [$thelabel cget -text] $newvalue ] != 0 } {
        $thelabel configure -bg red
    } else {
        $thelabel configure -bg lightgrey
    }
    $thelabel configure -text $newvalue
}

######################################################################
#
# MFC_Refresh -- Refresh the contents of the status window
#
######################################################################
proc MFC_Refresh { root args } {
    global MFCSim MFCSPU
    global SPUMFC 

    set sim $SPUMFC($root,sim)
    set spu $SPUMFC($root,spunum)
    set spucmd $SPUMFC($root,spucmd)
    set mfccmd $SPUMFC($root,mfccmd)

    # Update Interrupt State
    UpdateLabel $root.int_state.iclass0 "ICLASS0 : mask=[$mfccmd display imask 0] status=[$mfccmd display istatus 0] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 0] [$mfccmd display istatus 0]]]"
    UpdateLabel $root.int_state.iclass1 "ICLASS1 : mask=[$mfccmd display imask 1] status=[$mfccmd display istatus 1] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 1] [$mfccmd display istatus 1]]]"
    UpdateLabel $root.int_state.iclass2 "ICLASS2 : mask=[$mfccmd display imask 2] status=[$mfccmd display istatus 2] visible=[format64 "0x%016llX" [and64 [$mfccmd display imask 2] [$mfccmd display istatus 2]]]"
    UpdateLabel $root.int_state.cer "CER : [$mfccmd display priv cer]"
    UpdateLabel $root.int_state.ctfr "CTFR : [$mfccmd display priv dsir]"

    UpdateLabel $root.misc.mfc_control "DMA_Control : [$mfccmd display priv mfc_control]"
    UpdateLabel $root.misc.accr "ACCR : [$mfccmd display priv accr]"

    $root.dmaqStatus configure -text "Status : [$mfccmd display status]"

    # Update SPU Queue
    set lines [split [$mfccmd display spuq] "\n"]
    ${root}.spuq.head configure -state normal
    ${root}.spuq.head delete 1.0 end
    ${root}.spuq.head insert end "[lindex $lines 0]\n"
    ${root}.spuq.head insert end "[lindex $lines 1]\n"
    ${root}.spuq.head configure -state disabled
    ${root}.spuq.data delete 1.0 end
    foreach entry [lrange $lines 2 end-1] {
        ${root}.spuq.data insert end "$entry\n"
    }
    ${root}.spuq.data delete insert-1c end       ;# delete last newline char

    # Update Proxy Queue
    set lines [split [$mfccmd display puq] "\n"]
    ${root}.puq.head configure -state normal
    ${root}.puq.head delete 1.0 end
    ${root}.puq.head insert end "[lindex $lines 0]\n"
    ${root}.puq.head insert end "[lindex $lines 1]\n"
    ${root}.puq.head configure -state disabled
    ${root}.puq.data delete 1.0 end
    foreach entry [lrange $lines 2 end-1] {
        ${root}.puq.data insert end "$entry\n"
    }
    ${root}.puq.data delete insert-1c end       ;# delete last newline char
}

proc MFC_ScrollTogether { widgets view args } {
    foreach w $widgets {
        eval {$w $view} $args
    }
}
