#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  spuchannel.tcl
#
#------------------------------------------------------------------------

package require Widget_Balloon
package require BWidget 1.7

proc SPUChannel_BP {root chnum} {
	global SPUChannel

    set sim    $SPUChannel($root,sim)
    set mcmnum $SPUChannel($root,mcmnum)
    set spu    $SPUChannel($root,spunum)
    set spucmd $SPUChannel($root,spucmd)
    set mcmcmd $SPUChannel($root,mcmcmd)

    set idx $sim-$mcmnum-$spu
    global chanbrkInfo
    set chanbrkInfo(channel_$idx) $chnum
    
    if {$SPUChannel($root,$chnum,bp)} {
        Channel_breakpoint_dlg $root
        if {$SPUChannel($root,$chnum,bp)} {
            Channel_breakpoint_func $root $sim $mcmcmd $spucmd $spu $mcmnum
        }
    } else {
        Channel_breakpoint_func $root $sim $mcmcmd $spucmd $spu $mcmnum "clear"
    }
}

proc create_box {root group entries } {
	global SPUChannel

    frame $root.$group -borderwidth 2 -relief ridge
    foreach {chnum chname} $entries {
        if {[regexp "(\[\[:digit:\]\]+)D" $chnum match num]} {
            set chnum $num
            set deep 1
        } else {
            set deep 0
        }

        lappend SPUChannel($root,channels) $chnum $group
        set   frm $root.$group.ch$chnum
        frame $frm -borderwidth 2

        checkbutton $frm.bp -text "BP" -variable SPUChannel($root,$chnum,bp) -command [list \
            SPUChannel_BP $root $chnum                 \
            ]
        set spu $SPUChannel($root,spunum)
        set SPUChannel($spu,$chnum,bpstate) "Set Breakpoint"
        Widget_Balloon::Associate $SPUChannel($root,balloon) $frm.bp $SPUChannel($spu,$chnum,bpstate)

        if {$deep} {
            ComboBox ${frm}.value -editable false -textvariable SPUChannel($root,CH$chnum) -width 10 -justify right
        } else {
            label $frm.value -relief groove -textvariable SPUChannel($root,CH$chnum) -width 9 -justify right
        }
        label $frm.count -relief groove -textvariable SPUChannel($root,CHCNT$chnum) -width 2 -justify right
        label $frm.label -text " $chnum $chname" -pady 4
        grid  columnconfigure $frm 1 -minsize 90
        grid  $frm.bp $frm.value $frm.count $frm.label -sticky w
        pack  $frm -side top  -anchor w

        set SPUChannel($spu,$chnum,frm) $frm
        set SPUChannel($spu,$chnum,trig) 0
        set SPUChannel($spu,$chnum,deep) $deep
    }
}


######################################################################
#
# SPUChannel_Init -- Initalize the status window
#
######################################################################
proc SPUChannel_Init {root _upstate args } {
	global SPUChannel

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate SPUChannel $root       ;# create SPUChannel($root,x)

    set sim $SPUChannel($root,sim)
    set spu $SPUChannel($root,spunum)
    set spucmd $SPUChannel($root,spucmd)
    set mcmcmd $SPUChannel($root,mcmcmd)

    set SPUChannel($root,balloon) [Widget_Balloon::Create .ball_spu_chan_${spu}]

    create_box $root events   [list  0 "Read Event Status (RB)"    \
                                     1 "Write Event Mask (W)"  \
                                     2 "Write Event Acknowledgment (W)"      ]
    create_box $root signals  [list  3 "Signal Notification 1 (RB)" \
                                     4 "Signal Notification 2 (RB)" ]
    create_box $root decr     [list  7 "Write Decrementer (W)" \
                                     8 "Read Decrementer (R)" ]
    create_box $root dd2      [list  9 "Write Multisource Sync. Request (WB)" \
                                    11 "Read Event Mask (R)"  \
                                    12 "Read Tag-Group Query Mask (R)" ]
    create_box $root rstatus [list  13 "Read Machine Status (R)" ]
    create_box $root srr0    [list  14 "Write State Save-and-Restore (W)"    \
                                    15 "Read State Save-and-Restore (R)"    ]
    create_box $root dma     [list  16 "DMA Local Storage Address (W)"  \
                                    17 "DMA Effective Address High (W)"  \
                                    18 "DMA Effective Address Low (W)"   \
                                    19 "DMA Transfer Size (W)"  \
                                    20 "DMA Command Tag ID (W)"     \
                                    21 "DMA Command Opcode / Class ID (WB)" ]
    create_box $root tags    [list  22 "Write Tag-Group Query Mask (W)"                     \
                                    23 "Write Tag Status Update Request (WB)"    \
                                    24 "Read Tag-Group Status (RB)"   ]
    create_box $root dmalist [list  25 "Read List Stall-and-Notify Tag Status (RB)" \
                                    26 "Write List Stall-and-Notify Tag Ack. (W)"      \
                                    27 "Read Atomic Command Status (R)"         ]
    create_box $root mbox    [list  28 "Write Outbound Mailbox (WB)"         \
                                    29D "Read Inbound Mailbox (R)"        \
                                    30 "Write Outbound Interrupt Mailbox (WB)"   ]

    # status field
    frame $root.status -borderwidth 2 -relief ridge
    label $root.status.lbl -text "Status"
    label $root.status.reg -relief groove -width 12 -textvariable SPUChannel($root,status)
    pack  $root.status.lbl $root.status.reg -side left
    pack  $root.status -side top -anchor w

	table $root \
		$root.events  0,0 -fill both \
		$root.signals 1,0 -fill both \
		$root.decr    2,0 -fill both \
		$root.dd2     3,0 -fill both \
		$root.rstatus 4,0 -fill both \
		$root.srr0    5,0 -fill both \
		$root.dma     0,1 -fill both -rspan 2 \
		$root.tags    2,1 -fill both \
		$root.dmalist 3,1 -fill both \
		$root.mbox    4,1 -fill both \
		$root.status  5,1 -fill both \
        ;

    set title [treesim::get_path_name $SPUChannel($root,widget) $SPUChannel($root,treeidx) 1 ] 
    wm title $root "$title: Channels"

	SPUChannel_Refresh $root
	return "SPUChannel_Refresh"
}

######################################################################
#
# SPUChannel_Refresh -- Refresh the contents of the status window
#
######################################################################
proc SPUChannel_Refresh { root args } {
	global SPUChannel

	set sim $SPUChannel($root,sim)
	set spu $SPUChannel($root,spunum)
	set spucmd $SPUChannel($root,spucmd)

    foreach {chnum group} $SPUChannel($root,channels) {

        set frm $root.$group.ch$chnum
        set deep $SPUChannel($spu,$chnum,deep)

        # Query current channel count and values
        if {$deep} {
            set chinfo [$spucmd display channel $chnum vlist]
            set chcount [lindex $chinfo 0]
            set chvlist [lindex $chinfo 1]
            set chvalue [lindex $chvlist 0]
            $frm.value configure -values $chvlist
        } else {
            set chinfo [$spucmd display channel $chnum info]
            set chvalue [format "%08X" [lindex $chinfo 0] ]
            set chcount [lindex $chinfo 1]
        }

        # Determine highlighting
        if { $chvalue != $SPUChannel($root,CH$chnum) } {
            set value_backgrnd red
        } else {
            set value_backgrnd lightgrey
        }
        if { $chcount != $SPUChannel($root,CHCNT$chnum) } {
            set count_backgrnd red
        } else {
            set count_backgrnd lightgrey
        }

        if {$deep} {
            $frm.value configure -entrybg $value_backgrnd
        } else {
            $frm.value configure -bg $value_backgrnd
        }
        $frm.count configure -bg $count_backgrnd
        if {$SPUChannel($spu,$chnum,trig)} {
            $frm.bp    configure -bg yellow
            set SPUChannel($spu,$chnum,trig) 0
        } else {
            $frm.bp    configure -bg lightgrey
            Widget_Balloon::Message $frm.bp $SPUChannel($spu,$chnum,bpstate)
        }

        # Update variables
        set SPUChannel($root,CH$chnum)    $chvalue
        set SPUChannel($root,CHCNT$chnum) $chcount

    }

    # status
    set newstatus [$spucmd display status ext]
    set newstatus [lindex $newstatus 2]
    if { $SPUChannel($root,status)   != $newstatus } {
        $root.status.reg configure -bg red
    } else {
        $root.status.reg configure -bg lightgrey
    }
    set SPUChannel($root,status) $newstatus

}
#--------------------------------------------------------------------------
# Channel_breakpoint_dlg
#--------------------------------------------------------------------------
proc Channel_breakpoint_dlg { root } {
   global SPUChannel

   set sim    $SPUChannel($root,sim)
   set mcmnum $SPUChannel($root,mcmnum)
   set spu    $SPUChannel($root,spunum)
   set spucmd $SPUChannel($root,spucmd)
   set mcmcmd $SPUChannel($root,mcmcmd)

   set idx $sim-$mcmnum-$spu
   set chanbrkdlg .chanbrkdlg_$idx
   global chanbrkInfo

   set title "Breakpoint on Channel Access"

   # Create the dialog
   array set dlgInfo "top $chanbrkdlg title {$title} geom none"
   if { [ Dialog::create dlgInfo -borderwidth 5 ] } then {

      set topf [frame $chanbrkdlg.topf]

      set chanbrk [frame $topf.enable_chanbrk -relief sunken -bd 2]

      frame $chanbrk.channel
      label $chanbrk.channel.lbl -text "Set Breakpoint on Channel" -width 25
      label $chanbrk.channel.ent -width 3 -textvariable chanbrkInfo(channel_$idx)
      pack $chanbrk.channel.lbl $chanbrk.channel.ent -side left

      set matching [frame $chanbrk.matching -borderwidth 2 -relief groove ]

      checkbutton $matching.cbox -text "Break only if channel data & Mask matches Value" -borderwidth 2 \
                      -variable chanbrkInfo(checkmatch_$idx) -command "Channel_breakpoint_dlg_update $chanbrk $idx"

      frame $matching.mask
      label $matching.mask.lbl -text "Mask" -width 10
      entry $matching.mask.ent -width 10 -relief sunken -textvariable chanbrkInfo(mask_$idx)
      pack $matching.mask.lbl $matching.mask.ent -side left

      frame $matching.match
      label $matching.match.lbl -text "Value" -width 10
      entry $matching.match.ent -width 10 -relief sunken -textvariable chanbrkInfo(match_$idx)
      pack $matching.match.lbl $matching.match.ent -side left

      pack $chanbrk.channel -pady 5 -padx 50 -anchor w
      pack $matching.cbox -pady 3 -padx 10 -anchor w
      pack $matching.match -padx 50 -pady 5 -anchor w
      pack $matching.mask -padx 50 -anchor w
      pack $matching

      frame $topf.buttons
      button $topf.buttons.ok -text "  OK " -command "set chanbrkInfo(trigger_$idx) ok_pushed"
      button $topf.buttons.cancel -text "  Cancel " -command "set chanbrkInfo(trigger_$idx) cancel_pushed"
      pack $topf.buttons.ok $topf.buttons.cancel  -pady 5 -padx 10 -side left -fill x

      pack $topf.enable_chanbrk $topf.buttons -side top -padx 10 -pady 10
      pack $topf -fill both -pady 5 -padx 5 -side left -side top

      bind $topf <Return> "$topf.buttons.set invoke"
      bind $topf <Escape> "$topf.buttons.cancel invoke"

      Channel_breakpoint_dlg_update $chanbrk $idx
   }


   # reset the trigger variable
   set chanbrkInfo(trigger_$idx) "reset"

   # Save settings in case user cancels later
   array set save_chanbrkInfo [array get chanbrkInfo]


   # Wait for user to press the Ok or Cancel button, which will alter the trigger variable.
   Dialog::wait $chanbrkdlg chanbrkInfo(trigger_$idx)

   # Close the dialog box
   Dialog::dismiss $chanbrkdlg

   if { $chanbrkInfo(trigger_$idx) == "cancel_pushed" } then {
       # restore original settings
       array set chanbrkInfo [array get save_chanbrkInfo]
       set chnum $chanbrkInfo(channel_$idx)
       set SPUChannel($root,$chnum,bp) 0
   }
}

proc Channel_breakpoint_dlg_update {chanbrk idx} {
    global chanbrkInfo

    set matching $chanbrk.matching

    if {$chanbrkInfo(checkmatch_$idx) == "1" } {
       set state normal
    } else {
      set state disabled
    }

    $matching.mask.lbl configure -state $state
    $matching.mask.ent configure -state $state
    $matching.match.lbl configure -state $state
    $matching.match.ent configure -state $state
}


#--------------------------------------------------------------------------
# Channel_breakpoint_func
#--------------------------------------------------------------------------
proc Channel_breakpoint_func { root sim mcmcmd spucmd spu mcmnum {clear ""} } {
    global chanbrkInfo
    global SPUChannel

    set idx $sim-$mcmnum-$spu
    set chbp $chanbrkInfo(channel_$idx)

    if {$clear != ""} {
        $mcmcmd trigger clear spu $spu channel $chbp access
        set SPUChannel($spu,$chbp,bpstate) "Set Breakpoint"
    } else {
        if {$chanbrkInfo(checkmatch_$idx)} {
            set mask $chanbrkInfo(mask_$idx)
            set value  $chanbrkInfo(match_$idx)
            $mcmcmd trigger set spu $spu channel $chbp match $value $mask "spu_channel_break_point $sim $spucmd $spu $chbp Channel_$chbp-Match_$value"
            set SPUChannel($spu,$chbp,bpstate) "BP Armed (mask:$mask, val:$value)"
        } else {
            $mcmcmd trigger set spu $spu channel $chbp rw "spu_channel_break_point $sim $spucmd $spu $chbp Channel_$chbp-Access"
            set SPUChannel($spu,$chbp,bpstate) "BP Armed"
        }
    }

    set frm $SPUChannel($spu,$chbp,frm)
    Widget_Balloon::Message $frm.bp $SPUChannel($spu,$chbp,bpstate)
}

proc spu_channel_break_point { sim spucmd spu chnum comment args } {
    global SPUChannel

	set pc [$spucmd display pc]
    set value [$spucmd display memory $pc 4]
    set line [format "%08X:%08X:%s" $pc $value [$sim util spu_disasm $value]]

    array set data [lindex $args 0]
	puts "SPU $spu: Channel $chnum break point ($comment) -> $data(value), PC=$line"

    set SPUChannel($spu,$chnum,trig) 1
    set frm $SPUChannel($spu,$chnum,frm)
    Widget_Balloon::Message $frm.bp "BP triggered: PC=$line"
	simstop
}
