#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003,2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  spucore.tcl
#
#
#------------------------------------------------------------------------
######################################################################
#
# SPUCore_Init -- Initalize the status window
#
######################################################################
proc SPUCore_Init {root _upstate args } {
	global SPUCoreReg
    global SPUCore

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate SPUCore $root       ;# create SPUCore($root,x)

    set sim    $SPUCore($root,sim)
    set spu    $SPUCore($root,spunum)
    set spucmd $SPUCore($root,spucmd)

	# 0 - 64
	listbox $root.lbox0names -width 8 -height 16 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.spuregsSB0 $root.lbox0names $root.lbox0data" -takefocus 0
	for { set i 0 } { $i < 64 } { incr i } {
		$root.lbox0names insert end "REG$i"
	}
	listbox $root.lbox0data -width 35 -height 16 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.spuregsSB0 $root.lbox0names $root.lbox0data"
	for { set i 0 } { $i < 64 } { incr i } {
		$root.lbox0data insert end "[$spucmd display reg $i]"
	}
	scrollbar $root.spuregsSB0 -orient vert -command "TclUtils::ScrollBoth $root.lbox0names $root.lbox0data"
    bind $root.lbox0data <Button-1> { focus %W }
    bind $root.lbox0data <<ListboxSelect>> {ShowSPUReg %W [%W curselection] }

	# 64 - 128
	listbox $root.lbox1names -width 8 -height 16 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.spuregsSB1 $root.lbox1names $root.lbox1data" -takefocus 0
	for { set i 64 } { $i < 128 } { incr i } {
		$root.lbox1names insert end "REG$i"
	}
	listbox $root.lbox1data -width 35 -height 16 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.spuregsSB1 $root.lbox1names $root.lbox1data"
	for { set i 64 } { $i < 128 } { incr i } {
		$root.lbox1data insert end "[$spucmd display reg $i]"
	}
	scrollbar $root.spuregsSB1 -orient vert -command "TclUtils::ScrollBoth $root.lbox1names $root.lbox1data"
    bind $root.lbox1data <Button-1> { focus %W }
    bind $root.lbox1data <<ListboxSelect>> {ShowSPUReg %W [expr 64 + [%W curselection]] }

    # show selected reg as integers, floats, doubles
    set regn_0fmt [create_format_table $root 0]
    ShowSPUReg $regn_0fmt 0
    set regn_1fmt [create_format_table $root 1]
    ShowSPUReg $regn_1fmt 64

    # registers
    set others [frame $root.others -relief ridge -borderwidth 2]

    frame $others.fpscr
    label $others.fpscr.lbl -text "FPSCR "
    set   SPUCoreReg($root.fpscr) [$spucmd display FPSCR]
    label $others.fpscr.reg -relief groove -width 35 -font listbox_font -textvariable SPUCoreReg($root.fpscr)
    pack  $others.fpscr.lbl $others.fpscr.reg -side left
    pack  $others.fpscr -side top -anchor w

    frame $root.others.status
    label $others.status.lbl -text "Status"
    set   SPUCoreReg($root.status) [$spucmd display status ext]
    label $others.status.reg -relief groove -width 60 -font listbox_font -textvariable SPUCoreReg($root.status)
    pack  $others.status.lbl $others.status.reg -side left
    pack  $others.status -side top -anchor w

	table $root \
		$root.lbox0names	0,0 -fill y \
		$root.lbox0data		0,1 -fill both \
		$root.spuregsSB0	0,2 -fill y \
		$root.lbox1names	0,3 -fill y \
		$root.lbox1data		0,4 -fill both \
		$root.spuregsSB1	0,5 -fill y \
        $regn_0fmt          1,0 -cspan 3 -fill x \
        $regn_1fmt          1,3 -cspan 3 -fill x \
        $others             2,0 -cspan 5 -fill x

	#table configure $root r1 c0 c2 c3 c5 -resize none
	table configure $root c* r* -resize none
	table configure $root r0 c1 c4 -resize expand

    set title [treesim::get_path_name $SPUCore($root,widget) $SPUCore($root,treeidx) 1 ] 
    wm title $root "$title: Core"

	return "SPUCore_Refresh"
}

proc  create_format_table {root col} {
    global SPUCoreReg

    set topf [frame $root.regnfmt$col -relief ridge -borderwidth 2]

    set frma [frame $topf.frma ]
    label $frma.regnum -width 6 -font listbox_font -textvariable SPUCoreReg($root.regnum.$col)
    label $frma.fmt16 -relief groove -width 35 -font listbox_font -textvariable SPUCoreReg($root.fmt16.$col)
    pack  $frma.regnum $frma.fmt16 -side left

    label $topf.int0 -relief groove -justify right -width 12 -font listbox_font -textvariable SPUCoreReg($root.int0.$col)
    label $topf.int1 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.int1.$col)
    label $topf.int2 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.int2.$col)
    label $topf.int3 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.int3.$col)

    label $topf.flt0 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.flt0.$col)
    label $topf.flt1 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.flt1.$col)
    label $topf.flt2 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.flt2.$col)
    label $topf.flt3 -relief groove -justify right  -width 12 -font listbox_font -textvariable SPUCoreReg($root.flt3.$col)

    label $topf.dbl0 -relief groove -justify right  -width 18 -font listbox_font -textvariable SPUCoreReg($root.dbl0.$col)
    label $topf.dbl1 -relief groove -justify right  -width 18 -font listbox_font -textvariable SPUCoreReg($root.dbl1.$col)

    grid $topf.frma -row 0 -column 0 -columnspan 4 -sticky nw
    grid $topf.int0 $topf.int1 $topf.int2 $topf.int3 -row 1 -sticky nw
    grid $topf.flt0 $topf.flt1 $topf.flt2 $topf.flt3 -row 2 -sticky nw
    grid $topf.dbl0 -row 3 -column 0 -columnspan 2 -sticky ne
    grid $topf.dbl1 -row 3 -column 2 -columnspan 2 -sticky ne

    return $topf
}

######################################################################
#
# SPUCore_Refresh -- Refresh the contents of the status window
#
######################################################################
proc SPUCore_Refresh { root args } {
	global SPUCoreReg SPUCore

	set spu    $SPUCore($root,spunum)
    set spucmd $SPUCore($root,spucmd)

    set active_0 [$root.lbox0data index active ]
    set active_1 [$root.lbox1data index active ]

	# Set background colors
	for { set col 0 } { $col < 2 } { incr col } {
			for { set i [expr $col * 64] } { $i < [expr ($col + 1) * 64] } { incr i } {
				if { [$spucmd display reg $i] != [eval $root.lbox${col}data get [expr $i - ( $col * 64 )]] } {
					eval $root.lbox${col}names itemconfigure [expr $i - ( $col * 64 )] -bg red
					eval $root.lbox${col}data itemconfigure [expr $i - ( $col * 64 )] -bg red
				} else {
					eval $root.lbox${col}names itemconfigure [expr $i - ( $col * 64 )] -bg lightgrey
					eval $root.lbox${col}data itemconfigure [expr $i - ( $col * 64 )] -bg lightgrey
				}
			}
	}

	# Set new values
	for { set col 0 } { $col < 2 } { incr col } {
			for { set i [expr $col * 64] } { $i < [expr ($col + 1) * 64] } { incr i } {
				eval $root.lbox${col}data delete 0
			}
			for { set i [expr $col * 64] } { $i < [expr ($col + 1) * 64] } { incr i } {
				eval $root.lbox${col}data insert end "[$spucmd display reg $i]"
				eval $root.lbox${col}data itemconfigure [expr $i - ($col * 64)] -background [$root.lbox${col}names itemcget [expr $i - ($col * 64)] -background]

			}
			# After we update all the values, make sure we can see the modified ones
			TclUtils::ScrollBoth ${root}.lbox${col}names ${root}.lbox${col}data 0
			for { set i [expr $col * 64] } { $i < [expr ($col + 1) * 64] } { incr i } {
				set bgcolor [eval $root.lbox${col}names itemcget [expr $i - ($col * 64)] -background]
				if { [string compare $bgcolor "red"] == 0 } {
			        TclUtils::ScrollBoth ${root}.lbox${col}names ${root}.lbox${col}data [expr $i - ( $col * 64 )]
				}

			}
	}

    set newfpscr [$spucmd display FPSCR]
    if { $SPUCoreReg($root.fpscr) != $newfpscr } {
        $root.others.fpscr.reg configure -bg red
    } else {
        $root.others.fpscr.reg configure -bg lightgrey
    }
    set SPUCoreReg($root.fpscr) $newfpscr

    set newstatus  [$spucmd display status ext]
    if { $SPUCoreReg($root.status) != $newstatus } {
        $root.others.status.reg configure -bg red
    } else {
        $root.others.status.reg configure -bg lightgrey
    }
    set SPUCoreReg($root.status) $newstatus

    if {$active_0 != ""} {
        $root.lbox0data activate $active_0
        ShowSPUReg $root.regnfmt0 $active_0
    } elseif {$active_1 != ""} {
        $root.lbox1data activate $active_1
        ShowSPUReg $root.regnfmt1 $active_1
    }
}

proc ShowSPUReg {win reg } {

	global SPUCoreReg SPUCore

    set root [winfo parent $win]
	set spu    $SPUCore($root,spunum)
	set spucmd $SPUCore($root,spucmd)

   if {$reg < 64} {
       set col 0
   } else {
       set col 1
   }
   #puts "show spu reg $root, $reg"

   set SPUCoreReg($root.regnum.$col) "REG$reg"

   set hex32 [$spucmd display reg $reg]
   set SPUCoreReg($root.fmt16.$col) $hex32

   set bin32 [binary format H32 [string range $hex32 2 end]]    ;# skip over 0x and convert to binary

   binary scan $bin32 IIII SPUCoreReg($root.int0.$col) \
                           SPUCoreReg($root.int1.$col) \
                           SPUCoreReg($root.int2.$col) \
                           SPUCoreReg($root.int3.$col)

   set floats [$spucmd display reg $reg f]
   set doubles [$spucmd display reg $reg d]

   set SPUCoreReg($root.flt0.$col) [lindex $floats 0]
   set SPUCoreReg($root.flt1.$col) [lindex $floats 1]
   set SPUCoreReg($root.flt2.$col) [lindex $floats 2]
   set SPUCoreReg($root.flt3.$col) [lindex $floats 3]

   set SPUCoreReg($root.dbl0.$col) [lindex $doubles 0]
   set SPUCoreReg($root.dbl1.$col) [lindex $doubles 1]

}

