#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  spumemory.tcl
#
#------------------------------------------------------------------------

######################################################################
#
# SPUMemory_Init -- Initalize the status window
#
######################################################################
proc SPUMemory_Init {root _upstate args } {
    global SPUStart SPUSim  SPUEnd
    global SPUDeltaLines SPULength SPULengthFormat
    global SPUMemory SPUMemoryInstructions

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate SPUMemory $root       ;# create SPUMemory($root,x)

    set sim $SPUMemory($root,sim)
    set spu $SPUMemory($root,spunum)


    set SPUSim($root) $sim
    set SPUStart($root) 0x0
    set SPULength($root) 512
    set SPULengthFormat($root) 0
    set SPUEnd($root) [format "0x%X" [expr $SPUStart($root) + $SPULength($root)]]

    set start [frame ${root}.start]
    label ${start}.lbl -text "Start Addr:"
    entry ${start}.addr -textvariable SPUStart($root) -width 10
    grid ${start}.lbl ${start}.addr -padx 2 -columnspan 2

    set leng [frame ${root}.leng]
    label ${leng}.lbl -text "Length:"
    entry ${leng}.addr -textvariable SPULength($root) -width 8
    checkbutton ${leng}.form -text "Hex" -onvalue 1 -offvalue 0 -variable SPULengthFormat($root) -command "SPUMemory_ViewWindow $root"
    grid ${leng}.lbl ${leng}.addr ${leng}.form -padx 2 -columnspan 3

    button $root.refresh -text "Reload" -command "SPUMemory_Refresh $root"
    button $root.close -text "Close" -command "destroy $root"
    button $root.membrk -text "Breakpoints..." -command "SPUMemory_Breakpoint $root"

    #text $root.mem -wrap word -width 42 -height 14 -yscrollcommand "$root.memsy set" -font listbox_font
    listbox $root.mem -width 42 -height 14 -yscrollcommand "$root.memsy set" -font listbox_font
    scrollbar $root.memsy -orient vert -command "$root.mem yview"

    # instructions
    frame $root.instructions -borderwidth 2 -relief sunken
    checkbutton $root.instructions.cb -text "View As Instructions" -onvalue 1 -offvalue 0 -variable SPUMemoryInstructions($root) -command "SPUMemory_ViewToggle $root"
    pack $root.instructions.cb -side left

    # Start with no delta lines
    set SPUDeltaLines($root) {}

    table $root \
        $root.mem 0,0 -cspan 5 -fill both \
        $root.memsy 0,5 -cspan 1 -fill y \
        $root.start 1,0 -cspan 2 -fill both \
        $root.leng 1,2 -cspan 3 -fill both \
        $root.instructions 2,0 -cspan 2 -fill both \
        $root.membrk 2,2 -fill x \
        $root.refresh 2,3 -fill x \
        $root.close 2,4 -fill x 
    table configure $root c* r* -resize none
    table configure $root r0 c0 -resize expand

    set title [treesim::get_path_name $SPUMemory($root,widget) $SPUMemory($root,treeidx) 1 ] 
    wm title $root "$title: Local Store" 

    bind ${start}.addr <KeyPress-Return> "SPUMemory_ViewWindow $root" 
    bind ${start}.addr <KeyPress-Tab>    "SPUMemory_ViewWindow $root" 
    bind ${leng}.addr <KeyPress-Return>  "SPUMemory_ViewWindow $root"

    SPUMemory_Refresh $root
    return "SPUMemory_Refresh"
}

proc SPUMemory_Breakpoint { root } {
    global SPUMemory

    set mcmcmd $SPUMemory($root,mcmcmd)
    set spu $SPUMemory($root,spunum)
    set line [$root.mem get [$root.mem index @0,0] ]
    set startaddr 0x[string range $line 0 7]
    Memory_breakpoint_dlg $mcmcmd "spu" $spu $startaddr 0
}

proc SPUMemory_ViewToggle { root } {
    set iMem "$root.mem"
    $iMem delete 0 end
    SPUMemory_Refresh $root
}

proc SPUMemory_ViewWindow { root } {
    global SPUMemory SPUStart SPUEnd
    global SPUDeltaLines SPULengthFormat SPULength

    set spucmd $SPUMemory($root,spucmd)
    set ls_limit [format "0x%08X" [$spucmd display spr spu_amr]]

    set SPUStart($root) [format "0x%08X" $SPUStart($root)]
    if {$SPUStart($root) > $ls_limit} {
        set SPUStart($root) $ls_limit
    }

    set SPUEnd($root) [format "0x%08X" [expr $SPUStart($root) + $SPULength($root)]]
    if {$SPUEnd($root) > $ls_limit} {
        set SPUEnd($root) $ls_limit
        set SPULength($root) [expr $SPUEnd($root) - $SPUStart($root)]
    }

    if { $SPULengthFormat($root) } {
        set SPULength($root) [format "0x%X" $SPULength($root)]
    } else {
        set SPULength($root) [format "%d" $SPULength($root)]
    }

    set iMem "$root.mem"
    $iMem delete 0 end

    SPUMemory_Refresh $root
}

######################################################################
#
# SPUMemory_Refresh -- Refresh the contents of the status window
#
######################################################################
proc SPUMemory_Refresh { root args } {
    global SPUStart SPUEnd SPUSim
    global SPUDeltaLines SPULengthFormat SPULength
    global SPUMemory SPUMemoryInstructions

    set sim $SPUSim($root)
    set spu $SPUMemory($root,spunum)
    set spucmd $SPUMemory($root,spucmd)

    #busy hold .
    set iMem "$root.mem"
    set addr $SPUStart($root)

    # Remove difference coloration
    set NormalColor lightgrey
    set DeltaColor lightgrey
    if { [string length [$iMem get 0]] > 0 } {
        set DeltaColor red
        # Set the background of all the delta lines back to gray
        foreach l $SPUDeltaLines($root) {
            $iMem itemconfigure $l -bg $NormalColor
        }
    }

    set linenum 0
    # Clear List
    set SPUDeltaLines($root) {}

    while { $addr < $SPUEnd($root) } {

        if { $SPUMemoryInstructions($root) } {
            if { [catch {
                  set value [$spucmd display memory $addr 4]
                  set ascii [TclUtils::GenAscii $value]
                  set disasm [$sim util spu_disasm $value $addr]
               } ] == 1 } then {
                  set value -1
                  set ascii "****"
                  set disasm "????????"
            }

            set line [format "%08X : %08X : %s : %s" $addr $value $ascii $disasm]
            incr addr 4
        } else {
            set tmpaddr $addr

            for { set count 0 } { $count < 4 } { incr count } {
                    set value($count) [$spucmd display memory $tmpaddr 4]
                    incr tmpaddr 4     
                    set ascii($count) [TclUtils::GenAscii $value($count)]
            }

            set line [format "%08X : %08X %08X %08X %08X : %s%s%s%s" $addr $value(0) $value(1) $value(2) $value(3) $ascii(0) $ascii(1) $ascii(2) $ascii(3)]
            set addr $tmpaddr
        }

        set curline [$iMem get $linenum]
        if { [string compare $curline $line] != 0 } {
            #puts "XXX line $linenum mismatch len:[string length $curline] len:[string length $line] DeltaColor is $DeltaColor"
            $iMem delete $linenum $linenum
            $iMem insert $linenum $line

            # Save index to clear color next time
            if {[string compare $DeltaColor $NormalColor] != 0} {
                lappend SPUDeltaLines($root) $linenum
            }
        }
        incr linenum
    }

    # Color the background of all the delta lines back to red
    set i 0
    foreach l $SPUDeltaLines($root) {
        # Make it so we can see the first delta line
        if { $i == 0 } {
            $iMem see $l
        }
        #puts "Color bg of index $l to $DeltaColor"
        eval $iMem itemconfigure $l -bg $DeltaColor
        incr i
    }
}

