#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  sputracker.tcl
#------------------------------------------------------------------------

namespace eval sputrack {
    variable flyover
    set flyover(initialized) 0
}

proc spu_break_point { spucmd spu comment args } {
    set pc [$spucmd display pc]
    puts "SPU $spu $pc: break point ($comment)"
    simstop
}

proc SPUTrack_ToggleBreak {root mcmcmd spucmd spu txtwidget y} {
    global SPUTrackStart
    set cursor [$txtwidget index @0,$y]
    set line [ lindex [ split $cursor . ] 0 ]
    set addr [add64 $SPUTrackStart($root) [expr ($line-1) * 4]]
    set hexaddr [format64 "0x%016llX" $addr]
    if { [string length [$mcmcmd trigger display spu $spu pc addr $hexaddr]] > 0 } {
        puts "Clearing breakpoint at $hexaddr"
        if {[catch {$mcmcmd trigger clear spu $spu pc $hexaddr} result]} {
            puts "SPUTrack_ToggleBreak: FAILED to clear breakpoint '$result'"
            return -code error
        }
        textutils::untag_breakpoint $txtwidget $cursor
    } else {
        puts "Setting breakpoint at $hexaddr"
        if {[catch {$mcmcmd trigger set spu $spu pc $hexaddr "spu_break_point $spucmd $spu Break_at_$hexaddr"} result]} {
            puts "SPUTrack_ToggleBreak: FAILED to set breakpoint '$result'"
            return -code error
        }
        textutils::tag_breakpoint $txtwidget $cursor
    }
}

######################################################################
#
# SPUTrack_Init -- Initalize the status window
#
# ------------------------------------------------------------------
proc SPUTrack_Init {root _upstate args} {

    global SPUTrackEnd
    global SPUTrackLength SPUTrackStart
    global SPUTrackUserLength SPUTrackUserStart
    global SPUTrack

    upvar $_upstate  initstate
    TclUtils::copyarray initstate SPUTrack $root       ;# create SPUTrack($root,x)

    set sim $SPUTrack($root,sim)
    set spu $SPUTrack($root,spunum)
    set spucmd $SPUTrack($root,spucmd)
    set mfccmd $SPUTrack($root,mfccmd)
    set mcmcmd $SPUTrack($root,mcmcmd)

    set pc [$spucmd display pc]
    set SPUTrackStart($root) "0"
    set SPUTrackUserStart($root) "0x0"
    set SPUTrackEnd($root)   "-1"
    set SPUTrackLength($root)     64
    set SPUTrackUserLength($root) 64

    # starting address
    frame $root.start -borderwidth 2 -relief sunken
    label $root.start.label -text "Start Addr:" -pady 4
    entry $root.start.addr -textvariable SPUTrackUserStart($root)
    pack $root.start.label $root.start.addr -side left
    bind $root.start.addr <KeyPress-Return> "SPUTrack_Refresh $root"


    # length and format
    frame $root.leng -borderwidth 2 -relief sunken
    label $root.leng.label -text "Length:" -padx 4
    entry $root.leng.addr -textvariable SPUTrackUserLength($root) -width 8 -justify right
    bind $root.leng.addr <KeyPress-Return> "sputrack::length_changed $root"
    pack $root.leng.label $root.leng.addr -side left

    button $root.reload -text "Reload" -command "SPUTrack_Refresh $root"
    button $root.close -text "Close" -command "destroy $root"
    button $root.step -text "Step" -command "$spucmd step 1"

    text $root.sputrk -width 70 -height 14 -yscrollcommand "$root.sputrksy set" -font textbox_font  -tabs "12c" -wrap none
    scrollbar $root.sputrksy -orient vert -command "$root.sputrk yview"
    bind $root.sputrk <Double-1> "SPUTrack_ToggleBreak $root $mcmcmd $spucmd $spu %W %y"

    textutils::init_color_tags $root.sputrk

    table $root \
        $root.sputrk   0,0 -cspan 5 -fill both \
        $root.sputrksy 0,5 -cspan 1 -fill y \
        $root.start    1,0 -fill both \
        $root.leng     1,1 -fill both \
        $root.reload   1,2 -fill x    \
        $root.step   1,3 -fill x    \
        $root.close    1,4 -fill x
    table configure $root c* r* -resize none
    table configure $root r0 c0 -resize expand

    set title [treesim::get_path_name $SPUTrack($root,widget) $SPUTrack($root,treeidx) 1 ]
    wm title $root "$title: PC Tracker"

    SPUTrack_Refresh $root

    sputrack::flyover_init $root $spucmd

    return "SPUTrack_Refresh"
}

proc sputrack::length_changed {root} {
     SPUTrack_Refresh $root
}


######################################################################
#
# SPUTrack_Refresh -- Refresh the contents of the status window
#
######################################################################
proc SPUTrack_Refresh { root {force_reload 0} args } {
    global SPUTrack SPUTrackStart SPUTrackEnd
    global SPUTrackUserLength SPUTrackUserStart SPUTrackLength

    set sim    $SPUTrack($root,sim)
    set spu    $SPUTrack($root,spunum)
    set mcmcmd $SPUTrack($root,mcmcmd)
    set spucmd $SPUTrack($root,spucmd)
    set mfccmd $SPUTrack($root,mfccmd)
    set pc [$spucmd display pc]
    set iMem   "$root.sputrk"

    # If the user entered a new length or starting address...
    if {$SPUTrackUserLength($root) != $SPUTrackLength($root) ||
        $SPUTrackUserStart($root) != $SPUTrackStart($root)} {
       set SPUTrackUserLength($root) [ TclUtils::get_larger_power2 $SPUTrackUserLength($root)]
       set SPUTrackLength($root) $SPUTrackUserLength($root)
       set pc $SPUTrackUserStart($root)
       set force_reload 1
    }

    # See if we need to reload the contents
    if { $pc < $SPUTrackStart($root) ||
         $pc >= $SPUTrackEnd($root) ||
         $force_reload } {
        set SPUTrackStart($root) [expr $pc & ~( $SPUTrackLength($root) - 1 )]
        set SPUTrackUserStart($root) [format "0x%X" $SPUTrackStart($root)]
        set SPUTrackEnd($root) [expr $SPUTrackStart($root) + $SPUTrackLength($root)]
        set addr $SPUTrackStart($root)

        set bplist [$mcmcmd trigger display spu $spu pc list ]
        $iMem delete 1.0 end
        while { $addr < $SPUTrackEnd($root) } {
            set value [$spucmd display memory $addr 4]
            set ascii [TclUtils::GenAscii $value]
            if { [lsearch $bplist [format "0x%08X" $addr] ] >= 0 } {
                $iMem insert end "B " breakpoint
            } else {
                $iMem insert end "  "
            }
            set line [format "%08X : %08X : %s : %s\t\t\n" $addr $value $ascii [$sim util spu_disasm $value]]
            $iMem insert end $line
            incr addr 4
        }
        $iMem delete insert-1c end       ;# delete last newline char
    } else {
        # We do need to run through and update any disasm line that has changed
        set addr $SPUTrackStart($root)
        set linenum 1
        set bplist [$mcmcmd trigger display spu $spu pc list ]
        while { $addr < $SPUTrackEnd($root) } {
            set index $linenum.0
            set value [$spucmd display memory $addr 4]
            set oldvalue [$iMem get "$index+13chars" "$index+13chars wordend" ]
            if { [expr $value != "0x$oldvalue"] } {
                set ascii [TclUtils::GenAscii $value]
                set line [format "%08X : %08X : %s : %s\t\t" $addr $value $ascii [$sim util spu_disasm $value]]
                $iMem delete $linenum.2 $linenum.end
                $iMem insert $linenum.2 $line
            }
            incr addr 4
            incr linenum
        }
        textutils::untag_pcline $iMem $SPUTrack($root,last_pccursor)
    }

    # Now highlight the current pc
    set pc [$spucmd display pc]
    set pc_visible [expr $pc >= $SPUTrackStart($root) && $pc < $SPUTrackEnd($root) ]
    if {$pc_visible} {
        set pc_index [expr ( $pc-$SPUTrackStart($root) )/4 + 1]
        textutils::tag_pcline $iMem $pc_index.0
        $iMem see $pc_index.0
        set SPUTrack($root,last_pccursor) $pc_index.0
    }

}


#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc sputrack::flyover_init {root spucmd} {
   variable flyover

   if { !$flyover(initialized) } {

      set flyover(initialized) 1

      # add channel names to watch for
      # could get these from the model
      set channel_list [ list \
         0       "SPU_RdEventStat"          \
         1       "SPU_WrEventStat"          \
         2       "SPU_WrEventAck"           \
         3       "SPU_RdSigNotify1"         \
         4       "SPU_RdSigNotify2"         \
         7       "SPU_WrDec"                \
         8       "SPU_RdDec"                \
         9       "MFC_WrMSSyncReq"          \
         11      "SPU_RdEventMask"          \
         12      "MFC_RdTagMask"            \
         13      "SPU_RdMachStat"           \
         14      "SPU_WrSRR0"               \
         15      "SPU_RdSRR0"               \
         16      "MFC_LSA"                  \
         17      "MFC_EAH"                  \
         18      "MFC_EAL"                  \
         19      "MFC_Size"                 \
         20      "MFC_TagID"                \
         21      "MFC_Cmd"                  \
         22      "MFC_WrTagMask"            \
         23      "MFC_WrTagUpdate"          \
         24      "MFC_RdTagStat"            \
         25      "MFC_RdListStallStat"      \
         26      "MFC_WrListStallAck"       \
         27      "MFC_RdAtomicStat"         \
         28      "SPU_WrOutMbox"            \
         29      "SPU_RdInMbox"             \
         30      "SPU_WrOutIntrMbox"        \
      ]

      foreach {chnum chname} $channel_list {
         set flyover($chname) "display channel $chnum info"
         set flyover($chname,extra) "type channel chnum $chnum"
      }
   }

   flyover::register $root.sputrk "sputrack::flyover_handler $root $spucmd"
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc sputrack::register_handler { spucmd txtwin startidx } {

    set num [ $txtwin get $startidx "$startidx wordend" ]
    set txt "\$$num = "
    set val [$spucmd display reg $num]
    return [list $txt $val]
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc sputrack::id_keyword_handler { spucmd txtwin startidx {prefix ""}} {
    variable flyover

    # id/keyword
    set txt [ $txtwin get $startidx "$startidx wordend" ]

    # Now we have the target word under the mouse, see if it in our list of
    # trackable variables
    set cmd [ lindex [ array get flyover $txt ] 1 ]
    if { $cmd != "" } then {
       set extralist [ lindex [ array get flyover "$txt,extra" ] 1 ]
       if { $extralist != "" } then {
           array set extra $extralist
           if {$extra(type) == "channel"} {
               set txt "$prefix$txt (ch$extra(chnum)) = "
               set val [eval $spucmd $cmd]
               return [list $txt "$val (value count)"]
           }
       } else {
           set txt "$prefix$txt = "
           set val [eval $spucmd $cmd]
           return [list $txt $val]
       }
    }
    return {}
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc sputrack::flyover_handler { root spucmd txtwin startidx } {
   global SPUTrack
   variable flyover

   set rtn [list]

   set chr [ $txtwin get $startidx ]
   set prevchar [$txtwin get "$startidx-1chars" ]

   if {$chr == "$"} {
       set startidx [ $txtwin index "$startidx+1char" ]
       set chr [ $txtwin get $startidx ]
       set prevchar "$"
   }

   switch -glob -- $chr {
      [A-Za-z]  {
                   if {$prevchar != "$"} {
                      set prevchar ""
                   }
                   set rtn [id_keyword_handler $spucmd $txtwin $startidx $prevchar]
                }
      [0-9]     {
                    #
                    #  If over a number, look for $+number or number(reg)
                    #
                    if {$prevchar == "$"} {
                        set rtn [register_handler $spucmd $txtwin $startidx ]
                    } else {
                        set prevchar [$txtwin get "$startidx-2chars" ]
                        if {$prevchar == "."} {
                            set startidx [$txtwin index "$startidx-2chars" ]
                            set rtn [ textutils::branch_target $txtwin $startidx ]
                        } else {
                            set endidx [ $txtwin index "$startidx wordend" ]
                            set nextchar [$txtwin get "$endidx" ]
                            if {$nextchar == "("} {
                                set rtn [ sputrack::memory_address $spucmd $txtwin $startidx $endidx]
                            }
                        }
                    }
                }
      "."       {
                    # .-num, .+num
                    set rtn [ textutils::branch_target $txtwin $startidx ]
                }
      "-"       -
      "+"       {
                    # .-num, .+num
                    set prevchar [$txtwin get "$startidx-1chars" ]
                    if {$prevchar == "."} {
                        set startidx [$txtwin index "$startidx-1chars" ]
                        set rtn [ textutils::branch_target $txtwin $startidx ]
                    }
                }
   }

   return $rtn
}


#-----------------------------------------------------------------------------
# text at startidx is a number
# calculate num($reg)
#-----------------------------------------------------------------------------
proc sputrack::memory_address { spucmd txtwin startidx numendidx } {

   set regidx [$txtwin index "$numendidx+2chars" ]
   set regendidx [$txtwin index "$regidx wordend" ]
   set reg    [$txtwin get $regidx $regendidx ]
   set sign   [$txtwin get "$startidx-1chars" ]
   if {$sign == "-"} {
       set startidx [$txtwin index "$startidx-1chars" ]
   }
   set offset [$txtwin get $startidx $numendidx ]
   set txt    [$txtwin get $startidx "$regendidx+1chars" ]
   set addr   [ get_indirect_address $spucmd $offset $reg ]
   set data "?"
   catch {
       set data  [string trim [ $spucmd display memory $addr 4 4] ]
   }

   return [list "$txt=>\n" "$addr: $data" ]
}

#------------------------------------------------------------------
#
#------------------------------------------------------------------
proc sputrack::get_indirect_address {spucmd offset regnum } {

   if {$regnum == 0} {
       set regval 0
   } else {
       set regval [string range [$spucmd display reg $regnum] 0 9 ]
   }
   set spu_amr [ $spucmd display spr spu_amr]
   set addr [format "0x%X" [expr ($regval + $offset) & $spu_amr]]

   return $addr
}

