#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  spxlate.tcl
#
#------------------------------------------------------------------------
namespace eval SPU_xlate {
}

# ------------------------------------------------------------------
# SPU_xlate::Init -- Initalize the status window
# ------------------------------------------------------------------
proc SPU_xlate::Init {root _upstate args} {

    variable mystate 
    upvar $_upstate  initstate 
    TclUtils::copyarray initstate mystate $root       ;# create mystate($idx,x)

	set sim $mystate($root,sim)
    set spu $mystate($root,spunum)
    set spucmd $mystate($root,spucmd)
    set mfccmd $mystate($root,mfccmd)

    label $root.xlateLabel -text "SPE${spu} MFC Address Translation"

    ;#####  SLBs   ####################################################################
    set SLBs $root.slbs
    frame $SLBs -relief sunken -bd 2
    label $SLBs.label -text "SLBs"
    checkbutton $SLBs.validchk -text "Valid Only" -anchor w -variable SPU_xlate::mystate($root,slbvalid) -command "SPU_xlate::Refresh $root 1"
    #label $SLBs.datahdr -relief groove -textvariable SLBHeader($root) -font textbox_font
    text $SLBs.datahdr -relief groove -font textbox_font -height 1 -wrap none -width 70
	text $SLBs.data -wrap none -width 70 -height 5 -yscrollcommand "$SLBs.data_sy set" -font textbox_font
	scrollbar $SLBs.data_sy -orient vert -command "$SLBs.data yview"

    table $SLBs \
        $SLBs.label      0,0 -cspan 4 -fill x \
        $SLBs.validchk   0,3 -cspan 1 -fill x \
        $SLBs.datahdr    1,0 -cspan 5 -fill x -anchor w\
        $SLBs.data       2,0 -cspan 4 -fill both \
        $SLBs.data_sy    2,4 -cspan 1 -fill y

    table configure $SLBs c* r* -resize none
    table configure $SLBs r2 c0  -resize both

    ;#####  TLBs   ####################################################################
    set TLBs $root.tlbs
    frame $TLBs -relief sunken -bd 2
    label $TLBs.label -text "TLBs"
    checkbutton $TLBs.validchk -text "Valid Only" -anchor w -variable SPU_xlate::mystate($root,tlbvalid) -command "SPU_xlate::Refresh $root 1"
    #label $TLBs.datahdr -relief groove -textvariable TLBHeader($root) -font textbox_font
    text $TLBs.datahdr -relief groove -font textbox_font -height 1 -wrap none -width 70
    text $TLBs.data -wrap none -width 70 -height 5 -yscrollcommand "$TLBs.data_sy set" -font textbox_font
    scrollbar $TLBs.data_sy -orient vert -command "$TLBs.data yview"

    table $TLBs \
        $TLBs.label      0,0 -cspan 5 -fill x \
        $TLBs.validchk   0,3 -cspan 1 -fill x \
        $TLBs.datahdr    1,0 -cspan 5 -fill x -anchor w\
        $TLBs.data       2,0 -cspan 4 -fill both \
        $TLBs.data_sy    2,4 -cspan 1 -fill y

    table configure $TLBs c* r* -resize none
    table configure $TLBs r2 c0  -resize both

    ##### Address Translation Registers ###############################################
    frame $root.xlate_regs -relief sunken -bd 2
    label $root.xlate_regs.label -text "Translation Registers" -relief groove
    label $root.xlate_regs.dar -text   "DAR   : [$mfccmd display priv dar]"
    label $root.xlate_regs.dsisr -text "DSISR : [$mfccmd display priv dsisr]"
    label $root.xlate_regs.sdr -text   "SDR   : [$mfccmd display priv sdr]"
    label $root.xlate_regs.rmlr -text  "RMLR  : [$mfccmd display rmlr]"
    label $root.xlate_regs.rmor -text  "RMOR  : [$mfccmd display rmor]"

    table $root.xlate_regs \
        $root.xlate_regs.label      0,0 -cspan 2 -fill x \
        $root.xlate_regs.dar        1,0 -fill x \
        $root.xlate_regs.rmlr       1,1 -fill x \
        $root.xlate_regs.dsisr      2,0 -fill x \
        $root.xlate_regs.rmor       2,1 -fill x \
        $root.xlate_regs.sdr        3,0 -cspan 2 -fill x

    ##### Controls and Layout #######################################################
    button $root.close -text "Close" -command "destroy $root"

	table $root \
		$root.xlateLabel   0,0 -cspan 4 -fill x \
		$SLBs              1,0 -cspan 4 -fill both \
		$TLBs              2,0 -cspan 4 -fill both \
        $root.xlate_regs   3,0 -cspan 4 -fill both \
		$root.close        4,3 -fill x

	table configure $root c* r* -resize none
	table configure $root r1 c0 -resize both
	table configure $root r2 c0 -resize both

    set title [treesim::get_path_name $mystate($root,widget) $mystate($root,treeidx) 1 ] 
	wm title $root "$title: Address translation"

	SPU_xlate::Refresh $root 1
	return "SPU_xlate::Refresh"
}

# ------------------------------------------------------------------
# SPU_xlate::Refresh -- Refresh the contents of the status window
# ------------------------------------------------------------------
proc SPU_xlate::Refresh { root args } {
    variable mystate

    set spucmd $mystate($root,spucmd)
    set mfccmd $mystate($root,mfccmd)

    # get the slb data from the simulator
    set SLBs $root.slbs
    if { $mystate($root,slbvalid) } { set amount "valid"} else { set amount "all" }
	set slbs [$spucmd display slb $amount]
	set slbs [split $slbs "\n"]

    # put slb data into text block
    #set SLBHeader($root) [lindex $slbs 0]
    $SLBs.datahdr configure -state normal
    $SLBs.datahdr delete 1.0 end
    $SLBs.datahdr insert end [lindex $slbs 0]
    $SLBs.datahdr configure -state disabled
    $SLBs.data delete 1.0 end
    foreach slb [lrange $slbs 1 end-1] {
        $SLBs.data insert end "$slb\n"
    }
    $SLBs.data delete insert-1c end       ;# delete last newline char

    set TLBs $root.tlbs
    if { $mystate($root,tlbvalid) } { set amount "valid"} else { set amount "all" }
	set tlbs [$spucmd display tlb $amount]
    set tlbs [split $tlbs "\n"]

    #set TLBHeader($root) [lindex $tlbs 0]
    $TLBs.datahdr configure -state normal
    $TLBs.datahdr delete 1.0 end
    $TLBs.datahdr insert end [lindex $tlbs 0]
    $TLBs.datahdr configure -state disabled
    $TLBs.data delete 1.0 end
    foreach tlb [lrange $tlbs 1 end-1]  {
        $TLBs.data insert end "$tlb\n"
    }
    $TLBs.data delete insert-1c end        ;# delete last newline char

    # Update Translation Register Values
    UpdateLabel $root.xlate_regs.dar "DAR   : [$mfccmd display priv dar]"
    UpdateLabel $root.xlate_regs.dsisr "DSISR : [$mfccmd display priv dsisr]"
    UpdateLabel $root.xlate_regs.sdr "SDR   : [$mfccmd display priv sdr]"
    UpdateLabel $root.xlate_regs.rmlr "RMLR  : [$mfccmd display rmlr]"
    UpdateLabel $root.xlate_regs.rmor "RMOR  : [$mfccmd display rmor]"
}
