#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2005
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#   spu_stack_check.tcl
#
#   TCL procs that check for stack overflow in SPU programs.
#
#   Usage:
#
#   To enable stack overflow detection, issue
#
#       spu_stack_check::enable [spu_number]
#
#------------------------------------------------------------------------

namespace eval spu_stack_check {
   variable version       "1.0"
}

proc spu_stack_check::check_avail { sim benum args } {
    scan [lindex $args 0] " spu %d " spunum
    set r1 [ $sim mcm $benum spu $spunum display reg 1 x ]
    set w1 "0x[string range $r1 10 17]"
    scan $w1 "%x" avail
    if { $avail < 0 } {
	puts "STACK OVERFLOW DETECTED ON BE $benum SPU $spunum!"
	simstop
    }
}

proc spu_stack_check::is_enabled { spunum { benum 0 } { sim mysim } } {
    set mem_triggers [ $sim mcm $benum trigger display spu $spunum memory list ]
    for { set i 0 } { $i < [llength $mem_triggers] } { incr i } {
        set mem_trigger [ lindex $mem_triggers $i ]
        if { [ lindex $mem_trigger 3 ] == "spu_stack_check::check_avail" } {
            return 1
        }
    }
    return 0
}

proc spu_stack_check::enable { spunum { benum 0 } { sim mysim } } {
    $sim mcm $benum trigger set spu $spunum memory w 0x0 0x40000 "spu_stack_check::check_avail $sim $benum"
    puts "Stack overflow detection enabled on BE $benum SPU $spunum"
}

proc spu_stack_check::disable { spunum { benum 0 } { sim mysim } } {
    $sim mcm $benum trigger clear spu $spunum memory w 0x0 0x40000 "spu_stack_check::check_avail $sim $benum"
    puts "Stack overflow detection disabled on BE $benum SPU $spunum"
}
