#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  .systemsim.tcl
#
#  This is the default toplevel script that is run when using the CLI.
#------------------------------------------------------------------------

set IMAGES $env(IMAGES_DIR)/cell

# Initialize the systemsim tcl environment
source $env(LIB_DIR)/cell/mambo_init.tcl

# Configure and create the simulated machine
define dup cell myconf

# Call the config_hook proc if it is defined to modify the config
if {[string length [info procs config_hook]] > 0} {
    config_hook myconf
}

# To modify the amount of memory that is configured for each Cell BE
# processor in a system, uncomment and complete the following line
# Note: memory size larger that 1GB can only be requested on 64-bit
# host platforms.
#config_memory_size myconf <memsize_per_BE>

# Uncomment the following line for dual BE configuration support
#config_dual_be myconf

define machine myconf mysim

# Uncomment this to enable automatic starting of the default emitter readers
#EmitterReaders::init_default mysim linux_boot

# Start the GUI if -g option was given
MamboInit::gui $env(LIB_DIR)/cell/gui/gui.tcl

# Find the sysroot_disk to use
if {[file exists ./sysroot_disk]} {
   set sysrootfile ./sysroot_disk
}  elseif {[file exists $IMAGES/sysroot_disk]} {
   set sysrootfile $IMAGES/sysroot_disk
} else {
   puts "could not find sysroot_disk file"
   exit
}

# Find the OS image
if {[file exists ./vmlinux]} {
   set imagefile ./vmlinux
}  elseif {[file exists $IMAGES/vmlinux]} {
   set imagefile $IMAGES/vmlinux
} else {
   puts "could not find linux image file"
   exit
}

# Construct the emulated device tree
build_firmware_tree

# Uncomment the line below to enable bogus network support
#bogusnet::init 172.20.0

# Set boot parameters if desired
of::set_bootargs "lpj=8000000 console=hvc0 root=/dev/mambobd0 rw"

# Load the OS
mysim mcm 0 load vmlinux $imagefile 0x1000000

# Setup the bogus disk that Linux will use
if {[catch {mysim bogus disk init 0 $sysrootfile newcow sysroot_disk.cow 1024}]} {
  puts "bogus disk init for $sysrootfile failed"
  exit
}

# Back to the prompt
