#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  elfload.tcl
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# getfilename
#     Used to select or set a file. Requires options array
#     to be filled out from calling function.
#------------------------------------------------------------------------
proc getfilename { _upoptions } {

   upvar $_upoptions options

   set current_file $options(file_pref)

   if { [file isdirectory $current_file] } then {
      set working_directory $current_file
   } elseif { [file exists $current_file] ||
              [file exists [file dirname $current_file]] } then {
      set working_directory [file dirname $current_file]
   } else {
      set working_directory [pwd]
   }

   set file_types $options(file_types)

   set dlg_title $options(file_title)
   if { $dlg_title == "" } then {
      set dlg_title "Select file..."
   }

   set filename [ tk_getOpenFile -initialdir $working_directory -filetypes $file_types -title $dlg_title ]

   #focus -force $options(file_focus)

   return $filename
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc SelectElfFile { _upstate standalone args} {

    upvar $_upstate  initstate
    set sim $initstate(sim)
    set mcmcmd $initstate(mcmcmd)

    set ElfFile(file_title) "Open Elf Executable to load"
    set ElfFile(file_types) { { "All Files" { * } } }
    set ElfFile(file_pref)  [PrefManager::get_value ElfFile "" ]

    set filename [getfilename ElfFile]
    if { $filename != ""} {
       PrefManager::set_value ElfFile $filename
       LoadElfFile $mcmcmd $standalone $filename
    }
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc LoadElfFile { mcmcmd standalone filename args } {
    #puts "LoadElfFile standalone=$standalone called to load $filename into $simname\n"
    $mcmcmd load elf $filename
    if { $standalone == 1 } {
        $mcmcmd config standalone-app on
    }
    mambo_update 1
}
