#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------


namespace eval Fonts {
    variable version        "1.0"
    variable initialized    0
}

#------------------------------------------------------------------------
# Create the fonts used by the gui
#------------------------------------------------------------------------
proc Fonts::init {} {
   variable initialized  
   if {$initialized == 0} { 
       eval font create default_font [ get_pref fonts/default "-family Helvetica -size 10" ]
       eval font create listbox_font [ get_pref fonts/listbox "-family Courier -size 10" ]
       eval font create listbox_bold_font [ get_pref fonts/listbox "-family Courier -size 10 -weight bold" ]
       eval font create textbox_font [ get_pref fonts/textbox "-family Courier -size 10" ]
       eval font create small_textbox_font [ get_pref fonts/textbox "-family Courier -size 8" ]
       eval font create graph_legend_font [ get_pref fonts/textbox "-family Helvetica -size 8" ]
   }
   set initialized 1
}

#------------------------------------------------------------------------
# Fonts::create
#        Creates the Fonts dialog options dialog box.
#------------------------------------------------------------------------
proc Fonts::create { widget _owner_widget tgt_font } {

    variable fontwin
    variable owner_widget $_owner_widget
    variable ok_pushed

    if { ![ winfo exists $widget ] } then {

        toplevel $widget
        wm transient $widget [ winfo toplevel $owner_widget ]

        font create sample_font

        # Intercept window close and call the exit handler
        set top_widget [ winfo toplevel $widget ]
        wm protocol $top_widget WM_DELETE_WINDOW "Fonts::_on_close $top_widget"

        frame $widget.f
            frame $widget.f.fam -relief groove -borderwidth 3
                label $widget.f.fam.lab -text "Font" -font system_font
                frame $widget.f.fam.f
                    listbox $widget.f.fam.f.lb \
                        -height 5 -width 30 -font system_font \
                        -yscrollcommand "$widget.f.fam.f.sb set"
                    scrollbar $widget.f.fam.f.sb -orient vertical -command "$widget.f.fam.f.lb yview"
                    bind $widget.f.fam.f.lb <<ListboxSelect>> "Fonts::_listbox_preselect %W"
                    foreach f [ lsort [ font families ] ] {
                        $widget.f.fam.f.lb insert end $f
                    }
                    pack $widget.f.fam.f.sb $widget.f.fam.f.lb -side right -fill both -expand 1
                pack $widget.f.fam.lab $widget.f.fam.f -side top

            frame $widget.f.siz -relief groove -borderwidth 3
                label $widget.f.siz.lab -text "Size" -font system_font
                frame $widget.f.siz.f
                    listbox $widget.f.siz.f.lb \
                                -height 5 -width 30 -font system_font \
                                -yscrollcommand "$widget.f.siz.f.sb set"
                    scrollbar $widget.f.siz.f.sb -orient vertical -command "$widget.f.siz.f.lb yview"
                    bind $widget.f.siz.f.lb <<ListboxSelect>> "Fonts::_listbox_preselect %W"
                    set sizes [ list ]
                    for { set i 7 } { $i < 32 } { incr i } { lappend sizes $i }
                    foreach s $sizes {
                        $widget.f.siz.f.lb insert end $s
                    }
                    pack $widget.f.siz.f.sb $widget.f.siz.f.lb -side right -fill both -expand 1
                pack $widget.f.siz.lab $widget.f.siz.f -side top
            pack $widget.f.fam $widget.f.siz -side left -fill both -expand 1


        _sync_to_font $widget $tgt_font

        frame $widget.s -bd 2 -relief groove
            canvas $widget.s.c -height 50 -width 300
            $widget.s.c create text 150 25 -text "Sample Text" \
                    -font sample_font -justify center
            pack $widget.s.c -fill both -expand 1


        frame $widget.bf
            button $widget.bf.ok     -text "   OK   " -command "Fonts::_cmd_ok     $widget" -height 1 -font system_font
            button $widget.bf.cancel -text " Cancel " -command "Fonts::_cmd_cancel $widget" -height 1 -font system_font
            pack $widget.bf.cancel $widget.bf.ok -side right -padx 5 -pady 5

        bind $top_widget <Escape> "Fonts::_cmd_cancel $widget"
        bind $top_widget <Return> "Fonts::_cmd_ok     $widget"

        # Now pack these frames
        pack $widget.f $widget.s $widget.bf -side top -fill both -expand 1


    } else {
        focus $widget.bf.ok
        _sync_to_font $widget $tgt_font
        wm deiconify [ winfo toplevel $widget ]
    }

    update
    set geom [ get_pref win_geom/fontselect "550x250+100+100" ]
    wm geometry $widget $geom

}

#------------------------------------------------------------------------
# Fonts::_save_geometry
#------------------------------------------------------------------------
proc Fonts::_save_geometry { top_widget } {

    set_pref win_geom/fontselect [ winfo geometry $top_widget ]
}

#------------------------------------------------------------------------
# Fonts::-on_close
#------------------------------------------------------------------------
proc Fonts::-on_close { top_widget } {

    variable trigger
    variable owner_widget

    $top_widget.f.fam.f.lb itemconfigure active -bg {} -fg {}
    $top_widget.f.siz.f.lb itemconfigure active -bg {} -fg {}

    _save_geometry $top_widget

    focus $owner_widget
    wm withdraw $top_widget

    set trigger 1
}

#---------------------------------------------------------------------------
#  Fonts::_cmd_ok
#---------------------------------------------------------------------------
proc Fonts::_cmd_ok { top_widget } {

    variable chosen
    variable ok_pushed 1

    set chosen(fam)  [ $top_widget.f.fam.f.lb get active ]
    set chosen(size) [ $top_widget.f.siz.f.lb get active ]
    -on_close $top_widget
}

#---------------------------------------------------------------------------
#  Fonts::_cmd_cancel
#---------------------------------------------------------------------------
proc Fonts::_cmd_cancel { top_widget } {

    variable ok_pushed 0

    -on_close $top_widget
}

#---------------------------------------------------------------------------
#  Fonts::_listbox_preselect
#---------------------------------------------------------------------------
proc Fonts::_listbox_preselect { widget } {

    $widget itemconfigure active -bg {} -fg {}
    after 100 "Fonts::_listbox_postselect $widget"
}

#---------------------------------------------------------------------------
#  Fonts::_listbox_postselect
#---------------------------------------------------------------------------
proc Fonts::_listbox_postselect { widget } {

    global gvars

    $widget itemconfigure active -bg $gvars(sys_selbg) -fg $gvars(sys_selfg)
    set top_widget [ winfo toplevel $widget ]
    font configure sample_font -family [ $top_widget.f.fam.f.lb get active ] -size [ $top_widget.f.siz.f.lb get active ]
}

#------------------------------------------------------------------------
# Fonts::_sync_to_font
#------------------------------------------------------------------------
proc Fonts::_sync_to_font { widget _tgt_font } {

    variable chosen

    switch $_tgt_font {
        "def_font" { set tgt_font default_font }
        "lb_font"  { set tgt_font listbox_font }
        "tb_font"  { set tgt_font textbox_font }
    }

    set chosen(fam)  [ font configure $tgt_font -family ]
    set chosen(size) [ font configure $tgt_font -size   ]

    set w $widget.f.fam.f.lb
    $w selection clear 0 end
    for { set i 0 } { $i < [ $w size ] } { incr i } {
        if { [ string match -nocase [ $w get $i ] $chosen(fam) ] } then {
            $w activate $i
            $w see $i
            _listbox_preselect $w
        }
    }

    set w $widget.f.siz.f.lb
    $w selection clear 0 end
    for { set i 0 } { $i < [ $w size ] } { incr i } {
        if { [ string match -nocase [ $w get $i ] $chosen(size) ] } then {
            $w activate $i
            $w see $i
            _listbox_preselect $w
        }
    }
}

