#package require BLT
#namespace import blt::*

#source ${commonlibdir}/sysmem.tcl
#source ${commonlibdir}/value_expand.tcl
#source ${commonlibdir}/scrolltogether.tcl

# Assume 64-bit machine until discover otherwise
set GPRegWidth 64

array set nibbles { \
    0 "0000" 1 "0001" 2 "0010" 3 "0011" 4 "0100" 5 "0101" 6 "0110" 7 "0111" 8 "1000" 9 "1001" \
    a "1010" b "1011" c "1100" d "1101" e "1110" f "1111" \
    A "1010" B "1011" C "1100" D "1101" E "1110" F "1111" }

# MSR bits:  Display name of bit if bit is 1
#               0       1       2       3       4       5       6       7       8       9
set msrflags {"SF"    "TA"   "ISF"    "HV"      ""      ""      ""      ""      ""      ""  \
               ""      ""      ""      ""      ""      ""      ""      ""      ""      ""  \
               ""      ""      ""      ""      ""      ""      ""      ""      ""      ""  \
               ""      ""      ""      ""   "SIM"      ""      ""      ""   "VMX"      ""  \
               ""      ""      ""      ""      ""    "WE"    "CE"   "ILE"    "EE"    "PR"  \
             "FP"    "ME"   "FE0"    "SE"    "BE"   "FE1"      ""    "IP"    "IR"    "DR"  \
               ""      ""    "RI"    "LE"}

# CR fields: Display "<", ">", or "=" depending on bit 0, 1, 2.
#            Display "." if none or more than one bit in {0, 1, 2} is set (not an arithmetic result).
#            Display "V" if summary overflow bit is 1.

array set crflags { \
    0 "." 1 ".V" 2 "=" 3 "=V" 4 ">" 5 ">V" 6 "." 7 ".V" 8 "<" 9 "<V" \
    a "." b ".V" c "." d ".V" e "." f ".V" \
    A "." B ".V" C "." D ".V" E "." F ".V" }

proc bitstring {hexin len} {
    set hex $hexin
    global nibbles

    # Strip leading "0x" if present
    if {[string length $hex] > 2} {
       if {[string compare "0x" [string range $hex 0 1]] == 0} {
           set hex [string range $hex 2 end]
       }
    }

    # Insert leading zeros if needed
    set bits ""
    for {set zlen [expr $len - ([string length $hex] * 4)]} {$zlen > 0} {incr zlen -1} {
       append bits "0"
    }

    # Convert each hex nibble to 4 binary bits (as characters "0" and "1")
    for {set i 0} {$i < [string length $hex]} {incr i} {
       append bits $nibbles([string range $hex $i $i])
    }

    # Return a string of $len "0" and "1" characters
    return $bits
}

proc GPRegs_InsertPC {path pc cpucmd mcmcmd} {
    GPRegs_InsertReg $path "PC" $pc $pc pc pc
    $path insert end "    " {same}

    if {[catch {$cpucmd itranslate $pc} rAddr]} {
        set rAddr 0
    }

    if {[catch {$mcmcmd memory display $rAddr 4} iWord]} {
        set iWord 0
    }

    # If word we've fetched is an instruction prefix,
    # save it away and fetch the actual instruction word.
    set opCode [expr $iWord >> 26]
    if {$opCode == 1} {
       set prefix $iWord
       set rAddr [$cpucmd itranslate [expr $pc + 4]]
       set iWord [$mcmcmd memory display $rAddr 4]
    } else {
       set prefix 0
    }

    # Disassemble the prefix and instruction
    set asm [$mcmcmd util ppc_disasm2 $prefix $iWord $pc]
    $path insert end $asm {instr}
}


proc GPRegs_InsertReg {path name newval oldval sametag changedtag} {
    $path insert end [format "%5s " $name] {same}
    set new $newval
    set old $oldval
    if {[string compare "0x" [string range $new 0 1]] == 0} {
       set new [string range $new 2 end]
    }
    if {[string compare "0x" [string range $old 0 1]] == 0} {
       set old [string range $old 2 end]
    }
    if {[string compare $old $new] == 0} {
       $path insert end $new [list $sametag]
    } else {
       $path insert end $new [list $changedtag]
    }
}


proc GPRegs_InsertMSR {path newmsr oldmsr sametag changedtag} {
    global msrflags GPRegWidth
    set new $newmsr
    set old $oldmsr

    # Strip leading "0x" if present
    if {[string compare "0x" [string range $newmsr 0 1]] == 0} {
       set new [string range $newmsr 2 end]
    }
    if {[string compare "0x" [string range $oldmsr 0 1]] == 0} {
       set old [string range $oldmsr 2 end]
    }

    # Insert label
    $path insert end "\n  MSR " {sametag}

    # Insert each hex digit, tagging as changed/not changed
    for {set i 0} {$i < 16} {incr i} {
       set c [string range $new $i $i]
       if {[string compare $c [string range $old $i $i]] == 0} {
           $path insert end $c [list $sametag]
       } else {
           $path insert end $c [list $changedtag]
       }
    }

    # Insert names for MSR bits.
    # For bit 0, display as "64" or "32" (execution mode bit).
    # For the others, display name only if bit is 1
    $path insert end "    " {sametag}
    set newbits [bitstring $new 64]
    set oldbits [bitstring $old 64]
    set needcomma 0
    # Start with bit 0 or bit 32, depending on MSR width
    for {set i [expr 64 - $GPRegWidth]} {$i < 64} {incr i} {
       set newbit [string range $newbits $i $i]
       set oldbit [string range $oldbits $i $i]

       if {[string compare $newbit $oldbit] == 0} {
           set t [list $sametag]
       } else {
           set t [list $changedtag]
       }
       if {$i == 0} {
           if {[string compare "1" $newbit] == 0} {
              $path insert end "64" $t
           } else {
              $path insert end "32" $t
           }
           set needcomma 1
       } else {
           set flagname [lindex $msrflags $i]
           if {[string length $flagname] > 0} {
              if {[string compare "1" $newbit] == 0} {
                  if {$needcomma} {
                     $path insert end "," {sametag}
                  }
                  $path insert end $flagname $t
                  set needcomma 1
              }
           }
       }
    }

    $path insert end "\n"
}

proc GPRegs_InsertCR {path newcr oldcr sametag changedtag} {
    global crflags GPRegWidth
    set new $newcr
    set old $oldcr

    # Strip leading "0x" if present
    if {[string compare "0x" [string range $newcr 0 1]] == 0} {
       set new [string range $newcr 2 end]
    }
    if {[string compare "0x" [string range $oldcr 0 1]] == 0} {
       set old [string range $oldcr 2 end]
    }

    # Insert label
    if {$GPRegWidth == 64} {
       $path insert end "   CR         " [list $sametag]
    } else {
       $path insert end "   CR " [list $sametag]
    }

    # Insert each hex digit, tagging as changed/not changed
    for {set i 0} {$i < 8} {incr i} {
       set c [string range $new $i $i]
       if {[string compare $c [string range $old $i $i]] == 0} {
           $path insert end $c [list $sametag]
       } else {
           $path insert end $c [list $changedtag]
       }
    }

    # Insert mnemonic interpretation of CR fields (<, =, >, V)
    $path insert end "    " [list $sametag]
    for {set i 0} {$i < 8} {incr i} {
       if {$i != 0} {
           $path insert end " " [list $sametag]
       }
       set c [string range $new $i $i]
       if {[string compare $c [string range $old $i $i]] == 0} {
           $path insert end $crflags($c) [list $sametag]
       } else {
           $path insert end $crflags($c) [list $changedtag]
       }
    }
    $path insert end "\n"
}

######################################################################
#
# GPRegs_Init -- Initalize the GP registers window
#
# 64-bit machine:
#  |
#  |  MSR XXXXXXXXXXXXXXXX    64,EE,FP
#  |   CR         XXXXXXXX    = < > . . . . .
#  |
#  |   PC XXXXXXXXXXXXXXXX    ld  r23,48(r19)
#  |
#  |   LR XXXXXXXXXXXXXXXX   CTR XXXXXXXXXXXXXXXX   XER XXXXXXXXXXXXXXXX
#  |
#  |GPR 0 XXXXXXXXXXXXXXXX    16 XXXXXXXXXXXXXXXX    32 XXXXXXXXXXXXXXXX    48 XXXXXXXXXXXXXXXX
#  |    1 XXXXXXXXXXXXXXXX    17 XXXXXXXXXXXXXXXX    33 XXXXXXXXXXXXXXXX    49 XXXXXXXXXXXXXXXX
#  |    2 XXXXXXXXXXXXXXXX    18 XXXXXXXXXXXXXXXX    34 XXXXXXXXXXXXXXXX    50 XXXXXXXXXXXXXXXX
#  |      ...
#  |   15 XXXXXXXXXXXXXXXX    31 XXXXXXXXXXXXXXXX    47 XXXXXXXXXXXXXXXX    63 XXXXXXXXXXXXXXXX
#  |
#
# 32-bit machine:
#  |
#  |  MSR XXXXXXXX    EE,FP
#  |   CR XXXXXXXX    = < > . . . . .
#  |
#  |   PC XXXXXXXX    ld  r23,48(r19)
#  |
#  |   LR XXXXXXXX   CTR XXXXXXXX   XER XXXXXXXX
#  |
#  |GPR 0 XXXXXXXX    16 XXXXXXXX    32 XXXXXXXX    48 XXXXXXXX
#  |    1 XXXXXXXX    17 XXXXXXXX    33 XXXXXXXX    49 XXXXXXXX
#  |    2 XXXXXXXX    18 XXXXXXXX    34 XXXXXXXX    50 XXXXXXXX
#  |      ...
#  |   15 XXXXXXXX    31 XXXXXXXX    47 XXXXXXXX    63 XXXXXXXX
#  |
#
######################################################################

proc GPRegs_Init {root _upstate args} {
    global GPRegs
    global GPRegWidth

    upvar $_upstate  initstate
    TclUtils::copyarray initstate GPRegs $root      ;# create GPRegs($root,x)
    set cpucmd $GPRegs($root,cpucmd)
    set mcmcmd $GPRegs($root,mcmcmd)
    set ngpr [$cpucmd display ngpr]

    # Determine whether we have 64-bit or 32-bit regs
    set v [$cpucmd display gpr 0]
    if {[string length $v] < 16} {
       set GPRegWidth 32;          # default is 64, above
    }

    # Display GP regs in a set of columns.
    # Compute number of rows/columns based on the number of GPRs.
    # Register numbers run down the columns; last column may not be full.
    # (For now, hard-code the number of columns at 4.)
    set cols 4
    set rows [expr ($ngpr + $cols - 1) / $cols]

    set GPRegs($root,ngpr) $ngpr
    set GPRegs($root,rows) $rows
    set GPRegs($root,cols) $cols

    set GPRegs($root,ic)   [$cpucmd display instruction_count]

    # Text widget with GP register numbers and values.
    # We use a read-only text widget so we can control the colors of various parts.

    # Adjust column width for register size
    if {$GPRegWidth == 64} {
       set w 23
    } else {
       set w 15
    }
    set t [text $root.gprvals -width [expr ($w * $cols) + 2] -height [expr $rows + 9] -font listbox_font -wrap none]

    # Define tags -- one for changed reg contents, one for contents that haven't changed,
    #                one for PC's background, one for bold font for disassembled instruction.
    $t tag configure changed -background yellow
    $t tag configure same    -background [$t cget -background]
    $t tag configure pc      -background grey95
    $t tag configure instr   -font listbox_bold_font

    table $root $t 0,0 -fill y
    table configure $root c0 -resize none

    set title [treesim::get_path_name $GPRegs($root,widget) $GPRegs($root,treeidx) 1 ]
    wm title $root "$title: GPRs"

    GPRegs_Refresh $root 1

    return "GPRegs_Refresh"
}

######################################################################
#
# GPRegs_Refresh -- Refresh the contents of the status window
#
######################################################################
proc GPRegs_Refresh { root force_refresh args} {
    global GPRegs GPRegsVals

    set ngpr $GPRegs($root,ngpr)
    set cols $GPRegs($root,cols)
    set rows $GPRegs($root,rows)
    set cpucmd $GPRegs($root,cpucmd)
    set mcmcmd $GPRegs($root,mcmcmd)

    set ic [$cpucmd display instruction_count]

    set changed_tag "changed"
    set first_time 0
    if { [array names GPRegsVals $root,0 ] == "" } {
       set first_time 1
       set changed_tag "same"

       set GPRegs($root,msr) ""
       set GPRegs($root,cr)  ""
       set GPRegs($root,lr)  ""
       set GPRegs($root,ctr) ""
       set GPRegs($root,xer) ""

       for {set i 0} {$i < $ngpr} {incr i} {
           set GPRegsVals($root,$i) ""
       }
    }

    # leave if nothing has changed
    if { !$first_time &&
         !$force_refresh &&
         [string compare $GPRegs($root,ic) $ic] == 0} {
       return
    }
    set GPRegs($root,ic) $ic


    # Allow modifications to the text widget
    set t "$root.gprvals"
    $t configure -state normal

    # Delete all the existing text
    $t delete 1.0 end

    # Insert formatted MSR, with highlighted changes
    set newmsr [$cpucmd display spr msr]
    GPRegs_InsertMSR $t $newmsr $GPRegs($root,msr) same $changed_tag
    set GPRegs($root,msr) $newmsr

    # Insert formatted CR
    set newcr [$cpucmd display spr cr]
    GPRegs_InsertCR $t $newcr $GPRegs($root,cr) same $changed_tag
    set GPRegs($root,cr) $newcr

    # Insert PC and disassembled instruction
    $t insert end "\n";                   # skip a line
    set pc [$cpucmd display spr pc]
    GPRegs_InsertPC $t $pc $cpucmd $mcmcmd
    $t insert end "\n\n";          # skip a line

    # Insert LR, CTR, XER
    set lr [$cpucmd display spr lr]
    GPRegs_InsertReg $t "LR" $lr $GPRegs($root,lr) same $changed_tag
    set GPRegs($root,lr) $lr

    $t insert end " " {same}
    set ctr [$cpucmd display spr ctr]
    GPRegs_InsertReg $t "CTR" $ctr $GPRegs($root,ctr) same $changed_tag
    set GPRegs($root,ctr) $ctr
    $t insert end " " {same}

    set xer [$cpucmd display spr xer]
    GPRegs_InsertReg $t "XER" $xer $GPRegs($root,xer) same $changed_tag
    set GPRegs($root,xer) $xer
    $t insert end "\n\n"

    # Insert reg numbers and contents by row
    # Tag changed reg values so they'll display with a different background color
    for {set row 0} {$row < $rows} {incr row} {
       for {set col 0} {$col < $cols} {incr col} {
           set regnum [expr ($col * $rows) + $row]
           if {$regnum < $ngpr} {
              if {$regnum == 0} {
                  $t insert end "GPR 0 " {same}
              } else {
                  $t insert end [format "%5d " $regnum] {same}
              }
              # Fetch new values for the registers
              set newvals($regnum) [$cpucmd display gpr $regnum]
              if {[string range $newvals($regnum) 0 1] == "0x"} {
                  set newvals($regnum) [string range $newvals($regnum) 2 end]
              }
              if {[string compare $GPRegsVals($root,$regnum) $newvals($regnum)] != 0} {
            $t insert end $newvals($regnum) $changed_tag
              } else {
                  $t insert end $newvals($regnum) {same}
              }
              set GPRegsVals($root,$regnum) $newvals($regnum)
              $t insert end " " {same}
           }
       }
       $t insert end "\n"
    }

    # Disallow changes again
    $t configure -state disabled

}
