#!/usr/bin/bltwish

package require BLT
namespace import -force blt::*

##################################
# Generic Graph Zooming Support
# Start
##################################
proc ZoomSupport_Init {chart minimum_x minimum_y} {
    global zoomStack
    global MinimumX
    global MinimumY

    set MinimumX($chart) $minimum_x
    set MinimumY($chart) $minimum_y
    set zoomStack($chart) ""
    bind $chart <ButtonPress-1>   {SelectStart %W %x %y}
    bind $chart <B1-Motion>       {SelectMove  %W %x %y}
    bind $chart <ButtonRelease-1> {SelectEnd   %W %x %y}
    bind $chart <ButtonRelease-3> {ZoomOut     %W      }
}
proc ZoomIn {chart x0 y0 x1 y1 y2_0 y2_1} {
    global MinimumX
    global MinimumY

    if {($x0 == $x1) || ($y0 == $y1) || ($y2_0 == $y2_1)} {
	    # first and last points are the same
	    return
    }
    busy hold .
    update
    PushZoomStack $chart \
        [$chart xaxis cget -min] [$chart xaxis cget -max] \
	    [$chart yaxis cget -min] [$chart yaxis cget -max] \
	    [$chart y2axis cget -min] [$chart y2axis cget -max]

    if {$x0 > $x1} {
	    set temp $x0
	    set x0 $x1
	    set x1 $temp
    }
    if {$y0 > $y1} {
	    set temp $y0
	    set y0 $y1
	    set y1 $temp
    }
    if {$y2_0 > $y2_1} {
	    set temp $y2_0
	    set y2_0 $y2_1
	    set y2_1 $temp
    }

    #
    # Graphs can be setup to not allow to scale under some minimum for both X and Y
    #
    if {$x0 < $MinimumX($chart)} { set x0 $MinimumX($chart) }
    if {$x1 < $MinimumX($chart)} { set x1 $MinimumX($chart) }
    if {$y0 < $MinimumY($chart)} { set y0 $MinimumY($chart) }
    if {$y1 < $MinimumY($chart)} { set y1 $MinimumY($chart) }
    if {$y2_0 < $MinimumY($chart)} { set y2_0 $MinimumY($chart) }
    if {$y2_1 < $MinimumY($chart)} { set y2_1 $MinimumY($chart) }

	$chart xaxis configure -min $x0 -max $x1
	$chart yaxis configure -min $y0 -max $y1
	$chart y2axis configure -min $y2_0 -max $y2_1

    update
    busy release .
}
proc ZoomOut {chart} {
    global zoomStack

    if {![EmptyZoomStack $chart]} {
	    busy hold .
	    update
	    set level [llength $zoomStack($chart)]
	    ShowLevel $chart [expr $level - 1]
        # We pop from the zoom stack and execute the pop command which readjusts the chart back

        # We are going to ignore errors associated with logscale when zooming out
	    catch {eval [PopZoomStack $chart]} errors
        if { [string first logscale $errors] == -1 } {
            # If not logscale errors then present them
            #puts "Zoom out generated the following errors: $errors"
        }
	    busy release .
	    after 2000 "HideLevel $chart"
    }
}
proc ZoomOutComplete {chart} {
    while { ![EmptyZoomStack $chart] } {
        ZoomOut $chart
    }
}
proc GetCoords {win scrX scrY xVar yVar y2Var} {
    upvar $xVar x
    upvar $yVar y
    upvar $y2Var y2
    set coords [$win invtransform $scrX $scrY]
    set x [$win axis invtransform x $scrX]
    set y [$win axis invtransform y $scrY]
    set y2 [$win axis invtransform y2 $scrY]
}

proc SelectStart {win x y} {
    global x0 y0 y2_0
    GetCoords $win $x $y x0 y0 y2_0
    CreateRectangle $win
}

proc SelectMove {win x y} {
    global x0 y0 x1 y1 y2_1

    GetCoords $win $x $y x1 y1 y2_1
    DrawRectangle $win $x0 $y0 $x1 $y1
}

proc SelectEnd {win x y} {
    global x0 y0 y1 y2_0 y2_1
    GetCoords $win $x $y x1 y1 y2_1
    DestroyRectangle $win
    ZoomIn $win $x0 $y0 $x1 $y1 $y2_0 $y2_1
}

proc CreateRectangle {win} {
    global zoomStack
    $win marker create line -name ZoomRegion -dashes {4 2}
    # Display the level of the *next* zoom.
    ShowLevel $win [expr [llength $zoomStack($win)] + 1]
}

proc DrawRectangle {win x0 y0 x1 y1} {
    $win marker configure "ZoomRegion" \
	    -coords {$x0 $y0 $x1 $y0 $x1 $y1 $x0 $y1 $x0 $y0}
}

proc DestroyRectangle {win} {
    $win marker delete "ZoomRegion"
    HideLevel $win
}

proc EmptyZoomStack {chart} {
    global zoomStack
    return [expr {[llength $zoomStack($chart)] == 0}]
}

proc PushZoomStack {chart xmin xmax ymin ymax y2min y2max} {
    global zoomStack
    set fmt {
	    $chart xaxis configure -min "%s" -max "%s";
	    $chart yaxis configure -min "%s" -max "%s"
	    $chart y2axis configure -min "%s" -max "%s"
    }
    lappend zoomStack($chart) [format $fmt $xmin $xmax $ymin $ymax $y2min $y2max]
}

proc PopZoomStack {chart} {
    global zoomStack
    set cmd [lindex $zoomStack($chart) end]
    set zoomStack($chart) [lreplace $zoomStack($chart) end end]
    return $cmd
}

proc ShowLevel {chart level} {
    set text "Zoom #$level"

    if {[$chart marker exists "ZoomLevel"]} {
	    $chart marker configure "ZoomLevel" -text $text
    } else {
	    $chart marker create text -name "ZoomLevel" -text $text \
		    -coords {-Inf Inf} -anchor nw -bg {} -fg darkgreen
    }
    after 2000 HideLevel $chart
}

proc HideLevel {chart} {
	if { [winfo exists $chart] } {
        if {[$chart marker exists "ZoomLevel"]} {
            $chart marker delete "ZoomLevel"
        }
    }
}
##################################
# Generic Graph Zooming Support
# End
##################################
