#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  gui.tcl
#------------------------------------------------------------------------

namespace eval GUI {
   variable initialized   0
   variable updates
   set updates(started)   0
   set updates(completed) 0
}

#------------------------------------------------------------------------
#  shutdown
#     This is the function to call to shut down the gui.
#------------------------------------------------------------------------
proc GUI::shutdown {} {
    simstop
    set_pref win_geom/gmambo [ winfo geom . ]
    set_pref open_windows/latest [winmgr::get_open_window_paths ]
    PrefManager::save_file
    after 100 "exit"
}

#------------------------------------------------------------------------
# Setup default 'options' for widgets that do not specify otherwise.
#------------------------------------------------------------------------
proc GUI::set_default_options {} {
   option add *Listbox.background white
   option add *Entry.background white
   option add *background grey85
   option add *font default_font
   option add *labelfont default_font
   option add *Dialog.msg.wrapLength 8i
   . configure -background grey85
}

#------------------------------------------------------------------------
#  UpdateAll
#     Update all the widgets
#------------------------------------------------------------------------

proc GUI::UpdateAll { {force 0} } {
    variable updates

    if { [simdebug status UPDATE_GUI] == 1 } {
        puts "UpdateAll called with force = $force"
    }

    # There should be only one UpdateAll happening at once
    if { $updates(started) != $updates(completed) } {
        return;
    }
    incr updates(started)

    set update_error [catch {
        Controls::Refresh
        treesim::refresh .mambo.simlist
        winmgr::refresh_managed_windows $force
    } ]

    if {$update_error} {
       global errorInfo
       #puts "gui.tcl: Error trying to update windows"
       #puts "$errorInfo"
    }

    incr updates(completed)

    return ""
}

#------------------------------------------------------------------------
# GUI panel init function
#------------------------------------------------------------------------
proc GUI::init_widgets {} {

   # Setup main GUI panels
   Menubar::Init
   frame .mambo
   frame .mambo.simlist
   frame .mambo.controls

   set legend [frame .mambo.legend -relief groove -borderwidth 2]
   set run [label ${legend}.run -text "Running" -foreground darkgreen]
   set wait [label ${legend}.wait -text "Stalled" -foreground darkorange]
   set stop [label ${legend}.stop -text "Halted" -foreground darkred]
   grid $run $wait $stop

   set tree [treesim::init .mambo.simlist]

   Controls::Init .mambo.controls

   table .mambo \
       .mambo.simlist	0,0  -cspan 1  -fill both -anchor nw \
       .mambo.legend	1,0  -cspan 1  -fill both -anchor nw \
       .mambo.controls	0,1  -cspan 1  -fill x -anchor nw

   table configure .mambo c0 r0 r1 r2 r3 r4 -resize none
   table configure .mambo c0 r0 -resize expand

   pack .mambo -expand 1 -fill both

   # Restore main window position
   set geom [ get_pref win_geom/gmambo "650x400+200+200" ]
   wm geometry . $geom

   # Hook some shutdown sequences
   bind . <q> { GUI::shutdown }
   wm protocol . WM_DELETE_WINDOW "GUI::shutdown"

   UpdateAll

   # load any emitters specified in prefs file
   catch {Emitters::init}
}

#------------------------------------------------------------------------
#  Main GUI init function
#------------------------------------------------------------------------
proc GUI::init {} {
   global gvars
   variable initialized
   if {!$initialized} { 
       set initialized 1
       bootstrap
       PrefManager::bootstrap
       GUI::set_default_options
       Fonts::init
       set gvars(firstsim) [ lindex [ display machines ] 0 ]
       Control::init
       GUI::init_widgets
   }
}

#------------------------------------------------------------------------
# aliases
#------------------------------------------------------------------------
proc gui_init {} {
   GUI::init 
}

global first_UpdateAll
set first_UpdateAll 1
#------------------------------------------------------------------------
# This is called by the simulator's C code every 4 seconds while running.
#------------------------------------------------------------------------
proc UpdateAll { {force 0} } {
   #puts "UpdateAll $force"
   #puts "info level 1 = [info level 1]"
   if {$::GUI::initialized} { 
       GUI::UpdateAll $force
   } else {
       global first_UpdateAll
       if {$first_UpdateAll} { 
          set first_UpdateAll 0
          puts "GUI not initialized.  Execute tcl command 'gui_init'."
       }
   }
}
