#!/usr/bin/wish
#!wish

package require BLT
namespace import -force blt::*

#source ${commonlibdir}/_error_dialog.tcl
#source ${commonlibdir}/_select_file.tcl

set currentSim ""
##############################################################
#
#
##############################################################
proc HierSim_Refresh { win } {
    global currentSim HierSim

    set widget $win.box

    set simlist [display machines]
    # We need at least one simulator defined for now
    if { [llength $simlist] < 1 } {
        define machine apache mysim
        set simlist [display machines]
    }
    if {$HierSim(old)} {
        set hierpre ""
    } else {
        set hierpre "::$widget"
    }
    foreach s $simlist {
        set entries {}
        set simentry [${widget} find -full "$hierpre/$s"]
        if { [llength $simentry] < 1 } {
            # We need to add this simulator to the list
            set currentSim $s
            lappend entries / /$s
            foreach cpu [$s display cpulist] {
                set simentry [${widget} find -full "$hierpre/$s/$cpu"]
                if { [llength $simentry] < 1 } {
                    # We need to add this processor
                    lappend entries /$s/$cpu
                }
            }
            eval ${widget} insert end [lsort $entries]
        }
    }

    # Make Core entries call OpenCore when clicked
    foreach s $simlist {
        # Open folder that contain the machine entries
        if {$HierSim(old)} {
            ${widget} find -name "$s" -exec "%W open %n"
        } else {
            ${widget} find -name "$s" -exec "%W open %#"
        }
    }

    foreach s $simlist {
        # Now run through and color the processors as active (red) or not active (blue)
        # First color all the entry blue for inactive
        set cpulist [$s display cpulist]
        foreach cpu $cpulist {
            set entry [${widget} find -full "$hierpre/$s/$cpu"]
            if {$HierSim(old)} {
                ${widget} entry configure $entry -labelcolor blue
            } else {
                ${widget} entry configure $entry -foreground blue
            }
        }
        # Now color all the active entries red
        set activelist [$s display runlist]
        foreach cpu $activelist {
            set entry [${widget} find -full "$hierpre/$s/${cpu}"]
            if {$HierSim(old)} {
                ${widget} entry configure $entry -labelcolor red
            } else {
                ${widget} entry configure $entry -foreground red
            }
        }
    }

    # Update the font type if it has changed
    foreach entry [ $widget find ] {
        if {$HierSim(old)} {
            $widget entry configure $entry -labelfont default_font
        } else {
            $widget entry configure $entry -font default_font
        }
    }
}

proc HierSim_Init { win } {
    global currentSim HierSim blt_version

    set simlist [display machines]
    # We need at least one simulator defined for now
    if { [llength $simlist] < 1 } {
        define machine apache mysim
        set simlist [display machines]
    }
    set entries {}

    if {$blt_version == "2.4"} {
        set HierSim(old) 1
    } else {
        set HierSim(old) 0
    }

    # Create Hierbox
    if {$HierSim(old)} {
        hierbox  ${win}.box -separator "/" -autocreate true -scrollmode listbox -yscrollcommand "${win}.scroll set"
    } else {
        treeview ${win}.box -separator "/" -autocreate true -scrollmode listbox -yscrollcommand "${win}.scroll set"
    }
    scrollbar ${win}.scroll -command "${win}.box yview"

    $win.box entry configure root -label "Machines"

    table ${win} \
        ${win}.box        0,0  -fill both  \
        ${win}.scroll    0,1  -fill y
    table configure ${win} c* r* -resize none

    HierSim_Refresh $win

    return ${win}.box
}

#------------------------------------------------------------------
# Add Child entry into HierSim widget
#
# widget        - hiersim widget
# hierpath      - path of parent
# name          - name of new child
# cmd           - open command
#------------------------------------------------------------------
proc HierSim_AddChild { widget hierpath name opencmd} {

    set entry_index [ $widget insert end $hierpath/$name ]
    $widget entry configure $entry_index -opencommand $opencmd

    return $entry_index
}

#------------------------------------------------------------------
# Add entry into HierSim widget as a child of each sim
#
# name_and_opencmd_list is a list like { name opencmd name2 opencmd2 ... }
#
#------------------------------------------------------------------
proc HierSim_AddSimChildren { widget name_and_opencmd_list} {

    global HierSim

    set simlist [display machines]
    # We need at least one simulator defined for now
    if { [llength $simlist] < 1 } {
        define machine apache mysim
        set simlist [display machines]
    }

    if {$HierSim(old)} {
        set hierpre ""
    } else {
        set hierpre "::$widget"
    }
    set entries {}
    foreach s $simlist {
        foreach {name opencmd} $name_and_opencmd_list {
            set simentry [${widget} find -full "$hierpre/$s/$name"]
            if { [llength $simentry] < 1 } {
                set entry_index [$widget insert end "/$s/$name"]
                $widget entry configure $entry_index -opencommand $opencmd
            }
        }
    }
}


#------------------------------------------------------------------
# Add entry into HierSim widget as a child of each CPU
#
# name_and_opencmd_list is a list like { name opencmd name2 opencmd2 ... }
#
#------------------------------------------------------------------
proc HierSim_AddCPUChildren { widget cpuprefix name_and_opencmd_list} {
    global HierSim

    set simlist [display machines]
    # We need at least one simulator defined for now
    if { [llength $simlist] < 1 } {
        define machine apache mysim
        set simlist [display machines]
    }
    if {$HierSim(old)} {
        set hierpre ""
    } else {
        set hierpre "::$widget"
    }
    set entries {}
    set cpulen [string length $cpuprefix]
    incr cpulen -1
    foreach s $simlist {
        set cpulist [$s display cpulist]
        foreach cpu $cpulist {
            foreach {name opencmd} $name_and_opencmd_list {
               if { [string compare [string range $cpu 0 $cpulen] $cpuprefix] == 0 } {
                   set simentry [${widget} find -full "$hierpre/$s/$cpu/$name"]
                   if { [llength $simentry] < 1 } {
                       set entry_index [$widget insert end "/$s/$cpu/$name"]
                       $widget entry configure $entry_index -opencommand $opencmd
                   }
               }
            }
        }
    }
}

