#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003, 2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  hooks.tcl
#
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# Using hooks is very simple. To use a hook, you must first register
# an event ID and a callback function. After that, whenever someone
# calls "Hooks:callback $your_event_id" then the associated callback is run.
# Example:
#    ...the initial setup...
#    nspace1::callback_app_exiting {} { # the app is exiting, perform any last minute operations }
#    nspace2::callback_app_exiting {} { # the app is exiting, perform any last minute operations }
#    nspace3::callback_app_exiting {} { # the app is exiting, perform any last minute operations }
#    Hooks::register on_exit nspace1::callback_app_exiting
#    Hooks::register on_exit nspace2::callback_app_exiting
#    Hooks::register on_exit nspace3::callback_app_exiting
#    ...user presses 'Exit' button...
#    Hooks::callback on_exit
#
# In the above example, when the user presses the exit button, anyone who registered to be
# called back on the 'on_exit' event is called. In the above example, each of the three
# namespace 'callback_app_exiting' functions are called, giving each of the namespaces a
# chance to clean up (save prefs, etc) before the application closes.
#------------------------------------------------------------------------

namespace eval Hooks {

   variable version        "1.0"
   variable initialized    0
   variable callbacks
}

#------------------------------------------------------------------------
#  Hooks::register
#------------------------------------------------------------------------
proc Hooks::register {event callback_proc } {
    
    variable callbacks
    
    #puts "Hooks::register called: $callback_proc registered for event $event"
    if {[info exists callbacks($event)] == 0 || 
        [lsearch -exact $callbacks($event) $callback_proc] == -1 } {
        lappend callbacks($event) $callback_proc
    }
}

#------------------------------------------------------------------------
#  Hooks::callback
#------------------------------------------------------------------------
proc Hooks::callback {event args} {
    
    variable callbacks
    
    #puts "Hooks:callback called for event $event"
    if {[info exists callbacks($event)]} {
        foreach cbproc $callbacks($event) {
            eval $cbproc $args
        }
    } 
}
