#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  membrk.tcl
#
#------------------------------------------------------------------------

#--------------------------------------------------------------------------
# Memory_breakpoint_dlg
#--------------------------------------------------------------------------
proc Memory_breakpoint_dlg { sim type thrnum startaddr {addr_xlat 1} } {
   set idx $sim-$type-$thrnum
   set membrkdlg .membrkdlg_$idx
   global membrkInfo

   set title "Breakpoint on Memory Access"

   # Create the dialog
   array set dlgInfo "top $membrkdlg title {$title} geom none"
   if { [ Dialog::create dlgInfo -borderwidth 5 ] } then {

      set topf [frame $membrkdlg.topf]

      set membrk_tab [frame $topf.enable_membrk -relief sunken -bd 2]
      label $membrk_tab.lbl -text "Enable Memory Access Breakpoints"
      set control [frame $membrk_tab.control]
      set readchk  [checkbutton $control.readchk -text "Read Access" -anchor w -variable membrkInfo(readchk_$idx)]
      set writechk [checkbutton $control.writechk -text "Write Access" -anchor w -variable membrkInfo(writechk_$idx) ]
      if {$type == "system"} {
          set xlate [checkbutton $control.xlate -text "Use Address Translation" -anchor w -variable membrkInfo(xlate_$idx) ]
      }

      frame $membrk_tab.min
      label $membrk_tab.min.min_label -text "Min Address: " -width 13
      entry $membrk_tab.min.entry -width 10 -relief sunken -textvariable membrkInfo(min_bound_$idx)

      frame $membrk_tab.max
      label $membrk_tab.max.max_label -text "Max Address: " -width 13
      entry $membrk_tab.max.entry -width 10 -relief sunken -textvariable membrkInfo(max_bound_$idx)

      if {$type == "system"} {
          $membrk_tab.min.entry configure -width 18
          $membrk_tab.max.entry configure -width 18
      }

      set membrkInfo(readchk_$idx)    1

      pack $membrk_tab.lbl $readchk $writechk
      if {$type == "system"} {
          pack $xlate
      }
      pack $control -pady 3 -padx 10 -anchor w
      pack $membrk_tab.min.min_label $membrk_tab.min.entry -side left
      pack $membrk_tab.max.max_label $membrk_tab.max.entry -side left
      pack $membrk_tab.min -padx 10 -anchor w
      pack $membrk_tab.max -padx 10 -pady 5 -anchor w

      frame $topf.buttons
      button $topf.buttons.set     -text "   Set  " -command "set membrkInfo(trigger_$idx) set_pushed " -anchor w
      button $topf.buttons.clear   -text "  Clear "  -command "set membrkInfo(trigger_$idx) clear_pushed "
      button $topf.buttons.cancel -text "  Cancel " -command "set membrkInfo(trigger_$idx) cancel_pushed"
      pack $topf.buttons.set $topf.buttons.clear $topf.buttons.cancel  -pady 5 -padx 10 -side left -fill x

      pack $topf.enable_membrk $topf.buttons -side top -padx 10 -pady 10
      pack $topf -fill both -pady 5 -padx 5 -side left -side top

      bind $topf <Return> "$topf.buttons.set invoke"
      bind $topf <Escape> "$topf.buttons.cancel invoke"

   }

   set membrkInfo(xlate_$idx)      $addr_xlat
   set membrkInfo(min_bound_$idx)  $startaddr
   if {$type == "system"} {
      set membrkInfo(max_bound_$idx)  [format64 "0x%016llX" [add64 $startaddr 15]]
   } else {
      set membrkInfo(max_bound_$idx)  [format64 "0x%08X" [add64 $startaddr 15]]
   }

   # reset the trigger variable
   set membrkInfo(trigger_$idx) "reset"

   # Save settings in case user cancels later
   array set save_membrkInfo [array get membrkInfo]


   # Wait for user to press the Ok or Cancel button, which will alter the trigger variable.
   Dialog::wait $membrkdlg membrkInfo(trigger_$idx)

   # Close the dialog box
   Dialog::dismiss $membrkdlg

   if { $membrkInfo(trigger_$idx) == "set_pushed" } then {
       Memory_breakpoint_func $sim $type $thrnum
   } elseif { $membrkInfo(trigger_$idx) == "clear_pushed" } then {
       Memory_breakpoint_func $sim $type $thrnum "clear"
   } else {
       # restore original settings
       array set membrkInfo [array get save_membrkInfo]
   }
}

#--------------------------------------------------------------------------
# Memory_breakpoint_func
#--------------------------------------------------------------------------
proc Memory_breakpoint_func { sim type thrnum {clear ""} } {
    global membrkInfo
    set idx $sim-$type-$thrnum
    set spu $thrnum

    set addr1 $membrkInfo(min_bound_$idx)
    set addr2 $membrkInfo(max_bound_$idx)

    set rw ""
    if {$membrkInfo(readchk_$idx) == 1 } {
        append rw "r"
    }
    if {$membrkInfo(writechk_$idx) == 1 } {
        append rw "w"
    }

    if {$clear != ""} {
        if {$rw == ""} {
          set rw "rw"
        }
        if {$type == "system"} {
            $sim trigger clear memory system   $rw $addr1 $addr2
        } else {
            $sim trigger clear memory spu $spu $rw $addr1 $addr2
        }
    } else {
        if {$type == "system"} {
            set xlate $membrkInfo(xlate_$idx)
            $sim trigger set memory system   $rw $addr1 $addr2 $xlate "mem_break_point $sim MemAccess_at_$addr1-$addr2"
        } else {
            $sim trigger set memory spu $spu $rw $addr1 $addr2 "spu_mem_break_point $sim $spu MemAccess_at_$addr1-$addr2"
        }
    }
}

#
# args look like this : { address 0x12345 cpu 0:0 type R }
#
proc mem_break_point { sim comment args } {

    eval array set bpinfo $args
	set pc [$sim cpu $bpinfo(cpu) display spr pc]
	puts "$bpinfo(cpu) PC=$pc: memory trigger for Address=$bpinfo(address) ($comment)"
	simstop
}

proc spu_mem_break_point { sim spu comment args } {
	set pc [$sim spu $spu display pc]
	puts "SPU $spu $pc: memory break point ($comment)"
	simstop
}
