#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003,2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  memmap.tcl
#
#
#------------------------------------------------------------------------

######################################################################
#
# MemMap_Init -- Initalize the status window
#
######################################################################
proc MemMap_Init {root _upstate args} {
	global MemMapSim 
    global MemMap

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate MemMap $root      ;# create MemMap($root,x)
    set sim $MemMap($root,sim)

	set MemMapSim($root) $sim

    label $root.memmapLabel -text "Memory Map"

    ;#####  Memory Regions   ####################################################################
    set regions $root.regions
    frame $regions -relief sunken -bd 2
    label $regions.label -text "Regions"
    text $regions.datahdr -relief groove -font textbox_font -height 1 -wrap none -width 70
	text $regions.data -wrap none -width 70 -height 12 -yscrollcommand "$regions.data_sy set" -font textbox_font
	scrollbar $regions.data_sy -orient vert -command "$regions.data yview"

    table $regions \
        $regions.label      0,0 -cspan 4 -fill x \
        $regions.datahdr    1,0 -cspan 5 -fill x -anchor w\
        $regions.data       2,0 -cspan 4 -fill both \
        $regions.data_sy    2,4 -cspan 1 -fill y

    table configure $regions c* r* -resize none
    table configure $regions r2 c0  -resize both

    button $root.close -text "Close" -command "destroy $root"

	table $root \
		$root.memmapLabel  0,0 -cspan 4 -fill x \
		$regions           1,0 -cspan 4 -fill both \
		$root.close        2,3 -fill x

	table configure $root c* r* -resize none
	table configure $root r1 c0 -resize both

    set title [treesim::get_path_name $MemMap($root,widget) $MemMap($root,treeidx) 1 ] 
    wm title $root "$title: Memory Map" 

	MemMap_Refresh $root 1
	return "MemMap_Refresh"
}

######################################################################
#
# MemMap_Refresh -- Refresh the contents of the status window
#
######################################################################
proc MemMap_Refresh { root force args } {
	global MemMapSim SLBValid SLBHeader

	if { $force } {
        #set MemMapStart($root) "blah"
    }

	set sim $MemMapSim($root)

    # get the slb data from the simulator
    set regions $root.regions
	set mmap [$sim display memorymap listformat]
	#set mmap [split $mmap "\n"]

    # put slb data into text block
    $regions.datahdr configure -state normal
    $regions.datahdr delete 1.0 end
    set str [format "%20s %20s %20s" NAME START END]
    $regions.datahdr insert end $str
    $regions.datahdr configure -state disabled
    $regions.data delete 1.0 end
    foreach region $mmap {
        array set field $region
        set str [format "%20s %20s %20s" $field(NAME) $field(START) $field(END)]
        #$regions.data insert end "$region\n"
        $regions.data insert end "$str\n"
    }
    $regions.data delete insert-1c end       ;# delete last newline char
}
