#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  options.tcl
#
#
#------------------------------------------------------------------------


namespace eval Options {

   global env gvars
   variable version        "1.0"
   variable _objectcount   0
   variable data
   variable frozen         0

   source $gvars(tcl_common_dir)/gui/common_oo.tcl
   namespace import OOCommon::*
   namespace import ::blt::*
}

#---------------------------------------------------------------------------
#  Options::create
#---------------------------------------------------------------------------
proc Options::create { widget parent_namespace } {

    variable _objectcount
    variable data
    global gvars

    # Check for duplicate widgets
    if { [ winfo exists $widget.nb ] } then {
        error "Cannot create $widget"
    }

    set top_widget [ winfo toplevel $widget ]

    # increment instance count and set default instance values
    incr _objectcount
    set data($widget-instance)       $_objectcount
    setvar $widget widget            $widget
    setvar $widget top_widget        $top_widget
    setvar $widget parent_namespace  $parent_namespace
    setvar $widget tablist           [ list ]

    setglobal last_exit_mode "UNKNOWN"

    # Intercept window close and call the _on_close handler
    wm protocol $top_widget WM_DELETE_WINDOW "$widget _on_close"

    set nb [ tabset $widget.nb -font system_font ]
    pack $nb -side top -fill both -expand yes
    setvar $widget nb $nb

    frame $widget.buttonframe
      button $widget.buttonframe.ok     -text "   OK   " -command "Options::_cmd_ok     $widget" -height 1 -font system_font
      button $widget.buttonframe.cancel -text " Cancel " -command "Options::_cmd_cancel $widget" -height 1 -font system_font
      pack $widget.buttonframe.cancel $widget.buttonframe.ok -side right -padx 5 -pady 5
      pack $widget.buttonframe -side bottom -fill x -expand no

    bind $top_widget <Escape> "Options::_cmd_cancel $widget"
    bind $top_widget <Return> "Options::_cmd_ok     $widget"

    # Establish function that allows widget calls to get back into these functions
    rename $widget ::$widget:cmd
    proc ::$widget { command args } "return \[eval Options::\$command $widget \$args\]"

    set ::op $widget

    return $widget
}

#---------------------------------------------------------------------------
#  Options::_cmd_ok
#---------------------------------------------------------------------------
proc Options::_cmd_ok { widget } {

    variable frozen

    if { !$frozen } then {

        set pns     [ getvar $widget parent_namespace ]
        set tablist [ getvar $widget tablist ]

        # Call each tab's -on_ok handler
        foreach tab $tablist {
          eval "[subst $pns]::-on_ok_[subst $tab]"
        }

        _on_close $widget

        setglobal last_exit_mode "OK"
    }
}

#---------------------------------------------------------------------------
#  Options::_cmd_cancel
#---------------------------------------------------------------------------
proc Options::_cmd_cancel { widget } {

    variable frozen

    if { !$frozen } then {
        set pns     [ getvar $widget parent_namespace ]
        set tablist [ getvar $widget tablist ]


        # Call each tab's -on_ok handler
        foreach tab $tablist {
          eval "[subst $pns]::-on_cancel_[subst $tab]"
        }

        _on_close $widget

        setglobal last_exit_mode "CANCEL"
    }
}

#------------------------------------------------------------------------
#  Options::_on_close
#------------------------------------------------------------------------
proc Options::_on_close { widget } {
   set pns [ getvar $widget parent_namespace ]
   eval "[subst $pns]::-on_close [ getvar $widget top_widget ]"
}

#---------------------------------------------------------------------------
#  Options::insert
#---------------------------------------------------------------------------
proc Options::insert { widget index tab args } {

    global gvars

    set pns     [ getvar $widget parent_namespace ]
    set nb      [ getvar $widget nb ]
    set tablist [ getvar $widget tablist ]

    # Append new tab to the tablist
    lappend tablist $tab
    setvar $widget tablist $tablist

    # Setup appropriate callbacks for each tab
    set callbacks     "-createcmd  [subst $pns]::-createtab_$tab"
    #append callbacks  " -raisecmd  [subst $pns]::-raisetab_$tab"
    #append callbacks  " -leavecmd  [subst $pns]::-leavetab_$tab"

    # Add the tab
    set tabf [ frame $nb.[ $nb size ] ]
    [ subst $pns]::-createtab_$tab $tabf
    eval $nb insert $index $tab -window $tabf -font system_font -fill both $args
}

#---------------------------------------------------------------------------
#  Options::raise
#---------------------------------------------------------------------------
proc Options::raise { widget tab args } {
    global gvars
    set nb [ getvar $widget nb ]
    eval $nb activate [ $nb index -name $tab ]
}

#---------------------------------------------------------------------------
#  Options::get_last_exit_mode
#     In some cases we may need to see if a user pressed cancel to close
#     the option dialog. This is a simple method so we can peek at how the
#     user closed the option dialog box last.
#
#  Parms:
#     wait - if set to 1, perform a vwait on the last_exit_mode variable
#---------------------------------------------------------------------------
proc Options::get_last_exit_mode { { wait 0 } } {

    if { $wait } then {
        vwait [ getglobalname last_exit_mode ]
    }
    return [ getglobal last_exit_mode ]
}

#---------------------------------------------------------------------------
#  Options::freeze
#---------------------------------------------------------------------------
proc Options::freeze { widget } {

    variable frozen

    $widget.buttonframe.ok configure -state disabled
    $widget.buttonframe.cancel configure -state disabled
    set frozen 1
}

#---------------------------------------------------------------------------
#  Options::thaw
#---------------------------------------------------------------------------
proc Options::thaw { widget } {

    variable frozen

    $widget.buttonframe.ok configure -state normal
    $widget.buttonframe.cancel configure -state normal
    set frozen 0
}


#------------------------------------------------------------------------
#  Test functions
#------------------------------------------------------------------------

