#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  pci_utils.tcl
#
#
#------------------------------------------------------------------------
namespace eval pci {

proc find_device { dev_name { sim mysim } } {
    set devices [ $sim pci devices ]
    foreach device $devices {
        if { [lindex $device 0 ] == $dev_name } then {
            return $device
        }
        else {
            error "Device not found, device= $dev_name"
        }

    }
}

proc format_header { header } {

    set result ""
    for { set dword_num 0 } { $dword_num < 16 } { incr dword_num } {
        for { set i 0 } { $i < 4 } { incr i } {
            set elem [ lindex $header [ expr 4*$dword_num + $i ] ]
            set result $result[format "%02x " $elem]
        }
        puts "[format "%02d" $dword_num]: $result"
        set result ""
    }
    return $result
}

proc config_header { dev_name { sim mysim } } {
    set device [ find_device $dev_name ]
    set hdr [ $sim pci config_header [ lindex $device 0 ] [ lindex $device 1 ] [ lindex $device 2 ] [lindex $device 3 ] ]
    return $hdr
}

proc device_unit_address { info } {
    set addr "[format "%x" [ lindex $info 2 ] ]"
    set fn [ lindex $info 3 ]
    if { $fn != 0 } {
        set addr "{$addr},$fn"
    }
    return $addr
}

proc device_reg { info } {
    set bus [ lindex $info 1 ]
    set dev [ lindex $info 2 ]
    set fn  [ lindex $info 3 ]
    set reg [ expr ( $dev << 3 ) + $fn ]
    return [ list $reg 0 0 0 0 ]
}

}
