#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  treesim.tcl
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
#
# PPCCore_Init -- Initalize the status window
#
#------------------------------------------------------------------------
# Assume no FP, VMX registers
set PPCCoreNumFP  0
set PPCCoreNumVMX 0

proc PPCCore_Init {root _upstate args} {
    global PPCCoreSPRList PPCCoreSim PPCCoreCPU PPCCoreIC
    global PPCCore PPCCoreNumFP PPCCoreNumVMX

    upvar $_upstate  initstate
    TclUtils::copyarray initstate PPCCore $root      ;# create PPCCore($root,x)
    set sim $PPCCore($root,sim)
    set cpu $PPCCore($root,cpunum)
    set cpucmd $PPCCore($root,cpucmd)

#    # Get the instruction count; use it to attempt to detect redundant refresh calls
    set ic  [$cpucmd display cycles]

    # Tuck this data away so the Refresh can use it
    set PPCCoreSim($root) $sim
    set PPCCoreCPU($root) $cpu
    set PPCCoreIC($root)  $ic

    set PPCCoreSPRList($root) [lsort [$cpucmd display spr list]]

    # GPR Registers
    listbox $root.gprnames -width 8 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.gprSB $root.gprnames $root.gprdata"
    for { set i 0 } { $i < 32 } { incr i } {
        $root.gprnames insert end "GPR$i"
    }
    listbox $root.gprdata -width 20 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.gprSB $root.gprnames $root.gprdata"
    for { set i 0 } { $i < 32 } { incr i } {
        $root.gprdata insert end "[$cpucmd display gpr $i]"
    }
    scrollbar $root.gprSB -orient vert -command "TclUtils::ScrollBoth $root.gprnames $root.gprdata"
    bind $root.gprdata <Double-ButtonRelease-1> "PPC_Core_Show_Memory $root"

    #
    # Add FLOATING_POINT if appropriate
    #
    if { [ Config::is_defined CONFIG_FLOATING_POINT ] } {
        # FPR Registers
        set PPCCoreNumFP 32
        listbox $root.fprnames -width 8 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.fprSB $root.fprnames $root.fprdata"
        for { set i 0 } { $i < 32 } { incr i } {
            $root.fprnames insert end "FPR$i"
        }
        listbox $root.fprdata -width 20 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.fprSB $root.fprnames $root.fprdata"
        for { set i 0 } { $i < 32 } { incr i } {
            $root.fprdata insert end "[$cpucmd display fpr $i]"
        }
        scrollbar $root.fprSB -orient vert -command "TclUtils::ScrollBoth $root.fprnames $root.fprdata"
    }

    # SPR Registers
    listbox $root.sprnames -width 8 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.sprSB $root.sprnames $root.sprdata"
    foreach s $PPCCoreSPRList($root) {
        $root.sprnames insert end "$s"
    }
    listbox $root.sprdata -width 20 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.sprSB $root.sprnames $root.sprdata"
    foreach s $PPCCoreSPRList($root) {
        $root.sprdata insert end "[$cpucmd display spr $s]"
    }
    scrollbar $root.sprSB -orient vert -command "TclUtils::ScrollBoth $root.sprnames $root.sprdata"

    #
    # Add VMX if appropriate
    #

    if { [ Config::is_defined CONFIG_VMX32 ] } {
        set PPCCoreNumVMX 32
    }
    if { [ Config::is_defined CONFIG_VMX128 ] } {
        set PPCCoreNumVMX 128
    }
    if { $PPCCoreNumVMX != 0 } {
        # VMX Registers

        # Create the register list boxes
        listbox $root.vmxnames -width 8 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.vmxSB $root.vmxnames $root.vmxdata"
        for { set i 0 } { $i < $PPCCoreNumVMX } { incr i } {
            $root.vmxnames insert end "VMXR$i"
        }
        listbox $root.vmxdata -width 40 -height 32 -font listbox_font -yscrollcommand "TclUtils::ScrollbarSet $root.vmxSB $root.vmxnames $root.vmxdata"
        for { set i 0 } { $i < $PPCCoreNumVMX } { incr i } {
            $root.vmxdata insert end "[$cpucmd display vmxr $i]"
        }
        bind $root.vmxdata <Double-ButtonRelease-1> "VMX_ValueExpand $root $sim $cpu $cpucmd \[$root.vmxdata curselection\] 1"
        scrollbar $root.vmxSB -orient vert -command "TclUtils::ScrollBoth $root.vmxnames $root.vmxdata"
    }

    set row 0
    table $root $root.gprnames      0,$row -fill y
    table configure $root c$row -resize none
    table $root $root.gprdata       0,[incr row] -fill both
    table configure $root c$row -resize expand
    table $root $root.gprSB         0,[incr row] -fill y
    table configure $root c$row -resize none

    if { $PPCCoreNumFP > 0 } {
        table $root $root.fprnames      0,[incr row] -fill y
        table configure $root c$row -resize none
        table $root $root.fprdata       0,[incr row] -fill both
        table configure $root c$row -resize expand
        table $root $root.fprSB         0,[incr row] -fill y
        table configure $root c$row -resize none
    }
    if { $PPCCoreNumVMX > 0 } {
        table $root $root.vmxnames      0,[incr row] -fill y
        table configure $root c$row -resize none
        table $root $root.vmxdata       0,[incr row] -fill both
        table configure $root c$row -resize expand
        table $root $root.vmxSB         0,[incr row] -fill y
        table configure $root c$row -resize none
    }

    table $root $root.sprnames      0,[incr row] -fill y
    table configure $root c$row -resize none
    table $root $root.sprdata       0,[incr row] -fill both
    table configure $root c$row -resize expand
    table $root $root.sprSB         0,[incr row] -fill y
    table configure $root c$row -resize none

    set title [treesim::get_path_name $PPCCore($root,widget) $PPCCore($root,treeidx) 1 ]
    wm title $root "$title: Core"

    return "PPCCore_Refresh"
}

######################################################################
#
# PPCCore_Refresh -- Refresh the contents of the status window
#
######################################################################
proc PPCCore_Refresh { root args } {
    global PPCCoreSPRList PPCCoreSim PPCCoreCPU PPCCoreIC
    global PPCCore PPCCoreNumFP PPCCoreNumVMX

    set sim $PPCCoreSim($root)
    set cpu $PPCCoreCPU($root)
    set oic $PPCCoreIC($root)
    set cpucmd $PPCCore($root,cpucmd)

    # If cycle count hasn't changed, assume we've been called twice (or thrice, or ...)
    set ic [$cpucmd display cycles]
    if {[string compare $ic $oic] == 0} {
        return
    }
    set PPCCoreIC($root) $ic

    #######################################################################
    # Mark changed values with background color change

    # GPRs
    for { set i 0 } { $i < 32 } { incr i } {
        if { [$cpucmd display gpr $i] != [$root.gprdata get $i] } {
            $root.gprnames itemconfigure $i -bg yellow
        } else {
            $root.gprnames itemconfigure $i -bg lightgrey
        }
    }
    # FPRs
    for { set i 0 } { $i < $PPCCoreNumFP } { incr i } {
        if { [$cpucmd display fpr $i] != [$root.fprdata get $i] } {
            $root.fprnames itemconfigure $i -bg yellow
        } else {
            $root.fprnames itemconfigure $i -bg lightgrey
        }
    }
    # SPRs
    set i 0
    foreach s $PPCCoreSPRList($root) {
        if { [$cpucmd display spr $s] != [$root.sprdata get $i] } {
            $root.sprnames itemconfigure $i -bg yellow
        } else {
            $root.sprnames itemconfigure $i -bg lightgrey
        }

        # Always use almost-white as PC's background to make it easy to find.
        # Don't mark it differently when it changes, because it changes
        #   on every instruction.
        if { [string compare $s "pc"] == 0 } {
            $root.sprnames itemconfigure $i -bg grey95
        }
        incr i
    }
    # VMXRs
    if { [ Config::is_defined CONFIG_VMX ] } {
        for { set i 0 } { $i < $PPCCoreNumVMX } { incr i } {
            if { [$cpucmd display vmxr $i] != [$root.vmxdata get $i] } {
                $root.vmxnames itemconfigure $i -bg yellow
            } else {
                $root.vmxnames itemconfigure $i -bg lightgrey
            }
        }
    }

    #######################################################################
    # Update the values

    # Update the GPR registers
    $root.gprdata delete 0 end
    for { set i 0 } { $i < 32 } { incr i } {
        $root.gprdata insert end "[$cpucmd display gpr $i]"
        $root.gprdata itemconfigure $i -background \
            [$root.gprnames itemcget $i -background]
    }

    if { [ Config::is_defined CONFIG_FLOATING_POINT ] } {
        # Update the FPR registers
        $root.fprdata delete 0 end
        for { set i 0 } { $i < $PPCCoreNumFP } { incr i } {
            $root.fprdata insert end "[$cpucmd display fpr $i]"
            $root.fprdata itemconfigure $i -background \
                [$root.fprnames itemcget $i -background]
        }
    }

    # Update the SPR registers
    $root.sprdata delete 0 end
    set i 0
    foreach s $PPCCoreSPRList($root) {
        $root.sprdata insert end "[$cpucmd display spr $s]"
        $root.sprdata itemconfigure $i -background \
            [$root.sprnames itemcget $i -background]
        incr i
    }

    # Update the VMXR registers
    if { [ Config::is_defined CONFIG_VMX ] } {
        $root.vmxdata delete 0 end
        for { set i 0 } { $i < $PPCCoreNumVMX } { incr i } {
            $root.vmxdata insert end "[$cpucmd display vmxr $i]"
            $root.vmxdata itemconfigure $i -background \
                [$root.vmxnames itemcget $i -background]
        }
    }

}
#------------------------------------------------------------------------
# This will create a window containing the contents of a vmx register
#------------------------------------------------------------------------
proc VMX_ValueExpand { root sim cpu cpucmd vmxr raise } {

    set val [$cpucmd display vmxr $vmxr]

    set int0 [string range $val 2 9]
    set int1 [string range $val 10 17]
    set int2 [string range $val 18 25]
    set int3 [string range $val 26 35]

    #puts "display $val = 0x${int0} 0x${int1} 0x${int2} 0x${int3}"

    set title "[wm title $root] VMXR:$vmxr"
    set winname .[ TclUtils::safename valexpand_$title ]

    # If the window exists, just raise it
    if { [ValueExpand_Add $winname "VMX_ValueExpand $root $sim $cpu $cpucmd $vmxr 0"] == 0 } {
        # Already exists
        if { $raise == 1 } {
            raise $winname
        }
        ValueExpand_Refresh $winname "0x$int0 0x$int1 0x$int2 0x$int3"
        return
    }

    toplevel $winname
    ValueExpand_Init $winname $title "0x$int0 0x$int1 0x$int2 0x$int3"
}

proc PPC_Core_Show_Memory {root} {
    global PPCCore

    set cpucmd $PPCCore($root,cpucmd)
    set mcmcmd $PPCCore($root,mcmcmd)
    set cpu $PPCCore($root,cpunum)

    set addr [$cpucmd display gpr [$root.gprdata curselection] ]

    set sim $PPCCore($root,sim)
    SystemMemory_Open $sim $mcmcmd $addr PPC$cpu
}
