#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2007
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  ppcstack.tcl
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
#
# PPCStack_Init -- Initalize the stack window
#
#------------------------------------------------------------------------
proc PPCStack_Init {root _upstate args} {
    global PPCStack

    upvar $_upstate  initstate
    TclUtils::copyarray initstate PPCStack $root      ;# create PPCStack($root,x)

    set title [treesim::get_path_name $PPCStack($root,widget) $PPCStack($root,treeidx) 1 ] 
    wm title $root "$title: Stack Trace" 

    text $root.stack -font textbox_font -width 60 -height 20
    pack $root.stack -expand 1 -fill both

    PPCStack_Refresh $root

    return "PPCStack_Refresh"
}

#------------------------------------------------------------------------
#
# PPCStack_Refresh -- Refresh the contents of the stack window
#
#------------------------------------------------------------------------
proc PPCStack_Refresh { root args } {
    global PPCStack 

    $root.stack delete 1.0 end
    set sim $PPCStack($root,sim)
    set cpu $PPCStack($root,cpunum)

    set stack_trace [get_stack_trace $cpu $sim]

    for {set idx 0} {$idx < [llength $stack_trace]} {incr idx} {

        array set frame [lindex $stack_trace $idx]

        $root.stack insert end "\#$idx "

        $root.stack tag delete addr_tag$idx 
        $root.stack insert end "$frame(addr)" addr_tag$idx
        $root.stack tag bind addr_tag$idx <Double-Button-1> "PPCStack_Show_Memory $root $frame(addr)"

        $root.stack insert end " in $frame(label) at $frame(line)\n"
    }
}


proc PPCStack_Show_Memory {root addr} {
    global PPCStack

    set sim $PPCStack($root,sim)
    set cpu $PPCStack($root,cpunum)
    set mcmcmd $PPCStack($root,mcmcmd)

    SystemMemory_Open $sim $mcmcmd $addr PPC$cpu
}
