#!/usr/bin/bltwish
#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  process_details.tcl
#------------------------------------------------------------------------
proc Compute_ProcessDetail_Tag {machine process file ea} {
    set tag "processdetail_${machine}_${process}_${file}_${ea}"

    regsub -all {[/-]} $tag {_} tag

    return $tag
}

package require BLT
namespace import -force blt::*

proc UpdateFileGraph { machine pid file_index base_chart_name } {

    global  FileGraphData
    set FileGraphTag "${machine}_${pid}_${file_index}_filedata"
    set FileGraphData($FileGraphTag) [$machine processtree display tcl_stats proc_file_info $pid $file_index]

    array set FileGraphInfo $FileGraphData($FileGraphTag)

    set inst_chart ${base_chart_name}_inst
    set cache_chart ${base_chart_name}_cache

    # Create the Vectors
    foreach column $FileGraphInfo(columns) {
        set vecname V_${FileGraphTag}_${column}
        eval "global $vecname"

        # If the vector already exists, destroy it
        if { [llength [vector names ::${vecname}]] > 0 } {
            eval "vector destroy ${vecname}"
        }
        eval "vector create ${vecname}"
        eval "${vecname} set [list $FileGraphInfo($column)]"
    }

    # First delete all existing elements
    foreach CHART [list $inst_chart $cache_chart] {
        if { [llength [$CHART element names]] > 0 } {

            foreach el [$CHART element names] {
                $CHART element delete $el
            }
        }
    }

    foreach column $FileGraphInfo(columns) {
        if { ([string compare $column "ADDR"] != 0) &&
             ([string compare $column "INST"] != 0) &&
             ([string compare $column "INST_COUNT"] != 0) } {

            set xvals V_${FileGraphTag}_ADDR
            set yvals V_${FileGraphTag}_${column}

            if { [string first "CACHE" $column] == -1 } {
                set CHART $inst_chart
            } else {
                set CHART $cache_chart
            }
            # Cache Graph
            if { [llength [$CHART pen names $column]] == 0 } {
                $CHART pen create $column -foreground [GraphColor ${FileGraphTag}] -relief flat
            }
            ${CHART} element create $column -xdata $xvals -ydata $yvals -pen $column

        }
    }
}

proc ChangeGraphedData_File {chart machine tag} {

    global FileGraphColumns

    set graph_list ""

    foreach column $FileGraphColumns($tag) {
        eval "global ${tag}_${column}_toggle"
        eval "set tog_state \$${tag}_${column}_toggle"
        if { $tog_state == 1 } {
            lappend graph_list $column
        }
    }
    $chart element show $graph_list
}

proc ZoomGraphsOutComplete { chart1 chart2 } {

    foreach ch [list $chart1 $chart2] {
        ZoomOutComplete $ch
    }
}
proc ZoomGraphsOnFunction { chart1 chart2 } {
    global Func_Marker
    set coords [$chart1 marker cget $Func_Marker($chart1) -coords]
    set ymin [lindex [$chart1 axis limits y] 0]
    set ymax [lindex [$chart1 axis limits y] 1]
    set y2min [lindex [$chart1 axis limits y2] 0]
    set y2max [lindex [$chart1 axis limits y2] 1]
    eval "ZoomIn $chart1 [lindex $coords 0] $ymin [lindex $coords 4] $ymax $y2min $y2max"
    set ymin [lindex [$chart2 axis limits y] 0]
    set ymax [lindex [$chart2 axis limits y] 1]
    set y2min [lindex [$chart2 axis limits y2] 0]
    set y2max [lindex [$chart2 axis limits y2] 1]
    eval "ZoomIn $chart2 [lindex $coords 0] $ymin [lindex $coords 4] $ymax $y2min $y2max"
    #set Func_Marker(${ch}) [${ch} marker create polygon -under true -coords {0 0 0 0 0 Inf 0 Inf} -fill yellow -outline black]
}
proc scrollbothx {text1 text2 xview1 xview2} {
    $text1 xview $xview1 $xview2
    $text2 xview $xview1 $xview2
}
proc scrollbothy {text1 text2 yview1 yview2} {
    $text1 yview $yview1 $yview2
    $text2 yview $yview1 $yview2
}

######################################################################
#
#
proc Machine_ProcessFileDetail_Init { machine process file ea file_index mywin } {
    global RefreshDelay

    set tag [Compute_ProcessDetail_Tag ${machine} ${process} ${file} ${ea}]

    if { [llength $mywin] == 0 } {
        set winname ".${tag}"

        #
        # If the window exists, just call the refresh call and return
        #
        if { [winfo exists $winname] } {
            return
        }

        #
        # Otherwise, we need to create a window for this machine's status info
        #
        toplevel $winname
    } else {
        set winname $mywin
    }

    wm title $winname "$machine PID:$process File:$file EA:$ea"

    ################################################################
    # Save the associated Process File and EA for this Window
    # so we can use it when we do an update
    global ProcessID
    global Filename
    global EffectiveAddr
    set ProcessID($winname) $process
    set Filename($winname) $file
    set EffectiveAddr($winname) $ea

    set base_chart_name ${winname}.file_graph
    set inst_chart ${base_chart_name}_inst
    set cache_chart ${base_chart_name}_cache

    scrollbar ${winname}.inst_scrollx -orient horizontal -command "${inst_chart} axis view x"
    scrollbar ${winname}.inst_scrolly -orient vertical -command "${inst_chart} axis view y"
    barchart ${inst_chart} -title "Statistics Across File: $file (EA: $ea)" -width 1000 -height 200 -shadow white
    ZoomSupport_Init ${inst_chart} 0 0
    ${inst_chart} axis configure x -title "EA Offset" -scrollcommand "${winname}.inst_scrollx set"
    ${inst_chart} axis configure y -title "Counts" -scrollcommand "${winname}.inst_scrolly set"
    #${inst_chart} axis configure y2 -title "Counts" -hide false
    ${inst_chart} configure -barmode aligned

    # Don't mess with xticks for now...
    $inst_chart axis configure x -minorticks {0}
    $inst_chart axis configure x -majorticks {0}

    scrollbar ${winname}.cache_scrollx -orient horizontal -command "${cache_chart} axis view x"
    scrollbar ${winname}.cache_scrolly -orient vertical -command "${cache_chart} axis view y"
    barchart ${cache_chart} -title "Cache Statistics Across File: $file (EA: $ea)" -width 1000 -height 200 -shadow white
    ZoomSupport_Init ${cache_chart} 0 0
    ${cache_chart} axis configure x -title "EA Offset" -scrollcommand "${winname}.cache_scrollx set"
    ${cache_chart} axis configure y -title "Counts" -scrollcommand "${winname}.cache_scrolly set"
    #${inst_chart} axis configure y2 -title "Counts" -hide false
    ${cache_chart} configure -barmode stacked

    # Don't mess with xticks for now...
    $cache_chart axis configure x -minorticks {0}
    $cache_chart axis configure x -majorticks {0}

    set chart_list [list ${inst_chart} ${cache_chart}]
    global PT_Marker
    global Func_Marker
    global Func_Name_Marker
    foreach ch $chart_list {
        #${ch} configure -barmode stacked
        set PT_Marker(${ch}) [${ch} marker create text -coords {0 Inf} -text "0x0" -fill "" -outline red]
        set Func_Marker(${ch}) [${ch} marker create polygon -under true -coords {0 0 0 0 0 Inf 0 Inf} -fill yellow -outline black]
        set ymax [expr [string range [lindex [${ch} axis limits y] 1] 0 end-2] * 0.75]
        set Func_Name_Marker(${ch}) [${ch} marker create text -coords {0 $ymax} -text "" -fill "" -outline red]
        bind ${ch} <Shift-Motion>       "UpdateMotion_File  \%W \%x \%y $process $file_index \"$chart_list\" $winname $machine ${winname}.textboxes.assembly ${winname}.textboxes.assembly_label_fields"
        ${ch} legend configure -position bottom -font graph_legend_font
    }

    frame ${winname}.toggles
    # Get and save the stat names
    array set FileGraphInfo [$machine processtree display tcl_stats proc_file_info 0 0]
    global FileGraphColumns
    set FileGraphColumns($tag) $FileGraphInfo(columns)
    set x 0
    set y 0
    button ${winname}.toggles.zoomOnFunction -text "Zoom Graphs in on Function" -command "ZoomGraphsOnFunction ${inst_chart} ${cache_chart}"
    button ${winname}.toggles.zoomOutComplete -text "Zoom Graphs Out Completely" -command "ZoomGraphsOutComplete ${inst_chart} ${cache_chart}"
    table ${winname}.toggles ${winname}.toggles.zoomOnFunction $x,$y -fill x -cspan 2
    incr y 2
    table ${winname}.toggles ${winname}.toggles.zoomOutComplete $x,$y -fill x -cspan 2
    incr x
    set y 0
    foreach column $FileGraphColumns($tag) {
        if { [string first "CACHE" $column] == -1 } {
            set CHART $inst_chart
        } else {
            set CHART $cache_chart
        }
        # Toggle checkbox to choose whether to map data
        eval "global ${tag}_${column}_toggle"
        eval "set ${tag}_${column}_toggle 1"
        eval "checkbutton ${winname}.toggles.toggle_${column} -font graph_legend_font -text \"Graph_${column}\" -variable ${tag}_${column}_toggle -command \"ChangeGraphedData_File ${CHART} ${machine} ${tag}\""
        eval "table ${winname}.toggles ${winname}.toggles.toggle_${column} $x,$y -fill x -anchor w"
        incr y
        if { $y > 3 } {
            set y 0
            incr x
        }
    }

    frame ${winname}.textboxes
    label ${winname}.textboxes.assembly_label -justify center -text "Assembly:"
    text ${winname}.textboxes.assembly_label_fields -width 50 -height 1 -font textbox_font \
        -yscrollcommand "${winname}.textboxes.asm_scrolly set" \
        -xscrollcommand "${winname}.textboxes.asm_scrollx set"
    text ${winname}.textboxes.assembly -width 50 -height 15 -font textbox_font \
        -yscrollcommand "${winname}.textboxes.asm_scrolly set" \
        -xscrollcommand "${winname}.textboxes.asm_scrollx set"
    scrollbar ${winname}.textboxes.asm_scrollx -orient horizontal -command "scrollbothx ${winname}.textboxes.assembly ${winname}.textboxes.assembly_label_fields"
    scrollbar ${winname}.textboxes.asm_scrolly -orient vertical -command "${winname}.textboxes.assembly yview"

    table ${winname}.textboxes \
        ${winname}.textboxes.assembly_label 0,0 -cspan 2 -fill both \
        ${winname}.textboxes.assembly_label_fields       1,0 -fill x \
        ${winname}.textboxes.asm_scrolly    1,1 -rowspan 2 -fill y \
        ${winname}.textboxes.assembly       2,0 -fill both \
        ${winname}.textboxes.asm_scrollx    3,0 -fill x

    table configure ${winname}.textboxes c0 c1 c2 r0 r1 r2 r3 -resize none
    table configure ${winname}.textboxes c0 r2 -resize both

    table $winname \
        ${inst_chart} 0,2 -fill both \
        ${winname}.inst_scrolly 0,3 -fill y \
        ${winname}.inst_scrollx 1,2 -fill x \
        ${cache_chart} 2,2 -fill both \
        ${winname}.cache_scrolly 2,3 -fill y \
        ${winname}.cache_scrollx 3,2 -fill x \
        ${winname}.toggles 4,2 -fill x \
        ${winname}.textboxes 5,2 -fill both

    table configure $winname c0 r0 r1 r2 r3 r4 r5 c1 c2 c3 -resize none
    table configure $winname r0 r2 c2 r5 -resize both

    UpdateFileGraph $machine $process $file_index $base_chart_name

    update
}

proc UpdateMotion_File {chart x y pid file_index chart_list win machine asmtext asmlabel} {

    ${asmtext} delete 0.0 end
    ${asmlabel} delete 0.0 end

    global element_list
    ${chart} element closest $x $y element_list

    # If the element_list contains an index field then update text with location information
    if { [lsearch [array names element_list] index] != -1 } {
        set x_index $element_list(index)

        global  FileGraphData
        set FileGraphTag "${machine}_${pid}_${file_index}_filedata"
        array set FileGraphInfo $FileGraphData($FileGraphTag)

        # Get the EA offset at this index
        set addrvec V_${FileGraphTag}_ADDR
        eval "global $addrvec"
        eval "set addrvecval \$${addrvec}($x_index)"
        set point_addr [format64 "%lld" [string range $addrvecval 0 end-2]]
        set assembly(addr) [format64 "0x%016llX" [add64 $FileGraphInfo(ea) $point_addr]]

        # Get the encoding for this instruction
        set instvec V_${FileGraphTag}_INST
        eval "global $instvec"
        eval "set instvecval \$${instvec}($x_index)"
        set assembly(encoding) [format64 "0x%08llX" [string range $instvecval 0 end-2]]

        set label ""
        set func_start $assembly(addr)
        set func_end $assembly(addr)
        set func_name "Unknown"

        set filename [object load $FileGraphInfo(filename)]
        if { [string length $filename] > 0 } {
            set labelinfo [object label $FileGraphInfo(filename) $assembly(addr)]
            #puts "Label query for ea:$assembly(addr) got: $labelinfo"
            if { [string first "LABEL" $labelinfo] != -1 } {
                array set asm_info $labelinfo
                set label $asm_info(LABEL)
                set func_start $asm_info(FUNC_START)
                set func_end $asm_info(FUNC_END)
                set func_name $asm_info(FUNC)
            }
        }
        if { [string length $label] > 0 } {
            # Iterate through the EA instruction range for this procedure
            # and update the text window.   If we don't have data for it in the
            # the Vector, pull it from the object file itself.   We should also verify the
            # encoding in our vector and that found in the object file, match.
            # TODO

            # We want to show the other instruction in the neighborhood (bounded by the start and end of function)
            set start [sub64 $assembly(addr) 128]
            set end [add64 $assembly(addr) 128]
            if { [lessthan_u64 $func_start $start] != 1 } {
                set start $func_start
            }
            if { [lessthan_u64 $func_end $end] } {
                set end $func_end
            }
            #set start $func_start
            #set end $func_end

            # It can be expensive to pull the func data from the file so we will keep a cache of it here
            #global FuncData
            #set key "$FileGraphInfo(filename)_$func_name"
            #if { [array exists FuncData] && [llength [array names FuncData $key]] > 0 } {
            #    puts "Found cached funcdata for $key"
            #    set encodings $FuncData($key)
            #} else {
            #    puts "Did NOT find cached funcdata for $key"
            #    set encodings [object inst_data $FileGraphInfo(filename) $start $end]
            #    set FuncData($key) $encodings
            #    puts "Added FuncData for $key... [array names FuncData $key]"
            #}
            set encodings [object inst_data $FileGraphInfo(filename) $start $end]
            #puts "start $func_start end $func_end : got: $encodings"
            array set encoding $encodings
            set current_index 0.0

            # Update the assembly fields labels
            set disasm [format "%20s %18s %10s %10s %s\n" "Label" "Address" "Encoding" "Inst-Count" "Assembly"]
            ${asmlabel} insert end $disasm

            foreach ea [lsort [array names encoding]] {
                set offset [sub64 $ea $func_start]
                set label "$func_name+$offset"
                set addrvec V_${FileGraphTag}_ADDR
                eval "global $addrvec"
                set instvec V_${FileGraphTag}_INST_COUNT
                eval "global $instvec"
                eval "set vecindex \"[${addrvec} search [sub64 $ea $FileGraphInfo(ea)]]\""
                if { [llength $vecindex] > 0 } {
                    eval "set icount \$${instvec}($vecindex)"
                } else {
                    set icount 0
                }
                set disasm [format "%20s %18s %10s %10s %s\n" $label $ea $encoding($ea) $icount [$machine util ppc_disasm $encoding($ea) $ea] ]
                if { [compare64 $ea $assembly(addr)] == 0 } {
                    set current_index [${asmtext} index end]
                    ${asmtext} insert end $disasm CurrentLine
                } else {
                    ${asmtext} insert end $disasm
                }
                ${asmtext} tag configure CurrentLine -background red
            }
            ${asmtext} see $current_index
        } else {
            # Update the assembly fields labels
            set disasm [format "%20s %18s %10s %s\n" "Label" "Address" "Encoding" "Inst-Count" "Assembly"]
            ${asmlabel} insert end $disasm
            # No function data so just show the one instruction
            set disasm [format "%20s %18s %10s %s\n" $label $assembly(addr) $assembly(encoding) [$machine util ppc_disasm $assembly(encoding) $assembly(addr)] ]
            ${asmtext} insert end $disasm
        }

        global PT_Marker
        global Func_Marker
        global Func_Name_Marker
        foreach ch $chart_list {
            set label [format64 "0x%llX" [add64 $FileGraphInfo(ea) $point_addr]]
            ${ch} marker configure $PT_Marker($ch) -coords {$point_addr Inf} -text $label
            set func_start_index [sub64 $func_start $FileGraphInfo(ea)]
            set func_end_index [sub64 $func_end $FileGraphInfo(ea)]
            ${ch} marker configure $Func_Marker($ch) -coords {$func_start_index 0 $func_start_index Inf $func_end_index Inf $func_end_index 0}
            set ymax [expr [string range [lindex [${ch} axis limits y] 1] 0 end-2] * 0.75]
            ${ch} marker configure $Func_Name_Marker($ch) -coords {$point_addr $ymax} -text $func_name
        }
    }
}

