#!/usr/bin/bltwish
#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003,2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  spu_prof.tcl
#------------------------------------------------------------------------

package require BLT
namespace import -force blt::*

proc Compute_ProcTree_Tag {machine} {
    return "${machine}_proctree_stats"
}
proc ProcessStat_Fields { machine } {
    return [split [$machine processtree display tcl_stats process_columns] " "]
}

###########################################################
# read_processtree_data
#
#	This procedure actually queries the underlying
#   tcl interpreter to find out what statistics are available
#   associated with the local store.
#   It then creates a number of vectors filled with this data.
#   These vectors are then used to generate charts/graphs
#
proc read_processtree_data { machine } {
    set proctree_tag [Compute_ProcTree_Tag ${machine}]

    set column_index 0
    #
    # Get the list of local store fields available
    #
    foreach column [ProcessStat_Fields $machine] {
        if { [string compare $column "EXECNAME"] == 0 } {
            global execlist
            set execlist(${proctree_tag}) [$machine processtree display tcl_stats process_indices $column_index]
        } else {
            #
            # Create a global vector for this column of data
            #
            eval "global V_${proctree_tag}_${column}"

            # If the vector already exists, destroy it
            if { [llength [vector names ::V_${proctree_tag}_${column}]] > 0 } {
                eval "vector destroy V_${proctree_tag}_${column}"
            }
            eval "vector create V_${proctree_tag}_${column}"

            # Call back into the underlying interpreter asking for the data
            # associated with these columns
            eval "V_${proctree_tag}_${column} set [list [$machine processtree display tcl_stats process_indices $column_index]]"

            #
            # For the first column (could be any column 
            # since they all have the same number of entries)
            # we want to create an X-axis vector that just has a sequentially
            # numbered list that we can use to plot against.
            # If we use the LSADDR column, it can create gaps in our graph
            # (like with a vector of address every 128 bytes, blt will leave
            # gaps thinking we may want to plot something later in those gaps)
            #
            if { $column_index == 0 } {
                set index 0
                global V_${proctree_tag}_XAXIS
                if { [llength [vector names ::V_${proctree_tag}_XAXIS]] > 0 } {
                    eval "vector destroy V_${proctree_tag}_XAXIS"
                }
                eval "vector create V_${proctree_tag}_XAXIS"
                eval "set veclist \$V_${proctree_tag}_${column}(:)"
                foreach i $veclist {
                    eval "V_${proctree_tag}_XAXIS append $index"
                    incr index
                }
            }
        }
        incr column_index
    }
    
}


####################################################################
# GraphColor
#
#   For every graph we want to generate a stream of different colors
#   that the graphs use to plot each line.
#
#set base_colorlist {gray75 darkred darkmagenta magenta green4 gold4 yellow navy cyan blue red green purple orange brown pink tan lightblue}
set base_colorlist {gray75 darkred darkmagenta magenta green4 gold4 navy cyan blue red green purple orange brown pink tan lightblue}
proc GraphColor { tag } {
    global base_colorlist
    global colorlist
    global colorindex

    if { [llength [array names colorindex $tag]] <= 0 } {
        # Initialize the color setup for this tag
        set colorindex($tag) 0
    } 

    set retval [lindex $base_colorlist $colorindex($tag)]
    set colorindex($tag) [expr $colorindex($tag) + 1]
    if { $colorindex($tag) >= [llength $base_colorlist] } {
        set colorindex($tag) 0
    }

    return $retval
}

######################################################################
# FormatXTickLabels & ComputeXTicks
#
#   TODO: BLT was having trouble with the major x-ticks (like there 
#   were too many)  I just turned them off for now.   Should find 
#   out how to add some number of them.
#
proc FormatXTickLabels {chart val} {
    global graphOptions

	return ""
}
proc ComputeXTicks {chart axis vector} {
    $chart axis configure $axis -minorticks {0} 
    $chart axis configure $axis -majorticks {0} 
}

######################################################################
#
# PT_Graph_Logscale
#
#   This procedure is called by a checkbox button to turn on/off
#   the logscale for the y-axis in the graphs
#
proc PT_Graph_Logscale {chart checkbutt tag} {
    eval "global ${tag}_logscale"
    eval "set logscale_state \$${tag}_logscale"
    if { $logscale_state == 0 } {
        # When changing the logscale we occasionally get errors associated with the
        # current min and max.   If we reexecute the command it seems to work
        catch {$chart axis configure y -logscale false} foo
        $chart axis configure y -logscale false
        #$chart axis configure y2 -logscale false
    } else {
        # When changing the logscale we occasionally get errors associated with the
        # current min and max.   If we reexecute the command it seems to work
        catch {$chart axis configure y -logscale true} foo
        $chart axis configure y -logscale true
        #$chart axis configure y2 -logscale true
    }
}
proc ChangeGraphedData {chart machine} {

    set graph_list ""
    set fields [ProcessStat_Fields $machine] 
    set proctree_tag [Compute_ProcTree_Tag ${machine}]
    foreach column [lrange $fields 2 end] {
        eval "global ${proctree_tag}_${column}_toggle"
        eval "set tog_state \$${proctree_tag}_${column}_toggle"
        if { $tog_state == 1 } {
            lappend graph_list $column
        }
    }
    $chart element show $graph_list
}

######################################################################
#
# Machine_ProcessTreeStats_Init -- Initalize the monitor of the specified SPU's 
#                       local store
#
proc Machine_ProcessTreeStats_Init { machine mywin } {
	global RefreshDelay

    set proctree_tag [Compute_ProcTree_Tag ${machine}]

	if { [llength $mywin] == 0 } {
		set winname ".${proctree_tag}"

		#
		# If the window exists, just call the refresh call and return
		#
		if { [winfo exists $winname] } {
			return
		}

		#
		# Otherwise, we need to create a window for this machine's status info
		#
		toplevel $winname
	} else {
		set winname $mywin
	}

    global MachineNames
    set MachineNames($winname) $machine

	read_processtree_data $machine 

    set inst_chart ${winname}.process_inst_graph
    scrollbar ${winname}.inst_scrollx -orient horizontal -command "${inst_chart} axis view x"
    scrollbar ${winname}.inst_scrolly -orient vertical -command "${inst_chart} axis view y"
	barchart ${inst_chart} -title "Instruction-Count Distribution across Processes ($machine)" -height 200 -shadow white

    set cache_chart ${winname}.process_cache_graph
    scrollbar ${winname}.cache_scrollx -orient horizontal -command "${cache_chart} axis view x"
    scrollbar ${winname}.cache_scrolly -orient vertical -command "${cache_chart} axis view y"
	barchart ${cache_chart} -title "Cache Stats across Processes ($machine)" -height 200 -shadow white

    ZoomSupport_Init ${inst_chart} 0 0
    ZoomSupport_Init ${cache_chart} 0 0

    set fields [ProcessStat_Fields $machine] 
    #set x_axis [lindex $fields 0]
    set x_axis XAXIS
    global V_${proctree_tag}_XAXIS
    frame ${winname}.toggles
    set x 0
    set y 0
    foreach column [lrange $fields 2 end] {
        set xvals V_${proctree_tag}_${x_axis}
        set yvals V_${proctree_tag}_${column}

        if { [string first CACHE $column] == -1 } {
            # Then add it to the cache chart
            set CHART $inst_chart
            set GRAPH_TAG "_INST"
            set YAXIS y
        } else {
            set CHART $cache_chart
            set GRAPH_TAG "_CACHE"
            set YAXIS y
        }

        # Then add it to the instruction chart
        # Toggle checkbox to choose whether to map data
        eval "global ${proctree_tag}_${column}_toggle"
        eval "set ${proctree_tag}_${column}_toggle 1"

        eval "checkbutton ${winname}.toggles.toggle_${column} -font graph_legend_font -text \"Graph_${column}\" -variable ${proctree_tag}_${column}_toggle -command \"ChangeGraphedData ${CHART} ${machine}\""
        eval "table ${winname}.toggles ${winname}.toggles.toggle_${column} $x,$y -fill x -anchor w"
        incr y
        if { $y > 3 } {
            set y 0
            incr x
        }
        eval "$CHART pen create $column -foreground [GraphColor ${proctree_tag}_${GRAPH_TAG}] -relief flat"
        eval "$CHART element create $column -mapy $YAXIS -xdata $xvals -ydata $yvals -pen $column"
    }

    ########################################
    # Put in the log scale toggles
    set lstag "${proctree_tag}_inst"
    eval "global ${lstag}_logscale"
    eval "set ${lstag}_logscale 0"
    eval "checkbutton ${winname}.toggles.inst_logscale -font graph_legend_font -text \"Use Log Scale for Y-Axis Inst Graph\" -variable ${lstag}_logscale -command \"PT_Graph_Logscale ${inst_chart} ${winname}.inst_logscale ${lstag}\""
    set lstag "${proctree_tag}_cache"
    eval "global ${lstag}_logscale"
    eval "set ${lstag}_logscale 0"
    eval "checkbutton ${winname}.toggles.cache_logscale -font graph_legend_font -text \"Use Log Scale for Y-Axis Cache Graph\" -variable ${lstag}_logscale -command \"PT_Graph_Logscale ${cache_chart} ${winname}.cache_logscale ${lstag}\""
    incr x
    set y 0
    table ${winname}.toggles ${winname}.toggles.inst_logscale            $x,$y -fill x -anchor w
    incr y
    table ${winname}.toggles ${winname}.toggles.cache_logscale            $x,$y -fill x -anchor w

    ComputeXTicks ${inst_chart} x V_${proctree_tag}_${x_axis}
    ComputeXTicks ${cache_chart} x V_${proctree_tag}_${x_axis}

    ${inst_chart} axis configure x -title "Processes" -scrollcommand "${winname}.inst_scrollx set" -command FormatXTickLabels
    ${inst_chart} axis configure y -title "Counts" -scrollcommand "${winname}.inst_scrolly set"
    #${inst_chart} axis configure y2 -title "Non-Inst Counts" -hide false

    ${cache_chart} axis configure x -title "Processes" -scrollcommand "${winname}.cache_scrollx set" -command FormatXTickLabels
    ${cache_chart} axis configure y -title "Counts" -scrollcommand "${winname}.cache_scrolly set"
    #${cache_chart} axis configure y2 -title "Other Counts" -hide false

    frame ${winname}.textboxes
    label ${winname}.textboxes.files_label -justify center -text "Files:"
    set tree ${winname}.textboxes.files
    hierbox $tree -separator "^" -autocreate true -scrollmode listbox -xscrollcommand "$winname.textboxes.scrollx set" -yscrollcommand "$winname.textboxes.scrolly set" -width 800 -font small_textbox_font
    scrollbar $winname.textboxes.scrollx -orient horizontal -command "$tree xview"
    scrollbar $winname.textboxes.scrolly -orient vertical -command "$tree yview"
    #text ${winname}.textboxes.files -height 30 -font small_textbox_font
    label ${winname}.textboxes.details_label -justify center -text "Process Data Point Details:"
    text ${winname}.textboxes.details -height 15 -font small_textbox_font 
    #SPULS_Contents_Init ${winname}.textboxes.files $machine

	table ${winname}.textboxes \
		${winname}.textboxes.files_label            0,0 -cspan 1 -fill both \
		${winname}.textboxes.details_label          0,2 -fill both \
		${winname}.textboxes.files                  1,0 -cspan 1 -fill both \
        ${winname}.textboxes.scrolly                1,1 -fill y \
		${winname}.textboxes.details                1,2 -fill both \
		${winname}.textboxes.scrollx                2,0 -cspan 1 -fill x 

	table configure ${winname}.textboxes c0 c1 c2 r0 r1 r2 r3 r4 -resize none
	table configure ${winname}.textboxes c0 r1 -resize both

    set chart_list [list ${inst_chart} ${cache_chart}]
    global PT_Marker
    foreach ch $chart_list {
        ${ch} configure -barmode stacked
        set PT_Marker(${ch}) [${ch} marker create text -coords {0 Inf} -text "Unknown" -fill "" -outline red]
        bind ${ch} <Shift-Motion>       "UpdateMotion  \%W \%x \%y \"$chart_list\" $winname $machine ${winname}.textboxes.files"
        ${ch} legend configure -position bottom -font graph_legend_font
    }

	table $winname \
		${winname}.process_inst_graph       0,0 -fill both \
		${winname}.inst_scrolly             0,1 -fill y \
		${winname}.inst_scrollx             1,0 -fill x \
		${winname}.process_cache_graph      2,0 -fill both \
		${winname}.cache_scrolly            2,1 -fill y \
		${winname}.cache_scrollx            3,0 -fill x \
		${winname}.toggles                  4,0 -fill x -anchor w \
		${winname}.textboxes                5,0 -fill x 

	table configure $winname r0 r1 r2 r3 r4 r5 r6 c0 c1 c2 -resize none
	table configure $winname c0 r0 r2 -resize both

	update
}

proc FileOpenCommand {sim pid filename start_ea end_ea file_index} {
    Machine_ProcessFileDetail_Init $sim $pid $filename $start_ea $file_index ""
}

######################################################################
#
# UpdateMotion
#   This procedure is bound to the event of someone moving the mouse
#   across the graph.   It detects where the mouse it and provides more
#   data.
#
proc UpdateMotion {chart x y chart_list win machine files} {

    set details     ${win}.textboxes.details

    ${details} delete 0.0 end
    ${files} delete 0

    global element_list
    ${chart} element closest $x $y element_list

    # If the element_list contains an index field then update text with location information
    if { [lsearch [array names element_list] index] != -1 } {
        set x_index $element_list(index)

        # Now dump all the fields from the vectors for this index
        set fields [ProcessStat_Fields $machine] 
        set proctree_tag [Compute_ProcTree_Tag ${machine}]

        # We have been having trouble with BLT where it is returning an invalid index
        set vec V_${proctree_tag}_PID_INDEX
        eval "global $vec"
        eval "set len [${vec} length]"
        if { $x_index <= $len } {
            puts "$chart x:$x y:$y = x_index:$x_index"

            foreach column $fields {
                if { [string compare $column "EXECNAME"] == 0 } {
                    global execlist
                    set execname [lindex $execlist(${proctree_tag}) $x_index]
                    ${details} insert end [format "%30s % 16s\n" $column $execname]
                } else {
                    set vec V_${proctree_tag}_${column}
                    eval "global $vec"
                    eval "set val [expr int(\$${vec}($x_index))]"
                    eval "set val [format "%d" $val]"
                    eval "set min [expr int(\$${vec}(min))]"
                    eval "set max [expr int(\$${vec}(max))]"
                    ${details} insert end [format "%30s % 16s (Min=%10d, Max=%10d)\n" $column $val $min $max]
                }
            }

            global PT_Marker
            foreach ch $chart_list {
                ${ch} marker configure $PT_Marker($ch) -coords {$x_index Inf} -text "$x_index:$execname"
            }

            global MachineNames
            set sim $MachineNames($win)
            array set proc_info [$sim processtree display process $x_index]
            array set proc_stat_info $proc_info(stats)
            foreach stat [array names proc_stat_info] {
                set total($stat) $proc_stat_info($stat)
            }
            set idx 0
            foreach file $proc_info(files) {
                array set file_info $file
                set end [format64 "0x%016llX" [add64 $file_info(ea) $file_info(length)]]
                array set range_info [$sim processtree display tcl_stats proc_range_info $proc_info(pid) $file_info(ea) $end]
                set pcnt [percent64 $range_info(INST_COUNT) $proc_stat_info(INST_COUNT)]
                set file_root [format "^%2s) %s - %s %50s %10s %7s\n" $idx $file_info(ea) $end $file_info(filename) $range_info(INST_COUNT) $pcnt]
                set tag [${files} insert end $file_root]
                ${files} bind $tag <Double-ButtonRelease-1> "+FileOpenCommand $sim $proc_info(pid) $file_info(filename) $file_info(ea) $end $idx"
                foreach stat [array names range_info] {
                    set total($stat) [sub64 $total($stat) $range_info($stat)]
                    set pcnt [percent64 $range_info($stat) $proc_stat_info($stat)]
                    set label [format "%s^%30s: %10s %7s\n" $file_root $stat $range_info($stat) $pcnt]
                    ${files} insert end $label
                }
                incr idx
            }
            set pcnt [percent64 $total(INST_COUNT) $proc_stat_info(INST_COUNT)]
            set file_root [format "^%2s) %90s %10s %7s\n" $idx UNASSOCIATED $total(INST_COUNT) $pcnt]
            ${files} insert end $file_root 
            foreach stat [array names range_info] {
                set pcnt [percent64 $total($stat) $proc_stat_info($stat)]
                set label [format "%s^%30s: %10s %7s\n" $file_root $stat $total($stat) $pcnt]
                ${files} insert end $label
            }
            update
        }
    }
}
