#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  stats.tcl
#
#
#------------------------------------------------------------------------

######################################################################
#
# Stats_Init -- Initalize the status window
#
######################################################################
proc Stats_Init {root  _upstate args} {
    global StatsList StatDisplayed PPCCoreSim 
    global Stats

    upvar $_upstate  initstate 
    TclUtils::copyarray initstate Stats $root      ;# create Stats($root,x)
    set sim $Stats($root,sim)

    # Tuck this data away so the Refresh can use it
    set PPCCoreSim($root) $sim
    if { [catch {array set StatsArray [$sim stats listconfig]} errors] } {
        puts "Stats_Init: ERROR: stat list did not return an even numbered list"
        return;
    }
    set StatsList($root) [array names StatsArray]

    listbox $root.statsnames -width 30 -height 20 -font listbox_font
    listbox $root.statsdata -width 15 -height 20 -font listbox_font
    listbox $root.ratesnames -width 30 -height 10 -font listbox_font
    listbox $root.ratesdata -width 15 -height 10 -font listbox_font
    button $root.refresh -command "Stats_Refresh $root" -text "Refresh"

    foreach s $StatsList($root) {

        if { [catch {array set StatInfo $StatsArray($s)} errors] } {
            puts "Stats_Init: ERROR: stat info did not return an even numbered list $StatsArray($s) ERR:$errors"
            return;
        }

        if { [string first "%" $StatInfo(label)] != -1 } {
            $root.ratesnames insert end "$StatInfo(label)"
            $root.ratesdata insert end "[$sim stats display $s]"
        } else {
            $root.statsnames insert end "$StatInfo(label)"
            $root.statsdata insert end "[$sim stats display $s]"
        }
    }

    table $root \
        $root.statsnames    0,0 \
        $root.statsdata     0,1 \
        $root.ratesnames    1,0 \
        $root.ratesdata     1,1 \
        $root.refresh       2,0 -cspan 2 -fill x

    table configure $root c* r* -resize none

    set title [treesim::get_path_name $Stats($root,widget) $Stats($root,treeidx) 1 ] 
    wm title $root "$title: Stats" 

    return "Stats_Refresh"
}

######################################################################
#
# Stats_Refresh -- Refresh the contents of the status window
#
######################################################################
proc Stats_Refresh { root args } {
    global StatsList PPCCoreSim

    set sim $PPCCoreSim($root)

    if { [catch {array set StatsArray [$sim stats listconfig]} errors] } {
        puts "Stats_Init: ERROR: stat list did not return an even numbered list"
        return;
    }

    #######################################################################
    # Mark changed values with background color change
    set stat_idx 0
    set rates_idx 0
    foreach s $StatsList($root) {

        if { [catch {array set StatInfo $StatsArray($s)} errors] } {
            puts "Stats_Init: ERROR: stat info did not return an even numbered list $StatsArray($s) ERR:$errors"
            return;
        }

        if { [string first "%" $StatInfo(label)] != -1 } {
            if { [$sim stats display $s] != [$root.ratesdata get $rates_idx] } {
                $root.ratesnames itemconfigure $rates_idx -bg red
                $root.ratesdata itemconfigure $rates_idx -bg red
            } else {
                $root.ratesnames itemconfigure $rates_idx -bg lightgrey
                $root.ratesdata itemconfigure $rates_idx -bg lightgrey
            }
            incr rates_idx
        } else {
            if { [$sim stats display $s] != [$root.statsdata get $stat_idx] } {
                $root.statsnames itemconfigure $stat_idx -bg red
                $root.statsdata itemconfigure $stat_idx -bg red
            } else {
                $root.statsnames itemconfigure $stat_idx -bg lightgrey
                $root.statsdata itemconfigure $stat_idx -bg lightgrey
            }
            incr stat_idx
        }
    }

    #######################################################################
    # Update the values
    foreach s $StatsList($root) {
        if { [catch {array set StatInfo $StatsArray($s)} errors] } {
            puts "Stats_Init: ERROR: stat info did not return an even numbered list $StatsArray($s) ERR:$errors"
            return;
        }

        if { [string first "%" $StatInfo(label)] != -1 } {
            $root.ratesdata delete 0
        } else {
            $root.statsdata delete 0
        }
    }

    set stat_idx 0
    set rates_idx 0
    foreach s $StatsList($root) {

        if { [catch {array set StatInfo $StatsArray($s)} errors] } {
            puts "Stats_Init: ERROR: stat info did not return an even numbered list $StatsArray($s) ERR:$errors"
            return;
        }

        if { [string first "%" $StatInfo(label)] != -1 } {
            $root.ratesdata insert end "[$sim stats display $s]"
            $root.ratesdata itemconfigure $rates_idx -background \
                [$root.ratesnames itemcget $rates_idx -background]
            incr rates_idx
        } else {
            $root.statsdata insert end "[$sim stats display $s]"
            $root.statsdata itemconfigure $stat_idx -background \
                [$root.statsnames itemcget $stat_idx -background]
            incr stat_idx
        }
    }
}
