#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003,2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  sysmem.tcl
#
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
#
# SystemMemory_Init -- Initalize the status window
#
#------------------------------------------------------------------------
proc SystemMemory_Init_Common {root sim mcmcmd title args } {
    global cycleAdvance SystemMemoryStart SystemMemoryEnd SystemMemorySim SystemMemoryLength LengthFormat
    global SystemMemoryTranslate SystemMemoryBaseTitle sysmemmenutitle sysmemmenuindex SystemMemoryInstructions SystemMemoryCPU
    global SystemMemory

    set SystemMemorySim($root) $sim
    set SystemMemoryStart($root) 0
    set SystemMemoryLength($root) 256
    set LengthFormat($root) 0
    set SystemMemoryEnd($root) [format64 "0x%llX" [add64 $SystemMemoryStart($root) $SystemMemoryLength($root)]]
    set SystemMemoryBaseTitle($root) $title
    set SystemMemoryTranslate($root) 1
    set SystemMemoryInstructions($root) 0

    # starting address
    frame $root.start -borderwidth 2 -relief sunken
    label $root.start.label -text "Start Addr:" -pady 4
    entry $root.start.addr -textvariable SystemMemoryStart($root)
    pack $root.start.label $root.start.addr -side left


    # length and format
    frame $root.leng -borderwidth 2 -relief sunken
    label $root.leng.label -text "Length:" -padx 4
    entry $root.leng.addr -textvariable SystemMemoryLength($root) -width 8 -justify right
    checkbutton $root.leng.format -text "Hex" -onvalue 1 -offvalue 0 -variable LengthFormat($root) -command "SystemMemory_Refresh $root"
    pack $root.leng.label $root.leng.addr $root.leng.format -side left

    # buttons
    button $root.refresh -text "Reload" -command "SystemMemory_Refresh $root"
    button $root.membrk -text "Breakpoints..." -command "SystemMemory_Breakpoint $root"
    button $root.close -text "Close" -command "destroy $root"

    # address translation
    frame $root.translate -borderwidth 2 -relief sunken
    set xlate $root.translate
    checkbutton $xlate.chk -text "Use Address Translation" -onvalue 1 -offvalue 0 -variable SystemMemoryTranslate($root) -command "SystemMemory_Refresh $root"
    pack $xlate.chk -side left

    label $xlate.cpuLabel -text "Processor:"
    entry $xlate.cpu -state disabled -textvariable SystemMemoryCPU($root) -width 10

    menubutton $xlate.thread -text "Change..."  -direction below -takefocus 1 \
                                -relief raised  -borderwidth 2

    menu $xlate.thread.choices -tearoff 0

    # This is as many CPUs we want to show in the menu in one chunk
    set cpu_view_max 16

    set cpulist [$mcmcmd display cpulist]
    if { [llength $cpulist] > $cpu_view_max } {
        # Create Cascading Menus
        set sub_menus [expr [llength $cpulist] / $cpu_view_max]
        if { [expr [llength $cpulist] % $cpu_view_max] > 0 } {
            incr sub_menus
        }

        for { set sub 0 } { $sub < $sub_menus } { incr sub } {
            # Get sub CPU list

            set sub_cpu_list [lrange $cpulist [expr $cpu_view_max * $sub] [expr $cpu_view_max * ($sub + 1) - 1]]

            set sub_label "CPUS [lindex $sub_cpu_list 0] - [lindex $sub_cpu_list [expr [llength $sub_cpu_list] - 1]]"

            $xlate.thread.choices add cascade -label $sub_label -menu "$xlate.thread.choices.sub$sub"
            set sub_menu [menu $xlate.thread.choices.sub$sub]
            
            foreach cpu $sub_cpu_list {
                $sub_menu add radiobutton -label "$cpu" -variable SystemMemoryCPU($root) \
                               -command "SystemMemoryCPU_changed $root $cpu"
            }
        }

        # Now add CPU options to cascaded menus
    } else {
        # Just list all CPU options
        foreach cpu $cpulist {
            $xlate.thread.choices add radiobutton -label "$cpu" -variable SystemMemoryCPU($root) \
                           -command "SystemMemoryCPU_changed $root $cpu"
        }
    }

    $xlate.thread configure -menu $xlate.thread.choices
    set SystemMemoryCPU($root) [lindex $cpulist 0]
    SystemMemoryCPU_set $root $SystemMemoryCPU($root)

    pack $xlate.cpuLabel $xlate.cpu $xlate.thread -side left

    # instructions
    frame $root.instructions -borderwidth 2 -relief sunken
    checkbutton $root.instructions.cb -text "View As Instructions" -onvalue 1 -offvalue 0 -variable SystemMemoryInstructions($root) -command "SystemMemory_Refresh $root"
    pack $root.instructions.cb -side left

    text $root.mem -wrap word -width 70 -height 14 -yscrollcommand "$root.memsy set" -font textbox_font
    scrollbar $root.memsy -orient vert -command "$root.mem yview"

    table $root \
        $root.mem 0,0 -cspan 5 -fill both \
        $root.memsy 0,4 -cspan 1 -fill y \
        $root.start        1,0 -fill both \
        $root.leng         1,1 -fill both \
        $root.translate    2,0 -fill x \
        $root.instructions 2,1 -fill both \
        $root.refresh      1,3 -cspan 2 -fill x \
        $root.membrk       2,2 -fill x \
        $root.close        2,3 -cspan 2 -fill x
    table configure $root c* r* -resize none
    table configure $root r0 c0 -resize expand

    set sysmemmenutitle($root) "$SystemMemoryBaseTitle($root) : $SystemMemoryStart($root) - $SystemMemoryEnd($root)"
    wm title $root $sysmemmenutitle($root)
    bind $root.start.addr <KeyPress-Return> "SystemMemory_Refresh $root"
    bind $root.start.addr <KeyPress-Tab> "SystemMemory_Refresh $root"
    bind $root.leng.addr <KeyPress-Return> "SystemMemory_Refresh $root"
    bind $root.mem <Double-ButtonRelease-1> "SystemMemory_ToggleBreak $root $mcmcmd \[$root.mem index current\]"
    SystemMemory_Refresh $root
    return "SystemMemory_Refresh"
}
#------------------------------------------------------------------------
# this is called from ppc core to dump memory at addr in register
#------------------------------------------------------------------------
proc SystemMemory_Open {sim mcmcmd addr {cpu 0}} {


    set title "System Memory"
    set winname .[ TclUtils::safename "systemmem_${mcmcmd}_${addr}" ]

    global SystemMemory
    set SystemMemory($winname,mcmcmd) $mcmcmd

    # If the window exists, just raise it
    if { [winfo exists $winname] != 0 } {
        # Already exists
        raise $winname
    } else {
        toplevel $winname
        SystemMemory_Init_Common $winname $sim $mcmcmd $title nothing
    }

    if { $cpu != 0 } {
        SystemMemoryCPU_set $winname $cpu
    }

    global SystemMemoryStart 
    set SystemMemoryStart($winname) $addr
    SystemMemory_Refresh $winname nothing
}

# This is called from the tree widget
proc SystemMemory_Init {root _upstate args } {
    global cycleAdvance SystemMemoryStart SystemMemoryEnd SystemMemorySim SystemMemoryLength LengthFormat
    global SystemMemoryTranslate SystemMemoryBaseTitle sysmemmenutitle sysmemmenuindex SystemMemoryInstructions SystemMemoryCPU
    global SystemMemory

    upvar $_upstate  initstate
    TclUtils::copyarray initstate SystemMemory $root      ;# create SystemMemory($root,x)
    set sim $SystemMemory($root,sim)

    # if shared mode, use the display command for mcm 0
    if { [treesim::is_shared_mode $sim ] } {
       set SystemMemory($root,mcmcmd)  [alias create_unique $sim mcm 0]
    }
    set mcmcmd $SystemMemory($root,mcmcmd)

    set title [treesim::get_path_name $SystemMemory($root,widget) $SystemMemory($root,treeidx) 0 ]

    SystemMemory_Init_Common $root $sim $mcmcmd $title $args
}
proc SystemMemory_Retitle {root title} {
    global sysmemmenutitle SystemMemoryStart SystemMemoryEnd SystemMemoryBaseTitle
    set sysmemmenutitle($root) "$SystemMemoryBaseTitle($root) : $title"
    wm title $root $sysmemmenutitle($root)
}

proc SystemMemoryCPU_set {root cpu} {
    global SystemMemoryCPU
    set SystemMemoryCPU($root,num)      [string range $cpu 3 end]
    set cpu_type                        [string range $cpu 0 2]

    switch -regexp $cpu_type {
        {PP[CE]} {
            set SystemMemoryCPU($root,cpu_type) "cpu"
            set SystemMemoryCPU($root,dasm_cmd) "ppc_disasm"
        }
        {SP[UE]} {
            set SystemMemoryCPU($root,cpu_type) "spu"
            set SystemMemoryCPU($root,dasm_cmd) "spu_disasm"
        }
        default {
            return -code error "SystemMemoryCPU_set:: Unknown CPU type '$cpu_type'"
        }
    }
}

proc SystemMemoryCPU_changed {root cpu} {
   global SystemMemoryTranslate

   # need translation on to use cpu value
   set SystemMemoryTranslate($root) 1

   SystemMemoryCPU_set $root $cpu

   SystemMemory_Refresh $root
}

proc SystemMemory_AddrTrans {mcmcmd cputype cpunum transtype addr up_error} {
    upvar $up_error translate_error

    set translate_error 0

    if {[catch {
          set rAddr [$mcmcmd $cputype $cpunum $transtype $addr]
       } ] == 1 } then {
       set translate_error 1
       set rAddr $addr
    }

    return $rAddr
}

#------------------------------------------------------------------------
#
# SystemMemory_Refresh -- Refresh the contents of the status window
#
#------------------------------------------------------------------------
proc SystemMemory_Refresh { root args } {
    global cycleAdvance SystemMemoryStart SystemMemoryEnd SystemMemorySim SystemMemoryLength LengthFormat
    global SystemMemoryBaseTitle SystemMemoryTranslate sysmemmenutitle sysmemmenuindex SystemMemoryInstructions SystemMemoryCPU
    global SystemMemory

    set sim $SystemMemorySim($root)
    set mcmcmd  $SystemMemory($root,mcmcmd)
    set translate $SystemMemoryTranslate($root)
    set cpu_type $SystemMemoryCPU($root,cpu_type)
    set cpunum   $SystemMemoryCPU($root,num)
    set dasm_cmd $SystemMemoryCPU($root,dasm_cmd)

    set addr $SystemMemoryStart($root)
    set SystemMemoryStart($root) [format64 "0x%016llX" $SystemMemoryStart($root)]
    if { $LengthFormat($root) } {
        set SystemMemoryLength($root) [format64 "0x%llX" $SystemMemoryLength($root)]
    } else {
        set SystemMemoryLength($root) [format64 "%lld" $SystemMemoryLength($root)]
    }
    set SystemMemoryEnd($root) [format64 "0x%016llX" [add64 $SystemMemoryStart($root) $SystemMemoryLength($root)]]
    SystemMemory_Retitle $root "$SystemMemoryStart($root) - $SystemMemoryEnd($root)"

    set iMem "$root.mem"
    $iMem delete 1.0 end

    if { $SystemMemoryInstructions($root) } {
       set ea_bplist [$mcmcmd trigger display pc list]
       set ra_bplist [$mcmcmd trigger display real_pc list]
    }

    while { [lessthan_u64 $addr $SystemMemoryEnd($root)] } {

        set translate_error 0
        if { $SystemMemoryInstructions($root) } {
            # Display As Instructions
            if { $translate } {
                set rAddr [ SystemMemory_AddrTrans $mcmcmd $cpu_type $cpunum "itranslate" $addr translate_error ]
            } else {
                set rAddr $addr
            }

            set addr64 [format64 "%016llX" $addr]
            set raddr_str [format64 "%016llX" $rAddr]


            if {$translate_error ||
                 [catch {
                  set value [$mcmcmd memory display $rAddr 4]
                  set disasm [$sim util $dasm_cmd $value $addr]
               } ] == 1 } then {
                  set value -1
                  set disasm "????????"
            }
            set ascii [TclUtils::GenAscii $value]

            if { ( [search_break_list $ea_bplist 0x$addr64] == 1 ) || ( [search_break_list $ra_bplist $rAddr] == 1 ) } {
                set bp "B"
            } else {
                set bp " "
            }
            set line [format "%s %s : %s : %08X : %s : %s\n" $bp $addr64 $raddr_str $value $ascii $disasm]

            incr64 addr 4
        } else {
            # Display As Data
            if { $translate } {
                set rAddr [ SystemMemory_AddrTrans $mcmcmd $cpu_type $cpunum "dtranslate" $addr translate_error ]
            } else {
                set rAddr $addr
            }

            set raddr_str [format64 "%016llX" $addr]

            if {$translate_error ||
                 [catch {
                  set values [string trim [$mcmcmd memory display $rAddr 4 4]]
                  set ascii [GenAscii_list $values ]
               } ] == 1 } then {
                  set values "???????? ???????? ???????? ????????"
                  set ascii ""
            }

            set line [format "%s : %s : %s\n" $raddr_str $values $ascii ]
            incr64 addr 16
        }
        $iMem insert end $line
    }
    #busy release .
}

proc GenAscii_list { values } {
   set asciivalues ""
   foreach x $values {
       append asciivalues [TclUtils::GenAscii $x]
   }
   return $asciivalues
}

proc SystemMemory_ToggleBreak {root mcmcmd textidx} {
    global SystemMemoryInstructions SystemMemoryStart

    if { $SystemMemoryInstructions($root) } {

       set cur_line [lindex [split $textidx .] 0]
       set addr [add64 $SystemMemoryStart($root) [expr ($cur_line-1) * 4] ]

       set hexaddr [format64 "0x%016llX" $addr]
       if { [string length [$mcmcmd trigger display pc addr $hexaddr]] > 0 } {
           puts "Clearing breakpoint at $hexaddr"
           $mcmcmd trigger clear pc $hexaddr
           $mcmcmd trigger clear real_pc $hexaddr
       } else {
           puts "Setting breakpoint at $hexaddr"
           $mcmcmd trigger set pc $hexaddr "break_point $mcmcmd Break_at_$hexaddr"
           $mcmcmd trigger set real_pc $hexaddr "break_point $mcmcmd Break_at_$hexaddr"
       }

       SystemMemory_Refresh $root
       $root.mem see $textidx
    }
}


proc SystemMemory_Breakpoint { root } {
    global SystemMemory SystemMemoryTranslate
    global SystemMemoryCPU SystemMemoryStart

    set cpunum $SystemMemoryCPU($root,num)
    set mcmcmd $SystemMemory($root,mcmcmd)

    set startaddr $SystemMemoryStart($root)
    set xlat $SystemMemoryTranslate($root)
    Memory_breakpoint_dlg $mcmcmd "system" $cpunum $startaddr $xlat
}

