#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  testcmds.tcl
#
#  This is used for regression testing the tcl commands that are used 
#  by the gui.
#------------------------------------------------------------------------

namespace eval testcmds {
}

#------------------------------------------------------------------
# This proc simply gets the contents of a file and returns
# it in a tcl variable
#------------------------------------------------------------------
proc testcmds::get_file {filename} {
  set fd [open $filename "r"]
  set data [read $fd]
  close $fd
  return $data
}

#------------------------------------------------------------------
# Look for a string in the test file that indicates the start of the
# tcl commands to test
#------------------------------------------------------------------
proc testcmds::test_starting_line {input_lines} {
    set start_string "# start_of_test_commands #"
    set start_line 1
    foreach line $input_lines {
        if {[string index $line 0] == "#" &&
           $line == $start_string } { 
            break;
        }
        incr start_line
    }
    return $start_line;
}

#------------------------------------------------------------------
# Load the input file and find the test starting point
# execute the tcl commands in the file
#------------------------------------------------------------------
proc testcmds::run_test {infname} {
   set input_text [ get_file $infname ]
   set input_lines [split $input_text "\n"]
   set test_start [test_starting_line $input_lines ]
   set test_cmds [lrange $input_lines $test_start end ]

   if {$test_cmds == {} } {
      puts "test command start tag not found"
   }

   foreach line $test_cmds {

       if { $line == "" || 
            [string index $line 0] == "#" } {
          puts "$line"
          continue;
       }

       if { [catch {
                      puts "$line => [eval $line]" 
                   } err ] } {
          puts "$line => **error** $err"
       }
   }
}

