#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2004
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  textutils.tcl
#------------------------------------------------------------------------

namespace eval textutils {
}

proc textutils::get_color {category} {

    switch $category {
        "Default"       {return [ get_pref colors/Default     "black grey85"       ]}
        "Breakpoint"    {return [ get_pref colors/Breakpoint  "black red  "       ]}
        "Currentline"   {return [ get_pref colors/Currentline "black yellow "       ]}
        "Modified"      {return [ get_pref colors/Modified    "\#0000ff white"    ]}
        "Tooltips"      {return [ get_pref colors/Tooltips    "black lightyellow" ]}
        default         { return "green lightblue" }
    }
}

#---------------------------------------------------------------------------
#
# Create the window tags in priority lowest to highest.  If any tags get created 
# after this, they will have the highest priority unless they are lowered.
#
#---------------------------------------------------------------------------
proc textutils::init_color_tags {txtwidget}  {

   update_color_tag $txtwidget pc_cursor   Currentline
   update_color_tag $txtwidget breakpoint  Breakpoint
}

#---------------------------------------------------------------------------
#  add the color configuration tag
#---------------------------------------------------------------------------
proc textutils::update_color_tag { txtwidget tagname category {lower ""}} {
    #puts "tags: [$txtwidget tag names]"

    set colors [ get_color $category ]

    set fgclr [lindex $colors 0]
    set bgclr [lindex $colors 1]

    $txtwidget tag configure $tagname -foreground $fgclr -background $bgclr

    if {$lower != ""} { 
       # keep colors at low priority so other tags will show through
       $txtwidget tag lower $tagname 
    }
}

#------------------------------------------------------------------------
# textutils::tag
#     tag a line of text
#------------------------------------------------------------------------
proc textutils::tag_pcline { txtwidget cursor } {
   set line    [ lindex [ split $cursor . ] 0 ]
   $txtwidget tag add pc_cursor $line.0 $line.end
}

#------------------------------------------------------------------------
# textutils::untag 
#     Untag a line of text
#------------------------------------------------------------------------
proc textutils::untag_pcline { txtwidget cursor } {
   set line [ lindex [ split $cursor . ] 0 ]
   $txtwidget tag remove pc_cursor $line.0 $line.end
}

#------------------------------------------------------------------------
# textutils::tag_breakpoint
#------------------------------------------------------------------------
proc textutils::tag_breakpoint { txtwidget cursor } {
   set line [ lindex [ split $cursor . ] 0 ]
   $txtwidget delete $line.0
   $txtwidget insert $line.0 "B"
   $txtwidget tag add breakpoint $line.0 $line.2
}

#------------------------------------------------------------------------
# textutils::untag_breakpoint
#------------------------------------------------------------------------
proc textutils::untag_breakpoint { txtwidget cursor } {
   set line [ lindex [ split $cursor . ] 0 ]
   $txtwidget delete $line.0
   $txtwidget insert $line.0 " "
   $txtwidget tag remove breakpoint $line.0 $line.2
}

#----------------------------------------------------------------------------- 
# text at startidx is a '$' (or a '.') which indicates the current PC
# calculate $+num or $-num
# current PC is obtained from the line
#----------------------------------------------------------------------------- 
proc textutils::branch_target { txtwin startidx } {
   set line [ lindex [ split $startidx . ] 0 ]

   set pc 0x[$txtwin get $line.2 "$line.2 wordend"]
   set addrop [$txtwin get "$startidx+1chars" ]
   set numidx [$txtwin index "$startidx+2chars" ]
   set offset [$txtwin get $numidx "$numidx wordend" ]

   switch -exact -- $addrop {
       "+" { set addr [add64 $pc $offset] }
       "-" { set addr [sub64 $pc $offset] }
       default { return {} }
   }

   set txt    [$txtwin get $startidx "$numidx wordend" ]

   return [list "$txt=" [format64 "%llX" $addr] ]
}
