#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2004
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  TrackAll.tcl
#------------------------------------------------------------------------

namespace eval TrackAll {
   variable flyover
   set flyover(initialized) 0
}

#------------------------------------------------------------------------
#
# TrackAll::Init -- Initalize the status window
#
#------------------------------------------------------------------------
proc TrackAll::Init {root sim} {
    global TrackAllSim TrackAllStart TrackAllEnd TrackAllLength

    set TrackAllSim($root) $sim
    set TrackAllCPUs($root) [$sim display cpus]

    button $root.reload -text "Reload" -command "TrackAll::Refresh $root"
    button $root.close -text "Close" -command "destroy $root"

    #text $root.pctrk -width 70 -height 14 -yscrollcommand "$root.pctrksy set" -font small_textbox_font -tabs "12c" -wrap none
    text $root.pctrk -width 70 -height 18 -yscrollcommand "$root.pctrksy set" -font textbox_font -tabs "12c" -wrap none
    scrollbar $root.pctrksy -orient vert -command "$root.pctrk yview"

    textutils::init_color_tags $root.pctrk

    table $root \
        $root.pctrk 0,0 -cspan 4 -fill both \
        $root.pctrksy 0,4 -cspan 1 -fill y \
        $root.reload   1,2 -fill x    \
        $root.close    1,3 -fill x
    table configure $root c* r* -resize none
    table configure $root r0 c0 -resize expand

    wm title $root "$sim: Track All PCs"

    TrackAll::Refresh $root

    #TrackAll::flyover_init $root $cpucmd

    return "TrackAll::Refresh"
}

#------------------------------------------------------------------------
#
# TrackAll::Refresh -- Refresh the contents of the status window
#
#------------------------------------------------------------------------
proc TrackAll::Refresh { root } {
    global TrackAllSim TrackAllCPU TrackAllStart TrackAllEnd
    global TrackAllUserLength TrackAllUserStart TrackAllLength
    global TrackAll


    if { ![ winfo ismapped $root ] } then {
        return
    }

    set sim $TrackAllSim($root)

    set user_force_reload 0

    set iMem "$root.pctrk"

    # inst_per_cpu should be power of 2
    set inst_around_pc 2
    set inst_around_pc [TclUtils::get_larger_power2 $inst_around_pc]
    set inst_around_pc [mul64 $inst_around_pc 4]
    #set mask [invert64 [expr $inst_around_pc-1]]
    #puts "mask is $mask"
    set bplist [$sim trigger display pc list]
    $iMem delete 1.0 end
    $iMem tag configure CURRENT -background green
    $iMem tag configure INACTIVE -foreground darkgray
    $iMem tag configure INACTIVE_CURRENT -foreground darkgray -background darkred
    $iMem tag configure STALLED_CURRENT -background orange

    for { set mcm 0 } { $mcm < [$sim display number_of_MCMs] } { incr mcm } {
        foreach cpustring [$sim mcm $mcm display tcl_cpulist] {

            array set cpuinfo $cpustring

            switch $cpuinfo(name) {
            spu     {
                        # Handle SPU case
                        set pc [$sim mcm $mcm spu $cpuinfo(number) display pc]

                        set start_pc [sub64 $pc $inst_around_pc]
                        if { $start_pc < 0 } {
                            set start_pc 0
                        }
                        set end_pc [add64 $start_pc [mul64 $inst_around_pc 2]]
                        set end_pc [add64 $end_pc 4]
                        set iaddr $start_pc
                        if { [string compare $cpuinfo(state) "on"] == 0 } {
                            set default_tag ""
                            set default_current_tag "CURRENT"
                        } elseif { [string compare $cpuinfo(state) "stalled"] == 0 } {
                            set default_tag ""
                            set default_current_tag "STALLED_CURRENT"
                        } else {
                            set default_tag "INACTIVE"
                            set default_current_tag "INACTIVE_CURRENT"
                        }
                        $iMem insert end "MCM: $mcm SPU: $cpuinfo(number) STATE: $cpuinfo(state)\n" $default_tag
                        while { [lessthan_u64 $iaddr $end_pc] } {
                            set value [$sim mcm $mcm spu $cpuinfo(number) display memory $iaddr 4]
                            set ascii [TclUtils::GenAscii $value]

                            set addr64 [format64 "%016llX" $iaddr]
                            if { [lsearch $bplist 0x$addr64] >= 0 } {
                                $iMem insert end "B " breakpoint
                            } else {
                                $iMem insert end "  "
                            }
                            set line [format "%s : %08X : %s : %s\t\t\n" $addr64 $value $ascii [$sim util spu_disasm $value]]
                            if { [compare64 $iaddr $pc] == 0 } {
                                $iMem insert end $line $default_current_tag
                            } else {
                                $iMem insert end $line $default_tag
                            }

                            set iaddr [add64 $iaddr 4]
                        }
                    }
            default {
                        # Handle PPC case
                        set pc [$sim mcm $mcm cpu $cpuinfo(number) display spr pc]

                        set start_pc [sub64 $pc $inst_around_pc]
                        set end_pc [add64 $pc $inst_around_pc]
                        set end_pc [add64 $end_pc 4]
                        set iaddr $start_pc
                        if { [string compare $cpuinfo(state) "on"] == 0 } {
                            set default_tag ""
                            set default_current_tag "CURRENT"
                        } elseif { [string compare $cpuinfo(state) "stalled"] == 0 } {
                            set default_tag ""
                            set default_current_tag "STALLED_CURRENT"
                        } else {
                            set default_tag "INACTIVE"
                            set default_current_tag "INACTIVE_CURRENT"
                        }
                        $iMem insert end "MCM: $mcm CPU: $cpuinfo(number) STATE: $cpuinfo(state)\n" $default_tag
                        while { [lessthan_u64 $iaddr $end_pc] } {
                            set rAddr [$sim mcm $mcm cpu $cpuinfo(number) itranslate $iaddr]
                            if { [catch {set value [$sim mcm $mcm memory display $rAddr 4]} memerr ] } {
                                set value "bad-addr"
                                set ascii "...."
                                set disasm "...."
                            } else {
                                set ascii [TclUtils::GenAscii $value]
                                set disasm [$sim util ppc_disasm $value $iaddr]
                                set value [format "%08X" $value]
                            }

                            set addr64 [format64 "%016llX" $iaddr]
                            if { [lsearch $bplist 0x$addr64] >= 0 } {
                                $iMem insert end "B " breakpoint
                            } else {
                                $iMem insert end "  "
                            }
                            set line [format "%s : %s : %s : %s : %s\t\t\n" $addr64 $rAddr $value $ascii $disasm]
                            if { [compare64 $iaddr $pc] == 0 } {
                                $iMem insert end $line $default_current_tag
                            } else {
                                $iMem insert end $line $default_tag
                            }

                            set iaddr [add64 $iaddr 4]
                        }
                    }
            }

        }
    }
    $iMem delete insert-1c end       ;# delete last newline char
}

#------------------------------------------------------------------------
#  TrackAll::create
#
#------------------------------------------------------------------------
proc TrackAll::create { widget sim } {

    variable control_nb
    global gvars

    if { ![ winfo exists $widget ] } then {

        toplevel $widget

        wm transient $widget .

        # Intercept window close and call the exit handler
        set top_widget [ winfo toplevel $widget ]
        wm protocol $top_widget WM_DELETE_WINDOW "TrackAll::_on_close $top_widget"
        bind $top_widget <Configure> "TrackAll::_save_geometry $top_widget"
        bind $top_widget <Key-F5> "user_refresh"

        set geom [ get_pref win_geom/cpanel "800x600+80+80" ]

        TrackAll::Init $widget $sim

        wm geometry $top_widget $geom

    } else {

        wm deiconify [ winfo toplevel $widget ]
    }

    return $widget
}

#------------------------------------------------------------------------
# TrackAll::_save_geometry
#------------------------------------------------------------------------
proc TrackAll::_save_geometry { top_widget } {

   set_pref win_geom/cpanel [ winfo geometry $top_widget ]
}

#------------------------------------------------------------------------
# TrackAll::_on_close
#------------------------------------------------------------------------
proc TrackAll::_on_close { top_widget } {
   wm withdraw $top_widget
}



#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc TrackAll::flyover_init {root cpucmd} {
   variable flyover

   if { !$flyover(initialized) } {

      set flyover(initialized) 1

      # Setup the tracking variables array (gprs and fprs)
      for { set i 0 } { $i <= 31 } { incr i } {
         set flyover(r$i) "display gpr $i"
         set flyover(f$i) "display fpr $i"
         set flyover(v$i) "display vmxr $i"
         #set flyover(R[ format %02d $i]) "display gpr $i"
         #set flyover(F[ format %02d $i]) "display fpr $i"
      }

      set sprlist [$cpucmd display spr list]

      foreach spr $sprlist {
         set flyover($spr) "display spr $spr"
         #set flyover([ string toupper $spr ]) "display spr $spr"
      }
   }

   flyover::register $root.pctrk "TrackAll::flyover_handler $root $cpucmd"
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc TrackAll::flyover_handler { root cpucmd txtwin startidx } {
   global TrackAll
   variable flyover

   set rtn [list]

   set chr [ $txtwin get $startidx ]
   switch -glob -- $chr {
      [A-Za-z]  {
                   # id/keyword
                   set txt [ $txtwin get $startidx "$startidx wordend" ]

                   # Now we have the target word under the mouse, see if it in our list of
                   # trackable variables
                   set cmd [ lindex [ array get flyover $txt ] 1 ]
                   if { $cmd != "" } then {
                      set txt "$txt = "
                      set val [eval $cpucmd $cmd]
                      set rtn [list $txt $val]
                   }
                }
      [0-9]     {
                    #
                    #  If over a number, look for $+number or number(reg)
                    #
                    set prevchar [$txtwin get "$startidx-2chars" ]
                    if {$prevchar == "$"} {
                        set startidx [$txtwin index "$startidx-2chars" ]
                        set rtn [ textutils::branch_target $txtwin $startidx ]
                    } else {
                        set endidx [ $txtwin index "$startidx wordend" ]
                        set nextchar [$txtwin get "$endidx" ]
                        if {$nextchar == "("} {
                            set rtn [ TrackAll::memory_address $sim $cpucmd $txtwin $startidx $endidx]
                        }
                    }
                }
      "$"       {
                    # $-num, $+num
                    set rtn [ textutils::branch_target $txtwin $startidx ]
                }
      "-"       -
      "+"       {
                    # $-num, $+num
                    set prevchar [$txtwin get "$startidx-1chars" ]
                    if {$prevchar == "$"} {
                        set startidx [$txtwin index "$startidx-1chars" ]
                        set rtn [ textutils::branch_target $txtwin $startidx ]
                    }
                }

   }

   return $rtn
}

#-----------------------------------------------------------------------------
# text at startidx is a number
# calculate num(reg)
#-----------------------------------------------------------------------------
proc TrackAll::memory_address { mcmcmd cpucmd txtwin startidx numendidx } {

   set regidx [$txtwin index "$numendidx+2chars" ]
   set regendidx [$txtwin index "$regidx wordend" ]
   set reg    [$txtwin get $regidx $regendidx ]
   set sign   [$txtwin get "$startidx-1chars" ]
   if {$sign == "-"} {
       set startidx [$txtwin index "$startidx-1chars" ]
   }
   set offset [$txtwin get $startidx $numendidx ]
   set txt    [$txtwin get $startidx "$regendidx+1chars" ]
   set addrs  [ get_indirect_addresses $cpucmd $offset $reg ]
   set effaddr  [format64 "%llX" [lindex $addrs 0]]
   set realaddr  [lindex $addrs 1]
   set data "?"
   if {$realaddr != "?"} {
       catch {
           set data  [string trim [ $mcmcmd memory display $realaddr 4 4 ] ]
       }
   }

   return [list "$txt=>\n" "$effaddr: $realaddr: $data" ]
}


#------------------------------------------------------------------
#
#  input is a sting of the form offset(Rn)
#  offset can be of the form 0x1234 or 1234 or -1234
#
#------------------------------------------------------------------
proc get_indirect_addr_text {cpucmd addrstring} {

   set addrs {}
   set ok [regexp -nocase "(\[-\[:alnum:]]+)\[(]R(\[\[:digit:]]+)\[)]"  $addr  Match offset regnum]
   if {$ok} {
       set addrs [ get_indirect_addr $cpucmd $offset $regnum ]
   }
   return $addrs
}

#------------------------------------------------------------------
#
#------------------------------------------------------------------
proc get_indirect_addresses {cpucmd offset regnum } {

   if {$regnum == 0} {
       set regval 0
   } else {
       set regval [$cpucmd display gpr $regnum]
   }
   set effaddr [add64 $regval $offset]

   if {[catch {
         set realaddr [$cpucmd dtranslate $effaddr]
      } ] == 1 } then {
      set realaddr "?"
   }

   return "$effaddr $realaddr"
}
