#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  triggers.tcl
#
#
#------------------------------------------------------------------------

proc Trigger_Window { sim } {
	set winname .triggers_${sim}

	# If the window exists, just raise it
	if { [winfo exists $winname] } {
		raise $winname
		return
	}

	toplevel $winname
    Trigger_Init $winname $sim
}

######################################################################
#
# Trigger_Init -- Initalize the status window
#
######################################################################
proc Trigger_Init {root sim} {
	global TriggerSim

	set TriggerSim($root) $sim

	button $root.refresh -text "Refresh" -command "Trigger_Refresh $root"
	button $root.close -text "Close" -command "destroy $root"

	listbox $root.triggers -width 75 -height 14 -yscrollcommand "$root.pctrksy set" -font listbox_font
	scrollbar $root.pctrksy -orient vert -command "$root.triggers yview"

	table $root \
		$root.triggers 0,0 -cspan 4 -fill both \
		$root.pctrksy 0,4 -cspan 1 -fill y \
		$root.refresh 1,2 \
		$root.close 1,3 
	table configure $root c* r* -resize none
	table configure $root r0 c0 -resize expand

	wm title $root "$sim Triggers"

	Trigger_Refresh $root
	return
}

######################################################################
#
# Trigger_Refresh -- Refresh the contents of the status window
#
######################################################################
proc Trigger_Refresh { root args } {
	global cycleAdvance TriggerSim

	set sim $TriggerSim($root)
	set triggers "$root.triggers"

	$triggers delete 0 end

    set num_mcms [ treesim::get_num_mcms $sim ]

    for {set mcm 0} {$mcm < $num_mcms} {incr mcm} {

        set prefix ""
        if {$num_mcms > 1} { 
            set prefix "MCM $mcm: "
        }

        $triggers insert end "$prefix  PC triggers"
        set trigger_list [$sim mcm $mcm trigger display pc list]
        foreach trig $trigger_list {
            set setting [$sim mcm $mcm trigger display pc addr $trig]
            $triggers insert end "$prefix    $trig : \{$setting\}"
        }
        $triggers insert end ""

        $triggers insert end "$prefix  ASSOC triggers"
        set trigger_list [split [$sim mcm $mcm trigger display assoc list] "\n"]
        foreach trig $trigger_list {
            $triggers insert end "$prefix    $trig "
        }

        $triggers insert end "$prefix  CONSOLE triggers"
        set trigger_list [split [$sim mcm $mcm trigger display console list] "\n"]
        foreach trig $trigger_list {
            $triggers insert end "$prefix    $trig "
        }
        $triggers insert end ""

        $triggers insert end "$prefix  SYSTEM MEMORY triggers"
        set trigger_list [split [$sim mcm $mcm trigger display memory system list] "\n"]
        foreach trig $trigger_list {
            $triggers insert end "$prefix    $trig "
        }
        $triggers insert end ""

        if { [ Config::is_defined "MCONFIG_SPU" ] } {
            SPUUtils::show_triggers $triggers $sim $mcm $prefix
        }
    }
}

proc object_load {sim args} {

    scan [lindex $args 0] " filename %s address %s" filename addr
    object dir [file dirname $filename]
    object load [file tail $filename]
    #puts "[$sim display cycles] OBJECT_LOAD filename $filename ADDR $addr"

    if { [string match "*linux*" [file tail $filename]] == 1 } {
        vmlinux_loaded $sim [file tail $filename]
    }
}

proc new_machine {args} {
    scan [lindex $args 0] " machine %s" machine
    $machine trigger set assoc OBJECT_LOAD "object_load $machine"
}

proc Triggers_Setup {} {
    trigger set assoc NEW_MACHINE new_machine
}
