#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : latch.tcl
##
## Purpose    : TVU Latch GUI constructs
##
## CVS INFO:
##
## $Id: latch.tcl,v 1.1 2006/03/05 03:18:10 mkistler Exp $
##
#####################################################
#####################################################

namespace eval TVU_Latch {
    variable version        "1.0"
    variable declared
    variable data
}

package provide TVU_Latch $TVU_Latch::version
package require TVU_Cell

####
# Create a new latch widget
#
# \param    parent      Parent widget
# \param    lay_name    Name of layout
# \param    name        Name of this latch (must be unique)
# \param    tobj        TVU object to associate
# \param    loc         TVU location to associate (Widget title)
#
proc TVU_Latch::Create {parent lay_name name tobj loc} {
    set full_name ${lay_name}_${name}
    if {[llength [array names TVU_Latch::declared $full_name]] != 0} {
        puts "TVU_Latch::Create -- Latch to create '$full_name' already exists"
        return -code error
    }

    # Create the latch
    set latch [TVU_Cell::Create $parent $lay_name $full_name "latch" $loc $tobj [list $loc]]
    grid $latch

    # Save relevant data
    set TVU_Latch::declared($full_name) 1
    set TVU_Latch::data(tvu,$full_name) $tobj
    set TVU_Latch::data(parent,$full_name) $parent
    set TVU_Latch::data(widget,$full_name) $latch
    set TVU_Latch::data(loc,$full_name) $loc

    return $latch
}

####
# Create a new latch widget by duplicating a previously created widget
#
# \param    pname   Name of the parent latch
# \param    name    Name of this latch (must be unique)
# \param    dup     Name of the latch to duplicate (must be unique)
# \param    loc     TVU location to associate (Widget title)
#
proc TVU_Latch::Dup {pname name dup loc} {
    set full_name ${pname}_${name}
    set dup_name ${pname}_${dup}
    if {[llength [array names TVU_Latch::declared $dup_name]] == 0} {
        puts "TVU_Latch::Dup -- Latch to duplicate '$dup_name' is unknown"
        return -code error
    }

    set frm [TVU_Latch::Create $TVU_Latch::data(parent,$dup_name) $pname $name $TVU_Latch::data(tvu,$dup_name) $loc]
    return $frm
}

####
# Clear a latch widget
#
# \param    name    Name of the latch to clear
#
proc TVU_Latch::Clear {name} {
    # Clear the latch
    TVU_Cell::Clear $name "latch"
}

####
# Clear all matching latches
#
# \param    pat     Pattern to match latch names
#
proc TVU_Latch::ClearMatching {pat} {
    # Clear matching latch
    foreach p [array names TVU_Latch::declared $pat] {
        TVU_Latch::Clear $p
    }
}
