#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : matchbox.tcl
##
## Purpose    : TVU Match Dialog
##
## CVS INFO:
##
## $Id: matchbox.tcl,v 1.1 2006/03/05 03:18:10 mkistler Exp $
##
#####################################################
#####################################################

namespace eval TVU_MatchBox {
    variable version        "1.0"
    variable data
}

package provide TVU_MatchBox $TVU_MatchBox::version
package require TVU_Cell

####
# Activate a list selection
#
proc TVU_MatchBox::Select {w lay_name cbk} {
    set sel_list [$w curselection]
    if {[llength $sel_list] > 0} {
        set first_sel [lindex $sel_list 0]
        set sel [$w get $first_sel]

        set trid [lindex $sel 1]
        set oldlite [TVU_Cell::GetHilite $lay_name $trid]
        TVU_Cell::SetHilite [list yellow darkcyan] $lay_name $trid

        eval {$cbk} [lindex $sel 0]

        TVU_Cell::SetHilite $oldlite $lay_name $trid
    }
}

####
# Perform a search
#
proc TVU_MatchBox::Search {full_name tobj} {
    set TVU_MatchBox::data(lst,$full_name) [$tobj db match $TVU_MatchBox::data(bcyc,$full_name) \
        $TVU_MatchBox::data(ecyc,$full_name) $TVU_MatchBox::data(pat,$full_name)]
}

####
# Create a new matchbox dialog
#
# \param    tobj    TVU object to associate
#
proc TVU_MatchBox::Create {lay_name spun tobj cbk} {
    # Determine the name of this dialog
    set full_name "matchbox_spu${spun}"
    set wgt_name ".$full_name"

    # Check to see if dialog already exists
    if {[winfo exists $wgt_name]} {
        switch -- [wm state $wgt_name] {
            normal {
                raise $wgt_name
            }
            iconic {
                wm deiconify $wgt_name
                raise $wgt_name
            }
        }
        return $wgt_name
    }

    # Create Toplevel dialog
    set dlg [toplevel $wgt_name]
    wm title $dlg "spu${spun} Regex Search"

    # Range and pattern text entries
    set ctl_frm [frame ${dlg}.ctrl -relief groove -borderwidth 2]

    set low_lbl [label ${ctl_frm}.low_lbl -text "From(t):"]
    set low_ent [entry ${ctl_frm}.low_ent -textvariable TVU_MatchBox::data(bcyc,$full_name) -relief sunken -width 15 -foreground black -background white]

    set hi_lbl  [label ${ctl_frm}.hi_lbl -text "To(t):"]
    set hi_ent  [entry ${ctl_frm}.hi_ent -textvariable TVU_MatchBox::data(ecyc,$full_name) -relief sunken -width 15 -foreground black -background white]

    set pat_lbl [label ${ctl_frm}.pat_lbl -text "Pattern:"]
    set pat_ent [entry ${ctl_frm}.pat_ent -textvariable TVU_MatchBox::data(pat,$full_name) -relief sunken -width 20 -foreground black -background white]

    set sch_btn [button ${ctl_frm}.sch_btn -text "Search" -command [list TVU_MatchBox::Search $full_name $tobj]]

    grid $low_lbl $low_ent $hi_lbl $hi_ent $pat_lbl $pat_ent $sch_btn

    # Search results listbox
    set box_frm [frame ${dlg}.box -relief groove -borderwidth 2]
    set lst [listbox ${box_frm}.lst -listvariable TVU_MatchBox::data(lst,$full_name) \
        -xscrollcommand [list ${box_frm}.xscroll set] -yscrollcommand [list ${box_frm}.yscroll set]]
    set xsb [scrollbar ${box_frm}.xscroll -orient horizontal -command [list ${lst} xview]]
    set ysb [scrollbar ${box_frm}.yscroll -orient vertical -command [list ${lst} yview]]
    grid $lst $ysb -sticky news
    grid $xsb -sticky news
    grid rowconfigure $box_frm 0 -weight 1
    grid columnconfigure $box_frm 0 -weight 1

    bind $lst <ButtonRelease-1> [list TVU_MatchBox::Select %W $lay_name $cbk]

    grid $ctl_frm -sticky ew
    grid $box_frm -sticky ew

    # Save relevant data
    set TVU_MatchBox::data(spu,$full_name) $spun
    set TVU_MatchBox::data(tvu,$full_name) $tobj

    return $dlg
}
