#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : pipe.tcl
##
## Purpose    : TVU Pipeline GUI constructs
##
## CVS INFO:
##
## $Id: pipe.tcl,v 1.1 2006/03/05 03:18:10 mkistler Exp $
##
#####################################################
#####################################################

namespace eval TVU_Pipe {
    variable version        "1.0"
    variable declared
    variable data
}

package provide TVU_Pipe $TVU_Pipe::version
package require TVU_Cell

####
# Create a new pipeline widget
#
# \param    parent      Parent widget
# \param    lay_name    Name of layout
# \param    name        Name of this pipe (must be unique)
# \param    tobj        TVU object to associate
# \param    loc         TVU location to associate (Widget title)
# \param    stages      List of stage names
#
proc TVU_Pipe::Create {parent lay_name name tobj loc stages} {
    set full_name ${lay_name}_${name}
    if {[llength [array names TVU_Pipe::declared $full_name]] != 0} {
        puts "TVU_Pipe::Create -- Pipe to create '$full_name' already exists"
        return -code error
    }

    # Frame for grouping purposes
    set frm [frame ${parent}.${full_name}]
    set tlbl [label ${frm}.title -text "$loc"]
    grid $tlbl

    # Create the stages
    foreach s $stages {
        set s_frm [TVU_Cell::Create $frm $lay_name $full_name $s $s $tobj [list $loc $s]]
        grid $s_frm
    }

    # Save relevant data
    set TVU_Pipe::declared($full_name) 1
    set TVU_Pipe::data(tvu,$full_name) $tobj
    set TVU_Pipe::data(parent,$full_name) $parent
    set TVU_Pipe::data(widget,$full_name) $frm
    set TVU_Pipe::data(loc,$full_name) $loc
    set TVU_Pipe::data(stages,$full_name) $stages

    return $frm
}

####
# Create a new pipeline widget by duplicating a previously created widget
#
# \param    pname   Name of the parent pipe
# \param    name    Name of this pipe (must be unique)
# \param    dup     Name of the pipe to duplicate (must be unique)
# \param    loc     TVU location to associate (Widget title)
#
proc TVU_Pipe::Dup {pname name dup loc} {
    set full_name ${pname}_${name}
    set dup_name ${pname}_${dup}
    if {[llength [array names TVU_Pipe::declared $dup_name]] == 0} {
        puts "TVU_Pipe::Dup -- Pipe to duplicate '$dup_name' is unknown"
        return -code error
    }

    set frm [TVU_Pipe::Create $TVU_Pipe::data(parent,$dup_name) $pname $name $TVU_Pipe::data(tvu,$dup_name) $loc $TVU_Pipe::data(stages,$dup_name)]
    return $frm
}

####
# Clear a pipeline widget
#
# \param    name    Name of the pipe to clear
#
proc TVU_Pipe::Clear {name} {
    # Clear the stages
    foreach s $TVU_Pipe::data(stages,$name) {
        TVU_Cell::Clear $name $s
    }
}

####
# Clear all matching pipelines
#
# \param    pat     Pattern to match pipeline names
#
proc TVU_Pipe::ClearMatching {pat} {
    # Clear matching pipes
    foreach p [array names TVU_Pipe::declared $pat] {
        TVU_Pipe::Clear $p
    }
}
