#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  value_expand.tcl
#
#
#------------------------------------------------------------------------
set value_expand_window_list {}

proc UpdateValueExpands { } {
	global value_expand_window_list value_expand_command

	foreach cw $value_expand_window_list {
		eval $value_expand_command($cw)
	}
}
proc ValueExpand_Add { winname command } {
	global value_expand_window_list value_expand_command

	if { [lsearch -exact $value_expand_window_list $winname] == -1 } {
		lappend value_expand_window_list $winname
		set value_expand_command($winname) $command

		return 1
	}
	return 0
}
proc ValueExpand_Remove { winname } {
	global value_expand_window_list value_expand_command
	set oldlist $value_expand_window_list
	set value_expand_window_list {}
	foreach win $oldlist {
		if { $win != $winname } {
			lappend value_expand_window_list $win
		}
	}
}

######################################################################
#
# ValueExpand_Init -- Initalize a window expanding on the passed
#		in array of integer values
#
######################################################################
proc ValueExpand_Init {root title int_array} {

	set num_ints [llength $int_array]

	label $root.intsLabel -text "Integers:"
	listbox $root.ints -width 32 -height 16 -font listbox_font
	label $root.asciiLabel -text "ASCII:"
	listbox $root.ascii -width 8 -height 16 -font listbox_font
	label $root.floatsLabel -text "Floats:"
	listbox $root.floats -width 24 -height 16 -font listbox_font

	for { set int_index 0 } { $int_index < $num_ints } { incr int_index } {

		# Get the value out of the array
		set value [lindex $int_array $int_index]

		# Fill Integer Listbox for this Integer
		$root.ints insert end [format "%2d : 0x%08X : %d" $int_index $value $value]

		# Fill Float Listbox for this Integer
		$root.floats insert end [format "%2d : %f" $int_index [uint32_to_float $value]]

		# Fill Ascii Listbox for this Integer
		set v0 [expr ( ($value >> 24) & 0xff )]
		set v1 [expr ( ($value >> 16) & 0xff )]
		set v2 [expr ( ($value >> 8) & 0xff )]
		set v3 [expr ( ($value >> 0) & 0xff )]
		if { ( $v0 < 0x20 ) || ( $v0 > 0x7e ) } { set v0 [expr 0x2a + 0] }
		if { ( $v1 < 0x20 ) || ( $v1 > 0x7e ) } { set v1 [expr 0x2a + 0] }
		if { ( $v2 < 0x20 ) || ( $v2 > 0x7e ) } { set v2 [expr 0x2a + 0] }
		if { ( $v3 < 0x20 ) || ( $v3 > 0x7e ) } { set v3 [expr 0x2a + 0] }
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 0] $v0]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 1] $v1]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 2] $v2]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 3] $v3]

	}

	table $root \
		$root.intsLabel		0,0 \
		$root.floatsLabel	0,1 \
		$root.asciiLabel	0,2 \
		$root.ints			1,0 \
		$root.floats		1,1 \
		$root.ascii			1,2

	table configure $root c* r* -resize none

	wm title $root $title

	# Set binding to remove it from list if it is destroyed.
	bind $root <Destroy> "ValueExpand_Remove $root"
	return
}
proc ValueExpand_Refresh {root int_array} {

	set num_ints [llength $int_array]

	$root.ints delete 0 end
	$root.ascii delete 0 end
	$root.floats delete 0 end

	for { set int_index 0 } { $int_index < $num_ints } { incr int_index } {

		# Get the value out of the array
		set value [lindex $int_array $int_index]

		# Fill Integer Listbox for this Integer
		$root.ints insert end [format "%2d : 0x%08X : %d" $int_index $value $value]

		# Fill Float Listbox for this Integer
		$root.floats insert end [format "%2d : %f" $int_index [uint32_to_float $value]]

		# Fill Ascii Listbox for this Integer
		set v0 [expr ( ($value >> 24) & 0xff )]
		set v1 [expr ( ($value >> 16) & 0xff )]
		set v2 [expr ( ($value >> 8) & 0xff )]
		set v3 [expr ( ($value >> 0) & 0xff )]
		if { ( $v0 < 0x20 ) || ( $v0 > 0x7e ) } { set v0 [expr 0x2a + 0] }
		if { ( $v1 < 0x20 ) || ( $v1 > 0x7e ) } { set v1 [expr 0x2a + 0] }
		if { ( $v2 < 0x20 ) || ( $v2 > 0x7e ) } { set v2 [expr 0x2a + 0] }
		if { ( $v3 < 0x20 ) || ( $v3 > 0x7e ) } { set v3 [expr 0x2a + 0] }
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 0] $v0]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 1] $v1]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 2] $v2]
		$root.ascii insert end [format "%2d : %c" [expr 4 * $int_index + 3] $v3]

	}
	return
}
