#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : Balloon.tcl
##
## Purpose    : Balloon Help Widget
##
## CVS INFO:
##
## $Id: balloon.tcl,v 1.2 2006/06/23 16:27:32 dmurrell Exp $
##
#####################################################
#####################################################

namespace eval Widget_Balloon {
    variable version        "1.0"
    variable data
}

package provide Widget_Balloon $Widget_Balloon::version

####
# Enable
#
proc Widget_Balloon::Enable {win state} {
    if {$state} {
        set Widget_Balloon::data($win,enable) 1
    } else {
        Widget_Balloon::Cancel $win
        set Widget_Balloon::data($win,enable) 0
    }
}

####
# Show
#
proc Widget_Balloon::Show {win} {
    if {$Widget_Balloon::data($win,enable)} {
        set w $Widget_Balloon::data($win,top)
        set msg $Widget_Balloon::data($w,mlbl)
        $msg configure -text $Widget_Balloon::data($win,mtext)

        set x [expr [winfo rootx $win] + 10]
        set y [expr [winfo rooty $win] + [winfo height $win]]
        wm geometry $w +$x+$y
        wm deiconify $w
        raise $w
    }

    unset Widget_Balloon::data($win,pending)
}

####
# Cancel
#
proc Widget_Balloon::Cancel {win} {
    if {[info exists Widget_Balloon::data($win,pending)]} {
        after cancel $Widget_Balloon::data($win,pending)
        unset Widget_Balloon::data($win,pending)
    }
    wm withdraw $Widget_Balloon::data($win,top)
}

####
# Pending
#
proc Widget_Balloon::Pending {win} {
    Widget_Balloon::Cancel $win
    set Widget_Balloon::data($win,pending) [after 1000 [list Widget_Balloon::Show $win]]
}

####
# Message
#
proc Widget_Balloon::Message {win msg} {
    set Widget_Balloon::data($win,mtext) $msg
}

####
# Associate
#
proc Widget_Balloon::Associate {w win {msg ""}} {
    Widget_Balloon::Message $win $msg
    bind $win <Enter> {Widget_Balloon::Pending %W}
    bind $win <Leave> {Widget_Balloon::Cancel %W}

    set Widget_Balloon::data($win,top) $w
    set Widget_Balloon::data($win,enable) 1
}

####
# Create a new Balloon widget
#
proc Widget_Balloon::Create {w} {
    # Create Toplevel frame
    if {! [winfo exists $w]} {
        set top [toplevel $w]

        # Create message area
        set frm [frame ${w}.frm -background black -borderwidth 1 -relief flat]
        set msg [label ${frm}.msg -background lightyellow -foreground black]
        pack $msg -side left -fill y
        pack $frm

        wm overrideredirect $w 1
        wm withdraw $w

        set Widget_Balloon::data($w,mlbl) $msg
    }
    return $w
}
