#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : combobox.tcl
##
## Purpose    : Combo Box Widget
##
## CVS INFO:
##
## $Id: combobox.tcl,v 1.1 2006/03/05 03:18:10 mkistler Exp $
##
#####################################################
#####################################################

namespace eval Widget_ComboBox {
    variable version        "1.0"
    variable data
}

package provide Widget_ComboBox $Widget_ComboBox::version

####
# Withdraw the Listbox
#
proc Widget_ComboBox::Withdraw {w} {
    set dialog $Widget_ComboBox::data(box,$w)
    set ent $Widget_ComboBox::data(ent,$w)
    bind $ent <Enter> {}
    wm withdraw $dialog
}

####
# Reposition the Listbox
#
proc Widget_ComboBox::Reposition {w child x y} {
    set dialog $Widget_ComboBox::data(box,$w)
    if {[winfo ismapped $w]} {
        set x [winfo rootx $child]
        set y [expr [winfo rooty $child] + [winfo height $child]]
    }
    wm geometry $dialog +$x+$y
}

####
# Popup the Listbox
#
proc Widget_ComboBox::Popup {w x y} {
    set dialog $Widget_ComboBox::data(box,$w)
    set tvar $Widget_ComboBox::data(tvar,$w)
    upvar #0 $tvar val

    set idx [lsearch $Widget_ComboBox::data(lvar,$w) $val]
    if {$idx != -1} {
        $Widget_ComboBox::data(lst,$w) selection set $idx
    }

    # Determine the posting position, relative to the entry widget
    set ent $Widget_ComboBox::data(ent,$w)
    Widget_ComboBox::Reposition $w $ent $x $y
    bind $ent <Enter> [list Widget_ComboBox::Reposition $w $ent %X %Y]

    # Post the listbox
    wm deiconify $dialog
    raise $dialog
}

####
# Populate ListBox
#
proc Widget_ComboBox::Add {w item} {
    $Widget_ComboBox::data(lst,$w) insert end $item
}

####
# Activate a list selection
#
proc Widget_ComboBox::Select {w parent tvar dialog} {
    global $tvar
    set sel [$w curselection]
    set $tvar [$w get $sel]
    $Widget_ComboBox::data(ent,$parent) configure -foreground black -background white
    Widget_ComboBox::Withdraw $parent
}

####
# Entry
#
proc Widget_ComboBox::SetEntry {w parent tvar x y} {
    upvar #0 $tvar val
    set idx [lsearch $Widget_ComboBox::data(lvar,$parent) $val]
    if {$idx == -1} {
        $w configure -foreground red -background black
        Widget_ComboBox::Popup $parent $x $y
    } else {
        $w configure -foreground black -background white
        Widget_ComboBox::Withdraw $parent
    }
}

####
# Create a new combobox widget
#
proc Widget_ComboBox::Create {parent name tvar} {
    set lower_name [string tolower $name]

    # Create Toplevel frame
    set w [frame ${parent}.${lower_name} -relief groove -borderwidth 2]

    # Create Options Listbox 
    set box [toplevel .${lower_name}_box]
    set frm [frame ${box}.frm -relief groove -borderwidth 2]
    set lst [listbox ${frm}.lst -selectmode single -listvariable Widget_ComboBox::data(lvar,$w) \
        -xscrollcommand [list ${frm}.xscroll set] -yscrollcommand [list ${frm}.yscroll set]]
    set xsb [scrollbar ${frm}.xscroll -orient horizontal -command [list ${lst} xview]]
    set ysb [scrollbar ${frm}.yscroll -orient vertical -command [list ${lst} yview]]
    grid $lst $ysb -sticky news
    grid $xsb -sticky news
    grid rowconfigure $frm 0 -weight 1
    grid columnconfigure $frm 0 -weight 1
    bind $lst <ButtonRelease-1> [list Widget_ComboBox::Select %W $w $tvar $box]
    pack $frm
    wm overrideredirect $box 1
    wm withdraw $box

    # Create entry and browse button
    set ent [entry ${w}.ent -textvariable $tvar -relief sunken -width 15 -foreground black -background white]
    set btn [button ${w}.btn -text "..."]
    bind $btn <ButtonRelease-1> [list Widget_ComboBox::Popup $w %X %Y]
    bind $ent <Key> [list $ent configure -foreground white -background black]
    bind $ent <Return> [list Widget_ComboBox::SetEntry %W $w $tvar %X %Y]
    grid $ent $btn

    # Save relevant data
    set Widget_ComboBox::data(tvar,$w) $tvar
    set Widget_ComboBox::data(box,$w) $box
    set Widget_ComboBox::data(lst,$w) $lst
    set Widget_ComboBox::data(ent,$w) $ent

    return $w
}
