
# #############################################################
# procedures to be called by triggers
# triggers include breakpoints.
# triggers specify a condition and a function.
# they call the function when the condition is true.
proc registers_switch {sim  args} {
    global thread_table tid_table sdr_table lhr_table lsr_table ldr_table lir_table bar_table bbr_table llr_table first_time

    if { [catch "array set arginfo $args" errs] == 0 } {
        set mcm $arginfo(mcm)
        set cpu $arginfo(cpu)
        set thread $arginfo(thread)
    } else {
        puts "@@@@@@@@@@break_point: failed to parse args with errors $errs"
        set mcm 0
        set cpu 0
        set thread 0
    }

    set old_thread [$sim mcm $mcm cpu $cpu thread $thread display gpr 3]
    set new_thread [$sim mcm $mcm cpu $cpu thread $thread display gpr 4]

#    puts "old_thread:$old_thread , new_thread:$new_thread"
    set ldr [$sim mcm $mcm cpu $cpu thread $thread display spr 5]
    set lir [$sim mcm $mcm cpu $cpu thread $thread display spr 6]
    set lsr [$sim mcm $mcm cpu $cpu thread $thread display spr 7]
    set lhr [$sim mcm $mcm cpu $cpu thread $thread display spr 10]
    set sdr [$sim mcm $mcm cpu $cpu thread $thread display spr 11]
    set tid [$sim mcm $mcm cpu $cpu thread $thread display spr 12]
    set llr [$sim mcm $mcm cpu $cpu thread $thread display spr 4]
    set bar [$sim mcm $mcm cpu $cpu thread $thread display spr 14]
    set bbr [$sim mcm $mcm cpu $cpu thread $thread display spr 15]

    #list init
    if { [info exist first_time] == 0 } {
        set first_time 1
        set thread_table [list 0]
    }
  
    set index1 [lsearch $thread_table $old_thread]
    set index2 [lsearch $thread_table $new_thread]

    set flag 1
    if { $index1 == -1} {
        lappend thread_table $old_thread
        set index1 [expr [llength $thread_table]-1]
    }
    if { $index2 == -1 } {
        lappend thread_table $new_thread
        set index2 [expr [llength $thread_table]-1]
        set flag 0
    }

    if { $flag == 0} {
        set new_tid 0
        set new_sdr 0
        set new_lhr 0
        set new_lsr 0
        set new_ldr 0
        set new_lir 0
        set new_bar 0
        set new_bbr 0
	set new_llr 0
    } else {
        set new_tid $tid_table($index2) 
        set new_sdr $sdr_table($index2)
        set new_lhr $lhr_table($index2)
        set new_lsr $lsr_table($index2)
        set new_ldr $ldr_table($index2)
        set new_lir $lir_table($index2)
        set new_bar $bar_table($index2)
        set new_bbr $bbr_table($index2)
	set new_llr $llr_table($index2)
    }
     
    set tid_table($index1) $tid
    set sdr_table($index1) $sdr
    set lhr_table($index1) $lhr
    set lsr_table($index1) $lsr
    set ldr_table($index1) $ldr
    set lir_table($index1) $lir
    set bar_table($index1) $bar
    set bbr_table($index1) $bbr
    set llr_table($index1) $llr

    $sim mcm $mcm cpu $cpu thread $thread set spr 5  $new_ldr
    $sim mcm $mcm cpu $cpu thread $thread set spr 6  $new_lir
    $sim mcm $mcm cpu $cpu thread $thread set spr 7  $new_lsr
    $sim mcm $mcm cpu $cpu thread $thread set spr 10 $new_lhr
    $sim mcm $mcm cpu $cpu thread $thread set spr 11 $new_sdr
    $sim mcm $mcm cpu $cpu thread $thread set spr 12 $new_tid
    $sim mcm $mcm cpu $cpu thread $thread set spr 4  $new_llr
    $sim mcm $mcm cpu $cpu thread $thread set spr 14 $new_bar
    $sim mcm $mcm cpu $cpu thread $thread set spr 15 $new_bbr
}


