# #############################################################
# procedures to be called by triggers
# triggers include breakpoints.
# triggers specify a condition and a function.
# they call the function when the condition is true.

proc bet_registers_switch {sim  args} {
    global thread_table trc_table trtr_table trtfn_table trtv_table trtva_table first_time

    if { [catch "array set arginfo $args" errs] == 0 } {
        set mcm $arginfo(mcm)
        set cpu $arginfo(cpu)
        set thread $arginfo(thread)
    } else {
        puts "@@@@@@@@@@ break_point: failed to parse args with errors $errs @@@@@@@@@"
        set mcm 0
        set cpu 0
        set thread 0
    }

    set old_thread [$sim mcm $mcm cpu $mcm:$cpu:$thread display gpr 3]
    set new_thread [$sim mcm $mcm cpu $mcm:$cpu:$thread display gpr 4]

    set trc [$sim mcm $mcm cpu $mcm:$cpu:$thread display spr trc_ro]
    set trtr [$sim mcm $mcm cpu $mcm:$cpu:$thread display spr trtr_ro]
    set trtfn [$sim mcm $mcm cpu $mcm:$cpu:$thread display spr trtfn_ro]
    set trtv [$sim mcm $mcm cpu $mcm:$cpu:$thread display spr trtv_ro]
    set trtva [$sim mcm $mcm cpu $mcm:$cpu:$thread display spr trtva_ro]

    if { [info exist first_time] == 0 } {
        set first_time 1
        set thread_table [list 0]
    }

    set index1 [lsearch $thread_table $old_thread]
    set index2 [lsearch $thread_table $new_thread]

    set flag 1

    if { $index1 == -1} {
        lappend thread_table $old_thread
        set index1 [expr [llength $thread_table]-1]
    }
    if { $index2 == -1 } {
        lappend thread_table $new_thread
        set index2 [expr [llength $thread_table]-1]
        set flag 0
    }

    if { $flag == 0} {
        set new_trc 0
        set new_trtr 0
        set new_trtfn 0
        set new_trtv 0
        set new_trtva 0
    } else {
        set new_trc $trc_table($index2) 
        set new_trtr $trtr_table($index2)
        set new_trtfn $trtfn_table($index2)
        set new_trtv $trtv_table($index2)
        set new_trtva $trtva_table($index2)
    }

    set trc_table($index1) $trc
    set trtr_table($index1) $trtr
    set trtfn_table($index1) $trtfn
    set trtv_table($index1) $trtv
    set trtva_table($index1) $trtva

    $sim mcm $mcm cpu $mcm:$cpu:$thread set spr trc_wo $new_trc
    $sim mcm $mcm cpu $mcm:$cpu:$thread set spr trtr_wo $new_trtr
    $sim mcm $mcm cpu $mcm:$cpu:$thread set spr trtfn_wo $new_trtfn
    $sim mcm $mcm cpu $mcm:$cpu:$thread set spr trtv_wo $new_trtv
    $sim mcm $mcm cpu $mcm:$cpu:$thread set spr trtva_wo $new_trtva
}
