#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2003
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  debug.tcl
#
#------------------------------------------------------------------------

namespace eval Debug {}

#
#	Debug support routines
#
proc stop_simulator { sim reason args } {
	set pc [$sim cpu 0 display spr pc]
	set ipc [$sim cpu 0 display spr srr0]
	set clock [$sim display cycles]
	simstop
	puts "\n\nSimulator Stopped By Trigger @ Cycle $clock"
	puts [format "        %20s : $reason" Reason]
    eval "array set trigger_info $args"
    foreach name [array names trigger_info] {
        puts [format "        %20s : $trigger_info($name)" $name]
    }
    puts "\n"
}
proc dsi { sim args } {
	set pc [$sim cpu 0 display spr pc]
	set ipc [$sim cpu 0 display spr srr0]
	set dar [$sim cpu 0 display spr dar]
	set dsisr [$sim cpu 0 display spr dsisr]
	set clock [$sim display cycles]
	puts "$clock: $pc: data storage fault trigger at PC:$ipc DATA ADDR:$dar DSISR:$dsisr"
	if { $dsisr & 0x08000000 } {
		puts "Simulator stopped at Protection Fault"
		simstop
	}
}
proc isi { sim args } {
	set pc [$sim cpu 0 display spr pc]
	set ipc [$sim cpu 0 display spr srr0]
	set clock [$sim display cycles]
	puts "$clock: $pc: inst storage fault trigger at PC:$ipc"
	#simstop
}
proc isegfault { sim args } {
	set pc [$sim cpu 0 display spr pc]
	set ipc [$sim cpu 0 display spr srr0]
	set clock [$sim display cycles]
	puts "$clock: $pc: inst segmentation fault trigger at PC:$ipc"
	puts "SLB is [$sim cpu 0 display slb]"
	#simstop
}
proc dsegfault { sim args } {
	set pc [$sim cpu 0 display spr pc]
	set ipc [$sim cpu 0 display spr srr0]
	set dar [$sim cpu 0 display spr dar]
	set clock [$sim display cycles]
	puts "$clock: $pc: data segmentation fault trigger at PC:$ipc DATA ADDR:$dar"
	puts "SLB is [$sim cpu 0 display slb]"
	#simstop
}
proc debug_addr_trans { sim } {
	$sim trigger set pc 0x300 "dsi $sim"
	$sim trigger set pc 0x400 "isi $sim"
	$sim trigger set pc 0x480 "isegfault $sim"
	$sim trigger set pc 0x380 "dsegfault $sim"
}

#------------------------------------------------------------------------
#  Debug::help_autoload
#       Auto register help commands associated with this file
#------------------------------------------------------------------------
proc Debug::help_autoload {} {

    register_help_command "debug_addr_trans"   "Enables debug address translation"
}
