#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2003
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  emitter_readers.tcl
#
#------------------------------------------------------------------------

namespace eval EmitterReaders {
    variable error_displayed 0
}

#------------------------------------------------------------------------
#  EmitterReaders::init_default
#       Starts up some default readers
#------------------------------------------------------------------------
proc EmitterReaders::init_default { simulator reader_name } {

    global env

    # Return if emitters are not enabled
    if { [ info commands ereader ] == "" } then {
        return
    }

    start_reader stripstats_live [pid] $env(PROFILER_DIR)/live-graph
    start_reader reader [pid]
}

#------------------------------------------------------------------------
#  EmitterReaders::start_stripstats
#       Starts up some default readers
#------------------------------------------------------------------------
proc EmitterReaders::start_stripstats { simulator reader_name } {

    global env

    # Return if emitters are not enabled
    if { [ info commands ereader ] == "" } then {
        return
    }

    simemit set "MSR" 1
    simemit set "LPIDR" 1
    simemit set "Pid_Creatd" 1
    simemit set "Pid_Killed" 1
    simemit set "Pid_Exited" 1
    simemit set "Header_Record" 1
    simemit set "Footer_Record" 1
    simemit set "Instructions" 1
    simemit set "TLB_Inst_Miss" 1
    simemit set "TLB_Data_Miss" 1
    simemit set "SLB_Data_Miss" 1
    simemit set "SLB_Data_Miss" 1
    simemit set "L1_ICache_Miss" 1
    simemit set "L1_DCache_Miss" 1
    simemit set "L2_Cache_Miss" 1
    simemit set "Memory_Write" 1
    simemit set "Memory_Read" 1
    simemit set "Bus_Wait" 1
    ereader expect 1
    $simulator caches set on
    start_reader stripstats_live [pid] $env(PROFILER_DIR)/live-graph
}

#------------------------------------------------------------------------
#  EmitterReaders::start_reader
#------------------------------------------------------------------------
proc EmitterReaders::start_reader { reader args } {

    global env
    variable error_displayed

    set fq_reader $env(EXEC_DIR)/emitter/$reader

    if { [ info commands ereader ] != "" } then {
        if { ![ file exists $fq_reader ] } then {
            puts "EMITTER: Reader '$reader' was not found!"
        } elseif { [ catch { eval ereader start $fq_reader $args } ] } then {
            puts "EMITTER: Reader '$reader' failed to start!"
        }
    } else {
        if { $error_displayed == 0 } then {
            puts "EMITTER: No emitter support detected."
            set error_displayed 1
        }
    }
}

#------------------------------------------------------------------------
#  EmitterReaders::help_autoload
#       Auto register help commands associated with this file
#------------------------------------------------------------------------
proc EmitterReaders::help_autoload {} {

    register_help_command "EmitterReaders::init_default"    "Initialize some default emitter readers"
    register_help_command "EmitterReaders::start_reader"    "Start an emitter reader"
}
