#bltdebug 100

array set cursors {
    w left_side
    e right_side
    n top_side
    s bottom_side
    sw bottom_left_corner
    ne top_right_corner
    se bottom_right_corner
    nw top_left_corner
}


array set pageInfo {
    gripSize         8
    scale            0.25
    radioFont        -*-helvetica-medium-r-*-*-11-120-*-*-*-*-*-*
    labelFont        -*-helvetica-bold-r-*-*-12-120-*-*-*-*-*-*
    printCmd	     "lpr"
    printFile        "out.ps"
    printTo          "printCmd"
    printDecorations 0
}


proc SetUnits { units }  {
    global pageInfo
    switch -glob $units {
	"i*"    { set pageInfo(uscale) [winfo fpixels . 1i]  }
	"c*"    { set pageInfo(uscale) [winfo fpixels . 1c] }
	default { error "unknown unit \"$units\"" }
    }
    set pageInfo(units) [string index $units 0]
}


proc ConvertUnits { value } {
    global pageInfo
    set value [expr double($value) / $pageInfo(uscale)]
    return [format "%.1f%s" $value $pageInfo(units)]
}


proc SetCanvasSize { canvas width height } {
    global pageInfo

    set width [winfo pixels . $width]
    set height [winfo pixels . $height]
    $canvas configure -width $width -height $height
}

proc SetCanvasOrientation { canvas } {
    global pageInfo
    set width $pageInfo(paperWidth)
    set height $pageInfo(paperHeight)
    SetCanvasSize $canvas $width $height
}


proc GetPsOptions { graph } {
    global pageInfo

    foreach opt [$graph postscript configure] {
	set pageInfo([lindex $opt 0]) [lindex $opt 4]
    }
}

proc SetOutline { canvas } {
    global pageInfo
    foreach var { gripSize xMin yMin xMax yMax } {
	set $var $pageInfo($var)
    }
    set xMid [expr ($xMax + $xMin - $gripSize) * 0.5]
    set yMid [expr ($yMax + $yMin - $gripSize) * 0.5]
    $canvas coords image $xMin $yMin
    $canvas itemconfigure image \
	-width [expr $xMax - $xMin] -height [expr $yMax - $yMin]
    $canvas coords nw \
	$xMin $yMin [expr $xMin + $gripSize] [expr $yMin + $gripSize]
    $canvas coords se \
	[expr $xMax - $gripSize] [expr $yMax - $gripSize] $xMax $yMax
    $canvas coords ne \
	[expr $xMax - $gripSize] [expr $yMin + $gripSize] $xMax $yMin
    $canvas coords sw \
	$xMin $yMax [expr $xMin + $gripSize] [expr $yMax - $gripSize]
    SetCanvasOrientation $canvas
    $canvas coords n \
	$xMid $yMin [expr $xMid + $gripSize] [expr $yMin + $gripSize]
    $canvas coords s \
	$xMid [expr $yMax - $gripSize] [expr $xMid + $gripSize] $yMax
     $canvas coords e \
 	[expr $xMax - $gripSize] $yMid $xMax [expr $yMid + $gripSize]
     $canvas coords w \
 	$xMin $yMid [expr $xMin + $gripSize] [expr $yMid + $gripSize]
}

proc CreateOutline { canvas } {
    global pageInfo
    foreach var { gripSize xMin yMin xMax yMax } {
	set $var $pageInfo($var)
    }
    if { ![bitmap exists pattern8] } {
	bitmap define pattern8 { {8 8} {ff 00 ff 00 ff 00 ff 00 } }
    }
    $canvas create eps $xMin $yMin \
	-tags "outline image" \
	-width [expr $xMax - $xMin] \
	-height [expr $yMax - $yMin]
    set pageInfo(image) [image create photo]
    $pageInfo(graph) snap $pageInfo(image)
    $canvas itemconfigure image -image $pageInfo(image)
}

proc ComputePlotGeometry { graph } {
    global pageInfo

    GetPsOptions $graph
    set width [winfo width $graph]
    set height [winfo height $graph]
    if { $pageInfo(-width) > 0 } {
	set width $pageInfo(-width)
    }
    if { $pageInfo(-height) > 0 } {
	set height $pageInfo(-height)
    }

    set left [lindex $pageInfo(-padx) 0]
    set right [lindex $pageInfo(-padx) 1]
    set top [lindex $pageInfo(-pady) 0]
    set bottom [lindex $pageInfo(-pady) 1]
    set padx [expr $left + $right]
    set pady [expr $top + $bottom]

    if { $pageInfo(-paperwidth) > 0 } {
	set paperWidth $pageInfo(-paperwidth)
    } else {
	set paperWidth [expr $width + $padx]
    }
    if { $pageInfo(-paperheight) > 0 } {
	set paperHeight $pageInfo(-paperheight)
    } else {
	set paperHeight [expr $height + $pady]
    }
    if { $pageInfo(-landscape) } {
	set temp $paperWidth
	set paperWidth $paperHeight
	set paperHeight $temp
    }

    set scale 1.0
    if { $pageInfo(-maxpect) } {
	set xScale [expr ($paperWidth - $padx) / double($width)]
	set yScale [expr ($paperHeight - $pady) / double($height)]
	set scale [expr min($xScale,$yScale)]
	set bboxWidth [expr round($width * $scale)]
	set bboxHeight [expr round($height * $scale)]
    } else {
	if { ($width + $padx) > $paperWidth } {
	    set width [expr $paperWidth - $padx]
	}
	if { ($height + $pady) > $paperHeight } {
	    set height [expr $paperHeight - $pady]
	}
	set bboxWidth $width
	set bboxHeight $height
    }
    set x $left
    set y $top
    if { $pageInfo(-center) } {
	if { $paperWidth > $bboxWidth }  {
	    set x [expr ($paperWidth - $bboxWidth) / 2]
	}
	if { $paperHeight > $bboxHeight } {
	    set y [expr ($paperHeight - $bboxHeight) / 2]
	}
    }
    set pageInfo(xMin) [expr $x * $pageInfo(scale)]
    set pageInfo(yMin) [expr $y * $pageInfo(scale)]
    set pageInfo(xMax) [expr ($x + $bboxWidth) * $pageInfo(scale)]
    set pageInfo(yMax) [expr ($y + $bboxHeight) * $pageInfo(scale)]
    set pageInfo(paperHeight) [expr $paperHeight * $pageInfo(scale)]
    set pageInfo(paperWidth) [expr $paperWidth * $pageInfo(scale)]
}

proc PsDialog { graph } {
    global pageInfo

    set pageInfo(graph) $graph
    set top $graph.top
    toplevel $top
    option add *graph.top*Radiobutton.font $pageInfo(radioFont)
    GetPsOptions $graph
    ComputePlotGeometry $graph
    set canvas $top.layout
    canvas $canvas -confine yes \
	-width $pageInfo(paperWidth) -height $pageInfo(paperHeight) -bg gray \
	-bd 2 -relief sunken
    CreateOutline $canvas
    SetCanvasOrientation $canvas

    # Main title, canvas to show image of graph

    label $top.titleLabel -text "PostScript Options"
    table $top \
	0,0 $top.titleLabel -cspan 3 \
	1,0 $canvas -cspan 3
    table configure $top r0 -pady {8 8}

    # Orientation

    set row 2
    set col 0
    table configure $top r$row -pady { 8 0 }
    table configure $top c$col -padx { 15 0 }

    label $top.orientLabel -text "Orientation"
    radiobutton $top.portrait -text "Portrait" -value "0" \
	-variable pageInfo(-landscape) -command "ApplyPs"
    radiobutton $top.landscape -text "Landscape" -value "1" \
	-variable pageInfo(-landscape) -command "ApplyPs"
    table $top \
	$row,$col   $top.orientLabel -anchor w \
	$row+1,$col $top.portrait    -anchor w \
	$row+2,$col $top.landscape   -anchor w

    # Size

    incr col 1
    table configure $top c$col -padx { 15 0 }

    label $top.sizeLabel -text "Size"
    radiobutton $top.default -text "Default" -value "default" \
	    -variable pageInfo(plotSize) \
	    -command "SetPlotSize"
    radiobutton $top.maxpect -text "Max Aspect" -value "maxpect" \
	    -variable pageInfo(plotSize) \
	    -command "SetPlotSize"
    radiobutton $top.resize -text "Specify" -value "resize" \
	    -variable pageInfo(plotSize) \
	    -command "SizeDialog $graph {Specify Plot Size}"
    table $top \
	    $row,$col   $top.sizeLabel -anchor w \
	    $row+1,$col $top.default   -anchor w \
	    $row+2,$col $top.maxpect   -anchor w \
	    $row+3,$col $top.resize    -anchor w

    # Position

    incr col 1
    table configure $top c$col -padx { 15 15 }

    set pageInfo(oldPadX) $pageInfo(-padx)
    set pageInfo(oldPadY) $pageInfo(-pady)

    label $top.posLabel -text "Position"
    set pageInfo(position) $pageInfo(-center)
    radiobutton $top.center -text "Center" -value "1" \
	    -variable pageInfo(position) -command {
	set pageInfo(-center) 1
	CenterPlot
    }
    radiobutton $top.specify -text "Specify" -value "0" \
	    -variable pageInfo(position) -command {
	set pageInfo(-center) 0
	MoveDialog
    }
    table $top \
	    $row,$col   $top.posLabel -anchor w \
	    $row+1,$col $top.center   -anchor w \
	    $row+2,$col $top.specify  -anchor w

    # Print option:  with or without decorations

    incr row 4
    table configure $top r$row -pady {8 0}
    checkbutton $top.printDecor -text "Print background colors & decorations" \
	    -variable pageInfo(printDecorations)
    table $top \
	    $row,0 $top.printDecor -anchor w -cspan 3

    # Print command/file

    incr row 1
    table configure $top r$row -pady {8 0}
    label $top.printLabel -text "Print"
    radiobutton $top.toCmd -text "Command" -value "printCmd" \
	    -variable pageInfo(printTo) -command "
    $top.fileEntry configure -textvariable pageInfo(printCmd)
    "
    radiobutton $top.toFile -text "To file" -value "printFile" \
	    -variable pageInfo(printTo) -command "
    $top.fileEntry configure -textvariable pageInfo(printFile)
    "
    entry $top.fileEntry

    table $top \
	$row,0   $top.printLabel -anchor w \
	$row+1,0 $top.toCmd      -anchor w \
	$row+2,0 $top.toFile     -anchor w \
	$row+3,0 $top.fileEntry  -anchor w -fill x -cspan 3

    # Buttons for print/cancel

    incr row 4
    table configure $top r$row -pady { 8 8 }

    frame $top.buttons -relief flat

    button $top.buttons.print    -text "Print"    -command "
        PrintPs $graph
        destroy $top
    "
    pack $top.buttons.print -side left -padx 4
    button $top.buttons.cancel   -text "Cancel"  -command "destroy $top"
    pack $top.buttons.cancel -side left -padx 4
    table $top \
	    $row,0 $top.buttons -cspan 3 -fill x

    SetUnits "inches"
    foreach label [info commands $top.*Label] {
	$label configure -font $pageInfo(labelFont) -padx 4

    }

    # Set the default values of the radio buttons

    $top.portrait invoke
    $top.default invoke
    $top.center invoke
    $top.toCmd invoke
}

proc PrintPs { graph } {
    global pageInfo

    set psWidth  [expr ($pageInfo(xMax) - $pageInfo(xMin)) / $pageInfo(scale)]
    set psHeight [expr ($pageInfo(yMax) - $pageInfo(yMin)) / $pageInfo(scale)]

    if {$pageInfo(-landscape)} {
	# Landscape mode
	set psPadY [expr ($pageInfo(paperWidth) - $pageInfo(xMax)) / $pageInfo(scale)]
	set psPadX [expr $pageInfo(yMin) / $pageInfo(scale)]
    } else {
	# Portrait mode
	set psPadX [expr $pageInfo(xMin) / $pageInfo(scale)]
	set psPadY [expr $pageInfo(yMin) / $pageInfo(scale)]
    }

    if {$pageInfo(-maxpect)} {
	$graph postscript configure \
		-width $psWidth \
		-height $psHeight \
		-center 1 \
		-maxpect 1 \
		-landscape $pageInfo(-landscape) \
		-decorations $pageInfo(printDecorations)
    } else {
	$graph postscript configure \
		-width $psWidth \
		-height $psHeight \
		-center 0 \
		-maxpect 0 \
		-padx $psPadX \
		-pady $psPadY \
		-landscape $pageInfo(-landscape) \
		-decorations $pageInfo(printDecorations)
    }

    if {$pageInfo(printTo) == "printFile"} {

	# Print to postscript file

	$graph postscript output "$pageInfo(printFile)"
	puts stdout "wrote file \"$pageInfo(printFile)\"."
	flush stdout

    } else {

	# Print command, via pipe

	set outpipe [open "| $pageInfo(printCmd)" w]
	set ps [$graph postscript output]
	puts $outpipe $ps nonewline
	close $outpipe
    }
}

proc ApplyPs { } {
    global pageInfo

    set graph $pageInfo(graph)
    foreach option [$graph postscript configure] {
	set var [lindex $option 0]
	set old [lindex $option 4]
	if { [catch {$graph postscript configure $var $pageInfo($var)}] != 0 } {
	    $graph postscript configure $var $old
	    set pageInfo($var) $old
	}
    }
    ComputePlotGeometry $graph
    foreach var { -paperheight -paperwidth -width -height } {
	set pageInfo($var) [ConvertUnits $pageInfo($var)]
    }
    SetOutline $graph.top.layout
}

proc StartChange { w delta } {
    ChangeSize $w $delta
    global pageInfo
    set pageInfo(afterId) [after 300 RepeatChange $w $delta]
}

proc RepeatChange { w delta } {
    ChangeSize $w $delta
    global pageInfo
    set pageInfo(afterId) [after 100 RepeatChange $w $delta]
}

proc EndChange { w } {
    global pageInfo
    after cancel $pageInfo(afterId)
}

proc ChangeSize { w delta } {
    set f [winfo parent $w]
    set value [$f.entry get]
    set value [expr $value + $delta]
    if { $value < 0 } {
	set value 1
    }
    $f.entry delete 0 end
    $f.entry insert 0 $value
}

proc MakeSizeAdjustor { w label var } {
    frame $w
    label $w.label -text $label
    button $w.plus -text "+" -padx 1 -pady 0 -font \*symbol\*
    entry $w.entry -width 6 -textvariable "pageInfo($var)"
    button $w.minus -text "-" -padx 1 -pady 0 -font \*symbol\*
    label $w.units -text "in"
    bind $w.plus <ButtonPress-1>  { StartChange %W 0.1}
    bind $w.plus <ButtonRelease-1> { EndChange %W }
    bind $w.minus <ButtonPress-1>  { StartChange %W -0.1}
    bind $w.minus <ButtonRelease-1> { EndChange %W }
    table $w \
	0,1 $w.label \
	1,1 $w.entry -rspan 2 -fill y \
	1,0 $w.minus -padx 2 -pady 2 \
	2,0 $w.plus -padx 2 -pady { 0 2 } \
	1,2 $w.units -rspan 2 -fill y

}


proc SizeDialog { graph title } {
    global pageInfo
    set top .plotSize
    if { [winfo exists $top] } {
	return
    }
    toplevel $top
    label $top.title -text $title
    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.ok -text "Ok" -command "ApplyPs; destroy $top"
    MakeSizeAdjustor $top.plotWidth "Width" -width
    MakeSizeAdjustor $top.plotHeight "Height" -height
    table $top \
	0,0 $top.title -cspan 2 \
	1,0 $top.plotWidth \
	1,1 $top.plotHeight \
	2,0 $top.cancel -pady 4 -padx 4 -width 1i \
	2,1 $top.ok -pady 4 -padx 4 -width 1i
    set width [winfo fpixels . $pageInfo(-width)]
    set height [winfo fpixels . $pageInfo(-height)]
    if { $width == 0 } {
	set width [expr ($pageInfo(xMax) - $pageInfo(xMin)) / $pageInfo(scale)]
	set pageInfo(-width) [ConvertUnits $width]
    }
    if { $height == 0 } {
	set height [expr ($pageInfo(yMax) - $pageInfo(yMin)) / $pageInfo(scale)]
	set pageInfo(-height) [ConvertUnits $height]
    }
    set pageInfo(-maxpect) 0
}

proc SetPlotSize { } {
    global pageInfo
    set graph $pageInfo(graph)
    switch $pageInfo(plotSize) {
	default {
	    set pageInfo(-width) 0
	    set pageInfo(-height) 0
	    set pageInfo(-maxpect) 0
	    set pageInfo(-padx) $pageInfo(oldPadX)
	    set pageInfo(-pady) $pageInfo(oldPadY)
	} maxpect {
	    set pageInfo(-width) 0
	    set pageInfo(-height) 0
	    set pageInfo(-maxpect) 1
	    set pageInfo(-padx) $pageInfo(oldPadX)
	    set pageInfo(-pady) $pageInfo(oldPadY)
	} resize {
	    set pageInfo(-maxpect) 0
	}
    }
    ApplyPs
}


proc PaperSizeDialog { title } {
    set top .paperSize
    if { [winfo exists $top] } {
	return
    }
    toplevel $top
    label $top.title -text $title
    MakeSizeAdjustor $top.width "Width" -paperwidth
    MakeSizeAdjustor $top.height "Height" -paperheight
    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.ok -text "Ok" -command "ApplyPs; destroy $top"
    table $top \
	0,0 $top.title -cspan 2 \
	1,0 $top.width \
	1,1 $top.height \
	2,0 $top.cancel -pady 4 -padx 4 -width 1i \
	2,1 $top.ok -pady 4 -padx 4 -width 1i
}

proc CenterPlot { } {
    global pageInfo

    set pageInfo(-padx) $pageInfo(oldPadX)
    set pageInfo(-pady) $pageInfo(oldPadY)
    ApplyPs
}

proc ShowPageInfo {} {
    global pageInfo
    puts stdout "pageInfo() ..."
    set searchId [array startsearch pageInfo]
    while 1 {
	set name [array nextelement pageInfo $searchId]
	if {"" == $name} {
	    break
	}
	puts stdout "pageInfo($name) \"$pageInfo($name)\""
    }
    array donesearch pageInfo $searchId
    flush stdout
}
