########################################################
#
# _read_emitter.tcl
#
#   Tcl procs and data for reading "stripstats" files
#   generated by Mambo-PPC emitter function.
#
########################################################

# Global variables used to hold the various vectors

set column_count 0
set MAX_COLS 300

for {set col 0} {$col < $MAX_COLS} {incr col} {
    eval "vector create V$col"
}

##############################################
#
#   WriteVec array-no index new-value
#
#   Writes a value into specified vector
#
##############################################

proc WriteVec {i j val} {
    global MAX_COLS
    for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }

    eval "$i [set V$i($j) $val]"
}

###############################################
#
#   ReadVec array-no index
#
#   Reads a value from specfied vector
#
###############################################

proc ReadVec {i j} {
    global MAX_COLS
    for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }

    eval "$i [return V$i($j)]"
}

#############################################
#
#   VecCmd array-no command
#
#   Issues a command to specified vector
#
#############################################

proc VecCmd {i cmd} {
    global MAX_COLS
    for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }

    eval "return [V$i $cmd]"
}

###################################################
#
#   VecReinit array-no
#
#   Reinitializes specified vector
#
###################################################

proc VecReinit {i} {
    set vec "V$i"
    global $vec

    if [array exists $vec] {
	vector destroy $vec
    }
    vector create $vec
}


###############################################################
#
# ParseStrip
#
# The following procedure will read a stripstats file and
# set the following tcl variables accordingly:
#
# scalar "column_count" contains number of columns
#
# array "column_names()" contains name of data for each
#    column, indexed by column number
#
# array "column_indices()" contains column number for each
#    column, indexed by column name
#
# vectors V0, V1, ... contain the data for each column
#
# "flowControl", if specified, must be a list containing
#    these elements:
#        o number of lines (x-values) to return in initial read
#        o number of lines to be returned on each subsequent read
#        o number of seconds to wait between reads
#
###############################################################

set parseStrip(afterID) -1

proc ParseStrip {file {flowControl {}}} {
    global column_names column_indices column_count
    global parseStrip

    global MAX_COLS
	for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }
    puts "Parsing strip file '$file'\n";

    if {$parseStrip(afterID) != -1} {
	after cancel $parseStrip(afterID)
    }

    array set parseStrip {
	fileName          $file
	row               0
	initialLines      0
	subsequentLines   0
	waitTime          0
    }

    if {$flowControl != {}} {
	set parseStrip(initialLines) [lindex $flowControl 0]
	set parseStrip(subsequentLines) [lindex $flowControl 1]
	set parseStrip(waitTime) [lindex $flowControl 2]
    }

    if {[expr ! [file readable $file]]} {
	puts stdout "File $file not readable."
	return
    }

    set parseStrip(fileID) [open $file "r"]
    set get_data 0
    while { [gets $parseStrip(fileID) data] >= 0 } {
	if { $get_data == 1 } {
	    if { [string index $data 0] >= "0" &&
	    [string index $data 0] <= "9" } {
		set temp [split $data]
		set vals ""
		foreach x $temp {
		    if {$x != ""} {
			lappend vals $x
		    }
		}

		if { [llength $vals] != $column_count } {
		    puts [concat "Error: Number of values on row $parseStrip(row):[llength $vals]" \
			    "!= number of columns:$column_count"]
		    return
		}

		for {set col 0} {$col < $column_count} {incr col} {
		    set val [expr [lindex $vals $col] + 0]
		    WriteVec $col ++end $val
		}
		incr parseStrip(row)
		if {$parseStrip(initialLines) && [expr $parseStrip(row) >= $parseStrip(initialLines)]} {
		    set $parseStrip(afterID) [after [expr round($parseStrip(waitTime) * 1000)] {ParseStripSubsequent}]
		    return
		}

	    } else {
		puts "End of data reached\n"
		return
	    }
	} else {
	    set names [split $data]
	    if { [string compare [lindex $names 0] "INST"] == 0 } {
		set get_data 1
		set col 0
		foreach c $names {
		    if {$c != ""} {
			puts stdout "column name <$c>"
			set column_names($col) $c
			set column_indices($c) $col
			incr col
		    }
		}
		set column_count $col
		puts stdout "column count is $column_count"
	    }
	}

    }

    close $parseStrip(fileID);
}

proc ParseStripSubsequent {} {
    global column_names column_indices column_count
    global parseStrip
    global MAX_COLS
	for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }

    set maxRow [expr $parseStrip(row) + $parseStrip(subsequentLines)]

    while { [gets $parseStrip(fileID) data] >= 0 } {
	if { [string index $data 0] >= "0" &&
	[string index $data 0] <= "9" } {
	    set temp [split $data]
	    set vals ""
	    foreach x $temp {
		if {$x != ""} {
		    lappend vals $x
		}
	    }

	    if { [llength $vals] != $column_count } {
		puts [concat "Error: Number of values on row $parseStrip(row):[llength $vals]" \
			"!= number of columns:$column_count"]
		return
	    }

	    for {set col 0} {$col < $column_count} {incr col} {
		set val [expr [lindex $vals $col] + 0]
		WriteVec $col ++end $val
	    }
	    incr parseStrip(row)
	    if {$parseStrip(row) >= $maxRow} {
		after [expr round($parseStrip(waitTime) * 1000)] {ParseStripSubsequent}
		return
	    }
	} else {
	    puts "End of data reached\n"
	    close $parseStrip(fileID)
	    return
	}

    }
}

###############################################################
#
# ParseStripLive
#
# The following procedure will stripstats data from a running
# emitter reader (profiler)
#
# scalar "column_count" contains number of columns
#
# array "column_names()" contains name of data for each
#    column, indexed by column number
#
# array "column_indices()" contains column number for each
#    column, indexed by column name
#
# vectors V0, V1, ... contain the data for each column
#
# "flowControl", if specified, must be a list containing
#    these elements:
#        o number of lines (x-values) to return in initial read
#        o number of lines to be returned on each subsequent read
#        o number of seconds to wait between reads
#
###############################################################

proc ParseStripLive {} {
    global column_names column_indices column_count
    global MAX_COLS
	for {set col 0} {$col < $MAX_COLS} {incr col} {
        eval "global V$col"
    }

	set col 0
	set coldata [stripdata]
	foreach c [columnnames] {
	    if {$c != ""} {
		set column_names($col) $c
		set column_indices($c) $col
		V$col set [lindex $coldata $col]
		incr col
	    }
	}
	set column_count $col
}
