#package require BLT
#namespace import blt::*


######################################################################
#
# CommaInsert -- Put punctuating commas into numbers, so that
#                large integers can be parsed by eye.
#                1000000000  becomes 1,000,000,000
#
######################################################################

proc CommaInsert {val} {
    global graphOptions

    set retval ""

    # If $val formatted with "%f" contains a non-zero fraction part,
    # don't try to insert commas.  Just return the value unchanged.

    set sign ""
    if {$val < 0} {
	set sign "-"
	set val [expr 0 - $val]
    }

    set val [format "%.0f" $val]

    set lv [string length $val]
    if {$lv < 4} {
	return "$sign$val"
    }
    set ex [expr $lv % 3]
    if {$ex == 2} {
	set retval [string range $val 0 1]
	append retval ","
    } elseif {$ex == 1} {
	set retval [string range $val 0 0]
	append retval ","
    }
    while {$ex <= $lv - 3} {
	append retval [string range $val $ex [expr $ex + 2]]
	if {$ex < $lv - 3} {
	    append retval ","
	}
	set ex [expr $ex + 3]
    }
    return "$sign$retval"
}

######################################################################
#
# ProcBox_Init -- Initalize the process tracking hierbox
#
######################################################################
proc ProcBox_Init {root} {
	global refreshDelayMS ProcBox_Root

	set ProcBox_Root "$root"

	label ${ProcBox_Root}iCountLabel -text "Instruction Count :"
	label ${ProcBox_Root}iCount -text "0"
	label ${ProcBox_Root}activePIDLabel -text "Active PID :"
	label ${ProcBox_Root}activePID -text "1"

	hierbox ${ProcBox_Root}pids -allowduplicates true -autocreate true \
		-scrollmode listbox -yscrollcommand "${ProcBox_Root}pidsScrollbar set"
	scrollbar ${ProcBox_Root}pidsScrollbar -command "${ProcBox_Root}pids yview"

	scale ${ProcBox_Root}refreshDelay -orient horizontal -showvalue true \
		-from 1000 -to 30000 -variable refreshDelayMS -label "Refresh Delay:" -bigincrement 1000
	${ProcBox_Root}refreshDelay set 2000

	button ${ProcBox_Root}refresh -text "Refresh" -command { ProcBox_Refresh }

	table ${ProcBox_Root} \
		${ProcBox_Root}iCountLabel 0,0 -cspan 2 -fill x \
		${ProcBox_Root}iCount 1,0 -cspan 2         \
		${ProcBox_Root}activePIDLabel 2,0 -cspan 2 -fill x \
		${ProcBox_Root}activePID  3,0 -cspan 2         \
		${ProcBox_Root}refreshDelay 4,0 -cspan 2 -fill x \
		${ProcBox_Root}refresh 5,0 -cspan 2         \
		${ProcBox_Root}pids 6,0 -fill both \
		${ProcBox_Root}pidsScrollbar 6,1 -fill y
	table configure ${ProcBox_Root} c1 r0 r1 r2 r3 r4 r5 -resize none
	table configure ${ProcBox_Root} c0 -resize expand

	#	${ProcBox_Root}pids 6,0 -fill both -columnweight full \
	after $refreshDelayMS { ProcBox_Refresh }

	#FirstReadLiveData
    #ParseStripLive

	return
}

######################################################################
#
# ProcBox_Refresh -- Refresh the contents of the process tracking
#			hierbox
#
######################################################################
proc ProcBox_Refresh {} {
	global refreshDelayMS ProcBox_Root


	#busy hold .
	set iCount "${ProcBox_Root}iCount"
	set hbox "${ProcBox_Root}pids"
	$iCount configure -text [CommaInsert [cycles]]
	set activePID [lindex [activeproc] 0]
	${ProcBox_Root}activePID configure -text $activePID

	# Go find the active process, make it visible and select it.
	foreach active [$hbox find -name "PID=$activePID"] {
		set active_state [$hbox entry cget [$hbox entry children $active 0 0] -text]
		if { [string compare $active_state "DEAD"] != 0 } {
			$hbox selection clear 0 end
			$hbox selection set $active
			$hbox show $active
		}
	}

	set pr [getprocs]
	#puts $pr

	# Clear out the hierbox
	#$hbox delete 0

	foreach x $pr {

        if { [catch {array set ProcInfo $x} errors] } {
            puts "ProcBox_Refresh: ERROR: getprocs did not return an even numbered list to be put in array"
            return;
        }

		set pid $ProcInfo(PID)
		set ppid $ProcInfo(PPID)
		set procname $ProcInfo(PROCNAME)
		set start_time $ProcInfo(START_TIME)

		set found 0

        #
        # Go see if we can find an existing entry in the hierbox for this PID
        #
		foreach p [$hbox find -name "PID=$pid"] {
            #
            # OK, we found an entry with the same PID but does the start time match
            # we could be reusing PIDs
            #
			if { $start_time == [$hbox entry cget $p -data] } {
                #
                # We found it, now update it accordingly
                #
				set found 1
				if { [string compare $ProcInfo(STATE) "DEAD"] == 0 } {
					$hbox entry configure $p -labelcolor red
				}
				$hbox entry configure $p -text $ProcInfo(PROCNAME)
				# State
				$hbox entry configure [$hbox entry children $p 0 0] -text $ProcInfo(STATE)
				# Start Time
				$hbox entry configure [$hbox entry children $p 1 1] -text $ProcInfo(START_TIME)
				# End Time
				$hbox entry configure [$hbox entry children $p 2 2] -text $ProcInfo(END_TIME)
				# Modules Count
				set old_mod_count [$hbox entry cget [$hbox entry children $p 3 3] -text]
				$hbox entry configure [$hbox entry children $p 3 3] -text $ProcInfo(MODULES)
				set modlist_idx [$hbox index [$hbox entry children $p 4 4]]

				# Now add all the new modules
				set idx 0
				foreach mods $ProcInfo(MODULE_LIST) {
					if { $idx >= $old_mod_count } {
                        if { [catch {array set ModuleInfo $mods} errors] } {
                            puts "ProcBox_Refresh: ERROR: module list ($ProcInfo(MODULE_LIST)) does not return an even numbered list to be put in array"
                            return;
                        }

						set last_idx [$hbox insert -at $modlist_idx end $ModuleInfo(PATH)]
						set text_idx [$hbox insert -at $last_idx end "TEXT"]
						$hbox insert -at $text_idx end $ModuleInfo(TXT_EA) -label "EA:" -text $ModuleInfo(TXT_EA)
						$hbox insert -at $text_idx end $ModuleInfo(TXT_VSID) -label "VSID:" -text $ModuleInfo(TXT_VSID)
						$hbox insert -at $text_idx end $ModuleInfo(TXT_LEN) -label "LEN:" -text $ModuleInfo(TXT_LEN)
						set data_idx [$hbox insert -at $last_idx end "DATA"]
						$hbox insert -at $data_idx end $ModuleInfo(DATA_EA) -label "EA:" -text $ModuleInfo(DATA_EA)
						$hbox insert -at $data_idx end $ModuleInfo(DATA_VSID) -label "VSID:" -text $ModuleInfo(DATA_VSID)
						$hbox insert -at $data_idx end $ModuleInfo(DATA_LEN) -label "LEN:" -text $ModuleInfo(DATA_LEN)
					}
					incr idx
				}

				# Process Type: Kernel/User
				$hbox entry configure [$hbox entry children $p 5 5] -text $ProcInfo(PROC_TYPE)
			}
		}

        #
        # If we didn't find it we need to find out where we should add it in the hierarchy
        #
		if { $found == 0 } {

            #
			# We want to insert this process id as a child of its parent
            #
			if { $pid == $ppid } {
				set parent_idx 0
			} else {
                #
                # Find the parent PID
                #
				set parent_idx 0
				foreach parent [$hbox find -name "PID=$ppid"] {
					set parent_state [$hbox entry cget [$hbox entry children $parent 0 0] -text]
					if { [string compare $parent_state "DEAD"] != 0 } {
						set parent_idx [$hbox index [$hbox entry children $parent end end]]
					}
				}
			}

			set idx [$hbox insert -at $parent_idx end "PID=$pid" -button true -labelcolor blue -text $ProcInfo(PROCNAME) -data $start_time]
			$hbox open $parent_idx
			# State
			$hbox insert -at $idx end $ProcInfo(STATE) -label "State: " -text $ProcInfo(STATE)
			# Start Inst
			$hbox insert -at $idx end $start_time -label "Start Time:" -text $ProcInfo(START_TIME)
			# End Inst
			$hbox insert -at $idx end $ProcInfo(END_TIME) -label "End Time:" -text $ProcInfo(END_TIME)
			# Modules
			$hbox insert -at $idx end $ProcInfo(MODULES) -label "Modules:" -text $ProcInfo(MODULES)
			# Modules
			set modlist_idx [$hbox insert -at $idx end Modules -text Modules]
			foreach mods $ProcInfo(MODULE_LIST) {
                if { [catch {array set ModuleInfo $mods} errors] } {
                    puts "ProcBox_Refresh: ERROR: module list ($ProcInfo(MODULE_LIST)) does not return an even numbered list to be put in array"
                    return;
                }

				set last_idx [$hbox insert -at $modlist_idx end $ModuleInfo(PATH)]
				set text_idx [$hbox insert -at $last_idx end "TEXT"]
				$hbox insert -at $text_idx end $ModuleInfo(TXT_EA) -label "EA:" -text $ModuleInfo(TXT_EA)
				$hbox insert -at $text_idx end $ModuleInfo(TXT_VSID) -label "VSID:" -text $ModuleInfo(TXT_VSID)
				$hbox insert -at $text_idx end $ModuleInfo(TXT_LEN) -label "LEN:" -text $ModuleInfo(TXT_LEN)
				set data_idx [$hbox insert -at $last_idx end "DATA"]
				$hbox insert -at $data_idx end $ModuleInfo(DATA_EA) -label "EA:" -text $ModuleInfo(DATA_EA)
				$hbox insert -at $data_idx end $ModuleInfo(DATA_VSID) -label "VSID:" -text $ModuleInfo(DATA_VSID)
				$hbox insert -at $data_idx end $ModuleInfo(DATA_LEN) -label "LEN:" -text $ModuleInfo(DATA_LEN)
			}
			# Process Type
			$hbox insert -at $idx end $ProcInfo(PROC_TYPE) -label "Proc Type:" -text $ProcInfo(PROC_TYPE)
			# Children List
			$hbox insert -at $idx end $ppid -label "Children:"
		}
	}
	#$hbox open -recurse 0

	# Read Live Stripstats Data
	#SubsequentReadLiveData
	#CacheMiss_Refresh

	# Clear out any existing after jobs
	foreach i [after info] {
		after cancel $i
	}

	#if { $activePID != -1 } {
		after $refreshDelayMS { ProcBox_Refresh }
	#}
	#puts "Ending refresh\n"

	#busy release .
}

vector create ICACHE_MISS
vector create DCACHE_MISS
vector create L2CACHE_MISS
vector create ICACHE_INDEX
vector create DCACHE_INDEX
vector create L2CACHE_INDEX
vector create HISTORY_INDEX
vector create HISTORY_MISS

set L1I {}
proc TrackCache {cache graph element x y} {
	global L1I
	if { [$graph element closest $x $y info $element] } {
		puts "$cache index is $info(index)"
		set $cache $info(index)
	}
}
proc ClearTrack {cache} {
	global L1I
	set $cache {}
}
proc CacheTrackMotion {g title elem x y} {
	global history_cache
	global history_cache_index
	global CacheMiss_Root
	if { [$g element closest $x $y info $elem] } {
		$g configure -title "$title : x=$info(x) y=$info(y)"
	}
	set history_cache $elem
	set history_cache_index $info(index)
	$CacheMiss_Root.history configure -title "${history_cache}-Cache Miss History for Index $history_cache_index"
}
######################################################################
#
# CacheMiss_Init -- Initalize the process tracking hierbox
#
######################################################################
proc CacheMiss_Init {root} {
	global CacheMiss_Root
	global ICACHE_MISS
	global DCACHE_MISS
	global L2CACHE_MISS
	global ICACHE_INDEX
	global DCACHE_INDEX
	global L2CACHE_INDEX
	global HISTORY_INDEX
	global HISTORY_MISS
	global history_cache
	global history_cache_index

	puts "running cachemiss_init\n"

	set CacheMiss_Root "$root"

	ICACHE_INDEX set [icache_index]
	ICACHE_MISS set [icache_index_miss]
	barchart $CacheMiss_Root.icache -barmode stacked -title "I-Cache Misses Per Index (All Samples)"
	$CacheMiss_Root.icache element create l1i -xdata ICACHE_INDEX -ydata ICACHE_MISS
	$CacheMiss_Root.icache legend configure -hide yes
	$CacheMiss_Root.icache element bind l1i <Shift-Motion> \
		{CacheTrackMotion $CacheMiss_Root.icache "I-Cache Misses Per Index" l1i %x %y}

	DCACHE_INDEX set [dcache_index]
	DCACHE_MISS set [dcache_index_miss]
	barchart $CacheMiss_Root.dcache -barmode stacked -title "D-Cache Misses Per Index (All Samples)"
	$CacheMiss_Root.dcache element create l1d -xdata DCACHE_INDEX -ydata DCACHE_MISS
	$CacheMiss_Root.dcache legend configure -hide yes
	$CacheMiss_Root.dcache element bind l1d <Shift-Motion> \
		{CacheTrackMotion $CacheMiss_Root.dcache "D-Cache Misses Per Index" l1d %x %y}

	L2CACHE_INDEX set [l2cache_index]
	L2CACHE_MISS set [l2cache_index_miss]
	barchart $CacheMiss_Root.l2cache -barmode stacked -title "L2-Cache Misses Per Index (All Samples)"
	$CacheMiss_Root.l2cache element create l2 -xdata L2CACHE_INDEX -ydata L2CACHE_MISS
	$CacheMiss_Root.l2cache legend configure -hide yes
	$CacheMiss_Root.l2cache element bind l2 <Shift-Motion> \
		{CacheTrackMotion $CacheMiss_Root.l2cache "L2-Cache Misses Per Index" l2 %x %y}

	set history_cache l1i
	set history_cache_index 0
	HISTORY_INDEX set [history_index]
	HISTORY_MISS set [${history_cache}cache_index_history $history_cache_index]
	barchart $CacheMiss_Root.history -barmode stacked -title "${history_cache}-Cache Miss History for Index $history_cache_index"
	$CacheMiss_Root.history element create h1 -xdata HISTORY_INDEX -ydata HISTORY_MISS
	$CacheMiss_Root.history legend configure -hide yes

	table $CacheMiss_Root \
		$CacheMiss_Root.icache  0,0 -cspan 2 -fill x \
		$CacheMiss_Root.dcache  1,0 -cspan 2 -fill x \
		$CacheMiss_Root.l2cache 2,0 -cspan 2 -fill x \
		$CacheMiss_Root.history 3,0 -cspan 2 -fill x

	puts "leaving cachemiss_init"
}

######################################################################
#
# CacheMiss_Refresh -- Refresh the contents of the process tracking
#			hierbox
#
######################################################################
proc CacheMiss_Refresh {} {
	global CacheMiss_Root
	global ICACHE_INDEX
	global DCACHE_INDEX
	global L2CACHE_INDEX
	global ICACHE_MISS
	global DCACHE_MISS
	global L2CACHE_MISS
	global HISTORY_INDEX
	global HISTORY_MISS
	global history_cache
	global history_cache_index
	global L1I

	ICACHE_INDEX set [icache_index]
	$CacheMiss_Root.icache axis configure x -stepsize [expr ( $ICACHE_INDEX(max) + 1 ) / 16]
	if { [string compare "$L1I" ""] == 0 } {
		$CacheMiss_Root.icache configure -title "I-Cache Misses Per Index ( All Samples )"
		ICACHE_MISS set [icache_index_miss]
	} else {
		$CacheMiss_Root.icache configure -title "I-Cache Misses Per Index ( Sample # $L1I )"
		ICACHE_MISS set [icache_index_miss $L1I]
	}
	DCACHE_INDEX set [dcache_index]
	$CacheMiss_Root.dcache axis configure x -stepsize [expr ( $DCACHE_INDEX(max) + 1 ) / 16]
	DCACHE_MISS set [dcache_index_miss]
	L2CACHE_INDEX set [l2cache_index]
	$CacheMiss_Root.l2cache axis configure x -stepsize [expr ( $L2CACHE_INDEX(max) + 1 ) / 16]
	L2CACHE_MISS set [l2cache_index_miss]

	HISTORY_INDEX set [history_index]
	HISTORY_MISS set [${history_cache}cache_index_history $history_cache_index]
	$CacheMiss_Root.history axis configure x -stepsize [expr ( $HISTORY_INDEX(max) + 1 ) / 16]
}
