# --------------------------------------------------------------  
# (C)Copyright 2001,2007
# International Business Machines Corporation,
# Sony Computer Entertainment, Incorporated,
# Toshiba Corporation,
#
# All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the
# following conditions are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# - Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials
#   provided with the distribution.
#
# - Neither the name of IBM Corporation nor the names of its
#   contributors may be used to endorse or promote products
#   derived from this software without specific prior written
#   permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --------------------------------------------------------------
# PROLOG END TAG zYx                                              

# --------------------------------------------------------------
# sim_open (syscall w/ rb=1)
# --------------------------------------------------------------

	# Inputs
	#   r3 = pointer to file to be opened
	#   r4 = flags
	#
	# Outputs
	#   r3 = file descriptor of the file opened.
		
	.text
	.align	3
	.global	sim_open
	.type	sim_open, @function


sim_open:
	# Pack parms into r3 = (ptr, flags, xxx, xxx)

	rotqbyi $3, $3, 4
	rotqmbyi $4, $4, -4
	shlqbyi $3, $3, 12
	or	$3, $3, $4
	syscall $3, $3, 1
	bi 	$0


# --------------------------------------------------------------
# sim_close (syscall w/ rb=2)
# --------------------------------------------------------------

        # Inputs
	#   r3 = file descriptor to be closed.
	#
	# Outputs
	#   r3 = status
		
	.text
	.align	3
	.global	sim_close
	.type	sim_close, @function


sim_close:
	syscall $3, $3, 2
	bi $0

	
# --------------------------------------------------------------
# sim_read (syscall w/ rb=3)
# --------------------------------------------------------------

        # Inputs
	#   r3 = file descriptor
	#   r4 = buffer pointer
	#   r5 = number of bytes to read
	#
	# Outputs
	#   r3 = number of bytes read
		
        .text
	.align	3
	.global	sim_read
	.type	sim_read, @function


sim_read:
	# Pack parms into r3 = (fd, ptr, count, xxx)
	rotqbyi $3, $3, 4
	rotqmbyi $4, $4, -12
	rotqmbyi $5, $5, -12
	shlqbyi $3, $3, 12
	shlqbyi $4, $4, 8
	shlqbyi $5, $5, 4
	or	$3, $3, $4
	or	$3, $3, $5
	syscall $3, $3, 3
	bi 	$0


# --------------------------------------------------------------  
# sim_write (syscall w/ rb=4)
# --------------------------------------------------------------  
	
	# Inputs
	#   r3 = file descriptor
	#   r4 = buffer pointer
	#   r5 = number of bytes to write
	#
	# Outputs
	#   r3 = number of bytes wrote
		
	.text
	.align	3
	.global	sim_write
	.type	sim_write, @function


sim_write:
	# Pack parms into r3 = (fd, ptr, count, xxx)
	rotqbyi $3, $3, 4
	rotqmbyi $4, $4, -12
	rotqmbyi $5, $5, -12
	shlqbyi $3, $3, 12
	shlqbyi $4, $4, 8
	shlqbyi $5, $5, 4
	or	$3, $3, $4
	or	$3, $3, $5
	syscall $3, $3, 4
	bi	$0

        
# --------------------------------------------------------------  
# sim_start_timer (syscall w/ rb=13)
# --------------------------------------------------------------  
	
	# Inputs
	#   none
	#
	# Outputs
	#   none
		
	.text
	.align	3
	.global	sim_start_timer
	.type	sim_start_timer, @function


sim_start_timer:
	syscall $0, $0, 13
	bi 	$0

        
# --------------------------------------------------------------  
# sim_stop_timer (syscall w/ rb=14)
# --------------------------------------------------------------  
	
	# Inputs
	#   none
	#
	# Outputs
	#   none
		
	.text
	.align	3
	.global	sim_stop_timer
	.type	sim_stop_timer, @function


sim_stop_timer:
	syscall $0, $0, 14
	bi 	$0

        
        
# --------------------------------------------------------------  
# sim_printf (syscall w/ rb=21)
# --------------------------------------------------------------  

	.text
	.align	3
	.global	sim_printf
	.type	sim_printf, @function

        
sim_printf:
	syscall $0, $0, 21
	bi $0


# --------------------------------------------------------------  
# sim_instructions (syscall w/ rb=42)
# --------------------------------------------------------------  
	
	# Inputs
	#   none
	#
	# Outputs
	#   r3 = number of instructions executed
		
	.text
	.align	3
	.global	sim_instructions
	.type	sim_instructions, @function


sim_instructions:
	syscall $3, $3, 42
	bi $0

        
# --------------------------------------------------------------  
# sim_cycles (syscall w/ rb=43)
# --------------------------------------------------------------
	
	# Inputs
	#   none
	#
	# Outputs
	#   r3 = number of cycles executed
		
	.text
	.align	3
	.global	sim_cycles
	.type	sim_cycles, @function


sim_cycles:
	syscall $3, $3, 43
	bi $0


# --------------------------------------------------------------  
# sim_lseek (syscall w/ rb=52)
# --------------------------------------------------------------  
	
	# Inputs
	#   r3 = file descriptor
	#   r4 = offset
	#   r5 = whence
	#
	# Outputs
	#   r3 = offset
		
	.text
	.align	3
	.global	sim_lseek
	.type	sim_lseek, @function


sim_lseek:
	# Pack parms into r3 = (fd, offset, whence, xxx)
	rotqbyi $3, $3, 4
	rotqmbyi $4, $4, -12
	rotqmbyi $5, $5, -12
	shlqbyi $3, $3, 12
	shlqbyi $4, $4, 8
	shlqbyi $5, $5, 4
	or	$3, $3, $4
	or	$3, $3, $5
	syscall $3, $3, 52
	bi $0
